﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Web;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

/// <summary>
/// Summary description for MenuBuilder
/// </summary>
public static class MenuBuilder
{

    /// <summary>
    /// Builds left menu based on top menu location and pages accessible by user
    /// </summary>
    /// <param name="horizontalMenu"></param>
    /// <param name="pageTopMenuItem"></param>
    /// <param name="leftMenu"></param>
    /// <param name="pageLeftMenuItem"></param>
    /// <param name="breadCrumb"></param>
    /// <param name="accessiblePageList"></param>
    public static void BuildDynamicNavigationItems(string currentURL, Menu horizontalMenu, string pageTopMenuItem, Menu leftMenu, string pageLeftMenuItem, Label breadCrumb, string[] accessiblePageList)
    {   
        //menu items that need name manipulated after breadcrumb is built
        MenuItem menuItemNew = null;
              
        switch (pageTopMenuItem)
        {
            case "My Tasks":

                MenuItem parentReferral = new MenuItem("Referrals", "DVEIR_REFERRAL");
                parentReferral.Selectable = false;
                leftMenu.Items.Add(parentReferral);
                
                menuItemNew = new MenuItem("New", "DVEIR_REFERRAL", "", "Referrals.aspx?id=11");
                menuItemNew.ToolTip = "Select to view New Referrals";
                parentReferral.ChildItems.Add(menuItemNew);

                menuItemNew = new MenuItem("In Process", "DVEIR_REFERRAL", "", "Referrals.aspx?id=12");
                menuItemNew.ToolTip = "Select to view In Process Referrals";
                parentReferral.ChildItems.Add(menuItemNew);

                menuItemNew = new MenuItem("Returned", "DVEIR_REFERRAL", "", "Referrals.aspx?id=21");
                menuItemNew.ToolTip = "Select to view Returned Referrals";
                parentReferral.ChildItems.Add(menuItemNew);

                menuItemNew = new MenuItem("Ready for Review", "DVEIR_REFERRAL", "", "Referrals.aspx?id=13");
                menuItemNew.ToolTip = "Select to view Referrals Ready for Review";
                parentReferral.ChildItems.Add(menuItemNew);

                menuItemNew = new MenuItem("Ready to Send", "DVEIR_REFERRAL", "", "Referrals.aspx?id=14");
                menuItemNew.ToolTip = "Select to view Referrals Ready to Send to DVEIVR";
                parentReferral.ChildItems.Add(menuItemNew);

                menuItemNew = new MenuItem("Completed", "DVEIR_REFERRAL", "", "Referrals.aspx?id=15");
                menuItemNew.ToolTip = "Select to view Completed Referrals";
                parentReferral.ChildItems.Add(menuItemNew);
                
                break;

            case "Patients":

                menuItemNew = new MenuItem("Patient Search", "DVEIR PATIENTS", "", "Patients.aspx");
                menuItemNew.ToolTip = "Select to search for Patients in DVEIR";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("VISN/VAMC Search", "DVEIR PATIENTS LOC", "", "PatientLocations.aspx");
                menuItemNew.ToolTip = "Select to search for Patients by VISN/VAMC";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Patient Request", "DVEIR PATIENT REQUESTS", "", "PatientRequests.aspx");
                menuItemNew.ToolTip = "Select to request a Patient be added to DVEIR";
                leftMenu.Items.Add(menuItemNew);

                break;

            case "Reporting":

                menuItemNew = new MenuItem("Referral Overview", "REFERRAL_OVERVIEW_REPORT", "", "ReferralOverviewReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Referral Overview report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Annual Completion", "REFERRAL_YEARLY_BREAKDOWN_REPORT", "", "ReferralYearlyBreakdownReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Annual Completion report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Annual Diagnoses", "REFERRAL_YEARLY_DIAGNOSIS_REPORT", "", "ReferralYearlyDiagnosesReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Annual Diagnoses report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Annual Procedures", "AVERAGE_PROCEDURE_PER_DIAGNOSIS_REPORT", "", "AverageProcedureDiagnosesReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Annual Procedures report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Average Time", "AVERAGE_TIME_FOR_REFERRAL_COMPLETION_REPORT", "", "AverageTimeForReferralCompletionReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Average Time report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("User Event Log", "USER_EVENT_LOG_REPORT", "", "UserEventLogReport.aspx");
                menuItemNew.ToolTip = "Select to generate the User Event Log report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("DoD Interface Transfer", "DVEIR_REPORT_DOD_INTERFACE_TRANSFER", "", "DveirReport.aspx?id=" + DVEIRSharedConstant.DveirReportsDoDTransfer);
                menuItemNew.ToolTip = "Select to generate the DoD Interface Transfer report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Frequency of Fragments", "DVEIR_REPORT_FREQUENCY_OCULAR_EMBEDDED_FRAGMENTS", "", "DveirReport.aspx?id=" + DVEIRSharedConstant.DveirReportsFrequencyEmbeddedFragment);
                menuItemNew.ToolTip = "Select to generate the Frequency of Fragments report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Treatment for Fragments", "DVEIR_REPORT_TREATMENT_EMBEDDED_FRAGMENTS", "", "DveirReport.aspx?id=" + DVEIRSharedConstant.DveirReportsTreatmentEmbeddedFragment);
                menuItemNew.ToolTip = "Select to generate the Treatment for Fragments report";
                leftMenu.Items.Add(menuItemNew);

                menuItemNew = new MenuItem("Unique Patient", "UNIQUE_PATIENT_REPORT", "", "UniquePatientReport.aspx");
                menuItemNew.ToolTip = "Select to generate the Unique Patient report";
                leftMenu.Items.Add(menuItemNew);

                break;

            case "Dashboard":

                //leftMenu.Items.Add(new MenuItem("Main", "DVEIR DASHBOARD", "", "Dashboard.aspx"));

                break;
            case "Administration":

                menuItemNew = new MenuItem("Users", "DVEIR ADMIN LIST USERS", "", "AdminListUsers.aspx");
                menuItemNew.ToolTip = "Select to view and administer DVEIR users";
                leftMenu.Items.Add(menuItemNew);

                //leftMenu.Items.Add(new MenuItem("Add / Edit Users", "DVEIR ADMIN EDIT USERS", "", "AdminEditUsers.aspx"));

                menuItemNew = new MenuItem("Role Matrix", "DVEIR ADMIN USER ROLE", "", "AdminEditUsersRoles.aspx");
                menuItemNew.ToolTip = "Select to view and administer DVEIR roles";
                leftMenu.Items.Add(menuItemNew);

                //leftMenu.Items.Add(new MenuItem("Edit User VISN Access", "DVEIR ADMIN USER VISN", "", "AdminEditUsersVisn.aspx"));

                break;
            default:

                leftMenu.Items.Add(new MenuItem("Top Menu Not Registered"));
              
                break;
        }

        //Disable all menu items
        foreach (MenuItem item in leftMenu.Items)
                ChangeMenuState(item, false);

        //Enable selection for the accessible items only
        for (int i = 0; i < accessiblePageList.GetLength(0); i++)
            ChangeMenuState(leftMenu.FindItem(accessiblePageList[i]), true);

        BuildBreadCrumbDynamic(currentURL, pageTopMenuItem, leftMenu, pageLeftMenuItem, breadCrumb, accessiblePageList);
    }

    /// <summary>
    /// Builds bread crumb based item in selected menu
    /// </summary>
    /// <param name="horizontalMenu"></param>
    /// <param name="pageTopMenuItem"></param>
    /// <param name="leftMenu"></param>
    /// <param name="pageLeftMenuItem"></param>
    /// <param name="breadCrumb"></param>
    /// <param name="accessiblePageList"></param>
    public static void BuildBreadCrumbDynamic(string currentURL, string pageTopMenuItem, Menu leftMenu, string pageLeftMenuItem, Label breadCrumb, string[] accessiblePageList)
    {
        
        MenuItem currentMenuItem = null;



        //if (leftMenu.Items.Count > 0)
        //{
        //    currentMenuItem = leftMenu.Items[0];
        //}
        foreach (MenuItem item in leftMenu.Items)
        {
            currentMenuItem = SeachItemAndChildrenByURL(item, currentURL);
            if (currentMenuItem != null) break;
        }

        if (currentMenuItem == null)
        {
            currentMenuItem = leftMenu.FindItem(pageLeftMenuItem);
        }


        if (currentMenuItem != null)
        {
            if (currentMenuItem.Selectable)
            {
                currentMenuItem.Selected = true;
            }
            
            //initialize breadcrumb with currently selected menu item
            string breadCrumbPath = currentMenuItem.Text;

            //add to path all parents of selected menu item
            while (currentMenuItem.Parent != null)
            {
                //set current menutree location to parent
                currentMenuItem = currentMenuItem.Parent;

                breadCrumbPath = currentMenuItem.Text + " > " + breadCrumbPath;
            }

            //add top menu to beginning of breadcrumb
            breadCrumbPath = pageTopMenuItem + " > " + breadCrumbPath;

            breadCrumb.Text = breadCrumbPath;
        }
    }

    public static MenuItem SeachItemAndChildrenByURL(MenuItem item, string URL)
    {
        MenuItem foundItem = null;
         
        string[] pathElems = HttpContext.Current.Request.Url.PathAndQuery.Split(new char[] { '/' });
        string pageName =
        pathElems[pathElems.Length - 1];

        //foreach (MenuItem elem in menu)
        //{
        if (item.NavigateUrl == pageName)
            return item;

        if (item.ChildItems.Count > 0)
        {
            foreach (MenuItem subitem in item.ChildItems)
            {
                foundItem = SeachItemAndChildrenByURL(subitem, pageName);
                if (foundItem != null) 
                    return foundItem;
            }
        }

        return null;

    }

    /// <summary>
    /// Changes menu state recursively
    /// </summary>
    /// <param name="item"></param>
    /// <param name="flag"></param>
    private static void ChangeMenuState(MenuItem item, bool flag)
    {
        if (item != null)
        {
            if (flag == false) item.Selectable = flag;
            if (flag == true && item.ChildItems.Count == 0) item.Selectable = flag;
            if (item.ChildItems.Count > 0)
                foreach (MenuItem subitem in item.ChildItems)
                    ChangeMenuState(subitem, flag);
        }
    }

    /// <summary>
    /// Search and highlight appropriate menu item
    /// </summary>
    /// <param name="menu">Collection of Menu Items</param>
    /// <param name="value">Item to Search For</param>
    private static void SetMenuItemHighlight(MenuItemCollection menu, string value)
    {

        foreach ( MenuItem elem in menu)
        {
            if (elem.Value == value)
            {
                if (elem.Selectable)
                {
                    elem.Selected = true;
                    break;
                }
            }
            
            if (elem.ChildItems.Count > 0)
                SetMenuItemHighlight(elem.ChildItems, value);
        }
    }

 
}
