﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;

public partial class Patient : BasePage 
{

    DVEIRPatientManager patientManager = new DVEIRPatientManager();
    PATIENT patient;


    public override string TopMenuName()
    {
        return "Patients";
    }

    public override string AppObjectCode()
    {
        return "DVEIR PATIENT";
    }

    public override string MenuItemName()
    {
        return "DVEIR PATIENTS";
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            //BreadCrumbLabel.Text = BreadCrumbLabel.Text + Request.QueryString["id"].ToString();
 
            int patientId;
            if (int.TryParse(Request.QueryString["id"], out patientId))
            {
                
                getRefreshPatient(patientId);

                HiddenFieldPatientID.Value = patientId.ToString();

                LabelBreadCrumb.Text = LabelBreadCrumb.Text + " > Patient ID: " + patientId.ToString();
                
                //SetupDemographics();

                CreateLoggingRecord(patientId, DVEIRUserActivityType.PATIENT_SEARCH_SELECT, "");
            }
        }

        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewDVEIRReferrals.ClientID + "');", true);
    }

    private void getRefreshPatient(int oId)
    {
        patient = patientManager.SelectByID(oId);
        if (patient != null) patient.SetAsChangeTrackingRoot();

        //initial display panels
        PanelNoAccess.Visible = false;
        PanelIdNotFound.Visible = false;
        PanelObjectContent.Visible = false;

        if (patient == null)
        {
            PanelObjectContent.Visible = false;

            if (patientManager.PatientExists(oId))
            {
                PanelNoAccess.Visible = true;
            }
            else
            {
                PanelIdNotFound.Visible = true;
            }
        }
        else
        {
            PanelObjectContent.Visible = true;
        }
    }

    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);
        string patientXml = (string)ViewState["patientXml"];
        patient = SerializerUtil.Deserialize<PATIENT>(patientXml);

    }

    protected override object SaveViewState()
    {
        ViewState["patientXml"] = patient.Serialize();
        return base.SaveViewState();

    }

    private void SetupDemographics()
    {
        if (patient.REGISTRY_DEMOGRAPHICs.Count > 0)
        {
            REGISTRY_DEMOGRAPHIC demo = patient.REGISTRY_DEMOGRAPHICs.First();

            txtStreetAddress1.Text = demo.ADDRESS_LINE1;
            txtStreetAddress2.Text = demo.ADDRESS_LINE2;
            txtStreetAddress3.Text = demo.ADDRESS_LINE3;
            txtAddressCity.Text = demo.CITY;
            txtAddressState.Text = demo.STATE;
            txtAddressZip.Text = demo.POSTAL_CODE;
            txtAddressCountry.Text = demo.COUNTRY;
            txtHomePhone.Text = demo.HOME_PHONE;
            txtWorkPhone.Text = demo.WORK_PHONE;
            txtMobilePhone.Text = demo.CELL_PHONE;
            txtEmailAddress.Text = demo.EMAIL_ADDRESS;  
        }
    }

    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = e.Row.Cells[0].Text;

            Button btn = e.Row.Cells[3].Controls[0] as Button;
            if (btn != null)
            {
                btn.ToolTip = "Select the record for ID " + s1;
            }
        }
    }

    private void SaveCompletePatientDemographics()
    {

        REGISTRY_DEMOGRAPHIC demo;

        if (patient.REGISTRY_DEMOGRAPHICs.Count > 0)
        {

            demo = patient.REGISTRY_DEMOGRAPHICs.First();

            SaveDemographics(ref demo);

        }
        else
        {
            demo = new REGISTRY_DEMOGRAPHIC();

            SaveDemographics(ref demo);

            demo.PATIENT_ID = patient.PATIENT_ID;
            patient.REGISTRY_DEMOGRAPHICs.Add(demo);
        }
        //set questionnaire in-process status
        patientManager.Update(patient);
        getRefreshPatient(patient.PATIENT_ID);

    }

    private void SaveDemographics(ref REGISTRY_DEMOGRAPHIC rdemo)
    {

        rdemo.ADDRESS_LINE1 = txtStreetAddress1.Text.Trim() ;
        rdemo.ADDRESS_LINE2 = txtStreetAddress2.Text.Trim() ;
        rdemo.ADDRESS_LINE3 = txtStreetAddress3.Text.Trim() ;
        rdemo.CITY = txtAddressCity.Text.Trim() ;
        rdemo.STATE = txtAddressState.Text.Trim() ;
        rdemo.POSTAL_CODE = txtAddressZip.Text.Trim() ;
        rdemo.COUNTRY = txtAddressCountry.Text ;
        rdemo.HOME_PHONE = txtHomePhone.Text ;
        rdemo.WORK_PHONE = txtWorkPhone.Text ;
        rdemo.CELL_PHONE = txtMobilePhone.Text  ;
        rdemo.EMAIL_ADDRESS = txtEmailAddress.Text ;  
        //DVEIR registry
        rdemo.STD_REGISTRY_ID = CurrentUser.RegistryId; // 3;
        //alt address 
        rdemo.STD_ADDRESSTYPE_ID = 1;

        
    }

    public void GridViewDVEIRReferrals_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int index = Convert.ToInt32(e.CommandArgument);
            GridViewDVEIRReferrals.SelectedIndex = index;
            int id = Convert.ToInt32(GridViewDVEIRReferrals.SelectedValue);
            //calling the right page for different order lab order type
            Response.Redirect(string.Format("Referral.aspx?id={0}", id));


        }

    }

    protected void SaveButton_Click(object sender, EventArgs e)
    {
        
        SaveCompletePatientDemographics();

        SetupDemographics();

        (this.Master as RegistryMaster).SetDirtyFlag(false);
    }

    

}


