﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.DVEIR;
using System.Text;

public partial class PatientRequests : BasePage
{

    VAPatientManager _vaPatientManager = new VAPatientManager();
    DVEIRPatientManager _dveirPatientManager = new DVEIRPatientManager();
    Staging_DVEIR_PATIENTREQUEST _PatientRequest = new Staging_DVEIR_PATIENTREQUEST();

    private bool _resetDisplayMode = false;

    public enum DisplayModeType
    {
        Search,
        SearchResults,
        SubmitFromVaMasterList,
        SubmitManual,
        Submitted,
        Confirm

    }

    private DisplayModeType _displayMode;

    public DisplayModeType DisplayMode
    {
        get { return _displayMode; }
        set { _displayMode = value; }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            DisplayMode = DisplayModeType.Search;
            
            RangeValidatorTextBoxDateOfBirth.MinimumValue =
                RangeValidatorTextBoxDateOfBirthSubmit.MinimumValue = 
                    DateTime.MinValue.ToShortDateString();
            RangeValidatorTextBoxDateOfBirth.MaximumValue =
                RangeValidatorTextBoxDateOfBirthSubmit.MaximumValue = 
                    DateTime.Now.AddYears(-18).ToShortDateString();


            _resetDisplayMode = true;
        }

       // SetDisplayForMode();

        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewVAPatients.ClientID + "');", true);
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (_resetDisplayMode)
        {
            SetDisplayForMode();
        }
        
        DisplayErrorMessage();
    }

    protected void SetDisplayForMode()
    {
        
        switch (DisplayMode)
        {
            case DisplayModeType.Search:

                PanelPatientFields.GroupingText = "Select patient from VA master patient list&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""excludeSaveWarnings"">";

                LabelPageInstructions.Visible = true;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "To begin, search for the patient that you want to request to include in the registry.  At least one field below is required.  <br><br>If a SSN is entered into the search criteria, the other search fields will be cleared and not editable.";
                LabelSearchNote.Visible = true;

                LabelOrMessage.Visible = true;
                PanelManualRequestStart.Visible = true;

                ButtonSearchPatient.Visible = true;
                ButtonSubmitRequestFromVAMasterList.Visible = false;
                ButtonSubmitRequestManual.Visible = false;

                ButtonCancel.Visible = false;
                ButtonConfirm.Visible = false;
                ButtonSearchAgain.Visible = false;

                LabelSearchResultInformation.Visible =
                GridViewVAPatients.Visible = false;

                TextBoxPatientSSNSearch.Text = "";
                TextBoxPatientFirstNameSearch.Text = "";
                TextBoxMiddleName.Text = "";
                TextBoxPatientLastNameSearch.Text = "";
                TextBoxDateOfBirth.Text = "";

                SetRequiredFieldValidatorEnabledStatus(false);

                SetFieldReadOnlyStatus(false);

                HiddenFieldFocusOnBreadCrumb.Value = "0";
                HiddenFieldSearchMode.Value = "SearchMode";
                TextBoxPatientSSNSearch.Focus();

                break;

            case DisplayModeType.SearchResults:

                PanelPatientFields.GroupingText = "Select patient from VA master patient list&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""excludeSaveWarnings"">";

                LabelPageInstructions.Visible = true;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "To begin, search for the patient that you want to request to include in the registry.  At least one field below is required.";
                
                LabelSearchNote.Visible = true;

                LabelOrMessage.Visible = true;
                PanelManualRequestStart.Visible = true;

                ButtonSearchPatient.Visible = true;
                ButtonSubmitRequestFromVAMasterList.Visible = false;
                ButtonSubmitRequestManual.Visible = false;
                
                ButtonCancel.Visible = false;
                ButtonConfirm.Visible = false;
                ButtonSearchAgain.Visible = false;

                SetRequiredFieldValidatorEnabledStatus(false);
                
                SetFieldReadOnlyStatus(false);

                LabelSearchResultInformation.Visible = GridViewVAPatients.Visible;

                HiddenFieldSearchMode.Value = "SearchMode";

                HiddenFieldFocusOnBreadCrumb.Value = "0";

                break;

            case DisplayModeType.SubmitFromVaMasterList:

                PanelPatientFields.GroupingText = "Submit Request&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""placeholderDiv"">";

                LabelPageInstructions.Visible = false;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "Confirm your selection and click \"submit request\" to finish the request process which will add this patient to the VA eye injury data store.";
                LabelSearchNote.Visible = false;

                LabelOrMessage.Visible = false;
                PanelManualRequestStart.Visible = false;

                //make patient data read only where it exists
                if (TextBoxPatientSSNSearch.Text.Length !=0)
                    TextBoxPatientSSNSearch.Enabled = false;
                if (TextBoxPatientFirstNameSearch.Text.Length != 0)
                    TextBoxPatientFirstNameSearch.Enabled = false;
                if (TextBoxMiddleName.Text.Length != 0)
                    TextBoxMiddleName.Enabled = false;
                if (TextBoxPatientLastNameSearch.Text.Length != 0)
                    TextBoxPatientLastNameSearch.Enabled = false;
                if (TextBoxDateOfBirth.Text.Length != 0)
                    TextBoxDateOfBirth.Enabled = false;

                SetRequiredFieldValidatorEnabledStatus(true);
                
                ButtonSearchPatient.Visible = false;
                ButtonSubmitRequestFromVAMasterList.Visible = true;
                ButtonSubmitRequestManual.Visible = false;

                ButtonCancel.Visible = true;
                ButtonConfirm.Visible = false;
                ButtonSearchAgain.Visible = false;

                LabelSearchResultInformation.Visible = 
                    GridViewVAPatients.Visible = false;

                HiddenFieldFocusOnBreadCrumb.Value = "1";

                HiddenFieldSearchMode.Value = "";


                break;

            case DisplayModeType.SubmitManual:

                PanelPatientFields.GroupingText = "Submit Request&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""placeholderDiv"">";

                LabelPageInstructions.Visible = false;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "Enter the patient information and click submit request to continue. Required fields are marked with an asterisk.";

                LabelSearchNote.Visible = false;

                LabelOrMessage.Visible = false;
                PanelManualRequestStart.Visible = false;

                SetRequiredFieldValidatorEnabledStatus(true);

                ButtonSearchPatient.Visible = false;
                ButtonSubmitRequestFromVAMasterList.Visible = false;
                ButtonSubmitRequestManual.Visible = true;
                
                ButtonCancel.Visible = true;
                ButtonConfirm.Visible = false;
                ButtonSearchAgain.Visible = false;

                LabelSearchResultInformation.Visible = 
                    GridViewVAPatients.Visible = false;

                SetFieldReadOnlyStatus(false);

                TextBoxPatientSSNSearch.Focus();
                
                HiddenFieldFocusOnBreadCrumb.Value = "0";

                HiddenFieldSearchMode.Value = "";


                break;

            case DisplayModeType.Confirm:

                PanelPatientFields.GroupingText = "Submit Request&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""placeholderDiv"">";

                LabelPageInstructions.Visible = false;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "Confirm your selection and click \"submit request\" to finish the request process which will add this patient to the VA eye injury data store.";
                LabelSearchNote.Visible = false;

                LabelOrMessage.Visible = false;
                PanelManualRequestStart.Visible = false;

                SetRequiredFieldValidatorEnabledStatus(true);

                ButtonSearchPatient.Visible = false;
                ButtonSubmitRequestFromVAMasterList.Visible = false;
                ButtonSubmitRequestManual.Visible = false;
                
                ButtonCancel.Visible = true;
                ButtonConfirm.Visible = true;
                ButtonSearchAgain.Visible = false;

                LabelSearchResultInformation.Visible = 
                    GridViewVAPatients.Visible = false;

                SetFieldReadOnlyStatus(true);

                HiddenFieldFocusOnBreadCrumb.Value = "1";

                HiddenFieldSearchMode.Value = "";

                break;

            case DisplayModeType.Submitted:

                PanelPatientFields.GroupingText = "Submit Request&nbsp;&nbsp;";

                LiteralExcludeSaveWarningDiv.Text = @"<div class = ""placeholderDiv"">";

                LabelPageInstructions.Visible = false;
                LabelSearchInstruction.Visible = true;
                LabelSearchInstruction.Text = "Request Submitted. The request will be examined and processed as soon as possible.";

                LabelWarningNotificationMessage.Visible = true;
                LabelWarningNotificationMessage.Text =
                    "Request Submitted. The request will be examined and processed as soon as possible.";
                
                LabelSearchNote.Visible = false;

                LabelOrMessage.Visible = false;
                PanelManualRequestStart.Visible = false;

                ButtonSearchPatient.Visible = false;
                ButtonSubmitRequestFromVAMasterList.Visible = false;
                ButtonSubmitRequestManual.Visible = false;
                ButtonCancel.Visible = false;
                ButtonConfirm.Visible = false;
                ButtonSearchAgain.Visible = true;

                LabelSearchResultInformation.Visible = 
                    GridViewVAPatients.Visible = false;

                TextBoxPatientSSNSearch.ReadOnly = true;
                TextBoxPatientFirstNameSearch.ReadOnly = true;
                TextBoxMiddleName.ReadOnly = true;
                TextBoxPatientLastNameSearch.ReadOnly = true;
                TextBoxDateOfBirth.ReadOnly = true;

                SetRequiredFieldValidatorEnabledStatus(false);

                SetFieldReadOnlyStatus(true);

                HiddenFieldFocusOnBreadCrumb.Value = "1";

                HiddenFieldSearchMode.Value = "";

                break;            
        }

    }

    protected void SetRequiredFieldValidatorEnabledStatus(bool enabled)
    {
        //set enable status for all required field validators: SSN,First Name,Last Name,Date of Birth
        RequiredFieldValidatorDateOfBirth.Enabled = enabled;
        RequiredFieldValidatorFirstName.Enabled = enabled;
        RequiredFieldValidatorLastName.Enabled = enabled;
        RequiredFieldValidatorSSN.Enabled = enabled;
 
        SSNValidatorSubmit.Enabled = enabled;
        RegularExpressionValidatorFirstNameSubmit.Enabled = enabled;
        RegularExpressionValidatorMiddleNameSubmit.Enabled = enabled;
        RegularExpressionValidatorLastNameSubmit.Enabled = enabled;
        RangeValidatorTextBoxDateOfBirthSubmit.Enabled = enabled;
        RangeValidatorTextBoxDateOfBirth.Enabled = !RangeValidatorTextBoxDateOfBirthSubmit.Enabled;
        
        //change display of field labels
        if (enabled)
        {
            LabelforTextBoxDateOfBirth.Text = "Date of Birth (mm/dd/yyyy)*:";
            LabelforTextBoxPatientFirstNameSearch.Text = "First Name*:";
            LabelforTextBoxPatientLastNameSearch.Text = "Last Name*:";
            LabelforTextBoxSSNSearch.Text = "SSN*:";
        }
        else 
        {
            LabelforTextBoxDateOfBirth.Text = "Date of Birth (mm/dd/yyyy):";
            LabelforTextBoxPatientFirstNameSearch.Text = "First Name:";
            LabelforTextBoxPatientLastNameSearch.Text = "Last Name:";
            LabelforTextBoxSSNSearch.Text = "SSN:";
        }

    }

    protected void SetFieldReadOnlyStatus(bool readOnly)
    {
        TextBoxPatientSSNSearch.Enabled = !readOnly;
        TextBoxPatientFirstNameSearch.Enabled = !readOnly;
        TextBoxMiddleName.Enabled = !readOnly;
        TextBoxPatientLastNameSearch.Enabled = !readOnly;
        TextBoxDateOfBirth.Enabled = !readOnly;

        TextBoxPatientSSNSearch.ReadOnly = readOnly;
        TextBoxPatientFirstNameSearch.ReadOnly = readOnly;
        TextBoxMiddleName.ReadOnly = readOnly;
        TextBoxPatientLastNameSearch.ReadOnly = readOnly;
        TextBoxDateOfBirth.ReadOnly = readOnly;

    }

    protected void ButtonCancel_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = "";
        DisplayMode = DisplayModeType.Search;

        _resetDisplayMode = true;
    }

    protected void ButtonSearchAgain_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = ""; 
        DisplayMode = DisplayModeType.Search;

        _resetDisplayMode = true;
    }

    protected void ButtonSubmitRequestFromVAMasterList_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = "";
        CreateRecordForRequest();

        _resetDisplayMode = true;
    }

    protected void ButtonConfirm_Click(object sender, EventArgs e)
    {
        //to do
        LabelWarningNotificationMessage.Text = "";

        _resetDisplayMode = true;
    }

    protected void ButtonGoToManualEntry_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = "";
        DisplayMode = DisplayModeType.SubmitManual;

        _resetDisplayMode = true;
    }

    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = e.Row.Cells[0].Text;

            Button btn = e.Row.Cells[6].Controls[0] as Button;
            if (btn != null)
            {
                btn.ToolTip = "Select the record for Patient Number " + s1;
            }
        }
    }

    protected void ButtonSubmitRequestManual_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = "";
        
        DisplayMode = DisplayModeType.SubmitManual;
        
        //check if the patient is in the master patient list
        PATIENT patient = 
        GetPatientFromVaPatientList(TextBoxPatientSSNSearch.Text.Trim().ToUpper().Replace("-", ""))
        .FirstOrDefault();

        if (patient != null)
        { 
            //if the patient is in the VA master list than populate values from that patient

            LabelWarningNotificationMessage.Visible = true;
            LabelWarningNotificationMessage.Text =
                "The SSN you submitted matches a patient in the VA master patient list.  The patient’s data from the VA master patient list will take precedence over the data you entered and is displayed in the fields.  Confirm this is the patient that you want to request.";
            //ButtonSearchPatient.Focus();

            //pre-populate patient data
            TextBoxPatientSSNSearch.Text = patient.SSN;
            TextBoxPatientFirstNameSearch.Text = patient.FIRST_NAME;
            TextBoxMiddleName.Text = patient.MIDDLE_NAME;
            TextBoxPatientLastNameSearch.Text = patient.LAST_NAME;
            TextBoxDateOfBirth.Text = Helpers.getShortDateStringFromNullableDate(patient.BIRTH_DATE);


            DisplayMode = DisplayModeType.SubmitFromVaMasterList;

            _resetDisplayMode = true;

            return;

        }

        //perform the checks
        if (IsValidToSubmitRequest())
        {
            CreateRecordForRequest();
            DisplayMode = DisplayModeType.Submitted;
        }
      

        _resetDisplayMode = true;
    }

    protected void ButtonSearchPatient_Click(object sender, EventArgs e)
    {
        LabelWarningNotificationMessage.Text = "";

        //check if at least one search criteria was entered
        if (string.IsNullOrEmpty(
            TextBoxPatientSSNSearch.Text.Trim() +
            TextBoxPatientFirstNameSearch.Text.Trim() +
            TextBoxPatientLastNameSearch.Text.Trim() +
            TextBoxDateOfBirth.Text.Trim()))
        {
            LabelWarningNotificationMessage.Text = "At least one of the following fields is required in order to search: SSN, First Name, Last Name, Date of Birth";
            TextBoxPatientSSNSearch.Focus();
            return;
        }
        else
        {
            LabelWarningNotificationMessage.Text = "";
        }
        
        var searchParams = "Search Parameters: SSN:" + TextBoxPatientSSNSearch.Text.Trim() +
                           " First Name:" + TextBoxPatientFirstNameSearch.Text.Trim() +
                           " Middle Name:" + TextBoxMiddleName.Text.Trim() +
                           " Last Name:" + TextBoxPatientLastNameSearch.Text.Trim() +
                           " Date Of Birth:" + TextBoxDateOfBirth.Text.Trim();
        DVEIRUserActivityType dveirUserActivityType = (ButtonSearchPatient.Text.Trim() == "Search")
                                                          ? DVEIRUserActivityType.PATIENT_REQUEST_SEARCH
                                                          : DVEIRUserActivityType.PATIENT_REQUEST_REQUEST;
        CreateLoggingRecord(0, dveirUserActivityType, searchParams);

        ////patient exists in registry - serach by ssn
        //string ssn = TextBoxPatientSSNSearch.Text.Trim().Replace("-", "").ToUpper().Replace("P", "");
        //var patients = _dveirPatientManager.SelectBySearchCriteria(null, null, ssn, null, null, null, 0, 10);
        //int patientsCount = patients != null ? patients.Count() : 0;

        //if (patientsCount > 0)
        //{
        //    LabelVAPatientNotificationMessage.Visible = true;
        //    LabelVAPatientNotificationMessage.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", 
        //        "PatientRequests_ButtonSearchVAPatient_Click_Patient_with_SSN_")
        //    + TextBoxPatientSSNSearch.Text.Trim()
        //    +(string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", 
        //        "PatientRequests_ButtonSearchVAPatient_Click__already_does_exist_in_the_Repository_");
        //    ButtonSearchPatient.Focus();
        //    GridViewVAPatients.Visible = false;
        //    CreateLoggingRecord(patients.FirstOrDefault().PATIENT_ID,
        //                        DVEIRUserActivityType.PATIENT_REQUEST_NOT_SUBMITTED_ALREADY_IN_REGISTRY,
        //                        "");
        //    return;
        //}

        ////if list is empty and data is sufficient and button is 'Request'
        //if (ButtonSearchPatient.Text == "Request"
        //    && SufficientPatientData(
        //        TextBoxPatientSSNSearch.Text,
        //        TextBoxPatientFirstNameSearch.Text,
        //        TextBoxPatientLastNameSearch.Text,
        //        TextBoxDateOfBirth.Text
        //    ))
        //{
        //    CreateRecordForRequest();
        //    LabelVAPatientRequestMessage.Visible = false;
        //    TextBoxPatientSSNSearch.Enabled = false;
        //    TextBoxPatientFirstNameSearch.Enabled = false;
        //    TextBoxPatientLastNameSearch.Enabled = false;
        //    TextBoxMiddleName.Enabled = false;
        //    TextBoxDateOfBirth.Enabled = false;
        //    ButtonSearchPatient.Focus();
        //    //ButtonSearchPatient.Enabled = false;

        //    return;
        //}

        if (TextBoxPatientSSNSearch.Text.Trim().Length > 0)
        {
            if (!IsValidToSubmitRequest())
            {
                TextBoxPatientSSNSearch.Focus();
                return;
            }
        }

        GridViewVAPatients.DataBind();
        GridViewVAPatients.SelectedIndex = -1;

        //search returns a list of patients
        if (GridViewVAPatients.Rows.Count != 0)
        {
            LabelSearchResultInformation.Text = "<br />The following patients were found in the VA master patient list based on your search criteria.  To continue, select a patient from the results below.  If you do not see the patient you were looking for, revise your search or continue to the manual patient request entry below the search results.";
            ButtonSearchPatient.Focus();
            GridViewVAPatients.Visible = true;
        }
        else
        {
            LabelSearchResultInformation.Text = "<br />No patients were found in the VA master patient list based on your search criteria.  Revise your search or continue to the manual patient request entry below.";
            ButtonSearchPatient.Focus();
            GridViewVAPatients.Visible = true;
        }

        DisplayMode = DisplayModeType.SearchResults;
        
        ////if list is empty and not enough data
        //if (GridViewVAPatients.Rows.Count == 0
        //    && InsufficientPatientData(
        //        TextBoxPatientSSNSearch.Text,
        //        TextBoxPatientFirstNameSearch.Text,
        //        TextBoxPatientLastNameSearch.Text,
        //        TextBoxDateOfBirth.Text
        //    ))
        //{
        //    PanelSelectPatient.GroupingText = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_Create_Request_from_VA");
        //    LabelVAPatientRequestMessage.Visible = true;
        //    LabelVAPatientRequestMessage.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_Please_enter_SSN__First_Name__Last_Name_and_Date_of_Birth_to_request_a_patient_from_VA_");
        //    GridViewVAPatients.Visible = false;
        //    ButtonSearchPatient.Text = "Request";
        //    PromptForPatientData(true);
        //}

        ////if request was submitted go to search
        //if (ButtonSearchPatient.Text == 
        //    (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", 
        //    "PatientRequests_ButtonSearchVAPatient_Click_Request")) 
        //{
        //    //disable required validators for SSN,First Name,Last Name,Date of Birth
        //    RequiredFieldValidatorDateOfBirth.Enabled = false;
        //    RequiredFieldValidatorFirstName.Enabled = false;
        //    RequiredFieldValidatorLastName.Enabled = false;
        //    RequiredFieldValidatorSSN.Enabled = false;

        //    LabelforTextBoxDateOfBirth.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_Date_of_Birth__mm_dd_yyyy___");
        //    LabelforTextBoxSSNSearch.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_SSN__");
        //    LabelforTextBoxPatientFirstNameSearch.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_First_Name__");
        //    LabelforTextBoxPatientLastNameSearch.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_Last_Name__");

        //    //ButtonSearchPatient.Text = (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "PatientRequests_ButtonSearchVAPatient_Click_Search");
        //}

        _resetDisplayMode = true;
    }

    private void CreateRecordForRequest()
    {
        int returnStatus = -1;
        string ssn = TextBoxPatientSSNSearch.Text.Trim().ToUpper().Replace("-", "");

        _PatientRequest.SetAsChangeTrackingRoot();

        _PatientRequest.FIRST_NAME = TextBoxPatientFirstNameSearch.Text;
        _PatientRequest.MIDDLE_NAME = TextBoxMiddleName.Text;
        _PatientRequest.LAST_NAME = TextBoxPatientLastNameSearch.Text;
        _PatientRequest.SSN = TextBoxPatientSSNSearch.Text;
        _PatientRequest.BIRTH_DATE = DateTime.Parse(TextBoxDateOfBirth.Text);

        _PatientRequest.SetAsInsertOnSubmit();

        returnStatus = _vaPatientManager.AddPatient(_PatientRequest);

        (this.Master as RegistryMaster).SetDirtyFlag(false);

        DisplayMode = DisplayModeType.Submitted;
    }

    private bool IsValidToSubmitRequest()
    {
        string ssn = TextBoxPatientSSNSearch.Text.Trim().ToUpper().Replace("-", "");

        //check if patient is in registry
        if (CheckPatientInRegistry(ssn))
        {
            CreateLoggingRecord(_PatientRequest.ID, DVEIRUserActivityType.PATIENT_REQUEST_NOT_SUBMITTED_ALREADY_IN_REGISTRY, "");
            return false;
        }

        //check if patient is in Staging_DupPatient
        if (CheckStagingDupPatient(ssn))
        {
            CreateLoggingRecord(_PatientRequest.ID, DVEIRUserActivityType.PATIENT_REQUEST_NOT_SUBMITTED_DUPLICATE, "");
            return false;
        }

        //check if patient is in Staging_DVEIR_PATIENTREQUEST
        if (CheckStagingDveirPatientRequest(ssn))
        {
            CreateLoggingRecord(_PatientRequest.ID, DVEIRUserActivityType.PATIENT_REQUEST_NOT_SUBMITTED_ALREADY_REQUESTED, "");
            return false;
        }

        return true;
    }

    public void Patients_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Select")
        {
            int index = Convert.ToInt32(e.CommandArgument);
            GridViewVAPatients.SelectedIndex = index;
            int id = Convert.ToInt32(GridViewVAPatients.SelectedValue);
            PATIENT patient = _vaPatientManager.SelectByIdPatient(id).FirstOrDefault();
            string ssn = patient.SSN.Trim().ToUpper().Replace("-", "").Replace("P", "");
            
            //pre-populate patient data
            TextBoxPatientSSNSearch.Text = patient.SSN;
            TextBoxPatientFirstNameSearch.Text = patient.FIRST_NAME;
            TextBoxMiddleName.Text = patient.MIDDLE_NAME;
            TextBoxPatientLastNameSearch.Text = patient.LAST_NAME;
            TextBoxDateOfBirth.Text = Helpers.getShortDateStringFromNullableDate(patient.BIRTH_DATE);


            DisplayMode = DisplayModeType.SubmitFromVaMasterList;

            _resetDisplayMode = true;
        }
    }

    private bool CheckPatientInRegistry(string ssn)
    {

        ////patient exists in registry - serach by ssn
        var patients = _dveirPatientManager.SelectBySearchCriteria(null, null, ssn, null, null, null, 0, 10);
        int patientsCount = patients != null ? patients.Count() : 0;
        
        //requested patient exists in the Staging_DVEIR_PATIENTREQUEST table
        if (patientsCount > 0)
        {
            LabelWarningNotificationMessage.Visible = true;
            LabelWarningNotificationMessage.Text =
                "The patient you are trying to manually request is already in the Eye Injury Data Store. A manual request cannot be made. Use the patient search function (see left menu) to access this patient’s Eye Injury Data Store record.";
            
            return true;
        }
        return false;
    
    }

    private bool CheckStagingDveirPatientRequest(string ssn)
    {
        int stagingDveirPatientRequestPatientCount =
            _vaPatientManager.SelectBySearchCriteriaCountDveirPatientRequest(null, null, null, ssn, null);

        //requested patient exists in the Staging_DVEIR_PATIENTREQUEST table
        if (stagingDveirPatientRequestPatientCount > 0)
        {
            LabelWarningNotificationMessage.Visible = true;
            LabelWarningNotificationMessage.Text =
                "A manual request for this patient has already been submitted. The request will be examined and processed as soon as possible.";
            return true;
        }
        return false;
    }

    private bool CheckStagingDupPatient(string ssn)
    {
        int stagingDupPatientCount = 
            _vaPatientManager.SelectBySearchCriteriaCountStagingDupPatient( null, null, null, ssn, null);

        //requested patient exists in the stag_dup table
        if (stagingDupPatientCount > 0)
        {
            LabelWarningNotificationMessage.Visible = true;
            LabelWarningNotificationMessage.Text = 
                "The patient you are trying to manually request has a pending issue with his/her ICN or identity. A manual request cannot be made at this time.";
            ButtonSearchPatient.Focus();
            return true;
        }
        return false;
    }

    private IEnumerable<PATIENT> GetPatientFromVaPatientList(string ssn)
    {
        
        return
            _vaPatientManager.SelectBySearchCriteriaPatient("","","",ssn,"","",0,1);
    }

    private void DisplayErrorMessage()
    {
        if (LabelWarningNotificationMessage.Text != "")
        {
            LabelWarningNotificationMessage.Visible = true;

            StringBuilder oErrorScript = new StringBuilder();
            string sScript1 = @"<script type=""text/javascript"" language=""JavaScript"">";
            string sScript2a = @" alert('";
            string sScript2b = @"')";
            string sScript3 = @"</script>";
            oErrorScript.AppendLine(sScript1);
            oErrorScript.Append(sScript2a);

            
            string sErrorMessage = LabelWarningNotificationMessage.Text;
            oErrorScript.Append(sErrorMessage);

            oErrorScript.AppendLine(sScript2b);
            oErrorScript.AppendLine(sScript3);

            ClientScript.RegisterStartupScript(this.GetType(), "AlertMessage", oErrorScript.ToString());

            return;
        }
        else 
        {
            LabelWarningNotificationMessage.Visible = false;

        }
    }

    public override string TopMenuName()
    {
        return "Patients";
    }

    public override string AppObjectCode()
    {
        return "DVEIR PATIENT REQUESTS";
    }

    public override string MenuItemName()
    {
        return "DVEIR PATIENT REQUESTS";
    }
}
