﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

public partial class Patients : BasePage
{

    DVEIRPatientManager manager = new DVEIRPatientManager();
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            PanelSelectPatient.Visible = true;
            RangeValidatorTextBoxDateOfBirth.MinimumValue = DateTime.MinValue.ToShortDateString();
            RangeValidatorTextBoxDateOfBirth.MaximumValue = DateTime.MaxValue.ToShortDateString();
        }

        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewDVEIRPatients.ClientID + "');", true);
    }
    protected void ButtonSearchPatient_Click(object sender, EventArgs e)
    {
        GridViewDVEIRPatients.Visible = true;
        GridViewDVEIRPatients.DataBind();
        var searchParams = "Search Parameters: SSN:" + TextBoxPatientSSNSearch.Text.Trim() +
                           " First Name:" + TextBoxPatientFirstNameSearch.Text.Trim() +
                           " Last Name:" + TextBoxPatientLastNameSearch.Text.Trim() +
                           " Date Of Birth:" + TextBoxDateOfBirth.Text.Trim() +
                           " ICN:" + TextBoxPatientICNSearch.Text.Trim();
 
        CreateLoggingRecord(0, DVEIRUserActivityType.PATIENT_SEARCH, searchParams);
    }

    public void Patients_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int index = Convert.ToInt32(e.CommandArgument);
            GridViewDVEIRPatients.SelectedIndex = index;
            int id = Convert.ToInt32(GridViewDVEIRPatients.SelectedValue);
            //calling the right page for different order lab order type
            Response.Redirect(string.Format("Patient.aspx?id={0}", id));
            

        }

    }

    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = e.Row.Cells[0].Text;

            Button btn = e.Row.Cells[7].Controls[0] as Button;
            if (btn != null)
            {
                btn.ToolTip = "Select the record for Patient Number " + s1;
            }
        }
    }

    public void SearchCriteria_ServerValidate(object source, ServerValidateEventArgs args)
    {
        
        string sLastName = TextBoxPatientLastNameSearch.Text.Trim();
        string sFirstName = TextBoxPatientFirstNameSearch.Text.Trim();
        string sSSN = TextBoxPatientSSNSearch.Text.Trim();
        string sICN = TextBoxPatientICNSearch.Text.Trim();
        string sDateOfBirth = TextBoxDateOfBirth.Text.Trim();

        if (String.IsNullOrEmpty(sLastName + sFirstName + sSSN + sICN + sDateOfBirth))
        {
            (source as CustomValidator).ErrorMessage = "Please enter search information.";
            args.IsValid = false;
            return;
        }
        else
        {
            args.IsValid = true;
        }        
    }

    public override string TopMenuName()
    {
        return "Patients";
    }

    public override string AppObjectCode()
    {
        return "DVEIR PATIENTS";
    }

    public override string MenuItemName()
    {
        return "DVEIR PATIENTS";
    }
}
