﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;
using System.Text;

public partial class Referral : BasePage
{
    DVEIR_REFERRAL _referral;
    PATIENT patient;
    ER_ENCOUNTER _encounter;
    ER_EXAM exam;
    ER_OUTCOME outcome;
    //List<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments;

    readonly DVEIRReferralManager _referralManager = new DVEIRReferralManager();

    DVEIRPatientManager _patientManager = new DVEIRPatientManager();

    DVEIREyeExamManager _examManager = new DVEIREyeExamManager();

    //DVEIRReviewCommentsManager _reviewCommentsManager = new DVEIRReviewCommentsManager();

    private int EncounterId { get; set; }

    private DVEIRReferralView currentView { get; set; }

    private bool _TabChangePostBack = false;

    public Referral()
    {
        //Manager Save Notification Event Handler
        _referralManager.BOSaveSuccess += new BaseBO.BOSaveSuccessEvent(manager_BOSaveSuccess);
    }

    public override string TopMenuName()
    {
        return "My Tasks";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        //return "DVEIR_REFERRAL";
        return AppObjectCodeTypeSetting.ToString();
    }
    
    private DVEIRReferralStatus _appObjectCodeTypeSetting = DVEIRReferralStatus.DVEIR_REFERRAL;

    /// <summary>
    /// Set the value to the status of the referral 
    /// </summary>
    private DVEIRReferralStatus AppObjectCodeTypeSetting 
    {
        get
        {
            return _appObjectCodeTypeSetting;
        }

        set 
        { 
            _appObjectCodeTypeSetting = value; 
        }  
    }

    private enum firstLayerTabOrderNumber : int
    {
        Encounters = 0,
        Patient = 1,
        ProblemList = 2,
        ReviewComments = 3,

        //The following will be used for build 2
        PatientOcularHx = 4,
        MedicalHistory = 5,
        MentalHealth = 6,
        Injury = 7

    }

    private enum secondLayerTabOrderNumber : int
    {
        Provider = 0,
        ChiefComplaint = 1,
        Exam = 2,
        Diagnosis = 3,
        Procedures = 4
    }

    private bool InsertingNew;

    private bool DoNotRefreshActionButtons;

    protected void Page_Load(object sender, EventArgs e)
    {
        PanelObjectContent.DefaultButton = "ButtonSaveTop";
        
        //Create listeners for user controls events

        //encounter user control encounter selection event listener
        this.EncounterTabUserControl.EncounterSelected += new clinicalDataUserControls_EncounterTabUserControl.EncounterClickEvent(EncounterTabUserControl_EncounterSelected);

        //encounter user control encounter search event listener
        this.EncounterTabUserControl.EtlEncounterSearched += new clinicalDataUserControls_EncounterTabUserControl.EncounterSearchEvent(EncounterTabUserControl_EncounterSearched);

        //review comment user control comment selection event listener
        this.ReviewCommentsTabUserControl1.ReviewCommentSelected += new clinicalDataUserControls_ReviewCommentsTabUserControl.ReviewCommentClickEvent(ReviewCommentsTabUserControl1_ReviewCommentSelected);
        
        //eye exam user control imaging add event listener
        this.ExamTabUserControl.ImagingMethodologySelectedToAdd += new clinicalDataUserControls_ExamTabUserControl.ImagingAddClickEvent(ExamTabUserControl_ImagingMethodologySelectedToAdd);

        //eye exam user control imaging delete event listener
        this.ExamTabUserControl.ImagingMethodologySelectedToDelete +=new clinicalDataUserControls_ExamTabUserControl.ImagingDeleteClickEvent(ExamTabUserControl_ImagingMethodologySelectedToDelete);
        

        //set public property EncounterID to value in hiddenfield
        EncounterId = Helpers.getIntFromString(HiddenFieldEncounterId.Value);

        if (!IsPostBack)
        {
            //if a page is loading for the first time set the encounter id to 0
            EncounterId = 0;

            //if page is loading for first time get ID from querystring and set to hidden field to use on postbacks
            int? entryID = Helpers.getIntFromString(Request.QueryString["id"]);
            HiddenFieldReferralId.Value = entryID.GetValueOrDefault().ToString();



            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (entryID == null || entryID == 0)
            {
                //set entryID to zero so that a default linq entity will be instantiated
                entryID = 0;

                if (this.AllowInsert)
                {
                    //if request to begin a new item then initialize form for insert
                    //SetFormToInsertMode;

                    InsertingNew = true;
                }
                else
                {
                    //SetFormToNoInsertAllowedMode();
                    
                    //Modify breadcrumb to display inserting message
                    LabelBreadCrumb.Text = LabelBreadCrumb.Text + " > Inserting Referral";
                }

            }
            else
            {
                //if not a new item, initialize form elements for an existing item

            }

            //initial the tab menus for openingn a new item
            SetMenuDisplayMode("TopLevel");

            //set the tab display area (multi view) to the pre-selected tab
            MultiViewPatientClinicalData.ActiveViewIndex = Helpers.getIntFromString(MenuPatientClinicalData.SelectedValue);

            currentView = DVEIRReferralView.Encounters;

            //Create Entity objects and Set properties to state of form
            getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);

            if (_referral != null)
            {
                if (InsertingNew)
                {
                    LabelBreadCrumb.Text += "Create New Referral";
                }
                else
                {
                    LabelBreadCrumb.Text += " > "
                                         + _referral.STD_REFERRALST.NAME 
                                         + " > "
                                         + " Referral ID: "
                                         + _referral.REFERRAL_ID
                                         + " ("
                                         + patient.FullName
                                         + "&nbsp;&nbsp;:&nbsp;&nbsp;SSN *****"
                                         + patient.SSN.Substring(5)
                                         + "&nbsp;&nbsp;:&nbsp;&nbsp;DOB " + Helpers.getShortDateStringFromNullableDate(patient.BIRTH_DATE)
                                         + ")";
                }

                //PopulateSelectedTabControl(MultiViewPatientClinicalData.ActiveViewIndex);

                //Set hidden field to use to return to the status list the use was on before opening the referral
                HiddenFieldToReferralListId.Value = _referral.STD_REFERRALSTS_ID.ToString();

                CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_OPEN, "");
            }
            
        }

    }

    protected void getEntry(int id, int encounterID, DVEIRReferralView view)
    {
        //only load a referral if it has not been loaded during this request
        if (_referral == null)
        {
            Trace.Write("Begin getReferral");

            if (encounterID > 0)
            {
                _referral = _referralManager.SelectByID(id, encounterID, view);

                _encounter = _referral.ER_ENCOUNTERs.FirstOrDefault();

                if (_encounter != null)
                {
                    exam = _encounter.ER_EXAM;
                    outcome = _encounter.ER_OUTCOME;
                }

            }
            else
            {
                _referral = _referralManager.SelectByID(id, view);
            }

            Trace.Write("end getReferral");

        }

        //only load a patient if it has not been loaded during this request
        if (patient == null)
        {
            Trace.Write("Begin getPatient");

            patient = _patientManager.SelectByID(_referral.PATIENT_ID);

            Trace.Write("End getPatient");
        }

        //initial display panels
        PanelNoAccess.Visible = false;
        PanelIdNotFound.Visible = false;
        PanelObjectContent.Visible = false;

        if (_referral == null)
        {
            PanelObjectContent.Visible = false;

            if (_referralManager.ReferralExists(id))
            {
                PanelNoAccess.Visible = true;
            }
            else
            {
                PanelIdNotFound.Visible = true;
            }
        }
        else
        {
            PanelObjectContent.Visible = true;
             
            //set AppObjectCode for Role Access per status
            AppObjectCodeTypeSetting = (DVEIRReferralStatus)_referral.STD_REFERRALSTS_ID;
        
        }

        if (IsPostBack && !_TabChangePostBack)
        {
            LoadObjectFromControls();
        }

        //toggle _TabChangePostBack back to false 
        //so that it will only be true during the request cycle of the postback event that toggled it on
        _TabChangePostBack = false;
    
    }


    void Page_PreRender(object sender, EventArgs e)
    {
        if (_referral != null)
        {
            //This is the last event in the page life cycle where changes can be made to the page object
            //so enforce business rules on the object (which could possibly change the propertie of the object
            //then load the form controls from the object

            //Enforce business rules on the referral object
            //need to call EnforceBusinessRules method in the ReferralManager because some rules require a database call
            //pass in the local instance of referral to the method.  The method will return the object with modifications due to business rules.
            if (IsPostBack)
            {
                int ret;
                ret = _referralManager.EnforceBusinessRules(ref _referral);
            }

            //Load Controls from the Object
            LoadControlsFromObject();

            CheckCapabilities();

            PopulateActionButtons();
            
        }

        if (!this.ReadOnly )
        {
            const string scriptName = "__dveirReferralClientSideScript";
            if (!Page.ClientScript.IsClientScriptIncludeRegistered(scriptName))
            {
                Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName, GetPageJavaScript(),true);
            }
            
        }

        if (this.AllowUpdate)
        {
            ButtonSaveTop.Visible = true;
            ButtonSaveBottom.Visible = true;
        }
        else
        {
            ButtonSaveTop.Visible = false;
            ButtonSaveBottom.Visible = false;

            //Disable Buttons in Child Controls
            ExamTabUserControl.DisableActionButtons();

        }

        if (this.ReadOnly)
        {
            MakeFormReadOnly(AppObjectCode(), this.Controls);
        }
        
        //Helpers.SetInputControlsHighlight(this, "highlight");
       
    }

    #region Menu Display / Load Settings

    private void SetMenuDisplayMode(string MenuMode)
    {

        switch (MenuMode)
        {
            case "TopLevel":

                this.PanelEncounterButtonsTop.Visible = false;
                this.PanelEncounterButtonsBottom.Visible = false;
                this.PanelSecondLayerMenu.Visible = false;
                this.PanelTopLayerMenu.Visible = true;

                break;
            case "SecondLevel":

                this.PanelEncounterButtonsTop.Visible = true;
                this.PanelEncounterButtonsBottom.Visible = true;
                this.PanelSecondLayerMenu.Visible = true;
                this.PanelTopLayerMenu.Visible = false;

                break;

        }

    }

    private void PopulateSelectedTabControl(int SelectedTab)
    {
        switch (SelectedTab)
        {
            case (int)firstLayerTabOrderNumber.Encounters:
                //Encounter Tab
                EncounterTabUserControl.LoadControlsFromObject(_referral);
                if (_encounter == null || HiddenFieldEncounterId.Value == "0")
                {
                    CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_ENCOUNTER_TAB_OPEN, "");
                }
                break;
            case (int)firstLayerTabOrderNumber.Patient:
                //Patient
                PatientTabUserControl.LoadControlsFromObject(patient, _referral);
                CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_PATIENT_TAB_OPEN, "");
                break;

            case (int)firstLayerTabOrderNumber.ProblemList:
                //Problem List
                ProblemListTabUserControl.LoadControlsFromObject(patient);
                CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_PROBLEM_LIST_TAB_OPEN, "");
                break;
            case (int)firstLayerTabOrderNumber.ReviewComments:
                //Review Comments
                ReviewCommentsTabUserControl1.LoadControlsFromObject(_referral);
                CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_REVIEW_COMMENTS_TAB_OPEN, "");
                break;
            case (int)firstLayerTabOrderNumber.PatientOcularHx:
                //Patient Ocular Hx
                
                //Disabled for build 1
                //PatientOcularHxTabUserControl1.PatientId = referral.PATIENT_ID;
                break;
            case (int)firstLayerTabOrderNumber.MedicalHistory:
                //Medical History

                //Disabled for build 1
                //MedicalHistoryTabUserControl.PatientId = referral.PATIENT_ID;
                break;

            case (int)firstLayerTabOrderNumber.MentalHealth:
                //Mental Health / Substance Abuse

                //Disabled for build 1
                //MHSATabUserControl.PatientId = referral.PATIENT_ID;
                break;

            case (int)firstLayerTabOrderNumber.Injury:
                //Injury
                //Disabled for build 1
                //InjuryTabUserControl1.PatientId = referral.PATIENT_ID;
                break;

            default:

                //do nothing   
                break;
        }
    }

    private int PopulateObjectFromSelectedTabControl(int SelectedTab)
    {
        int returnValue = 0;

        switch (SelectedTab)
        {
            case (int)firstLayerTabOrderNumber.Encounters:
                //Encounter Tab
                
                //Encounter is read only - do nothing

                break;
            case (int)firstLayerTabOrderNumber.Patient:
                //Patient

                returnValue = PatientTabUserControl.LoadObjectFromControls(patient, _referral);

                break;
            case (int)firstLayerTabOrderNumber.ProblemList:
                //Problem List

                //returnValue = TabUserControl.LoadObjectFromControls(ref patient);

                break;
            case (int)firstLayerTabOrderNumber.ReviewComments:
                //Review Comments

                returnValue = ReviewCommentsTabUserControl1.LoadObjectFromControls(ref _referral);

                break;
            case (int)firstLayerTabOrderNumber.PatientOcularHx:
                //Patient Ocular Hx

                //Disabled for build 1
                //PatientOcularHxTabUserControl1.PatientId = referral.PATIENT_ID;
                break;
            case (int)firstLayerTabOrderNumber.MedicalHistory:
                //Medical History

                //Disabled for build 1
                //MedicalHistoryTabUserControl.PatientId = referral.PATIENT_ID;
                break;

            case (int)firstLayerTabOrderNumber.MentalHealth:
                //Mental Health / Substance Abuse

                //Disabled for build 1
                //MHSATabUserControl.PatientId = referral.PATIENT_ID;
                break;

            case (int)firstLayerTabOrderNumber.Injury:
                //Injury
                //Disabled for build 1
                //InjuryTabUserControl1.PatientId = referral.PATIENT_ID;
                break;

            default:

                //do nothing 

                returnValue = 0;
                break;
        }

        return returnValue;
    }

    private void PopulateSecondLayerSelectedTabControl(int SelectedTab)
    {
        if (_encounter != null)
        {
            switch (SelectedTab)
            {
                case (int)secondLayerTabOrderNumber.Provider:
                    //Provider Tab
                    ProviderTabUserControl.LoadControlsFromObject(_encounter);
                    //if( HiddenFieldEncounterId.Value != "0")
                    //{
                        CreateLoggingRecord(_encounter.ER_ENCOUNTER_ID, DVEIRUserActivityType.ENCOUNTER_PROVIDER_TAB_OPEN, "");
                    //}
                    break;
                case (int)secondLayerTabOrderNumber.ChiefComplaint:
                    //ChiefComplaint
                    ChiefComplaintTabUserControl.LoadControlsFromObject(_encounter);
                    CreateLoggingRecord(_encounter.ER_ENCOUNTER_ID, DVEIRUserActivityType.ENCOUNTER_CHIEF_COMPLAINT_TAB_OPEN, "");
                    break;
                case (int)secondLayerTabOrderNumber.Exam:
                    //Exam
                    ExamTabUserControl.LoadControlsFromObject(_encounter, exam, outcome);
                    CreateLoggingRecord(_encounter.ER_ENCOUNTER_ID, DVEIRUserActivityType.ENCOUNTER_EXAM_TAB_OPEN, "");
                    break;
                case (int)secondLayerTabOrderNumber.Diagnosis:
                    //Diagnosis
                    DiagnosisTabUserControl.LoadControlsFromObject(_encounter, outcome);
                    CreateLoggingRecord(_encounter.ER_ENCOUNTER_ID, DVEIRUserActivityType.ENCOUNTER_DIAGNOSIS_TAB_OPEN, "");
                    break;
                case (int)secondLayerTabOrderNumber.Procedures:
                    //Procedures
                    ProceduresTabUserControl.LoadControlsFromObject(_encounter);
                    CreateLoggingRecord(_encounter.ER_ENCOUNTER_ID, DVEIRUserActivityType.ENCOUNTER_PROCEDURES_TAB_OPEN, "");
                    break;
                default:
                    //do nothing   
                    break;
            }
        }
    }

    private int PopulateObjectFromSecondLayerSelectedTabControl(int SelectedTab)
    {
        int returnValue = 0;

        switch (SelectedTab)
        {
            case (int)secondLayerTabOrderNumber.Provider:

                //Provider Tab
                returnValue = ProviderTabUserControl.LoadObjectFromControls(ref _encounter);

                break;

            case (int)secondLayerTabOrderNumber.ChiefComplaint:

                //ChiefComplaint
                returnValue = ChiefComplaintTabUserControl.LoadObjectFromControls(ref _encounter);
                break;

            case (int)secondLayerTabOrderNumber.Exam:

                //Exam
                returnValue = ExamTabUserControl.LoadObjectFromControls(ref _encounter, ref exam, ref outcome);
                break;

            case (int)secondLayerTabOrderNumber.Diagnosis:

                //Diagnosis
                returnValue = DiagnosisTabUserControl.LoadObjectFromControls(ref _encounter, ref outcome);
                break;

            case (int)secondLayerTabOrderNumber.Procedures:

                //Procedures
                returnValue = ProceduresTabUserControl.LoadObjectFromControls(ref _encounter);
                break;

            default:

                //do nothing 
                returnValue = 0; 
                break;

        }

        return returnValue;
    }

    #endregion

    #region UserControl Events

    void EncounterTabUserControl_EncounterSelected(object sender, clinicalDataUserControls_EncounterTabUserControl.EncounterClickEventArgs e)
    {
        SetMenuDisplayMode("SecondLevel");

        EncounterId = e.EncounterID;
        HiddenFieldEncounterId.Value = EncounterId.ToString();

        currentView = DVEIRReferralView.Provider;
        
        MultiViewPatientEncounterData.ActiveViewIndex = 0;
        MenuPatientEncounterData.Items[0].Selected = true;

        MultiViewPatientEncounterData.ActiveViewIndex = 0;
        MenuPatientEncounterData.Items[0].Selected = true;
        CreateLoggingRecord(EncounterId, DVEIRUserActivityType.ENCOUNTER_OPEN, "");

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
    }

    void EncounterTabUserControl_EncounterSearched(object sender)
    {
        ((RegistryMasterNoLeft)Master).SetDirtyFlag(false);
    }

    protected override bool OnBubbleEvent(object sender, EventArgs e)
    {
           if (e.GetType() == typeof(GoToReviewCommentListEventArgs))
           {
               ReturnToReviewCommentList_Click(sender, e);
               return true;
           }

        return false;
    }

    void ReturnToReviewCommentList_Click(object sender, EventArgs e)
    {
        //first save the data 
        ///////////////////////////////////////////////////////////////////////
 
        //get the entity object with the view of the current page.  this will load the control data into the object
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
    
        //int returnSaveValue = SaveItem();
        int returnSaveValue = SaveItem();
        SetSaveStatusLabel(returnSaveValue);

        //if the save was successful, retrieve entity object for the new view 
        ///////////////////////////////////////////////////////////////////////
        if (returnSaveValue > 0)
        {
            currentView = DVEIRReferralView.ReviewComments;

            int _activeViewIndex = (int)firstLayerTabOrderNumber.ReviewComments;
            int _menuSelected = (int)firstLayerTabOrderNumber.ReviewComments;

            MultiViewPatientClinicalData.ActiveViewIndex = _activeViewIndex;
            MenuPatientClinicalData.Items[_menuSelected].Selected = true;

            MultiViewPatientEncounterData.ActiveViewIndex = 0;

            SetMenuDisplayMode("TopLevel");

            //Create Entity objects and Set properties to state of form
            getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);

            HiddenFieldEncounterId.Value = "0";

            ReviewCommentsTabUserControl1.ClearReviewCommentsGridSelection();
            EncounterTabUserControl.ClearEncounterGridSelection();

            GoToReviewCommentListEventArgs reviewCommentEvent = (GoToReviewCommentListEventArgs)e;

            if (reviewCommentEvent.Type == "referral")
            {
                if (reviewCommentEvent.ReviewCommentID != 0)
                {
                    ER_REFERRAL_REVIEWCOMMENT reviewCommentSelected = new DVEIRReferralReviewCommentsManager().SelectByID(reviewCommentEvent.ReviewCommentID);

                    //to do : add code to navigate to the review comment in the grid view
                    ReviewCommentsTabUserControl1.LoadWithReviewCommentID = reviewCommentSelected.ID;
                    ReviewCommentsTabUserControl1.LoadWithReviewCommentType = "referral";
                }
            }
            else if (reviewCommentEvent.Type == "encounter")
            {
                if (reviewCommentEvent.ReviewCommentID != 0)
                {
                    ER_ENCOUNTER_REVIEWCOMMENT reviewCommentSelected = new DVEIRReviewCommentsManager().SelectByID(reviewCommentEvent.ReviewCommentID);

                    //to do : add code to navigate to the review comment in the grid view
                    ReviewCommentsTabUserControl1.LoadWithReviewCommentID = reviewCommentSelected.ID;
                    ReviewCommentsTabUserControl1.LoadWithReviewCommentType = "encounter";
                }
            }
        }


    }

    
    void ReviewCommentsTabUserControl1_ReviewCommentSelected(object sender, clinicalDataUserControls_ReviewCommentsTabUserControl.ReviewCommentClickEventArgs e)
    {
        _TabChangePostBack = true;
        //set is dirty to false since the user is changing tabs
        ((RegistryMasterNoLeft)Master).SetDirtyFlag(false);
        
        ////first save the data 
        /////////////////////////////////////////////////////////////////////////
 
        ////get the entity object with the view of the current page.  this will load the control data into the object
        //currentView = SetCurrentViewFromSelectedMenu();

        ////Create Entity objects and Set properties to state of form
        //getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
    
        ////int returnSaveValue = SaveItem();
        //Page.Validate();
        //int returnSaveValue = SaveItem();
        //SetSaveStatusLabel(returnSaveValue);

        ////if the save was successful, retrieve entity object for the new view 
        /////////////////////////////////////////////////////////////////////////
        //if (returnSaveValue > 0)
        //{}
            ////suppress the save message since this will act like an autosave
            //SuppressSaveMessage = false;

        if (e.ReferralComment)
            ReferralCommentSelected(e.ReviewCommentID);
        else
            EncounterCommentSelected(e.ReviewCommentID);
    }

    private void ReferralCommentSelected(int id)
    {
        ER_REFERRAL_REVIEWCOMMENT reviewCommentSelected = new DVEIRReferralReviewCommentsManager().SelectByID(id);

        if (reviewCommentSelected != null)
        {
            SetMenuDisplayMode("TopLevel");
            currentView = DVEIRReferralView.Patient;
            MultiViewPatientClinicalData.ActiveViewIndex = (int)firstLayerTabOrderNumber.Patient;
            MenuPatientClinicalData.Items[(int)firstLayerTabOrderNumber.Patient].Selected = true;

            //Create Entity objects and Set properties to state of form
            getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), 0, currentView);
            ER_REFERRAL_REVIEWCOMMENT reviewCommentsFlattened =
                _referral.ER_REFERRAL_REVIEWCOMMENTs
                .Where(rc => rc.ID == id).FirstOrDefault();

            if (reviewCommentsFlattened != null)
            {
                reviewCommentsFlattened.IsSelectedFromCommentsTab = true;
            }
        }
    }

    private void EncounterCommentSelected(int id)
    {
        ER_ENCOUNTER_REVIEWCOMMENT reviewCommentSelected = new DVEIRReviewCommentsManager().SelectByID(id);

        if (reviewCommentSelected != null)
        {
            SetMenuDisplayMode("SecondLevel");

            EncounterId = reviewCommentSelected.ER_ENCOUNTER_ID;
            HiddenFieldEncounterId.Value = EncounterId.ToString();

            currentView = SetCurrentViewFromLocationPage(reviewCommentSelected.LOCATIONPAGE);

            int _activeViewIndex = 0;
            int _menuSelected = 0;

            switch (reviewCommentSelected.LOCATIONPAGE)
            {
                case "Provider":
                    _activeViewIndex = 0;
                    _menuSelected = 0;
                    break;
                case "Chief Complaint":
                    _activeViewIndex = 1;
                    _menuSelected = 1;
                    break;
                case "Exam":
                    _activeViewIndex = 2;
                    _menuSelected = 2;
                    break;
                case "Diagnosis":
                    _activeViewIndex = 3;
                    _menuSelected = 3;
                    break;
                case "Procedures":
                    _activeViewIndex = 4;
                    _menuSelected = 4;
                    break;
            }

            MultiViewPatientEncounterData.ActiveViewIndex = _activeViewIndex;
            MenuPatientEncounterData.Items[_menuSelected].Selected = true;

            //Create Entity objects and Set properties to state of form
            getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);


            //from e in referral.ER_ENCOUNTERs 

            ER_ENCOUNTER_REVIEWCOMMENT reviewCommentsFlattened =
                _referral.ER_ENCOUNTERs.SelectMany(enc => enc.ER_ENCOUNTER_REVIEWCOMMENTs)
                .Where(rc => rc.ID == id).FirstOrDefault();

            if (reviewCommentsFlattened != null)
            {
                reviewCommentsFlattened.IsSelectedFromCommentsTab = true;
            }

        }
    }

    private DVEIRReferralView SetCurrentViewFromLocationPage(string locationPage)
    {
        switch (locationPage)
        {
            case "Provider":
                return DVEIRReferralView.Provider;
            case "Chief Complaint":
                return DVEIRReferralView.ChiefComplaint;
            case "Exam":
                return DVEIRReferralView.Exam;
            case "Diagnosis":
                return DVEIRReferralView.Diagnosis;
            case "Procedures":
                return DVEIRReferralView.Procedures;
            default:
                return DVEIRReferralView.All;
        }

    }

    private DVEIRReferralView SetCurrentViewFromSelectedMenu()
    {
        if (this.PanelTopLayerMenu.Visible)
        {
            switch (MultiViewPatientClinicalData.ActiveViewIndex)
            {
                case 0: //"Encounters"
                    return DVEIRReferralView.Encounters;
                case 1: //"Patient"
                    return DVEIRReferralView.Patient;
                case 2: //"ProblemList"
                    return DVEIRReferralView.ProblemList;
                case 3: //"ReviewComments"
                    return DVEIRReferralView.ReviewComments;
                default:
                    return DVEIRReferralView.All;
            }
        }

        if (this.PanelSecondLayerMenu.Visible)
        {
            switch (MultiViewPatientEncounterData.ActiveViewIndex)
            {
                case 0: //"Provider"
                    return DVEIRReferralView.Provider;
                case 1: //"Chief Complaint"
                    return DVEIRReferralView.ChiefComplaint;
                case 2: //"Exam"
                    return DVEIRReferralView.Exam;
                case 3: //"Diagnosis"
                    return DVEIRReferralView.Diagnosis;
                case 4: //"Procedures"
                    return DVEIRReferralView.Procedures;
                default:
                    return DVEIRReferralView.All;
            }
        }

        //else
        return DVEIRReferralView.All;
    }

    void ExamTabUserControl_ImagingMethodologySelectedToAdd(object sender, clinicalDataUserControls_ExamTabUserControl.ImagingAddClickEventArgs e)
    {
        currentView = DVEIRReferralView.Exam;

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
        
        ER_IMAGING_DTL newItem = _examManager.AddImagingItemToExam(exam.ER_EXAM_ID, e.NewImagingItem);
        
        if (newItem != null)
        {
            exam.ER_IMAGING_DTLs.Add(newItem);

        }
    }

    void ExamTabUserControl_ImagingMethodologySelectedToDelete(object sender, clinicalDataUserControls_ExamTabUserControl.ImagingDeleteClickEventArgs e)
    {
        currentView = DVEIRReferralView.Exam;

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
        
        int returnStatus = _examManager.DeleteImagingItemFromExam(e.ImagingDtlId);

        if (returnStatus == 0)
        {
            exam.ER_IMAGING_DTLs.Remove(
                                        (from i in exam.ER_IMAGING_DTLs where i.ER_IMAGING_DTL_ID == e.ImagingDtlId
                                        select i).FirstOrDefault()
                                        );
        }
    }

    #endregion


    #region Postback Events
    protected void MenuPatientClinicalData_MenuItemClick(object sender, MenuEventArgs e)
    {
        _TabChangePostBack = true;
        //set is dirty to false since the user is changing tabs
        ((RegistryMasterNoLeft)Master).SetDirtyFlag(false);
        
        int index = Int32.Parse(e.Item.Value);
        MultiViewPatientClinicalData.ActiveViewIndex = index;

        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);

        //Page.Validate();

        ////Save Data on changing tab
        //int returnSaveValue = SaveItem();
        //SetSaveStatusLabel(returnSaveValue);

        (this.Master as RegistryMasterNoLeft).SetDirtyFlag(false);

    }
    protected void MenuPatientEncounterData_MenuItemClick(object sender, MenuEventArgs e)
    {

        _TabChangePostBack = true;
        //set is dirty to false since the user is changing tabs
        ((RegistryMasterNoLeft)Master).SetDirtyFlag(false);

        int index = Int32.Parse(e.Item.Value);
        MultiViewPatientEncounterData.ActiveViewIndex = index;

        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);

        //Page.Validate();

        ////Save Data on changing tab
        //int returnSaveValue = SaveItem();
        //SetSaveStatusLabel(returnSaveValue);

        (this.Master as RegistryMasterNoLeft).SetDirtyFlag(false);
    }
    protected void ButtonCloseEncounter_Click(object sender, EventArgs e)
    {
        SetMenuDisplayMode("TopLevel");
        
        currentView = DVEIRReferralView.Encounters;

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value),0, currentView);
                
        //Save Data on changing tab
        //int returnSaveValue = SaveItem();
        (this.Master as RegistryMasterNoLeft).SetDirtyFlag(false);

        HiddenFieldEncounterId.Value = "0";

        //SetSaveStatusLabel(returnSaveValue);
        EncounterTabUserControl.DataBind();
        EncounterTabUserControl.ClearEncounterGridSelection();
        
    }
    protected void ButtonSave_Click(object sender, EventArgs e)
    {
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);


        //int returnSaveValue = SaveItem();
        int returnSaveValue = SaveItem();
        SetSaveStatusLabel(returnSaveValue);

        if (returnSaveValue > 0)
        {
            //this.Page_Load(this, new EventArgs());
        }
    }
    protected void ButtonSendBackToAbstractor_Click(object sender, EventArgs e)
    {
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);

        PerformActionAndSaveItem("DVEIR SENT BACK TO ABSTRACTOR");
    }
    protected void ButtonCancel_Click(object sender, EventArgs e)
    {
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);
        
        CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_CANCEL, "");
        
        //TO DO : return to referral menu that user was on when opening referral.
        Response.Redirect("Referrals.aspx?id=" + HiddenFieldToReferralListId.Value );
    }
    protected void ButtonSendForReview_Click(object sender, EventArgs e)
    {
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);


        PerformActionAndSaveItem("DVEIR MARKED READY FOR REVIEW");
    }
    protected void ButtonApproveForSendToDVEIR_Click(object sender, EventArgs e)
    {
        currentView = SetCurrentViewFromSelectedMenu();

        //Create Entity objects and Set properties to state of form
        getEntry(Helpers.getIntFromString(HiddenFieldReferralId.Value), EncounterId, currentView);


        //save item or perform action and save item
        PerformActionAndSaveItem("DVEIR MARKED READY TO SEND TO DVIER");
    }
    
    #endregion
    #region Private Methods
    private int PerformActionAndSaveItem(string activityCode)
    {
        int returnValue = -1;

        //if encounter exists the user has an encounter item open and has clicked the save button.  
        //Therefore set the updatedby field so that the top level (encounter) gets flagged as changed even if only children have changes
        if (_encounter != null)
        {
            _encounter.UPDATEDBY = "ItemUpdate";
        }

        if (activityCode != null)   //perform action and save item
        {
            returnValue = _referralManager.PerformActivityAndSave( _referral, activityCode);
            
            if (returnValue >= 0)
            {
                CreateLoggingRecord(_referral.REFERRAL_ID, activityCode, "");
                Response.Redirect("Referrals.aspx?id=" + HiddenFieldToReferralListId.Value);
            }
            else
            {
                SetSaveStatusLabel(returnValue);
            }
        }
        else
        {
            returnValue = SaveItem();   //save item
        }
        return returnValue;
    }
    private void SetSaveStatusLabel(int ReturnSaveValue)
    {
        if (ReturnSaveValue > 0)
        {
            //LabelSaveStatusMessage.Text = "Save Successful";
            LabelSaveStatusMessage.Text = "";
        }

        if (ReturnSaveValue == 0)
        {
            LabelSaveStatusMessage.Text = "";
        }

        if (ReturnSaveValue < 0)
        {
            LabelSaveStatusMessage.Text = "Error Saving Data. Make a copy of data before closing the browser then contact application administrator.";
            DoNotRefreshActionButtons = true;
        }
    }
 
    private int SaveItem()
    {
        bool saveSuccessOccurred = false;

        if (ReadOnly) return 0;

        if (Page.IsValid)
        {
            //if encounter exists the user has an encounter item open and has clicked the save button.  
            //Therefore set the updatedby field so that the top level (encounter) gets flagged as changed even if only children have changes
            if (_encounter != null)
            {
                _encounter.UPDATEDBY = "ItemUpdate";
            }

            int originalReferralStatus = _referral.STD_REFERRALSTS_ID;
            
            IsSaveSuccess =
            saveSuccessOccurred = _referralManager.SaveItem(_referral, patient);

            if (IsSaveSuccess)
            {
                SuppressSaveMessage = false;
                ((HiddenField)Master.FindControl("HiddenFieldShowSaveMessage")).Value = "true";
                CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_SAVE, "");

                if (originalReferralStatus != _referral.STD_REFERRALSTS_ID
                    &&
                    _referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_IN_PROCESS)
                {
                    CreateLoggingRecord(_referral.REFERRAL_ID, DVEIRUserActivityType.REFERRAL_SEND_TO_IN_PROCESS, "");
                    //modify bread crumb
                    LabelBreadCrumb.Text = LabelBreadCrumb.Text.Replace("> New >", " > " + _referral.STD_REFERRALST.NAME + " > ");

                }
            }

            SetBrokenRuleErrorDisplay();
        }

        if (saveSuccessOccurred)
        {
            ((RegistryMasterNoLeft)Master).SetDirtyFlag(false);
            return 1;
        }
        return -1;
    }

    private void SetBrokenRuleErrorDisplay()
    {
        if (_referral.BrokenRules.Count == 0
            || patient.BrokenRules.Count == 0
            || _encounter.BrokenRules.Count == 0
            || exam.BrokenRules.Count == 0
            || outcome.BrokenRules.Count == 0
            )
        {
            ErrorList1.Visible = false;
        }
        else
        {
            ErrorList1.Visible = true;
        }

        var AggregateBrokenRules = _referral.BrokenRules;

        if (patient != null) AggregateBrokenRules.Concat(patient.BrokenRules);

        if (_encounter != null) AggregateBrokenRules.Concat(_encounter.BrokenRules);

        if (exam != null) AggregateBrokenRules.Concat(exam.BrokenRules);

        if (outcome != null) AggregateBrokenRules.Concat(outcome.BrokenRules);

        ErrorList1.BrokenRules = AggregateBrokenRules;
    }


    private string GetPageJavaScript()
    {
        StringBuilder myStringBuilder = new StringBuilder();
           
        //myStringBuilder.Append("function SetTextBoxOtherInjuryDescriptionEnabledStatus()");
        //myStringBuilder.Append("{");
        //myStringBuilder.Append("var TextBoxOtherInjuryDescription = document.getElementById('" + TextBoxOtherInjuryDescription.ClientID + "');");
        //myStringBuilder.Append("if ($get('" + DropDownListOtherInjury.ClientID + "').value  != 'Y')");
        //myStringBuilder.Append("{");
        //myStringBuilder.Append("TextBoxOtherInjuryDescription.value = '';");
        //myStringBuilder.Append("TextBoxOtherInjuryDescription.disabled = true;");
        //myStringBuilder.Append("}");
        //myStringBuilder.Append("else");
        //myStringBuilder.Append("{");
        //myStringBuilder.Append("TextBoxOtherInjuryDescription.disabled = false;");
        //myStringBuilder.Append("}");
        //myStringBuilder.Append("}");

        return myStringBuilder.ToString();

    }
    #endregion
    #region Object/Form Mapping
    private void LoadControlsFromObject()
    {
        Trace.Write("Begin LoadControlsFromObject");

        //clear all form elements so they will only be populated if data exists in object
        ClearAllControlValues(this.Controls);

        //populate dropdowns
        //TO DO : modify drop down population to add archived item if persisted in entity 
        //          and remove "-- Select a Value --" when an item is selected and field is not nullable 
        PopulateDropDowns();

        if (_referral != null && PanelTopLayerMenu.Visible == true)
        {
            PopulateSelectedTabControl(MultiViewPatientClinicalData.ActiveViewIndex);
        }

        if (_encounter != null && PanelSecondLayerMenu.Visible == true)
        {
            PopulateSecondLayerSelectedTabControl(MultiViewPatientEncounterData.ActiveViewIndex);
        }

        Trace.Write("End LoadControlsFromObject");
        
    }

    private void LoadObjectFromControls()
    {
        Trace.Write("Begin LoadObjectFromControls");

        //this.PanelSecondLayerMenu.Visible = true;
        //this.PanelTopLayerMenu.Visible = false;

        if (_referral != null)
        {
            PopulateObjectFromSelectedTabControl(MultiViewPatientClinicalData.ActiveViewIndex);
        }

        if (_encounter != null)
        {
            PopulateObjectFromSecondLayerSelectedTabControl(MultiViewPatientEncounterData.ActiveViewIndex);
        }

        Trace.Write("End LoadObjectFromControls");

    }

    private void PopulateDropDowns()
    {

    }

    private void PopulateActionButtons()
    {
        if (DoNotRefreshActionButtons) return;
        
        //initiate all action buttons as not displayed
        //tableCellForButtonSendForReviewTop.Visible = false;
        //tableCellForButtonSendForReviewBottom.Visible = false;
        ButtonSendForReviewTop.Visible = false;
        ButtonSendForReviewBottom.Visible = false;

        //tableCellForButtonSendBackToAbstractorTop.Visible = false;
        //tableCellForButtonSendBackToAbstractorBottom.Visible = false;
        ButtonSendBackToAbstractorTop.Visible = false;
        ButtonSendBackToAbstractorBottom.Visible = false;

        //tableCellForButtonApproveForSendToDVEIRTop.Visible = false;
        //tableCellForButtonApproveForSendToDVEIRBottom.Visible = false;
        ButtonApproveForSendToDVEIRTop.Visible = false;
        ButtonApproveForSendToDVEIRBottom.Visible = false;

        //11 DVEIR_NEW
        //    DVEIR DATA Abstractor - Edit
        //    DVEIR MEDICAL PROFESSIONAL - Read Only

        //12 DVEIR_IN_PROCESS
        //    DVEIR DATA Abstractor - Edit
        //    DVEIR MEDICAL PROFESSIONAL - Read Only

        //13 DVEIR_READY_FOR_REVIEW
        //    DVEIR MEDICAL PROFESSIONAL - Edit
        //    DVEIR DATA Abstractor - Read Only

        //14 DVEIR_READY_TO_SEND_TO_DVEIR
        //    DVEIR DATA Abstractor - Read Only
        //    DVEIR MEDICAL PROFESSIONAL - Read Only

        //15 DVEIR_COMPLETED	Completed
        //    DVEIR DATA Abstractor - Read Only
        //    DVEIR MEDICAL PROFESSIONAL - Read Only

        switch ((DVEIRReferralStatus)_referral.STD_REFERRALSTS_ID)
        {
            case DVEIRReferralStatus.DVEIR_NEW:

                if (
                    CurrentUser.IsSuperUser
                    ||
                    CurrentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    )
                {
                    //tableCellForButtonSendForReviewTop.Visible = true;
                    //tableCellForButtonSendForReviewBottom.Visible = true;
                    ButtonSendForReviewTop.Visible = true;
                    ButtonSendForReviewBottom.Visible = true;
                }
                break;


            case DVEIRReferralStatus.DVEIR_IN_PROCESS:

                if (
                    CurrentUser.IsSuperUser
                    ||
                    CurrentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    )
                {
                    //tableCellForButtonSendForReviewTop.Visible = true;
                    //tableCellForButtonSendForReviewBottom.Visible = true;
                    ButtonSendForReviewTop.Visible = true;
                    ButtonSendForReviewBottom.Visible = true;

                }
                break;

            case DVEIRReferralStatus.DVEIR_RETURNED:

                if (
                    CurrentUser.IsSuperUser
                    ||
                    CurrentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    )
                {
                    //tableCellForButtonSendForReviewTop.Visible = true;
                    //tableCellForButtonSendForReviewBottom.Visible = true;
                    ButtonSendForReviewTop.Visible = true;
                    ButtonSendForReviewBottom.Visible = true;

                }
                break;

            case DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW:

                if (
                    CurrentUser.IsSuperUser
                    ||
                    //CurrentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    CurrentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    //tableCellForButtonSendBackToAbstractorTop.Visible = true;
                    //tableCellForButtonSendBackToAbstractorBottom.Visible = true;
                    ButtonSendBackToAbstractorTop.Visible = true;
                    ButtonSendBackToAbstractorBottom.Visible = true;
                }
                else
                {
                    ButtonSendBackToAbstractorTop.Visible = false;
                    ButtonSendBackToAbstractorBottom.Visible = false;
                }

                if (
                    CurrentUser.IsSuperUser
                    ||
                    CurrentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    //tableCellForButtonApproveForSendToDVEIRTop.Visible = true;
                    //tableCellForButtonApproveForSendToDVEIRBottom.Visible = true;
                    ButtonApproveForSendToDVEIRTop.Visible = true;
                    ButtonApproveForSendToDVEIRBottom.Visible = true;
                }
                break;

            case DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR:

                if (
                    CurrentUser.IsSuperUser
                    ||
                    //CurrentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    CurrentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    //tableCellForButtonSendBackToAbstractorTop.Visible = true;
                    //tableCellForButtonSendBackToAbstractorBottom.Visible = true;
                    ButtonSendBackToAbstractorTop.Visible = true;
                    ButtonSendBackToAbstractorBottom.Visible = true;
                    this.AllowUpdate = true;
                }
                else
                {
                    ButtonSendBackToAbstractorTop.Visible = false;
                    ButtonSendBackToAbstractorBottom.Visible = false;
                    this.AllowUpdate = false;
                }

                //set read only to all
                this.ReadOnly = true;

                break;

            case DVEIRReferralStatus.DVEIR_COMPLETED:

                //do nothing, no actions allowed in this status

                //set read only to all
                this.ReadOnly = true;
                this.AllowUpdate = false;

                
                break;
            
        }
           

    }

    #endregion
}
