﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;
using System.Text;

public partial class controls_BaseControl : BaseClinicalDataControl
{
    private DVEIRBaseControlManager _baseControlManager = new DVEIRBaseControlManager();

    protected void Page_Load(object sender, EventArgs e)
    {
        Page.PreRender += new EventHandler(Page_PreRender);
    }

    public string Text
    {
        get
        {
            return HiddenFieldID.Value;
        }
        set
        {
            HiddenFieldID.Value = value;
            switch (BaseControlType)
            {
                case DveirBaseControlType.TextBox:
                    TextBoxID.Text = value;
                    break;
                case DveirBaseControlType.RadioButtonList:
                    if (RadioButtonListID.Items.Count == 0)
                    {
                        RadioButtonListID.DataSource = _baseControlManager.SelectLookupList(LookupCategory);
                        RadioButtonListID.DataTextField = "NAME";
                        RadioButtonListID.DataValueField = "ID";
                        RadioButtonListID.DataBind();

                    }
                    foreach (ListItem radioListItem in RadioButtonListID.Items)
                    {
                        if (radioListItem.Value == value.Trim())
                        {
                            radioListItem.Selected = true;
                            break;
                        }
                    }
                    if (string.IsNullOrEmpty(RadioButtonListID.SelectedValue))
                    {
                        foreach (ListItem radioListItem in RadioButtonListID.Items)
                        {
                            if (radioListItem.Text == value.Trim())
                            {
                                radioListItem.Selected = true;
                                break;
                            }
                        }
                    }
                    break;
                case DveirBaseControlType.Checkbox:
                    if (value == "true")
                    {
                        CheckBoxID.Checked = true;
                    }
                    else
                    {
                        CheckBoxID.Checked = false;
                    }
                    break;
                case DveirBaseControlType.DropdownList:
                    if (DropDownListID.Items.Count == 1)
                    {
                        DropDownListID.DataSource = _baseControlManager.SelectLookupList(LookupCategory);
                        DropDownListID.DataTextField = "NAME";
                        DropDownListID.DataValueField = "ID";
                        DropDownListID.DataBind();

                    }
                    foreach (ListItem item in DropDownListID.Items)
                    {
                        if (item.Value == value.Trim())
                        {
                            item.Selected = true;
                            break;
                        }
                    }
                    if (string.IsNullOrEmpty(DropDownListID.SelectedValue))
                    {
                        foreach (ListItem item in DropDownListID.Items)
                        {
                            if (item.Text == value.Trim())
                            {
                                item.Selected = true;
                                break;
                            }
                        }
                    }
                    break;
            }
        }

    }

    public DveirBaseControlType BaseControlType
    {
        get
        {
            if (Session["cotrolType" + ClientID] == null)
            {
                return DveirBaseControlType.None;
            }
            else
            {
                return (DveirBaseControlType)Session["cotrolType" + ClientID];
            }
        }
        set
        {
            Session["cotrolType" + ClientID] = value;
        }

    }

    public string LookupCategory
    {
        get
        {
            return (string)Session["LookupCategory" + ClientID];
        }
        set
        {
            Session["LookupCategory" + ClientID] = value;
        }
    }
    public int IndentForLabel
    {
        get
        {
            if (Session["IndentForLabel" + ClientID] == null)
            {
                return 0;
            }
            else
            {
                return (int)Session["IndentForLabel" + ClientID];
            }
        }
        set
        {
            Session["IndentForLabel" + ClientID] = value;
        }
    }

    public int BaseControlId
    {
        get
        {
            if (Session["BaseControlId" + ClientID] == null)
            {
                return 0;
            }
            else
            {
                return (int)Session["BaseControlId" + ClientID];
            }
        }
        set
        {
            Session["BaseControlId" + ClientID] = value;
        }
    }

    public int MaxLength
    {
         set
        {
            if (value > 0)
            {
                TextBoxID.MaxLength = value;
                TextBoxID.Width = value * 10;
            }
        }
    }

    public bool HideLabel
    {
        get
        {
            if (Session["HideLabel" + ClientID] == null)
            {
                return false;
            }
            else
            {
                return (bool)Session["HideLabel" + ClientID];
            }
        }
        set
        {
            Session["HideLabel" + ClientID] = value;
        }
    }

    public bool IsMultilineTextBox
    {
        get
        {
            if (Session["IsMultilineTextBox" + ClientID] == null)
            {
                return false;
            }
            else
            {
                return (bool)Session["IsMultilineTextBox" + ClientID];
            }
        }
        set
        {
            Session["IsMultilineTextBox" + ClientID] = value;
        }
    }
    public string ToolTip
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                LabelID.ToolTip = value;
                TextBoxID.ToolTip = value;
                RadioButtonListID.ToolTip = value;
                DropDownListID.ToolTip = value;
                CheckBoxID.ToolTip = value;
            }
        }
    }

    public string CheckBoxText
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                CheckBoxID.Text = value;
            }
        }
    }

    public string BaseControlName
    {
        get
        {
            if (Session["BaseControlName" + ClientID] == null)
            {
                return string.Empty;
            }
            else
            {
                return (string)Session["BaseControlName" + ClientID];
            }
        }
        set
        {
            Session["BaseControlName" + ClientID] = value;
        }
    }
    public string ConditionText
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                LabelSeeBelow.Text = value;
                LabelSeeBelow.ToolTip = value;
            }
            else
            {
                LabelSeeBelow.Text = string.Empty;
                LabelSeeBelow.Width = 0;
            }
        }
    }

    public bool IsRequired
    {
        set
        {
            if (value)
            {
                RequiredFieldValidatorID.Visible = true;
                RequiredFieldValidatorID.Enabled = true;
            }
            else
            {
                RequiredFieldValidatorID.Visible = false;
                RequiredFieldValidatorID.Enabled = false;
            }
        }
    }

    public int ControlWidth
    {
        set
        {
            if (value > 0)
            {
                TextBoxID.Width = value;
            }
        }
    }

    public string ValidationExpression
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                RegularExpressionValidatorTextBoxID.ValidationExpression = value;
                RegularExpressionValidatorTextBoxID.Visible = true;
                RegularExpressionValidatorTextBoxID.Enabled = true;
            }
            else
            {
                RegularExpressionValidatorTextBoxID.Visible = false;
                RegularExpressionValidatorTextBoxID.Enabled = false;
            }
        }
    }

    public string ValidationGroup
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                if (RegularExpressionValidatorTextBoxID.Enabled)
                {
                    RegularExpressionValidatorTextBoxID.ValidationGroup = value;
                }

                RequiredFieldValidatorID.ValidationGroup = value;
            }
        }
    }

    public string ValidationErrorMessage
    {
        set
        {
            if (!string.IsNullOrEmpty(value))
            {
                RegularExpressionValidatorTextBoxID.ErrorMessage = value;
                RegularExpressionValidatorTextBoxID.Visible = true;
                RegularExpressionValidatorTextBoxID.Enabled = true;
            }
            else
            {
                RegularExpressionValidatorTextBoxID.Visible = false;
                RegularExpressionValidatorTextBoxID.Enabled = false;
            }
        }
    }

    void Page_PreRender(object sender, EventArgs e)
    {
        if (BaseControlType != DveirBaseControlType.None)
        {
            ShowBaseControl(BaseControlType);
        }
    }

    private void ShowBaseControl(DveirBaseControlType type)
    {
        LabelID.Visible = true;
        LabelID.Enabled = true;
        LabelID2.Visible = false;
        LabelID2.Enabled = false;
        LabelID.Text = GetExtraSpace();
        if (!HideLabel)
        {
            LabelID.Text = LabelID.Text + BaseControlName;
        }

        if (BaseControlName.Length > 60)
        {
            LabelID.Text = GetExtraSpace();
            LabelID2.Visible = true;
            LabelID2.Enabled = true;
            LabelID2.Text = BaseControlName + "<br />";
        }
        switch (type)
        {
            case DveirBaseControlType.TextBox:
                LabelID.AssociatedControlID = "TextBoxID";
                LabelID2.AssociatedControlID = "TextBoxID";
                TextBoxID.Visible = true;
                TextBoxID.Enabled = true;
                RequiredFieldValidatorID.ControlToValidate = "TextBoxID";
                RadioButtonListID.Visible = false;
                RadioButtonListID.Enabled = false;
                CheckBoxID.Visible = false;
                CheckBoxID.Enabled = false;
                DropDownListID.Visible = false;
                DropDownListID.Enabled = false;
                if (IsMultilineTextBox)
                {
                    TextBoxID.TextMode = TextBoxMode.MultiLine;
                    TextBoxID.Rows = 5;
                }
                break;
            case DveirBaseControlType.RadioButtonList:
                LabelID.AssociatedControlID = "RadioButtonListID";
                LabelID2.AssociatedControlID = "RadioButtonListID";
                RadioButtonListID.Visible = true;
                RadioButtonListID.Enabled = true;
                RequiredFieldValidatorID.ControlToValidate = "RadioButtonListID";
                TextBoxID.Visible = false;
                TextBoxID.Enabled = false;
                RegularExpressionValidatorTextBoxID.Visible = false;
                RegularExpressionValidatorTextBoxID.Enabled = false;
                CheckBoxID.Visible = false;
                CheckBoxID.Enabled = false;
                DropDownListID.Visible = false;
                DropDownListID.Enabled = false;
                if (RadioButtonListID.Items.Count == 0)
                {
                    RadioButtonListID.Items.Clear();
                    RadioButtonListID.DataSource = _baseControlManager.SelectLookupList(LookupCategory);
                    RadioButtonListID.DataTextField = "NAME";
                    RadioButtonListID.DataValueField = "ID";
                    RadioButtonListID.DataBind();
                }
                break;
            case DveirBaseControlType.Checkbox:
                LabelID.AssociatedControlID = "CheckBoxID";
                LabelID2.AssociatedControlID = "CheckBoxID";
                LabelID.Text = GetExtraSpace();
                RadioButtonListID.Visible = false;
                RadioButtonListID.Enabled = false;
                RequiredFieldValidatorID.ControlToValidate = "CheckBoxID";
                TextBoxID.Visible = false;
                TextBoxID.Enabled = false;
                RegularExpressionValidatorTextBoxID.Visible = false;
                RegularExpressionValidatorTextBoxID.Enabled = false;
                CheckBoxID.Visible = true;
                CheckBoxID.Enabled = true;
                DropDownListID.Visible = false;
                DropDownListID.Enabled = false;
                break;
            case DveirBaseControlType.DropdownList:
                LabelID.AssociatedControlID = "DropDownListID";
                LabelID2.AssociatedControlID = "DropDownListID";
                RadioButtonListID.Visible = false;
                RadioButtonListID.Enabled = false;
                RequiredFieldValidatorID.ControlToValidate = "DropDownListID";
                TextBoxID.Visible = false;
                TextBoxID.Enabled = false;
                RegularExpressionValidatorTextBoxID.Visible = false;
                RegularExpressionValidatorTextBoxID.Enabled = false;
                CheckBoxID.Visible = false;
                CheckBoxID.Enabled = false;
                DropDownListID.Visible = true;
                DropDownListID.Enabled = true;
                if (DropDownListID.Items.Count == 1)
                {
                    DropDownListID.DataSource = _baseControlManager.SelectLookupList(LookupCategory);
                    DropDownListID.DataTextField = "NAME";
                    DropDownListID.DataValueField = "ID";
                    DropDownListID.DataBind();
                }
                break;
        }
        AddAttributes();

        if (RequiredFieldValidatorID.Visible)
        {
            RequiredFieldValidatorID.ErrorMessage = "'" + BaseControlName + "' is required.";
        }
    }


    /// <summary>
    /// Get extra space for label text based on indent value
    /// </summary>
    /// <param name="indent"></param>
    /// <returns></returns>
    private string GetExtraSpace()
    {
        string results = string.Empty;

        if (IndentForLabel > 0)
        {
            for (int i = 1; i <= IndentForLabel; i++)
            {
                results = results + "&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;";
            }
        }

        return results;
    }


    /// <summary>
    /// Add attributes to controls to invoke javascript function to copy data to hiddenfield.
    /// </summary>
    private void AddAttributes()
    {
        switch (BaseControlType)
        {
            case DveirBaseControlType.TextBox:
                if (TextBoxID.Attributes["onchange"] == null)
                {
                    TextBoxID.Attributes.Add("onchange",
                        "SetBaseControlTextValue(this, '" + HiddenFieldID.ClientID + "')");
                }
                break;
            case DveirBaseControlType.RadioButtonList:
                RadioButtonListJavaScripts();
                break;
            case DveirBaseControlType.DropdownList:
                if (DropDownListID.Attributes["onchange"] == null)
                {
                    DropDownListID.Attributes.Add("onchange",
                        "SetBaseControlTextValue(this, '" + HiddenFieldID.ClientID + "')");
                }
                break;
            case DveirBaseControlType.Checkbox:
                if (CheckBoxID.Attributes["onclick"] == null)
                {
                    CheckBoxID.Attributes.Add("onclick",
                        "SetBaseControlCheckBoxTextValue(this, '" + HiddenFieldID.ClientID + "')");
                }
                break;
        }
    }

    public void RadioButtonListJavaScripts()
    {

        string scriptName = "__ToggleRadioButtonList";

        if (!Page.ClientScript.IsClientScriptBlockRegistered(Page.GetType(), scriptName))
        {
            Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName, GetPageJavaScript(), true);
        }
    }
    
    private string GetPageJavaScript()
    {
        StringBuilder myStringBuilder = new StringBuilder();

        myStringBuilder.Append(" function SetRadioButtonValueOnly(clientId, index, hiddenFieldId) {"
                        + Environment.NewLine);
        myStringBuilder.Append("    var radioButtonList = document.getElementById(clientId);"
                                + Environment.NewLine);
        myStringBuilder.Append("    var radioButton = document.getElementById(clientId + '_' + index);"
                                + Environment.NewLine + Environment.NewLine);
        myStringBuilder.Append("    SetBaseControlTextValue(radioButton, hiddenFieldId);" + Environment.NewLine);
        myStringBuilder.Append("}" + Environment.NewLine);

        return myStringBuilder.ToString();

    }

    protected void RadioButtonListID_onDataBound(object sender, EventArgs e)
    {
        RadioButtonList radioButtonList = (RadioButtonList)sender;
        int index = 0;
        foreach (ListItem li in radioButtonList.Items)
        {
            li.Attributes.Add("onclick", "javascript:SetRadioButtonValueOnly('" + radioButtonList.ClientID + "', " + index + ", '" + HiddenFieldID.ClientID + "')");
            index++;
        }
    }


}
