﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_EncounterTabUserControl : BaseControl
{
    private DVEIREncounterManager _encounterManager = new DVEIREncounterManager();

    #region public events
    
    /// <summary>
    /// Event to raise when an encounter is selected.  The EventArgs for the event will contain the selected encounter id.
    /// </summary>
    public event EncounterClickEvent EncounterSelected;

    public delegate void EncounterClickEvent(object sender, EncounterClickEventArgs e);

    public class EncounterClickEventArgs : EventArgs
    {
        public int EncounterID {get;set;}
    }

    /// <summary>
    /// Event to raise when ETL encounters are searched.  
    /// </summary>
    public event EncounterSearchEvent EtlEncounterSearched;
    public delegate void EncounterSearchEvent(object sender);

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewEncounters.ClientID + "');", true);
        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewSearch.ClientID + "');", true);
    }
 
    #region Postback Events

    protected void GridViewEncounters_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            //get the id of the selected item
            int index = Convert.ToInt32(e.CommandArgument);
            GridViewEncounters.SelectedIndex = index;
            int id = Convert.ToInt32(GridViewEncounters.SelectedValue);

            //Raise an Event to give notice that a item has been selected

            if (this.EncounterSelected != null) //check for listeners
            {
                //create an instance of the custom eventArgs in order to populate the id selected
                EncounterClickEventArgs eventArgs = new EncounterClickEventArgs();
                eventArgs.EncounterID = id;

                //raise event
                this.EncounterSelected(this, eventArgs);
            }

        }

    }

    protected void GridViewSearch_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Add")
        {
            //get the id of the selected item
            int index = Convert.ToInt32(e.CommandArgument);
            GridViewSearch.SelectedIndex = index;
            int id = Convert.ToInt32(GridViewSearch.SelectedValue);
            
            //Get the ER_ENCOUNTER_ETL object
            ER_ENCOUNTER_ETL encounterEtl = null;
            List<ER_ENCOUNTER_ETL> encounterList = (List<ER_ENCOUNTER_ETL>) Session["ETLEncounters"];
            foreach (ER_ENCOUNTER_ETL item in encounterList)
            {
                if (item.ER_ENCOUNTER_ID == id)
                {
                    encounterEtl = item;
                    break;
                }
            }

            if (encounterEtl != null)
            {
                if (_encounterManager.EncounterExists(encounterEtl.REFERRAL_ID,
                        (int) encounterEtl.PROVIDER_ID, encounterEtl.STA6A, encounterEtl.VISIT_DATE))
                {
                    Alert.Show("This encounter already exists in the Data Store.");
                }
                else
                {

                    AddNewEncounter(encounterEtl);
                    //Display encounter list panel
                    PanelEncounter.Visible = true;
                    PanelEncounter.DataBind();
                    PanelSearch.Visible = false;
                    ClearSearchScreen();
                }
            }
        }
    }

    /// <summary>
    /// Add a new encounter from ETL tables to data store.
    /// </summary>
    /// <param name="encounterEtl"></param>
    /// <returns></returns>
    private int AddNewEncounter(ER_ENCOUNTER_ETL encounterEtl)
    {
        //Convert ER_ENCOUNTER_ETL object to ER_ENCOUNTER object
        ER_ENCOUNTER encounter = ConvertEncounterObject(encounterEtl);

        //Exam object
        encounter.ER_EXAM = ConvertExamObject(encounterEtl);
        encounter.ER_EXAM.SetAsInsertOnSubmit();

        //Outcome object
        encounter.ER_OUTCOME = ConvertOutcomeObject(encounterEtl);
        encounter.ER_OUTCOME.SetAsInsertOnSubmit();

        //Encounter Diagnosis objects
        foreach (ER_DIAG_DTL_ETL item in encounterEtl.ER_DIAG_DTL_ETLs)
        {
            encounter.ER_DIAG_DTLs.Add(ConvertDiagnosisDetailObject(item));
            encounter.ER_DIAG_DTLs.Last().SetAsInsertOnSubmit();
        }

        //Encounter Procedures objects
        foreach (ER_PROCEDURE_DTL_ETL item in encounterEtl.ER_PROCEDURE_DTL_ETLs)
        {
            encounter.ER_PROCEDURE_DTLs.Add(ConvertProcedureDetailObject(item));
            encounter.ER_PROCEDURE_DTLs.Last().SetAsInsertOnSubmit();
        }

        //Save ER_ENCOUNTER object to database
        return _encounterManager.Update(encounter);
    }

    /// <summary>
    /// Convert an ER_ENCOUNTER_ETL object to an ER_ENCOUNTER object
    /// </summary>
    /// <param name="encounterEtl"></param>
    /// <returns></returns>
    private ER_ENCOUNTER ConvertEncounterObject(ER_ENCOUNTER_ETL encounterEtl)
    {
        // New ER_ENCOUNTER object
        ER_ENCOUNTER encounter = new ER_ENCOUNTER();
        encounter.SetAsChangeTrackingRoot();
        encounter.SetAsInsertOnSubmit();

        encounter.REFERRAL_ID = encounterEtl.REFERRAL_ID;
        encounter.PROVIDER_ID = encounterEtl.PROVIDER_ID;
        encounter.STA6A = encounterEtl.STA6A;
        encounter.VISIT_DATE = encounterEtl.VISIT_DATE;
        encounter.CHIEF_COMPLAINT_TEXT = encounterEtl.CHIEF_COMPLAINT_TEXT;
        encounter.TBI_IND = encounterEtl.TBI_IND;
        encounter.TBI_READ_PRBLM_IND = encounterEtl.TBI_READ_PRBLM_IND;
        encounter.TBI_DIPLOPIA_IND = encounterEtl.TBI_DIPLOPIA_IND;
        encounter.TBI_DAZZLING_IND = encounterEtl.TBI_DAZZLING_IND;
        encounter.TBI_PHOTOPHOBIA_IND = encounterEtl.TBI_PHOTOPHOBIA_IND;
        encounter.OTHER_NEUROLOGICAL_IND = encounterEtl.OTHER_NEUROLOGICAL_IND;
        encounter.NEUROLOGICAL_COMMENT_TEXT = encounterEtl.NEUROLOGICAL_COMMENT_TEXT;
        encounter.DIAGNOSIS_SUMMARY_COMMENT_TEXT = encounterEtl.DIAGNOSIS_SUMMARY_COMMENT_TEXT;
        encounter.PROCEDURE_SUMMARY_COMMENT_TEXT = encounterEtl.PROCEDURE_SUMMARY_COMMENT_TEXT;
        encounter.CURRENTLY_HOSPITALIZED_IND = encounterEtl.CURRENTLY_HOSPITALIZED_IND;
        encounter.TBI_EYE_STRAIN_IND = encounterEtl.TBI_EYE_STRAIN_IND;
        encounter.TBI_BLURRED_VISION_IND = encounterEtl.TBI_BLURRED_VISION_IND;
        encounter.PRIMARYCLINICSTOP = encounterEtl.PRIMARYCLINICSTOP;
        encounter.PRIMARYCLINICSTOPDESC = encounterEtl.PRIMARYCLINICSTOPDESC;
        encounter.SendTo_DoD_DVEIR_FLAG = encounterEtl.SendTo_DoD_DVEIR_FLAG;
        encounter.Ready_to_Send_to_DVEIR_FLAG = encounterEtl.Ready_to_Send_to_DVEIR_FLAG;

        return encounter;
    }


    /// <summary>
    /// Convert an ER_EXAM_ETL object to an ER_EXAM object
    /// </summary>
    /// <param name="examEtl"></param>
    /// <returns></returns>
    private ER_EXAM ConvertExamObject(ER_ENCOUNTER_ETL encounterEtl)
    {
        // New ER_EXAM object
        ER_EXAM exam = new ER_EXAM();
        ER_EXAM_ETL examEtl = encounterEtl.ER_EXAM_ETL;

        if (examEtl == null)
        {
            exam.PATIENT_ID = encounterEtl.REFERRAL.PATIENT_ID;
        }
        else
        {
            exam.PATIENT_ID = examEtl.PATIENT_ID;
            exam.PATIENT_VISIT_COUNT = examEtl.PATIENT_VISIT_COUNT;
            exam.LEFT_REFRACTION_NUMBER = examEtl.LEFT_REFRACTION_NUMBER;
            exam.RIGHT_REFRACTION_NUMBER = examEtl.RIGHT_REFRACTION_NUMBER;
            exam.LEFT_COLOR_RESPONSE_NUMBER = examEtl.LEFT_COLOR_RESPONSE_NUMBER;
            exam.RIGHT_COLOR_RESPONSE_NUMBER = examEtl.RIGHT_COLOR_RESPONSE_NUMBER;
            exam.LEFT_CONFRONTATION_NUMBER = examEtl.LEFT_CONFRONTATION_NUMBER;
            exam.RIGHT_CONFRONTATION_NUMBER = examEtl.RIGHT_CONFRONTATION_NUMBER;
            exam.LEFT_MEAN_DEVIATION_NUMBER = examEtl.LEFT_MEAN_DEVIATION_NUMBER;
            exam.RIGHT_MEAN_DEVIATION_NUMBER = examEtl.RIGHT_MEAN_DEVIATION_NUMBER;
            exam.LEFT_STD_DEVIATION_NUMBER = examEtl.LEFT_STD_DEVIATION_NUMBER;
            exam.RIGHT_STD_DEVIATION_NUMBER = examEtl.RIGHT_STD_DEVIATION_NUMBER;
            exam.LEFT_FIXATION_LOSS_NUMBER = examEtl.LEFT_FIXATION_LOSS_NUMBER;
            exam.RIGHT_FIXATION_LOSS_NUMBER = examEtl.RIGHT_FIXATION_LOSS_NUMBER;
            exam.EXAM_DATE = examEtl.EXAM_DATE;
            exam.COMMENT_TEXT = examEtl.COMMENT_TEXT;
            exam.LEFT_SPHERE_NUMBER = examEtl.LEFT_SPHERE_NUMBER;
            exam.RIGHT_SPHERE_NUMBER = examEtl.RIGHT_SPHERE_NUMBER;
            exam.LEFT_CYLINDER_NUMBER = examEtl.LEFT_CYLINDER_NUMBER;
            exam.RIGHT_CYLINDER_NUMBER = examEtl.RIGHT_CYLINDER_NUMBER;
            exam.LEFT_MEAN_DEVIATION_COMMENTS = examEtl.LEFT_MEAN_DEVIATION_COMMENTS;
            exam.RIGHT_MEAN_DEVIATION_COMMENTS = examEtl.RIGHT_MEAN_DEVIATION_COMMENTS;
            exam.LEFT_REFRACTION_UNKNOWN_FLAG = examEtl.LEFT_REFRACTION_UNKNOWN_FLAG;
            exam.RIGHT_REFRACTION_UNKNOWN_FLAG = examEtl.RIGHT_REFRACTION_UNKNOWN_FLAG;
            exam.LEFT_KINETIC_VISUAL_FIELDS_GOLDMAN_COMMENTS = examEtl.LEFT_KINETIC_VISUAL_FIELDS_GOLDMAN_COMMENTS;
            exam.RIGHT_KINETIC_VISUAL_FIELDS_GOLDMAN_COMMENTS = examEtl.RIGHT_KINETIC_VISUAL_FIELDS_GOLDMAN_COMMENTS;
            exam.LEFT_CONFRONTATION_PERFORMED = examEtl.LEFT_CONFRONTATION_PERFORMED;
            exam.RIGHT_CONFRONTATION_PERFORMED = examEtl.RIGHT_CONFRONTATION_PERFORMED;
            exam.LEFT_CONFRONTATION_STATUS = examEtl.LEFT_CONFRONTATION_STATUS;
            exam.RIGHT_CONFRONTATION_STATUS = examEtl.RIGHT_CONFRONTATION_STATUS;
            exam.LEFT_REFERENCE_SCALE = examEtl.LEFT_REFERENCE_SCALE;
            exam.RIGHT_REFERENCE_SCALE = examEtl.RIGHT_REFERENCE_SCALE;
        }
        return exam;
    }

    /// <summary>
    /// Convert an ER_OUTCOME_ETL object to an ER_OUTCOME object
    /// </summary>
    /// <param name="outcomeEtl"></param>
    /// <returns></returns>
    private ER_OUTCOME ConvertOutcomeObject(ER_ENCOUNTER_ETL encounterEtl)
    {
        // New ER_OUTCOME object
        ER_OUTCOME outcome = new ER_OUTCOME();
        ER_OUTCOME_ETL outcomeEtl = encounterEtl.ER_OUTCOME_ETL;

        if (outcomeEtl == null)
        {
            outcome.PATIENT_ID = encounterEtl.REFERRAL.PATIENT_ID;
        }
        else
        {
            outcome.PATIENT_ID = outcomeEtl.PATIENT_ID;
            outcome.STD_ER_VISION_STATUS_ID = outcomeEtl.STD_ER_VISION_STATUS_ID;
            outcome.DUTY_RETURN_IND = outcomeEtl.DUTY_RETURN_IND;
            outcome.DUTY_RETURN_DSCR = outcomeEtl.DUTY_RETURN_DSCR;
            outcome.CURRENT_DISABILITY_RATING_NUMB = outcomeEtl.CURRENT_DISABILITY_RATING_NUMB;
            outcome.DISABILITY_RATING_DATE = outcomeEtl.DISABILITY_RATING_DATE;
            outcome.REHABILITATION_REFERRAL_IND = outcomeEtl.REHABILITATION_REFERRAL_IND;
            outcome.VFQ_NUMBER = outcomeEtl.VFQ_NUMBER;
            outcome.STD_ER_VFQ_QUESTIONNAIRE_TYPE_ID = outcomeEtl.STD_ER_VFQ_QUESTIONNAIRE_TYPE_ID;
        }
        return outcome;
    }

    /// <summary>
    /// Convert an ER_DIAG_DTL_ETL object to an ER_DIAG_DTL object
    /// </summary>
    /// <param name="DiagDetailEtl"></param>
    /// <returns></returns>
    private ER_DIAG_DTL ConvertDiagnosisDetailObject(ER_DIAG_DTL_ETL diagDetailEtl)
    {
        // New ER_DIAG_DTL object
        ER_DIAG_DTL diagDetail = null;

        if (diagDetailEtl != null)
        {
            diagDetail = new ER_DIAG_DTL();
            diagDetail.COMMENT_TEXT = diagDetailEtl.COMMENT_TEXT;
            diagDetail.CODE = diagDetailEtl.CODE;
            diagDetail.DESCRIPTION_TEXT = diagDetailEtl.DESCRIPTION_TEXT;
            diagDetail.ICD_CODE_ID = diagDetailEtl.ICD_CODE_ID;
        }

        return diagDetail;
    }

    /// <summary>
    /// Convert an ER_PROCEDURE_DTL_ETL object to an ER_PROCEDURE_DTL object
    /// </summary>
    /// <param name="procedureDetailEtl"></param>
    /// <returns></returns>
    private ER_PROCEDURE_DTL ConvertProcedureDetailObject(ER_PROCEDURE_DTL_ETL procedureDetailEtl)
    {
        // New ER_PROCEDURE_DTL object
        ER_PROCEDURE_DTL procedureDetail = null;

        if (procedureDetailEtl != null)
        {
            procedureDetail = new ER_PROCEDURE_DTL();
            procedureDetail.COMMENT_TEXT = procedureDetailEtl.COMMENT_TEXT;
            procedureDetail.ER_ENCOUNTER_ID = procedureDetailEtl.ER_ENCOUNTER_ID;
            procedureDetail.CODE = procedureDetailEtl.CODE;
            procedureDetail.DESCRIPTION_TEXT = procedureDetailEtl.DESCRIPTION_TEXT;
            procedureDetail.ICD9_FLAG = procedureDetailEtl.ICD9_FLAG;
            procedureDetail.CPT_CODE_ID = procedureDetailEtl.CPT_CODE_ID;
        }
        return procedureDetail;
    }
    protected void ButtonAddEncounter_Click(object sender, EventArgs e)
    {
        PanelEncounter.Visible = false;
        PanelSearch.Visible = true;
    }

    private void AddEncounter(int etlEncounterId)
    {
    }

    protected void ButtonCancelAdd_Click(object sender, EventArgs e)
    {
        PanelSearch.Visible = false;
        ClearSearchScreen();
        PanelEncounter.Visible = true;
    }

    private void ClearSearchScreen()
    {
        TextBoxDate.Text = string.Empty;
        GridViewSearch.DataSource = null;
        GridViewSearch.DataBind();
        GridViewSearch.SelectedIndex = -1;
    }

    protected void ButtonSearch_Click(object sender, EventArgs e)
    {
        DateTime date;
        if (DateTime.TryParse(TextBoxDate.Text, out date))
        {
            List<ER_ENCOUNTER_ETL> encounters = _encounterManager.SelectByDate(Convert.ToInt32(HiddenFieldReferralId.Value), date).ToList();
            GridViewSearch.DataSource = encounters;
            GridViewSearch.DataBind();

            Session["ETLEncounters"] = encounters;
        }

        if (this.EtlEncounterSearched != null) //check for listeners
        {
            //raise event
            this.EtlEncounterSearched(this);
        }
    }

    protected void GridViewEncounters_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = string.Empty; // e.Row.Cells[0].Text;
            Label lbl = e.Row.Cells[0].Controls[1] as Label;
            if (lbl != null)
            {
                s1 = lbl.Text;
            }

            Button btn = e.Row.Cells[5].Controls[0] as Button;
            if (btn != null)
            {
                btn.ToolTip = "Select the record for Visit Date " + s1;
            }
        }
    }

    protected void GridViewSearch_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = string.Empty; // e.Row.Cells[0].Text;
            Label lbl = e.Row.Cells[0].Controls[1] as Label;
            if (lbl != null)
            {
                s1 = lbl.Text;
            }

            Button btn = e.Row.Cells[2].Controls[0] as Button;
            if (btn != null)
            {
                btn.ToolTip = "Select the record for Visit Date " + s1;
            }
        }
    }

    #endregion


    #region Object/Form Mapping


    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eyeExam"></param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(DVEIR_REFERRAL referral)
    {
        switch ((DVEIRReferralStatus)referral.STD_REFERRALSTS_ID)
        {
            case DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR:
            case DVEIRReferralStatus.DVEIR_COMPLETED:
                ButtonAddEncounter.Visible = false;
                break;
        }

        //populate dropdowns
        PopulateDropDowns(referral);

        HiddenFieldReferralId.Value = referral.REFERRAL_ID.ToString();

        return 0;

    }

    private void LoadObjectFromControls(ref DVEIR_REFERRAL referral)
    {

    }

    private void PopulateDropDowns(DVEIR_REFERRAL referral)
    {

    }

    public void ClearEncounterGridSelection ()
    {
        GridViewEncounters.SelectedIndex = -1;

    }

    #endregion

}
