﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_ExamTabAcuityTestUserControl : BaseClinicalDataControl
{
    private string panelName = "Set Panel Name in ASPX";

    public List<KeyValuePair<int, string>> examData { get; set; }
    public bool PinholeVisible { get; set; }
    public string PanelName 
    {
        get
        {
            return panelName;
        }
        
        set
        {
           PanelVisualAcuity.GroupingText = value;
        }
    }
    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }
    public bool IsNearDistance { get; set; }
    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }
    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }
    /// <summary>
    /// Set the value to the test number performed 
    /// </summary>
    public STD_ER_ACUITY_TYPE.TestNumberType TestNumber { get; set; }
    public string TestNumberCodeValue
    {
        get
        {
            return STD_ER_ACUITY_TYPE.GetTestNumberCodeValue(TestNumber);
        }

    }
    public int TestNumberID
    {
        get
        {
            return STD_ER_ACUITY_TYPE.GetTestNumberId(TestNumber);
        }

    }



    protected void Page_Load(object sender, EventArgs e)
    {
        ReviewCommentUserControlAcuity.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlAcuity.LocationPage = this.LocationPage;
        ReviewCommentUserControlAcuity.AnchorId = this.AnchorId;


        if (!PinholeVisible
                && RadioButtonListVisualAcuityCorrected.Items.Count == 4)
        {
            RadioButtonListVisualAcuityCorrected.Items.Remove(
                RadioButtonListVisualAcuityCorrected.Items[1]);
        }

    }

    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
     /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ExamTestPerformedUserControl.ExamTestName = LocationFriendlyName;
        ExamTestResultUserControl.ExamTestName = LocationFriendlyName;
        ExamCommentsUserControl.ExamTestName = LocationFriendlyName;
        ER_EXAM exam = encounter.ER_EXAM;

        int testPerformedId = 0;
        int testResultsId = 0;
        int testCommentsId = 0;
        int refractionSphere = 0;
        int refractionCylinder = 0;
        int refractionAxis = 0;
        int refractionUnknownFlag = 0;
        int acuityOther = 0;


        //populate dropdowns
        PopulateDropDowns();

        if (LocationFriendlyName.IndexOf("1 - Best Corrected") > 0)
        {
            testPerformedId = DVEIRSharedConstant.ExamDataElementVisualAcuity1TestPerformed;
            testResultsId = DVEIRSharedConstant.ExamDataElementVisualAcuity1TestResult;
            testCommentsId = DVEIRSharedConstant.ExamDataElementVisualAcuity1Comments;
            refractionSphere = DVEIRSharedConstant.ExamDataElementRefraction1Sphere;
            refractionCylinder = DVEIRSharedConstant.ExamDataElementRefraction1Cylinder;
            refractionAxis = DVEIRSharedConstant.ExamDataElementRefraction1Axis;
            refractionUnknownFlag = DVEIRSharedConstant.ExamDataElementRefraction1UnknownFlag;
            acuityOther = DVEIRSharedConstant.ExamDataElementVisualAcuity1Other;
        }
        else
        {
            testPerformedId = DVEIRSharedConstant.ExamDataElementVisualAcuity2TestPerformed;
            testResultsId = DVEIRSharedConstant.ExamDataElementVisualAcuity2TestResult;
            testCommentsId = DVEIRSharedConstant.ExamDataElementVisualAcuity2Comments;
            refractionSphere = DVEIRSharedConstant.ExamDataElementRefraction2Sphere;
            refractionCylinder = DVEIRSharedConstant.ExamDataElementRefraction2Cylinder;
            refractionAxis = DVEIRSharedConstant.ExamDataElementRefraction2Axis;
            refractionUnknownFlag = DVEIRSharedConstant.ExamDataElementRefraction2UnknownFlag;
            acuityOther = DVEIRSharedConstant.ExamDataElementVisualAcuity2Other;
        }

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            testPerformedId, DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

        DropDownListVisualAcuity.ToolTip = "Visual Acuity " + ((int)TestNumber + 1) + " Value for " + LocationFriendlyName.Substring(LocationFriendlyName.IndexOf("-") + 1).Trim();

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            testResultsId, DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            //Filter Data for control
            ER_ACUITY_EXAM_DTL acuityDetail = GetAcuityDetail(exam);

            if (acuityDetail != null)
            {

                //Visual Acuity
                //TextBoxVisualAcuity.MaxLength = 6;
                DropDownListVisualAcuity.SelectedValue = acuityDetail.COMMENT_TEXT;

                if (DropDownListVisualAcuity.SelectedValue == "Other")
                {
                    TextBoxOtherVisualAcuity.Text = Helpers.GetExamDetailValue(encounter,
            acuityOther, DveirBaseControlType.TextBox, getRightEye, getLeftEye); ;
                }

                //Less than 20
                DropDownListVisualAcuityLessThan20200.SelectedValue = acuityDetail.STD_ER_ACUITY_MTHD_ID.ToString();

                //Vision Corrected Indicator
                if (acuityDetail.VISION_CORRECTED_IND.HasValue)
                {
                    RadioButtonListVisualAcuityCorrected.Text = acuityDetail.VISION_CORRECTED_IND.Value.ToString();
                }
                else
                {
                    RadioButtonListVisualAcuityCorrected.ClearSelection();
                }
            }
        }

        //Refraction
        TextBoxAxis.MaxLength = 3;
        TextBoxAxis.Text = Helpers.GetExamDetailValue(encounter,
            refractionAxis, DveirBaseControlType.TextBox, getRightEye, getLeftEye);
        TextBoxAxis.ToolTip = "Enter value for Axis (XXX)";

        FormatAxisText();

        //Sphere
        TextBoxSphere.MaxLength = 6;
        TextBoxSphere.Text = Helpers.GetExamDetailValue(encounter,
            refractionSphere, DveirBaseControlType.TextBox, getRightEye, getLeftEye);
        TextBoxSphere.ToolTip = "Enter value for Sphere (+/-XX.XX)";

        //Cylinder
        TextBoxCylinder.MaxLength = 6;
        TextBoxCylinder.Text = Helpers.GetExamDetailValue(encounter,
            refractionCylinder, DveirBaseControlType.TextBox, getRightEye, getLeftEye);
        TextBoxCylinder.ToolTip = "Enter value for Cylinder (+/-XX.XX)";

        //Refraction Unknown
        CheckBoxRefractionUnknown.Checked = (Helpers.GetExamDetailValue(encounter,
            refractionUnknownFlag, DveirBaseControlType.TextBox, getRightEye, getLeftEye) == "true");

        if (!string.IsNullOrEmpty(TextBoxAxis.Text)
                || !string.IsNullOrEmpty(TextBoxSphere.Text)
                    || !string.IsNullOrEmpty(TextBoxCylinder.Text)
           )
        {
            CheckBoxRefractionUnknown.Checked = false;
        }

        TextBoxAxis.Enabled =
            TextBoxSphere.Enabled =
            TextBoxCylinder.Enabled = !CheckBoxRefractionUnknown.Checked; 
        
        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            testCommentsId, DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        ReviewCommentUserControlAcuity.LoadControlsFromObject(encounter);

        return 0;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        int returnStatus = 0;

        int testPerformedId = 0;
        int testResultsId = 0;
        int testCommentsId = 0;
        int refractionSphere = 0;
        int refractionCylinder = 0;
        int refractionAxis = 0;
        int refractionUnknownFlag = 0;
        int acuityOther = 0;

        if (LocationFriendlyName.IndexOf("1 - Best Corrected") > 0)
        {
            testPerformedId = DVEIRSharedConstant.ExamDataElementVisualAcuity1TestPerformed;
            testResultsId = DVEIRSharedConstant.ExamDataElementVisualAcuity1TestResult;
            testCommentsId = DVEIRSharedConstant.ExamDataElementVisualAcuity1Comments;
            refractionSphere = DVEIRSharedConstant.ExamDataElementRefraction1Sphere;
            refractionCylinder = DVEIRSharedConstant.ExamDataElementRefraction1Cylinder;
            refractionAxis = DVEIRSharedConstant.ExamDataElementRefraction1Axis;
            refractionUnknownFlag = DVEIRSharedConstant.ExamDataElementRefraction1UnknownFlag;
            acuityOther = DVEIRSharedConstant.ExamDataElementVisualAcuity1Other;
        }
        else
        {
            testPerformedId = DVEIRSharedConstant.ExamDataElementVisualAcuity2TestPerformed;
            testResultsId = DVEIRSharedConstant.ExamDataElementVisualAcuity2TestResult;
            testCommentsId = DVEIRSharedConstant.ExamDataElementVisualAcuity2Comments;
            refractionSphere = DVEIRSharedConstant.ExamDataElementRefraction2Sphere;
            refractionCylinder = DVEIRSharedConstant.ExamDataElementRefraction2Cylinder;
            refractionAxis = DVEIRSharedConstant.ExamDataElementRefraction2Axis;
            refractionUnknownFlag = DVEIRSharedConstant.ExamDataElementRefraction2UnknownFlag;
            acuityOther = DVEIRSharedConstant.ExamDataElementVisualAcuity2Other;
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            testPerformedId, ExamTestPerformedUserControl.TestPerformed, getRightEye, getLeftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                testResultsId,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            foreach (ER_ACUITY_EXAM_DTL detail in encounter.ER_EXAM.ER_ACUITY_EXAM_DTLs)
            {
                if (detail.RIGHT_FLAG == getRightEye
                    && detail.LEFT_FLAG == getLeftEye
                    && detail.STD_ER_ACUITY_TYPE_ID == TestNumberID)
                {
                    detail.SetAsDeleteOnSubmit();
                    break;
                }
            }

            ER_ACUITY_EXAM_DTL acuityDetail = null;

            if (!string.IsNullOrEmpty(DropDownListVisualAcuity.SelectedValue)
                    || !string.IsNullOrEmpty(DropDownListVisualAcuityLessThan20200.SelectedValue))
            {
                acuityDetail = new ER_ACUITY_EXAM_DTL();
                acuityDetail.ER_EXAM_ID = encounter.ER_EXAM.ER_EXAM_ID;
                acuityDetail.LEFT_FLAG = getLeftEye;
                acuityDetail.RIGHT_FLAG = getRightEye;
                acuityDetail.STD_ER_ACUITY_TYPE_ID = TestNumberID;

                //Visual Acuity
                acuityDetail.COMMENT_TEXT = DropDownListVisualAcuity.SelectedValue;
                Helpers.LoadOneObjectForExamDetail(ref encounter,
                     acuityOther, TextBoxOtherVisualAcuity.Text, getRightEye, getLeftEye,
                     DveirBaseControlType.TextBox);

                //Less than 20
                acuityDetail.STD_ER_ACUITY_MTHD_ID = Helpers.getNullableIntFromString(DropDownListVisualAcuityLessThan20200.SelectedValue);

                //Vision Corrected Indicator
                acuityDetail.VISION_CORRECTED_IND = Helpers.ConvertStringToNullable<char>(RadioButtonListVisualAcuityCorrected.Text);

                encounter.ER_EXAM.ER_ACUITY_EXAM_DTLs.Add(acuityDetail);
                encounter.ER_EXAM.ER_ACUITY_EXAM_DTLs.Last().SetAsInsertOnSubmit();

            }

            Helpers.LoadOneObjectForExamDetail(ref encounter,
                refractionSphere, TextBoxSphere.Text, getRightEye, getLeftEye,
                DveirBaseControlType.TextBox);
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                refractionCylinder, TextBoxCylinder.Text, getRightEye, getLeftEye,
                DveirBaseControlType.TextBox);

            FormatAxisText();

            Helpers.LoadOneObjectForExamDetail(ref encounter,
                refractionAxis, TextBoxAxis.Text, getRightEye, getLeftEye,
                DveirBaseControlType.TextBox);

            if (CheckBoxRefractionUnknown.Checked)
            {
                Helpers.LoadOneObjectForExamDetail(ref encounter,
                   refractionUnknownFlag, "true", getRightEye, getLeftEye,
                   DveirBaseControlType.TextBox);
            }
            else
            {
                Helpers.LoadOneObjectForExamDetail(ref encounter,
                   refractionUnknownFlag, "false", getRightEye, getLeftEye,
                   DveirBaseControlType.TextBox);
            }
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
              testResultsId,
              ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);

            Helpers.RemoveOneRowForExamDetail(ref encounter,
                refractionSphere, TextBoxSphere.Text, getRightEye, getLeftEye);
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                refractionCylinder, TextBoxCylinder.Text, getRightEye, getLeftEye);
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                refractionAxis, TextBoxAxis.Text, getRightEye, getLeftEye);

            if (CheckBoxRefractionUnknown.Checked)
            {
                Helpers.RemoveOneRowForExamDetail(ref encounter,
                   refractionUnknownFlag, "true", getRightEye, getLeftEye);
            }
            else
            {
                Helpers.RemoveOneRowForExamDetail(ref encounter,
                   refractionUnknownFlag, "false", getRightEye, getLeftEye);
            }
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            testCommentsId, ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlAcuity.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }

    private ER_ACUITY_EXAM_DTL GetAcuityDetail(ER_EXAM exam)
    {
        ER_ACUITY_EXAM_DTL acuityDetail;

        //Filter Data for control
        acuityDetail = (from e in exam.ER_ACUITY_EXAM_DTLs
                        where e.LEFT_FLAG == getLeftEye
                        where e.RIGHT_FLAG == getRightEye
                        where e.STD_ER_ACUITY_TYPE_ID == TestNumberID
                       select e).FirstOrDefault();
        return acuityDetail;

    }

    private void PopulateDropDowns()
    {
        if (DropDownListVisualAcuity.Items.Count <= 1)
        {
            string acuityCategory = string.Empty;
            if (IsNearDistance)
            {
                acuityCategory = "VISUAL_ACUITY_NEAR_DISTANCE";
            }
            else
            {
                acuityCategory = "VISUAL_ACUITY_FAR_DISTANCE";
            }

            DVEIRBaseControlManager baseControlManager = new DVEIRBaseControlManager();
            DropDownListVisualAcuity.DataSource = baseControlManager.SelectLookupList(acuityCategory);
            DropDownListVisualAcuity.DataTextField = "NAME";
            DropDownListVisualAcuity.DataValueField = "NAME";
            DropDownListVisualAcuity.DataBind();

            if (DropDownListVisualAcuityLessThan20200.Items.Count == 1)
            {
                DropDownListVisualAcuityLessThan20200.DataBind();
            }
        }

    }

    /// <summary>
    /// VHAISPBLANCT | RTC 20713 | Additional logic to store/display Axis value as 3 digits with leading zeros
    /// </summary>
    private void FormatAxisText()
    {
        if (TextBoxAxis.Text.Length > 3)
        {
            TextBoxAxis.Text = TextBoxAxis.Text.LastChars(3);
        }
        else if (TextBoxAxis.Text.Length == 2)
        {
            TextBoxAxis.Text = "0" + TextBoxAxis.Text;
        }
        else if (TextBoxAxis.Text.Length == 1)
        {
            TextBoxAxis.Text = "00" + TextBoxAxis.Text;
        }
    }
 
    #endregion

}
