﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class controls_ExamTabAngleRecessionUserControl : BaseClinicalDataControl
{
    private DVEIRBaseControlManager _baseControlManager = new DVEIRBaseControlManager();

    public string LabelforTestPerformed { get; set; }

    public string PanelName
    {
        get
        {
            return PanelAngleRecession.GroupingText;
        }

        set
        {
            PanelAngleRecession.GroupingText = value;
        }
    }


    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }


    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }

    protected void Page_Load(object sender, EventArgs e)
    {
        ReviewCommentUserControlAngleRecession.LocationPage = this.LocationPage;
        ReviewCommentUserControlAngleRecession.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlAngleRecession.AnchorId = this.AnchorId;
        LoadBaseControls();
    }


    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        foreach (ER_ANGLE_RECESSION_EXAM_DTL item in exam.ER_ANGLE_RECESSION_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == getRightEye && item.LEFT_FLAG == getLeftEye)
            {
                item.SetAsDeleteOnSubmit();
            }
        }

        int returnStatus = 0;

        if (exam == null)
        {
            exam = new ER_EXAM();
            exam.ER_EXAM_ID = encounter.ER_ENCOUNTER_ID;
            if (encounter.REFERRAL != null)
            {
                exam.PATIENT_ID = encounter.REFERRAL.PATIENT_ID;
            }
            encounter.ER_EXAM = exam;
            encounter.ER_EXAM.SetAsInsertOnSubmit();
        }

        LoadObjectFromControls(ref encounter, getRightEye, getLeftEye);

        returnStatus = ReviewCommentUserControlAngleRecession.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }


    /// <summary>
    /// Load encounter exam data for right eye or left eye
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadObjectFromControls(ref ER_ENCOUNTER encounter, bool rightEye, bool leftEye)
    {
        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementAngleRecessionTestPerformed,
            ExamTestPerformedUserControl.TestPerformed, rightEye, leftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the Angle Recession test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementAngleRecessionTestResult,
                ExamTestResultUserControl.TestResult, rightEye, leftEye,
                DveirBaseControlType.RadioButtonList);

            LoadOneObjectFromControl(ref encounter, BaseControlAngleRecessionClockHours, rightEye, leftEye);
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementAngleRecessionTestResult,
                ExamTestResultUserControl.TestResult, rightEye, leftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementAngleRecessionComments,
            ExamCommentsUserControl.Comments, rightEye, leftEye,
            DveirBaseControlType.TextBox);
    }


    /// <summary>
    /// Load one ER_ANGLE_RECESSION_EXAM_DTL object with the data from one AngleRecession user control
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="baseControl"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadOneObjectFromControl(ref ER_ENCOUNTER encounter,
        controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        if (!string.IsNullOrEmpty(baseControl.Text))
        {
            ER_ANGLE_RECESSION_EXAM_DTL newExamDetail = new ER_ANGLE_RECESSION_EXAM_DTL();
            newExamDetail.ER_EXAM_ID = encounter.ER_EXAM.ER_EXAM_ID;
            newExamDetail.STD_ER_ANGLE_RECESSION_ISSUE_ID = baseControl.BaseControlId;
            newExamDetail.EXAM_DATE = encounter.ER_EXAM.EXAM_DATE;
            newExamDetail.RIGHT_FLAG = rightEye;
            newExamDetail.LEFT_FLAG = leftEye;
            newExamDetail.COMMENT_TEXT = baseControl.Text;

            encounter.ER_EXAM.ER_ANGLE_RECESSION_EXAM_DTLs.Add(newExamDetail);
            encounter.ER_EXAM.ER_ANGLE_RECESSION_EXAM_DTLs.Last().SetAsInsertOnSubmit();
        }
    }


    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        LoadBaseControls();
        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        // Left Eye
        if (getLeftEye)
        {
            LoadControlsFromObject(encounter, false, true);
        }
        else  //Right eye
        {
            LoadControlsFromObject(encounter, true, false);
        }

        return 0;
    }


    /// <summary>
    /// Load controls from object for right eye or left eye
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadControlsFromObject(ER_ENCOUNTER encounter, bool rightEye, bool leftEye)
    {
        if (!string.IsNullOrEmpty(LabelforTestPerformed))
        {
            ExamTestPerformedUserControl.LabelForTestPerformed = LabelforTestPerformed;
        }

        BaseControlAngleRecessionClockHours.HideLabel = true;
        
        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementAngleRecessionTestPerformed,
            DveirBaseControlType.RadioButtonList, rightEye, leftEye);

        // Only load the following controls if the Angle Recession test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementAngleRecessionTestResult,
            DveirBaseControlType.RadioButtonList, rightEye, leftEye);

            LoadOneControl(encounter, BaseControlAngleRecessionClockHours, rightEye, leftEye);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementAngleRecessionComments,
            DveirBaseControlType.TextBox, rightEye, leftEye);

        ReviewCommentUserControlAngleRecession.LoadControlsFromObject(encounter);
    }

    private void LoadOneControl(ER_ENCOUNTER encounter, controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        foreach (ER_ANGLE_RECESSION_EXAM_DTL item in encounter.ER_EXAM.ER_ANGLE_RECESSION_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye
                && item.STD_ER_ANGLE_RECESSION_ISSUE_ID == baseControl.BaseControlId)
            {
                baseControl.Text = item.COMMENT_TEXT;
                break;
            }
        }
    }

    /// <summary>
    /// Load all the base controls based on the control property data from database
    /// </summary>
    private void LoadBaseControls()
    {
        List<VIEW_GUI_CONTROL> guiControls = _baseControlManager.SelectGuiControlList("ANGLE_RECESSION").ToList();
        foreach (VIEW_GUI_CONTROL dbItem in guiControls)
        {
            Control userControl = PanelAngleRecession.FindControl(dbItem.USER_CONTROL_ID);
            if (userControl != null)
            {
                controls_BaseControl baseControl = (controls_BaseControl)userControl;
                baseControl.BaseControlType = (DveirBaseControlType)dbItem.ISSUE_DETAIL_TYPE_ID;
                baseControl.IsRequired = dbItem.REQUIRED_FLAG;
                baseControl.ToolTip = dbItem.TOOLTIP;
                baseControl.BaseControlName = dbItem.NAME;
                baseControl.IndentForLabel = dbItem.INDENT_WIDTH;
                baseControl.IsMultilineTextBox = dbItem.MULTILINE_FLAG;
                baseControl.BaseControlId = dbItem.ID;
                if (dbItem.MAX_LENGTH > 0)
                {
                    baseControl.MaxLength = (int)dbItem.MAX_LENGTH;
                }

                if (!string.IsNullOrEmpty(dbItem.VALIDATION_VALUE))
                {
                    baseControl.ValidationExpression = dbItem.VALIDATION_VALUE;
                }
                if (!string.IsNullOrEmpty(dbItem.VALIDATION_MESSAGE))
                {
                    baseControl.ValidationErrorMessage = dbItem.VALIDATION_MESSAGE;
                }
                baseControl.LocationFriendlyName = dbItem.NAME;
                baseControl.LookupCategory = dbItem.LOOKUP_CATEGORY;
                baseControl.ConditionText = dbItem.CONDITION_TEXT;
                if (dbItem.CONTROL_WIDTH > 0)
                {
                    baseControl.ControlWidth = (int)dbItem.CONTROL_WIDTH;
                }
            }
        }
    }
}
