﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_ExamTabColorVisionTestUserControl : BaseClinicalDataControl
{
    private string panelName = "Set Panel Name in ASPX";

    public List<KeyValuePair<int, string>> examData { get; set; }
    
    public string PanelName 
    {
        get
        {
            return panelName;
        }
        
        set
        {
            PanelColorVision.GroupingText = value;
        }
    }

    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }
    
   
    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }

    protected void Page_Load(object sender, EventArgs e)
    {
        //TO DO : need to add javascript to handle the proper values of the items

        ReviewCommentUserControlVisionTest.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlVisionTest.LocationPage = this.LocationPage;
        ReviewCommentUserControlVisionTest.AnchorId = this.AnchorId;
    }

    #region Postback Events
    
    #endregion

    #region Private Methods
    
    #endregion

   
    #region Postback Events


    #endregion


    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        //use following as sample if tool tips need to be created that included Right Eye/Left Eye value
        TextBoxColorVision.ToolTip = "Number of Correct Responses Over Number of Possible Responses for Color Vision " + DVEIR_REFERRAL.GetEyeSettingDisplay(EyeSetting);

        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementColorVisionTestPerformed,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementColorVisionTestResult,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            //Color Vision
            TextBoxColorVision.MaxLength = 100;
            TextBoxColorVision.Text = (getLeftEye ? exam.LEFT_COLOR_RESPONSE_NUMBER : exam.RIGHT_COLOR_RESPONSE_NUMBER);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementColorVisionComments,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        ReviewCommentUserControlVisionTest.LoadControlsFromObject(encounter);

        return 0;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        int returnStatus = 0;

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementColorVisionTestPerformed,
            ExamTestPerformedUserControl.TestPerformed, getRightEye, getLeftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementColorVisionTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            if (getLeftEye)
            {
                //Color Vision
                exam.LEFT_COLOR_RESPONSE_NUMBER = TextBoxColorVision.Text;

            }
            else  //Right eye
            {
                //Color Vision
                exam.RIGHT_COLOR_RESPONSE_NUMBER = TextBoxColorVision.Text;

            }
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementColorVisionTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);

        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementColorVisionComments,
            ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlVisionTest.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }

    #endregion
}
