﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;
using System.Text;

public partial class clinicalDataUserControls_ExamTabConfrontationVisualTestUserControl : BaseClinicalDataControl
{
    private string panelName = "Set Panel Name in ASPX";

    public List<KeyValuePair<int, string>> examData { get; set; }
    
    public string PanelName 
    {
        get
        {
            return panelName;
        }
        
        set
        {
            PanelConfrontationVisualFields.GroupingText = value;
        }
    }

    
    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }
    
    
    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }


 

    protected void Page_Load(object sender, EventArgs e)
    {
        //TO DO : need to add javascript to handle the proper values of the items

        ReviewCommentUserControlConfrontationVisualFields.LocationPage = this.LocationPage;
        ReviewCommentUserControlConfrontationVisualFields.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlConfrontationVisualFields.AnchorId = this.AnchorId;
    }

    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementConfrontationTestPerformed,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementConfrontationTestResult,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementConfrontationComments,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        ReviewCommentUserControlConfrontationVisualFields.LoadControlsFromObject(encounter);

        return 0;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        int returnStatus = 0;

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementConfrontationTestPerformed,
            ExamTestPerformedUserControl.TestPerformed, getRightEye, getLeftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementConfrontationTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementConfrontationTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementConfrontationComments,
            ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlConfrontationVisualFields.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }

    #endregion

}
