﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq; 
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_ExamTabDynamicTestsUserControl : BaseClinicalDataControl
{
    private string panelName = "Set Panel Name in ASPX";
    private EyeSettingType _EyeSetting;
    private const string DROP_DOWN_INITIAL_VALUE = "-- Select a Value --";

    private enum IssueDetailType
    {
        Error,          //Undefined
        String,         //TextBox
        SingleSelect,   //DropDownList
        Boolean,        //RadioButtonList
        MultipleSelect  //CheckBoxList
    }
    private enum IssueDetailValidationType
    {
        Error,          //Undefined
        RegExp,         //Regular Expression
        Date,           //Date
        Range           //Range
    }

    public List<KeyValuePair<int, string>> ExamData { get; set; }
    public bool MultiLine { get; set; }
    public string PanelName
    {
        get
        {
            return panelName;
        }

        set
        {
            PanelTestResults.GroupingText = value;
        }
    }
    
    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting
    {
        get { return _EyeSetting; }
        set { _EyeSetting = value; }
    }

    public bool HideTestPerformedControls { get; set; }
    public string LabelforTestPerformed { get; set; }

    protected void Page_Load(object sender, EventArgs e)
    {
        string scriptName = "__dveirExamTabDynamicTestUserControlTextBoxClientSideScript";
        if (!Page.ClientScript.IsClientScriptBlockRegistered(scriptName))
        {
            Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName, GetTextBoxJavaScript(), true);
        }

        string scriptName2 = "__dveirExamTabDynamicTestUserControlJQueryScript";
        if (!Page.ClientScript.IsClientScriptIncludeRegistered(scriptName2))
        {
            Page.ClientScript.RegisterClientScriptInclude(Page.GetType(), scriptName2,"controls/dveirExamTabDynamicTestUserControlJQueryScript.js");
        }

        ReviewCommentUserControl.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControl.LocationPage = this.LocationPage;
        ReviewCommentUserControl.AnchorId = this.AnchorId;
           
    }

    #region Postback Events
    
    #endregion

    #region Private Methods

    private string GetTextBoxJavaScript()
    {
        StringBuilder myStringBuilder = new StringBuilder();

        myStringBuilder.Append("function checkBoxListOnClick(");
        myStringBuilder.Append("elementRef,");
        myStringBuilder.Append("textBoxName,");
        myStringBuilder.Append("requiredValidatorControlName,");
        myStringBuilder.Append("regexValidatorControlName,");
        myStringBuilder.Append("reqFlagRef,");
        myStringBuilder.Append("validationTypeControlName,");
        myStringBuilder.Append("textBoxLastValueName)");
        myStringBuilder.Append("{");
        myStringBuilder.Append("var TextBoxRef = document.getElementById(textBoxName);");
        myStringBuilder.Append("var TextBoxLastValue = document.getElementById(textBoxLastValueName);");
        myStringBuilder.Append("var RequiredFieldFlagRef = document.getElementById(reqFlagRef);");
        myStringBuilder.Append("var RequiredValidatorRef = document.getElementById(requiredValidatorControlName);");
        myStringBuilder.Append("var RegExValidatorRef = document.getElementById(regexValidatorControlName);");
        myStringBuilder.Append("var ValidationTypeRef = document.getElementById(validationTypeControlName);");
        myStringBuilder.Append("if (elementRef.checked == false)");
        myStringBuilder.Append("{");
        myStringBuilder.Append("    TextBoxRef.disabled = true;");
        myStringBuilder.Append("    TextBoxLastValue.value = TextBoxRef.value;");
        myStringBuilder.Append("    TextBoxRef.value = '';");
        myStringBuilder.Append("    if(RequiredFieldFlagRef.value == 'True'){");
        myStringBuilder.Append("        ValidatorEnable(RequiredValidatorRef, false);");
        myStringBuilder.Append("        RequiredValidatorRef.isvalid = true;");
        myStringBuilder.Append("        ValidatorUpdateDisplay(RequiredValidatorRef);");
        myStringBuilder.Append("    }");
        myStringBuilder.Append("    if(ValidationTypeRef.value == 1){");
        myStringBuilder.Append("        ValidatorEnable(RegExValidatorRef, false);");
        myStringBuilder.Append("        RegExValidatorRef.isvalid = true;");
        myStringBuilder.Append("        ValidatorUpdateDisplay(RegExValidatorRef);");
        myStringBuilder.Append("    }");
        myStringBuilder.Append("}");
        myStringBuilder.Append("else");
        myStringBuilder.Append("{");
        myStringBuilder.Append("    TextBoxRef.disabled = false;");
        myStringBuilder.Append("    TextBoxRef.value = TextBoxLastValue.value;");
        myStringBuilder.Append("    if(RequiredFieldFlagRef.value == 'True'){");
        myStringBuilder.Append("        ValidatorEnable(RequiredValidatorRef, true);");
        myStringBuilder.Append("        ValidatorUpdateDisplay(RequiredValidatorRef);");
        myStringBuilder.Append("    }");
        myStringBuilder.Append("    if(ValidationTypeRef.value == 1){");
        myStringBuilder.Append("        ValidatorEnable(RegExValidatorRef, true);");
        myStringBuilder.Append("        ValidatorUpdateDisplay(RegExValidatorRef);");
        myStringBuilder.Append("    }");
        myStringBuilder.Append("}");
        myStringBuilder.Append("return true;");
        myStringBuilder.Append("}");
        
        return myStringBuilder.ToString();
    }

    #endregion

    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="examList"><typeparam name="T">List to populate checkboxes </typeparam></param>
    /// <param name="KeyField">Field that represents the checkbox value</param>
    /// <param name="ValueField">Field that represents the additional data value</param>
    /// <returns>int</returns>
    //public void LoadControlsFromObject<T>(List<T> examList) where T : class, LINQEntityBase
    public int LoadControlsFromObject<T>(List<T> examList, string KeyField, string ValueField, object checkBoxListDataSource, ER_ENCOUNTER encounter, int categoryIdTestPerformed,
        int categoryIdTestResult, int categoryIdComments)
        where T : LINQEntityBase, new()
    {

        //EyeSetting
        bool getLeftEye = (EyeSetting == EyeSettingType.LeftEye ? true : false);
        bool getRightEye = (EyeSetting == EyeSettingType.RightEye ? true : false);

        if (EyeSetting == EyeSettingType.BothEyes)
        {
            getLeftEye = true;
            getRightEye = true;
        }

        HiddenFieldCategoryIdTestPerformed.Value = categoryIdTestPerformed.ToString();
        HiddenFieldCategoryIdTestResult.Value = categoryIdTestResult.ToString();
        HiddenFieldCategoryIdComments.Value = categoryIdComments.ToString();

        if (HideTestPerformedControls)
        {
            ExamTestPerformedUserControl.Visible = false;
            ExamTestResultUserControl.Visible = false;
            ExamCommentsUserControl.Visible = false;
        }
        else
        {
            ExamTestPerformedUserControl.ExamTestName = LocationFriendlyName;
            ExamTestResultUserControl.ExamTestName = LocationFriendlyName;
            ExamCommentsUserControl.ExamTestName = LocationFriendlyName;

            if (!string.IsNullOrEmpty(LabelforTestPerformed))
            {
                ExamTestPerformedUserControl.LabelForTestPerformed = LabelforTestPerformed;
            }

            ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
                categoryIdTestPerformed, DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            // Only load the following controls if the test was performed
            if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
            {
                ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
                categoryIdTestResult, DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);
            }
            ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
               categoryIdComments, DveirBaseControlType.TextBox, getRightEye, getLeftEye);
        }

        ExamData = (from e in examList
                    where
                        (bool)(from p in e.GetType().GetProperties()
                               where p.Name.Equals("LEFT_FLAG")
                               select p.GetValue(e, null)).FirstOrDefault()
                        == getLeftEye
                        &&
                        (bool)(from p in e.GetType().GetProperties()
                               where p.Name.Equals("RIGHT_FLAG")
                               select p.GetValue(e, null)).FirstOrDefault()
                        == getRightEye

                    select new KeyValuePair<int, string>(
                        (int)(from p in e.GetType().GetProperties()
                              where p.Name.Equals(KeyField)
                              select p.GetValue(e, null)).FirstOrDefault()
                          ,
                          (string)(from p in e.GetType().GetProperties()
                                   where p.Name.Equals(ValueField)
                                   select p.GetValue(e, null)).FirstOrDefault()


                        )).ToList();

        RepeaterTestResults.DataSource = checkBoxListDataSource;

        RepeaterTestResults.DataBind();
        
        ReviewCommentUserControl.LoadControlsFromObject(encounter);

        return 0;
    }

    protected void RepeaterTestResults_OnItemDataBound(object sender, RepeaterItemEventArgs e)
    {
        if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
        {
            Label labelforTextBoxExamIndicatorValue = (Label)e.Item.FindControl("LabelforTextBoxExamIndicatorValue");
            CheckBox checkBoxExamIndicator = (CheckBox)e.Item.FindControl("CheckBoxExamIndicator");
            HiddenField hiddenFieldLastIndicatorValue = (HiddenField)e.Item.FindControl("HiddenFieldLastIndicatorValue");
            HiddenField hiddenFieldDetailType = (HiddenField)e.Item.FindControl("HiddenFieldDetailType");
            HiddenField hiddenFieldHasDetail = (HiddenField)e.Item.FindControl("HiddenFieldHasDetail");
            HiddenField hiddenFieldDetailRequiredFlag = (HiddenField)e.Item.FindControl("HiddenFieldDetailRequiredFlag");
            HiddenField hiddenFieldValidationType = (HiddenField)e.Item.FindControl("HiddenFieldValidationType");
            HiddenField hiddenFieldValidationMessage = (HiddenField)e.Item.FindControl("HiddenFieldValidationMessage");
            
            RegularExpressionValidator regularExpressionValidatorExamIndicatorValue =
                (RegularExpressionValidator)e.Item.FindControl("RegularExpressionValidatorExamIndicatorValue");

            RequiredFieldValidator requiredFieldValidatorExamIndicatorValue =
                (RequiredFieldValidator)e.Item.FindControl("RequiredFieldValidatorExamIndicatorValue");

            if (hiddenFieldHasDetail.Value == "True")
            {
                //set detail control 
                switch (int.Parse(hiddenFieldDetailType.Value))
                {
                    case (int)IssueDetailType.String:
                        SetTextBox(
                            e,
                            checkBoxExamIndicator,
                            hiddenFieldLastIndicatorValue,
                            hiddenFieldDetailRequiredFlag,
                            hiddenFieldValidationType,
                            hiddenFieldValidationMessage,
                            requiredFieldValidatorExamIndicatorValue,
                            regularExpressionValidatorExamIndicatorValue,
                            labelforTextBoxExamIndicatorValue);
                        break;
                    case (int)IssueDetailType.SingleSelect:
                        SetDropDown(
                            e,
                            checkBoxExamIndicator,
                            hiddenFieldLastIndicatorValue,
                            hiddenFieldDetailRequiredFlag,
                            hiddenFieldValidationType,
                            hiddenFieldValidationMessage,
                            requiredFieldValidatorExamIndicatorValue,
                            regularExpressionValidatorExamIndicatorValue,
                            labelforTextBoxExamIndicatorValue);
                        break;
                    case (int)IssueDetailType.MultipleSelect:
                        break;
                    case (int)IssueDetailType.Boolean:
                        break;
                    case (int)IssueDetailType.Error:
                        //todo: display error
                        break;
                    default:
                        break;
                }
            }
            else
            { 
                //Test Indicator does not have additional data.  Set checkbox to true based on 

                foreach (var examItem in ExamData)
                {
                    int repeaterRowValue = Helpers.getIntFromString(checkBoxExamIndicator.Attributes["Value"]);

                    if (examItem.Key == repeaterRowValue)
                    {
                        checkBoxExamIndicator.Checked = true;
                        break;
                    }

                }
            }
        }
    }

    private void SetTextBox(
        RepeaterItemEventArgs e, 
        CheckBox checkBoxExamIndicator, 
        HiddenField hiddenFieldLastIndicatorValue, 
        HiddenField hiddenFieldDetailRequiredFlag,
        HiddenField hiddenFieldValidationType,
        HiddenField hiddenFieldValidationMessage,
        RequiredFieldValidator requiredFieldValidatorExamIndicatorValue,
        RegularExpressionValidator regularExpressionValidatorExamIndicatorValue,
        Label labelforTextBoxExamIndicatorValue)
    {
        var textBoxExamIndicatorValue = (TextBox) e.Item.FindControl("TextBoxExamIndicatorValue");

        if (MultiLine)
        {
            textBoxExamIndicatorValue.TextMode = TextBoxMode.MultiLine;
        }

        //initialize the comment box as disabled.  If its associated checkbox is true it will be set to enabled.
        //textBoxExamIndicatorValue.Enabled = false;
        textBoxExamIndicatorValue.Visible = true;

        foreach (var examItem in ExamData)
        {
            int repeaterRowValue = Helpers.getIntFromString( checkBoxExamIndicator.Attributes["Value"]);

            if (examItem.Key == repeaterRowValue)
            {
                checkBoxExamIndicator.Checked = true;
                textBoxExamIndicatorValue.Text = examItem.Value;
                textBoxExamIndicatorValue.Enabled = true;
                hiddenFieldLastIndicatorValue.Value = examItem.Value;
                
                break;
            }
        }

        //Validators
        SetValidators(
            e,
            labelforTextBoxExamIndicatorValue,
            requiredFieldValidatorExamIndicatorValue,
            hiddenFieldDetailRequiredFlag,
            hiddenFieldValidationMessage,
            regularExpressionValidatorExamIndicatorValue,
            textBoxExamIndicatorValue,
            false);

        textBoxExamIndicatorValue.ToolTip = 
            checkBoxExamIndicator.Text + 
            (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetTextBox__") +
            DVEIR_REFERRAL.GetEyeSettingDisplay(EyeSetting) + 
            (string) System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetTextBox____") +
            labelforTextBoxExamIndicatorValue.Text;
        
        //set javascript  
        SetJavascript(
            textBoxExamIndicatorValue,
            checkBoxExamIndicator,
            requiredFieldValidatorExamIndicatorValue, 
            regularExpressionValidatorExamIndicatorValue,
            hiddenFieldDetailRequiredFlag,
            hiddenFieldValidationType,
            hiddenFieldLastIndicatorValue);

    }

    private void SetValidators(
        RepeaterItemEventArgs e, 
        Label labelforTextBoxExamIndicatorValue,
        RequiredFieldValidator requiredFieldValidatorExamIndicatorValue, 
        HiddenField hiddenFieldDetailRequiredFlag,
        HiddenField hiddenFieldValidationMessage,
        RegularExpressionValidator regularExpressionValidatorExamIndicatorValue,
        WebControl controlExamIndicatorValue,
        bool isDropDown
        )
    {

        //set required validator
        if (bool.Parse(hiddenFieldDetailRequiredFlag.Value))
        {
            requiredFieldValidatorExamIndicatorValue.ControlToValidate = controlExamIndicatorValue.ID;
            requiredFieldValidatorExamIndicatorValue.Enabled = true;
            if (isDropDown)
            {
                requiredFieldValidatorExamIndicatorValue.InitialValue = "0";
            }
            else
            {
                requiredFieldValidatorExamIndicatorValue.InitialValue = "";
            }
            requiredFieldValidatorExamIndicatorValue.ErrorMessage = //(string)System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetValidators_Selection_one_of_the_following_values___")
                                                                    "'" + labelforTextBoxExamIndicatorValue.Text +
                                                                    (string)System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetValidators_is_required");

        }
        //else
        //{
        //    this.Page.Controls.Remove(requiredFieldValidatorExamIndicatorValue);
        //}

        //set Regex Validators
        var hiddenFieldValidationValue = (HiddenField)e.Item.FindControl("HiddenFieldValidationValue");

        if (regularExpressionValidatorExamIndicatorValue != null)
        {
            //set default validation and message to allow up to 4000 characters (4000 is the lenght of the field in the database) 
            regularExpressionValidatorExamIndicatorValue.ErrorMessage =
                labelforTextBoxExamIndicatorValue.Text +
                (string)System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx", "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetValidators__needs_to_be_less_than_4000_characters_");
            regularExpressionValidatorExamIndicatorValue.ValidationExpression = "^[\\s\\S]{0,4000}$";

            //override validation if database has different value
            if (!string.IsNullOrEmpty(hiddenFieldValidationValue.Value))
            {
                regularExpressionValidatorExamIndicatorValue.ValidationExpression = hiddenFieldValidationValue.Value;
            }

            //var hiddenFieldDetailList = (HiddenField)e.Item.FindControl("HiddenFieldDetailList");
            //if (!string.IsNullOrEmpty(hiddenFieldDetailList.Value))
            if (!string.IsNullOrEmpty(hiddenFieldValidationMessage.Value))
            {
                regularExpressionValidatorExamIndicatorValue.ErrorMessage = hiddenFieldValidationMessage.Value;  //hiddenFieldDetailList.Value;
            }

            regularExpressionValidatorExamIndicatorValue.Enabled = true;

        }
    }

    private void SetJavascript(
        WebControl examIndicatorValue,
        CheckBox checkBoxExamIndicator,
        RequiredFieldValidator requiredFieldValidatorExamIndicatorValue,
        RegularExpressionValidator regularExpressionValidatorExamIndicatorValue,
        HiddenField hiddenFieldDetailRequiredFlag,
        HiddenField hiddenFieldValidationType,
        HiddenField hiddenFieldLastIndicatorValue)
    {
        checkBoxExamIndicator.Attributes.Add("onclick",
                                             "checkBoxListOnClick(this,'" +
                                             examIndicatorValue.ClientID + "', '" +
                                             requiredFieldValidatorExamIndicatorValue.ClientID + "', '" +
                                             regularExpressionValidatorExamIndicatorValue.ClientID + "', '" +
                                             hiddenFieldDetailRequiredFlag.ClientID + "', '" +
                                             hiddenFieldValidationType.ClientID + "', '" +
                                             hiddenFieldLastIndicatorValue.ClientID + "');");
    }

    private void SetDropDown(
        RepeaterItemEventArgs e, 
        CheckBox checkBoxExamIndicator, 
        HiddenField hiddenFieldLastIndicatorValue,
        HiddenField hiddenFieldDetailRequiredFlag,
        HiddenField hiddenFieldValidationType,
        HiddenField hiddenFieldValidationMessage,
        RequiredFieldValidator requiredFieldValidatorExamIndicatorValue,
        RegularExpressionValidator regularExpressionValidatorExamIndicatorValue,
        Label labelforTextBoxExamIndicatorValue)
    {
        var hiddenFieldDetailList = (HiddenField)e.Item.FindControl("HiddenFieldDetailList");
        DropDownList dropDownListExamIndicatorValue =
            (DropDownList) e.Item.FindControl("DropDownListExamIndicatorValue");

        hiddenFieldLastIndicatorValue.Value = "0"; //DROP_DOWN_INITIAL_VALUE;

        //List<string> detailList = new List<string>();
        //detailList.Add(DROP_DOWN_INITIAL_VALUE);
        //detailList.AddRange( hiddenFieldDetailList.Value.Split(",").ToList());

        List<KeyValuePair<string, string>> detailList = new List<KeyValuePair<string, string>>();
        
        //set a default empty value to the dropdown list. This will be used to determine if a selection has been made.
        KeyValuePair<string, string> dropDownItem = new KeyValuePair<string,string>("0",DROP_DOWN_INITIAL_VALUE);
        detailList.Add(dropDownItem);
        
        string[] dropdownListItems = hiddenFieldDetailList.Value.Split(",");
        
        foreach (string item in dropdownListItems)
        {
            dropDownItem = new KeyValuePair<string, string>(item, item);
            detailList.Add(dropDownItem);
        }

        dropDownListExamIndicatorValue.Visible = true;
        //dropDownListExamIndicatorValue.Enabled = false;
        labelforTextBoxExamIndicatorValue.AssociatedControlID = "dropDownListExamIndicatorValue";

        //datasource
        dropDownListExamIndicatorValue.DataSource = detailList;
        dropDownListExamIndicatorValue.DataBind();
        
        //tooltip
        dropDownListExamIndicatorValue.ToolTip =
            (string)
            System.Web.HttpContext.GetLocalResourceObject("~/controls/ErrorList.ascx",
                                                          "clinicalDataUserControls_ExamTabDynamicTestsUserControl_SetDropDown_Please_select_a_value_from_the_list_");
        //value
        foreach (var examItem in ExamData)
        {
            int repeaterRowValue = Helpers.getIntFromString(checkBoxExamIndicator.Attributes["Value"]);

            if (examItem.Key == repeaterRowValue)
            {
                if (string.IsNullOrEmpty(examItem.Value))
                {
                    dropDownListExamIndicatorValue.SelectedIndex = 0;
                }
                else
                {
                    //dropDownListExamIndicatorValue.SelectedIndex = detailList.IndexOf(examItem.Value);
                    dropDownListExamIndicatorValue.Text = examItem.Value;

                    if(dropDownListExamIndicatorValue.SelectedIndex == -1)
                    {
                        dropDownListExamIndicatorValue.Items.Add(examItem.Value);
                        dropDownListExamIndicatorValue.SelectedIndex = dropDownListExamIndicatorValue.Items.Count;
                    }
                }

                checkBoxExamIndicator.Checked = true;
                dropDownListExamIndicatorValue.Enabled = true;

                break;
            }
        }

        //Javascript
        SetJavascript(
            dropDownListExamIndicatorValue,
            checkBoxExamIndicator,
            requiredFieldValidatorExamIndicatorValue,
            regularExpressionValidatorExamIndicatorValue,
            hiddenFieldDetailRequiredFlag,
            hiddenFieldValidationType,
            hiddenFieldLastIndicatorValue);

        //Validators
        SetValidators(
            e,
            labelforTextBoxExamIndicatorValue,
            requiredFieldValidatorExamIndicatorValue,
            hiddenFieldDetailRequiredFlag,
            hiddenFieldValidationMessage,
            regularExpressionValidatorExamIndicatorValue,
            dropDownListExamIndicatorValue,
            true);

    }

    
    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eyeExam"></param>
    /// <returns>int</returns>
    public int LoadObjectFromControls<T>(ER_EXAM exam, List<T> examList, string idPropertyName, ER_ENCOUNTER encounter)
        where T : LINQEntityBase, new()
    {
        
        int returnStatus = 0;

        //EyeSetting
        bool getLeftEye = (EyeSetting == EyeSettingType.LeftEye ? true : false);
        bool getRightEye = (EyeSetting == EyeSettingType.RightEye ? true : false);
        bool testPerformed = false;

        if (EyeSetting == EyeSettingType.BothEyes)
        {
            getLeftEye = true;
            getRightEye = true;
        }

        if (ExamTestPerformedUserControl.Visible)
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                Helpers.getIntFromString(HiddenFieldCategoryIdTestPerformed.Value),
                ExamTestPerformedUserControl.TestPerformed,
                getRightEye, getLeftEye, DveirBaseControlType.RadioButtonList);

            testPerformed = (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString());

            Helpers.LoadOneObjectForExamDetail(ref encounter,
                Helpers.getIntFromString(HiddenFieldCategoryIdTestResult.Value),
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            Helpers.LoadOneObjectForExamDetail(ref encounter,
               Helpers.getIntFromString(HiddenFieldCategoryIdComments.Value),
               ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
               DveirBaseControlType.TextBox);
        }


        T existingItem  = default(T);

        foreach (RepeaterItem item in RepeaterTestResults.Items)
        {

            CheckBox checkBoxExamIndicator = (CheckBox)item.FindControl("CheckBoxExamIndicator");
            Label labelforTextBoxExamIndicatorValue = (Label)item.FindControl("LabelforTextBoxExamIndicatorValue");
            TextBox textBoxExamIndicatorValue = (TextBox)item.FindControl("TextBoxExamIndicatorValue");
            DropDownList dropDownListExamIndicatorValue = (DropDownList)item.FindControl("DropDownListExamIndicatorValue");
            HiddenField HiddenFieldTextBoxValue = (HiddenField)item.FindControl("HiddenFieldTextBoxValue");

            RegularExpressionValidator regularExpressionValidatorExamIndicatorValue =
                (RegularExpressionValidator)item.FindControl("RegularExpressionValidatorExamIndicatorValue");

            RequiredFieldValidator requiredFieldValidatorExamIndicatorValue =
                (RequiredFieldValidator)item.FindControl("RequiredFieldValidatorExamIndicatorValue");

            int repeaterRowValue = Helpers.getIntFromString(checkBoxExamIndicator.Attributes["Value"]);

            existingItem = (
                        from e in examList
                        where

                            (int)(from p in e.GetType().GetProperties()
                              where p.Name.EndsWith("ISSUE_ID")
                              select p.GetValue(e, null)).FirstOrDefault() 
                                == repeaterRowValue
                            &&
                            (bool)(from p in e.GetType().GetProperties()
                                   where p.Name.Equals("LEFT_FLAG")
                                   select p.GetValue(e, null)).FirstOrDefault()
                                == getLeftEye
                            &&
                            (bool)(from p in e.GetType().GetProperties()
                                   where p.Name.Equals("RIGHT_FLAG")
                                   select p.GetValue(e, null)).FirstOrDefault()
                                == getRightEye

                      select e).FirstOrDefault();

            // if ExamTestPerformedUserControl is not visible, the objects need to be loaded as well.
            if (checkBoxExamIndicator.Checked && (testPerformed || ExamTestPerformedUserControl.Visible == false))
            {
                //add to list
                if (existingItem == null)
                {
                    System.Reflection.PropertyInfo propertyInfo = null;
                    
                    T newItem = new T();

                    propertyInfo = newItem.GetType().GetProperty("ER_EXAM_ID");
                    if (propertyInfo != null)
                        propertyInfo.SetValue(newItem, exam.ER_EXAM_ID, null);

                    propertyInfo = newItem.GetType().GetProperty(idPropertyName);
                    if (propertyInfo != null)
                        propertyInfo.SetValue(newItem, repeaterRowValue, null);

                    propertyInfo = newItem.GetType().GetProperty("COMMENT_TEXT");
                    if (propertyInfo != null)
                    {

                        if (textBoxExamIndicatorValue.Visible)
                        {
                            propertyInfo.SetValue(newItem, textBoxExamIndicatorValue.Text, null);
                        }
                        else if (dropDownListExamIndicatorValue.Visible)
                        {
                            propertyInfo.SetValue(newItem,
                                                  dropDownListExamIndicatorValue.SelectedIndex == 0
                                                      ? null
                                                      : dropDownListExamIndicatorValue.SelectedItem.Text, null);
                        }
                    }

                    if (DVEIR_REFERRAL.GetEyeSettingDatabaseField(_EyeSetting) == "LEFT_FLAG")
                    {
                        propertyInfo = newItem.GetType().GetProperty("LEFT_FLAG");
                        if (propertyInfo != null)
                            propertyInfo.SetValue(newItem, true, null);
                    }

                    if (DVEIR_REFERRAL.GetEyeSettingDatabaseField(_EyeSetting) == "RIGHT_FLAG")
                    {
                        propertyInfo = newItem.GetType().GetProperty("RIGHT_FLAG");
                        if (propertyInfo != null)
                            propertyInfo.SetValue(newItem, true, null);
                    }

                    if (DVEIR_REFERRAL.GetEyeSettingDatabaseField(_EyeSetting) == "BOTH_EYES")
                    {
                        propertyInfo = newItem.GetType().GetProperty("LEFT_FLAG");
                        if (propertyInfo != null)
                            propertyInfo.SetValue(newItem, true, null);

                        propertyInfo = newItem.GetType().GetProperty("RIGHT_FLAG");
                        if (propertyInfo != null)
                            propertyInfo.SetValue(newItem, true, null);
                    }
                    
                    DVEIREyeExamManager examManager = new DVEIREyeExamManager();
                    examManager.AddChild(exam, newItem);

                }
                else
                {
                    //existingItem.COMMENT_TEXT = textBoxExamIndicatorValue.Text;

                    System.Reflection.PropertyInfo propertyInfo = null;
                    propertyInfo = existingItem.GetType().GetProperty("COMMENT_TEXT"); 

                    if (propertyInfo != null)
                    {
                        if (textBoxExamIndicatorValue.Visible)
                        {
                            propertyInfo.SetValue(existingItem, textBoxExamIndicatorValue.Text, null);  
                        }
                        else if (dropDownListExamIndicatorValue.Visible)
                        {
                            propertyInfo.SetValue(existingItem,
                                                  dropDownListExamIndicatorValue.SelectedIndex == 0
                                                      ? null
                                                      : dropDownListExamIndicatorValue.SelectedItem.Text, null);
                        }
                    }
                }
            }
            else
            {
                //remove from list
                //if the item is not selected, then it needs to be taken out if it is in the list
                if (existingItem != null)
                {
                    existingItem.SetAsDeleteOnSubmit();
                }

                //disable validators on the detail form elements since the data is discarded
                regularExpressionValidatorExamIndicatorValue.Enabled = false;
                requiredFieldValidatorExamIndicatorValue.Enabled = false;
            }

        }

        returnStatus = ReviewCommentUserControl.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }
 
    #endregion

}
