﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class controls_ExamTabIntraocularPressureUserControl : BaseClinicalDataControl
{
    public string PanelName
    {
        get
        {
            return PanelIntraocularPressure.GroupingText;
        }

        set
        {
            PanelIntraocularPressure.GroupingText = value;
        }
    }


    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }


    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }

    protected void Page_Load(object sender, EventArgs e)
    {
        ReviewCommentUserControlIntraocularPressure.LocationPage = this.LocationPage;
        ReviewCommentUserControlIntraocularPressure.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlIntraocularPressure.AnchorId = this.AnchorId;

    }
    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        int returnStatus = 0;

        if (exam == null)
        {
            exam = new ER_EXAM();
            exam.ER_EXAM_ID = encounter.ER_ENCOUNTER_ID;
            if (encounter.REFERRAL != null)
            {
                exam.PATIENT_ID = encounter.REFERRAL.PATIENT_ID;
            }
            encounter.ER_EXAM = exam;
            encounter.ER_EXAM.SetAsInsertOnSubmit();
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureTestPerformed,
            ExamTestPerformedUserControl.TestPerformed,getRightEye, getLeftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the Intraocular Pressure test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementIntraocularPressureTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementIntraocularPressureNumber,
                TextBoxIntraocularPressure.Text, getRightEye, getLeftEye,
                DveirBaseControlType.TextBox);

        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementIntraocularPressureTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);

            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementIntraocularPressureNumber,
                TextBoxIntraocularPressure.Text, getRightEye, getLeftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureComments,
            ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlIntraocularPressure.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }


    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {

        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureTestPerformed,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

        // Only load the following controls if the Intraocular Pressure test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureTestResult,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            TextBoxIntraocularPressure.Text = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureNumber,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementIntraocularPressureComments,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        ReviewCommentUserControlIntraocularPressure.LoadControlsFromObject(encounter);

        return 0;
    }


}
