﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class controls_ExamTabMotilityUserControl : BaseClinicalDataControl
{
    private DVEIRBaseControlManager _baseControlManager = new DVEIRBaseControlManager();

    public List<KeyValuePair<int, string>> examData { get; set; }

    public string PanelName
    {
        get
        {
            return PanelMotility.GroupingText;
        }

        set
        {
            PanelMotility.GroupingText = value;
        }
    }


    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }


    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }

 
    protected void Page_Load(object sender, EventArgs e)
    {
        ReviewCommentUserControlMotility.LocationPage = this.LocationPage;
        ReviewCommentUserControlMotility.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlMotility.AnchorId = this.AnchorId;
        LoadBaseControls();
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        foreach (ER_MOTILITY_EXAM_DTL item in exam.ER_MOTILITY_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == getRightEye && item.LEFT_FLAG == getLeftEye)
            {
                item.SetAsDeleteOnSubmit();
            }
        }

        int returnStatus = 0;

        if (exam == null)
        {
            exam = new ER_EXAM();
            exam.ER_EXAM_ID = encounter.ER_ENCOUNTER_ID;
            if (encounter.REFERRAL != null)
            {
                exam.PATIENT_ID = encounter.REFERRAL.PATIENT_ID;
            }
            encounter.ER_EXAM = exam;
            encounter.ER_EXAM.SetAsInsertOnSubmit();
        }

        // Left Eye
        if (getLeftEye)
        {
            LoadObjectFromControls(ref encounter, false, true);
        }
        else  //Right eye
        {
            LoadObjectFromControls(ref encounter, true, false);
        }

        returnStatus = ReviewCommentUserControlMotility.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }


    /// <summary>
    /// Load encounter exam data for right eye or left eye
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadObjectFromControls(ref ER_ENCOUNTER encounter, bool rightEye, bool leftEye)
    {
        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementMotilityTestPerformed,
            ExamTestPerformedUserControl.TestPerformed, rightEye, leftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the motility test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementMotilityTestResult,
                ExamTestResultUserControl.TestResult, rightEye, leftEye,
                DveirBaseControlType.RadioButtonList);

            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPursuit, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilitySaccades, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityNystagmus, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityNystagmusUpbeat, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityNystagmusDownbeat, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityNystagmusToTheRight, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityNystagmusToTheLeft, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoria, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaExophoris, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaExophorisPdDis, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaExophorisPdNear, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaEsophoria, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaEsophoriaPdDis, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityPhoriaEsophoriaPdNear, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropia, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaExotropia, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaExotropiaPdDis, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaExotropiaPdNear, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaEsotropia, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaEsotropiaPdDis, rightEye, leftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlMotilityTropiaEsotropiaPdNear, rightEye, leftEye);
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementMotilityTestResult,
                ExamTestResultUserControl.TestResult, rightEye, leftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementMotilityComments,
            ExamCommentsUserControl.Comments, rightEye, leftEye,
            DveirBaseControlType.TextBox);
    }


    /// <summary>
    /// Load one ER_MOTILITY_EXAM_DTL object with the data from one motility user control
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="baseControl"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void  LoadOneObjectFromControl(ref ER_ENCOUNTER encounter, 
        controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        if (!string.IsNullOrEmpty(baseControl.Text))
        {
            ER_MOTILITY_EXAM_DTL newExamDetail = new ER_MOTILITY_EXAM_DTL();
            newExamDetail.ER_EXAM_ID = encounter.ER_EXAM.ER_EXAM_ID;
            newExamDetail.STD_ER_MOTILITY_ISSUE_ID = baseControl.BaseControlId;
            newExamDetail.EXAM_DATE = encounter.ER_EXAM.EXAM_DATE;
            newExamDetail.RIGHT_FLAG = rightEye;
            newExamDetail.LEFT_FLAG = leftEye;
            if (baseControl.BaseControlType == DveirBaseControlType.TextBox)
            {
                newExamDetail.COMMENT_TEXT = baseControl.Text;
            }
            else
            {
                newExamDetail.ER_Exam_Lookup_ID = Convert.ToInt32(baseControl.Text);
            }

            encounter.ER_EXAM.ER_MOTILITY_EXAM_DTLs.Add(newExamDetail);
            encounter.ER_EXAM.ER_MOTILITY_EXAM_DTLs.Last().SetAsInsertOnSubmit();
        }
    }


    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        LoadBaseControls();
        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        // Left Eye
        if (getLeftEye)
        {
            LoadControlsFromObject(encounter, false, true);
        }
        else  //Right eye
        {
            LoadControlsFromObject(encounter, true, false);
        }

        return 0;
    }


    /// <summary>
    /// Load controls from object for right eye or left eye
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadControlsFromObject(ER_ENCOUNTER encounter, bool rightEye, bool leftEye)
    {
        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter, 
            DVEIRSharedConstant.ExamDataElementMotilityTestPerformed,
            DveirBaseControlType.RadioButtonList, rightEye, leftEye);

        // Only load the following controls if the motility test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter, 
            DVEIRSharedConstant.ExamDataElementMotilityTestResult,
            DveirBaseControlType.RadioButtonList, rightEye, leftEye);

            LoadOneControl(encounter, BaseControlMotilityPursuit, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilitySaccades, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityNystagmus, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityNystagmusUpbeat, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityNystagmusDownbeat, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityNystagmusToTheRight, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityNystagmusToTheLeft, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoria, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaExophoris, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaExophorisPdDis, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaExophorisPdNear, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaEsophoria, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaEsophoriaPdDis, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityPhoriaEsophoriaPdNear, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropia, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaExotropia, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaExotropiaPdDis, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaExotropiaPdNear, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaEsotropia, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaEsotropiaPdDis, rightEye, leftEye);
            LoadOneControl(encounter, BaseControlMotilityTropiaEsotropiaPdNear, rightEye, leftEye);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementMotilityComments,
            DveirBaseControlType.TextBox, rightEye, leftEye);

        ReviewCommentUserControlMotility.LoadControlsFromObject(encounter);
    }

    private void LoadOneControl(ER_ENCOUNTER encounter, controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        foreach (ER_MOTILITY_EXAM_DTL item in encounter.ER_EXAM.ER_MOTILITY_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye
                && item.STD_ER_MOTILITY_ISSUE_ID == baseControl.BaseControlId)
            {
                if (baseControl.BaseControlType == DveirBaseControlType.RadioButtonList
                    && item.ER_Exam_Lookup_ID > 0)
                {
                    baseControl.Text = item.ER_Exam_Lookup_ID.ToString();
                }
                
                if (baseControl.BaseControlType == DveirBaseControlType.TextBox
                    && !string.IsNullOrEmpty(item.COMMENT_TEXT))
                {
                    baseControl.Text = item.COMMENT_TEXT;
                }
                break;
            }
        }
    }

    /// <summary>
    /// Load all the base controls based on the control property data from database
    /// </summary>
    private void LoadBaseControls()
    {
        List<VIEW_GUI_CONTROL> guiControls = _baseControlManager.SelectGuiControlList("MOTILITY").ToList();
        foreach (VIEW_GUI_CONTROL dbItem in guiControls)
        {
            Control userControl = PanelMotility.FindControl(dbItem.USER_CONTROL_ID);
            if (userControl != null)
            {
                controls_BaseControl baseControl = (controls_BaseControl)userControl;
                baseControl.BaseControlType = (DveirBaseControlType)dbItem.ISSUE_DETAIL_TYPE_ID;
                baseControl.IsRequired = dbItem.REQUIRED_FLAG;
                baseControl.ToolTip = dbItem.TOOLTIP;
                baseControl.BaseControlName = dbItem.NAME;
                baseControl.IndentForLabel = dbItem.INDENT_WIDTH;
                baseControl.IsMultilineTextBox = dbItem.MULTILINE_FLAG;
                baseControl.BaseControlId = dbItem.ID;
                if (dbItem.MAX_LENGTH > 0)
                {
                    baseControl.MaxLength = (int)dbItem.MAX_LENGTH;
                }

                if (!string.IsNullOrEmpty(dbItem.VALIDATION_VALUE))
                {
                    baseControl.ValidationExpression = dbItem.VALIDATION_VALUE;
                }
                if (!string.IsNullOrEmpty(dbItem.VALIDATION_MESSAGE))
                {
                    baseControl.ValidationErrorMessage = dbItem.VALIDATION_MESSAGE;
                }
                baseControl.LocationFriendlyName = dbItem.NAME;
                baseControl.LookupCategory = dbItem.LOOKUP_CATEGORY;
                baseControl.ConditionText = dbItem.CONDITION_TEXT;
                if (dbItem.CONTROL_WIDTH > 0)
                {
                    baseControl.ControlWidth = (int) dbItem.CONTROL_WIDTH;
                }
            }
       }
    }

}
