﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class controls_ExamTabRetinaUserControl : BaseClinicalDataControl
{
    private DVEIRBaseControlManager _baseControlManager = new DVEIRBaseControlManager();

    public string PanelName
    {
        get
        {
            return PanelRetina.GroupingText;
        }

        set
        {
            PanelRetina.GroupingText = value;
        }
    }


    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }


    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }

    protected void Page_Load(object sender, EventArgs e)
    {
        ReviewCommentUserControlRetina.LocationPage = this.LocationPage;
        ReviewCommentUserControlRetina.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlRetina.AnchorId = this.AnchorId;
        LoadBaseControls();
    }


    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        foreach (ER_RETINA_EXAM_DTL item in exam.ER_RETINA_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == getRightEye && item.LEFT_FLAG == getLeftEye)
            {
                item.SetAsDeleteOnSubmit();
            }
        }

        int returnStatus = 0;

        if (exam == null)
        {
            exam = new ER_EXAM();
            exam.ER_EXAM_ID = encounter.ER_ENCOUNTER_ID;
            if (encounter.REFERRAL != null)
            {
                exam.PATIENT_ID = encounter.REFERRAL.PATIENT_ID;
            }
            encounter.ER_EXAM = exam;
            encounter.ER_EXAM.SetAsInsertOnSubmit();
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementRetinaTestPerformed,
            ExamTestPerformedUserControl.TestPerformed, getRightEye, getLeftEye,
            DveirBaseControlType.RadioButtonList);

        // Only load the following controls if the Retina test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementRetinaTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            LoadOneObjectFromControl(ref encounter, BaseControlRetinaTearDefect, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaTearDefectDropDown, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentTractional, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentTractionalRepaired, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentRhegmatogenous, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentRhegmatogenousRepaired, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentMaculaIsOn, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentMaculaIsOnRepaired, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentMaculaIsOff, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaDetachmentMaculaIsOffRepaired, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaMacularHoleCyst, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaEpiretinalMembrane, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaRetinalScar, getRightEye, getLeftEye);
            LoadOneObjectFromControl(ref encounter, BaseControlRetinaOther, getRightEye, getLeftEye);
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementRetinaTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementRetinaComments,
            ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlRetina.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }



    /// <summary>
    /// Load one ER_RETINA_EXAM_DTL object with the data from one pupil user control
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="baseControl"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    private void LoadOneObjectFromControl(ref ER_ENCOUNTER encounter,
        controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        if (!string.IsNullOrEmpty(baseControl.Text))
        {
            ER_RETINA_EXAM_DTL newExamDetail = new ER_RETINA_EXAM_DTL();
            newExamDetail.ER_EXAM_ID = encounter.ER_EXAM.ER_EXAM_ID;
            newExamDetail.STD_ER_RETINA_ISSUE_ID = baseControl.BaseControlId;
            newExamDetail.EXAM_DATE = encounter.ER_EXAM.EXAM_DATE;
            newExamDetail.RIGHT_FLAG = rightEye;
            newExamDetail.LEFT_FLAG = leftEye;
            newExamDetail.COMMENT_TEXT = baseControl.Text;

            encounter.ER_EXAM.ER_RETINA_EXAM_DTLs.Add(newExamDetail);
            encounter.ER_EXAM.ER_RETINA_EXAM_DTLs.Last().SetAsInsertOnSubmit();
        }
    }


    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        LoadBaseControls();
        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementRetinaTestPerformed,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

        // Only load the following controls if the Retina test was performed
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementRetinaTestResult,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            LoadOneControl(encounter, BaseControlRetinaTearDefect, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaTearDefectDropDown, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentTractional, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentTractionalRepaired, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentRhegmatogenous, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentRhegmatogenousRepaired, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentMaculaIsOn, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentMaculaIsOnRepaired, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentMaculaIsOff, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaDetachmentMaculaIsOffRepaired, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaMacularHoleCyst, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaEpiretinalMembrane, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaRetinalScar, getRightEye, getLeftEye);
            LoadOneControl(encounter, BaseControlRetinaOther, getRightEye, getLeftEye);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementRetinaComments,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);

        ReviewCommentUserControlRetina.LoadControlsFromObject(encounter);

        return 0;
    }


    private void LoadOneControl(ER_ENCOUNTER encounter, controls_BaseControl baseControl, bool rightEye, bool leftEye)
    {
        foreach (ER_RETINA_EXAM_DTL item in encounter.ER_EXAM.ER_RETINA_EXAM_DTLs)
        {
            if (item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye
                && item.STD_ER_RETINA_ISSUE_ID == baseControl.BaseControlId)
            {
                baseControl.Text = item.COMMENT_TEXT;
                break;
            }
        }
    }

    /// <summary>
    /// Load all the base controls based on the control property data from database
    /// </summary>
    private void LoadBaseControls()
    {
        List<VIEW_GUI_CONTROL> guiControls = _baseControlManager.SelectGuiControlList("RETINA").ToList();
        foreach (VIEW_GUI_CONTROL dbItem in guiControls)
        {
            Control userControl = PanelRetina.FindControl(dbItem.USER_CONTROL_ID);
            if (userControl != null)
            {
                controls_BaseControl baseControl = (controls_BaseControl)userControl;
                baseControl.BaseControlType = (DveirBaseControlType)dbItem.ISSUE_DETAIL_TYPE_ID;
                if (baseControl.BaseControlType == DveirBaseControlType.Checkbox)
                {
                    baseControl.CheckBoxText = dbItem.NAME;
                }
                baseControl.IsRequired = dbItem.REQUIRED_FLAG;
                baseControl.ToolTip = dbItem.TOOLTIP;
                baseControl.BaseControlName = dbItem.NAME;
                baseControl.IndentForLabel = dbItem.INDENT_WIDTH;
                baseControl.IsMultilineTextBox = dbItem.MULTILINE_FLAG;
                baseControl.BaseControlId = dbItem.ID;
                if (dbItem.MAX_LENGTH > 0)
                {
                    baseControl.MaxLength = (int)dbItem.MAX_LENGTH;
                }

                if (!string.IsNullOrEmpty(dbItem.VALIDATION_VALUE))
                {
                    baseControl.ValidationExpression = dbItem.VALIDATION_VALUE;
                }
                if (!string.IsNullOrEmpty(dbItem.VALIDATION_MESSAGE))
                {
                    baseControl.ValidationErrorMessage = dbItem.VALIDATION_MESSAGE;
                }
                baseControl.LocationFriendlyName = dbItem.NAME;
                baseControl.LookupCategory = dbItem.LOOKUP_CATEGORY;
                baseControl.ConditionText = dbItem.CONDITION_TEXT;
                if (dbItem.CONTROL_WIDTH > 0)
                {
                    baseControl.ControlWidth = (int)dbItem.CONTROL_WIDTH;
                }
            }
        }
    }
}
