﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_ExamTabVisualFieldTestUserControl : BaseClinicalDataControl
{
    private string panelName = "Set Panel Name in ASPX";

    public List<KeyValuePair<int, string>> examData { get; set; }
    
    public string PanelName 
    {
        get
        {
            return panelName;
        }
        
        set
        {
            PanelVisualField.GroupingText = value;
        }
    }

    
    /// <summary>
    /// Set the value to the eye that the test was performed on (Right Eye or Left Eye)
    /// </summary>
    public EyeSettingType EyeSetting { get; set; }
    
    public bool getLeftEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.LeftEye ? true : false);
        }

    }

    public bool getRightEye
    {
        get
        {
            return (EyeSetting == EyeSettingType.RightEye ? true : false);
        }

    }


 

    protected void Page_Load(object sender, EventArgs e)
    {
        //TO DO : need to add javascript to handle the proper values of the items

        ReviewCommentUserControlVisualField.LocationPage = this.LocationPage;
        ReviewCommentUserControlVisualField.LocationFriendlyName = this.LocationFriendlyName;
        ReviewCommentUserControlVisualField.AnchorId = this.AnchorId;
    }

    #region Postback Events
    
    #endregion

    #region Private Methods
    
    #endregion

   
    #region Postback Events


    #endregion


    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        TextBoxVisualFieldMeanDeviation.ToolTip = "Enter Mean Deviation " + DVEIR_REFERRAL.GetEyeSettingDisplay(EyeSetting) + " (+5.00 to -35.35)";
        TextBoxVisualFieldPatternStandardDeviation.ToolTip = "Enter Pattern Standard Deviation " + DVEIR_REFERRAL.GetEyeSettingDisplay(EyeSetting) + " (00.00 to 35.35)";
        TextBoxVisualFieldFixationLoses.ToolTip = "Enter Fixation Losses " + DVEIR_REFERRAL.GetEyeSettingDisplay(EyeSetting) + " (00/01 to 35/35)";

        ExamTestPerformedUserControl.EyeSetting = EyeSetting;
        ExamTestResultUserControl.EyeSetting = EyeSetting;
        ExamCommentsUserControl.EyeSetting = EyeSetting;

        ExamTestPerformedUserControl.TestPerformed = Helpers.GetExamDetailValue(encounter,
DVEIRSharedConstant.ExamDataElementHumphreyTestPerformed,
DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);
        
        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            ExamTestResultUserControl.TestResult = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementHumphreyTestResult,
            DveirBaseControlType.RadioButtonList, getRightEye, getLeftEye);

            //Scale
            if (getLeftEye)
            {
                if (exam.LEFT_REFERENCE_SCALE.HasValue)
                {
                    RadioButtonVisualFieldScale.Text = (exam.LEFT_REFERENCE_SCALE.Value.ToString());
                }
                else
                {
                    RadioButtonVisualFieldScale.ClearSelection();
                }
            }
            else
            {
                if (exam.RIGHT_REFERENCE_SCALE.HasValue)
                {
                    RadioButtonVisualFieldScale.Text = (exam.RIGHT_REFERENCE_SCALE.Value.ToString());
                }
                else
                {
                    RadioButtonVisualFieldScale.ClearSelection();
                }
            }

            //Mean Deviation
            TextBoxVisualFieldMeanDeviation.MaxLength = 6;
            TextBoxVisualFieldMeanDeviation.Text = (getLeftEye ? exam.LEFT_MEAN_DEVIATION_NUMBER : exam.RIGHT_MEAN_DEVIATION_NUMBER);

            //Pattern Standard Deviation
            TextBoxVisualFieldPatternStandardDeviation.MaxLength = 5;
            TextBoxVisualFieldPatternStandardDeviation.Text = (getLeftEye ? exam.LEFT_STD_DEVIATION_NUMBER : exam.RIGHT_STD_DEVIATION_NUMBER);

            //Fixation Loses
            TextBoxVisualFieldFixationLoses.MaxLength = 5;
            TextBoxVisualFieldFixationLoses.Text = (getLeftEye ? exam.LEFT_FIXATION_LOSS_NUMBER : exam.RIGHT_FIXATION_LOSS_NUMBER);
        }

        ExamCommentsUserControl.Comments = Helpers.GetExamDetailValue(encounter,
            DVEIRSharedConstant.ExamDataElementHumphreyComments,
            DveirBaseControlType.TextBox, getRightEye, getLeftEye);
        
        ReviewCommentUserControlVisualField.LoadControlsFromObject(encounter);

        return 0;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter)
    {
        ER_EXAM exam = encounter.ER_EXAM;

        int returnStatus = 0;

        Helpers.LoadOneObjectForExamDetail(ref encounter,
          DVEIRSharedConstant.ExamDataElementHumphreyTestPerformed,
          ExamTestPerformedUserControl.TestPerformed, getRightEye, getLeftEye,
          DveirBaseControlType.RadioButtonList);

        if (ExamTestPerformedUserControl.TestPerformed == DVEIRSharedConstant.TestPerformedLookupIdYes.ToString())
        {
            Helpers.LoadOneObjectForExamDetail(ref encounter,
                DVEIRSharedConstant.ExamDataElementHumphreyTestResult,
                ExamTestResultUserControl.TestResult, getRightEye, getLeftEye,
                DveirBaseControlType.RadioButtonList);

            if (getLeftEye)
            {
                //Scale
                exam.LEFT_REFERENCE_SCALE = Helpers.ConvertStringToNullable<char>(RadioButtonVisualFieldScale.Text);

                //Mean Deviation
                exam.LEFT_MEAN_DEVIATION_NUMBER = TextBoxVisualFieldMeanDeviation.Text;

                //Pattern Standard Deviation
                exam.LEFT_STD_DEVIATION_NUMBER = TextBoxVisualFieldPatternStandardDeviation.Text;

                //Fixation Loses
                exam.LEFT_FIXATION_LOSS_NUMBER = TextBoxVisualFieldFixationLoses.Text;

                //Mean Deviation Comments
                exam.LEFT_MEAN_DEVIATION_COMMENTS = ExamCommentsUserControl.Comments;


            }
            else  //Right eye
            {
                //Scale
                exam.RIGHT_REFERENCE_SCALE = Helpers.ConvertStringToNullable<char>(RadioButtonVisualFieldScale.Text);

                //Mean Deviation
                exam.RIGHT_MEAN_DEVIATION_NUMBER = TextBoxVisualFieldMeanDeviation.Text;

                //Pattern Standard Deviation
                exam.RIGHT_STD_DEVIATION_NUMBER = TextBoxVisualFieldPatternStandardDeviation.Text;

                //Fixation Loses
                exam.RIGHT_FIXATION_LOSS_NUMBER = TextBoxVisualFieldFixationLoses.Text;

                //Mean Deviation Comments
                exam.RIGHT_MEAN_DEVIATION_COMMENTS = ExamCommentsUserControl.Comments;
            }
        }
        else
        {
            Helpers.RemoveOneRowForExamDetail(ref encounter,
               DVEIRSharedConstant.ExamDataElementHumphreyTestResult,
               ExamTestResultUserControl.TestResult, getRightEye, getLeftEye);
        }

        Helpers.LoadOneObjectForExamDetail(ref encounter,
            DVEIRSharedConstant.ExamDataElementHumphreyComments,
            ExamCommentsUserControl.Comments, getRightEye, getLeftEye,
            DveirBaseControlType.TextBox);

        returnStatus = ReviewCommentUserControlVisualField.LoadObjectFromControls(ref encounter, this.getIdentifyingData());

        return returnStatus;

    }
 
    #endregion

}
