﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class controls_PatientTabUserControl : BaseControl
{
    private string _currentRegistry = Helpers.GetConfigEntry("Registry");

    protected void Page_Load(object sender, EventArgs e)
    {
    }
    
    #region Postback Events
    
    #endregion

    #region Private Methods

    #endregion

    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eyeExam"></param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(PATIENT patient, REFERRAL referral)
    {
        //populate dropdowns
        //TO DO : modify drop down population to add archived item if persisted in entity 
        //          and remove "-- Select a Value --" when an item is selected and field is not nullable 
        PopulateDropDowns(patient);

        //populate controls

        //Last Name
        LabelLastNameValue.Text = patient.LAST_NAME;
        
        //first name
        LabelFirstNameValue.Text = patient.FIRST_NAME;
        
        //middle name
        LabelMiddleNameValue.Text = patient.MIDDLE_NAME;

        //date of birth
        LabelDateOfBirthValue.Text = Helpers.getShortDateStringFromNullableDate(patient.BIRTH_DATE);

        //data of death
        LabelDateOfDeathValue.Text = Helpers.getShortDateStringFromNullableDate(patient.DEATH_DATE);

        //race
        LabelRaceValue.Text = (patient.STD_RACE != null ? patient.STD_RACE.NAME : "");

        //ssn
        LabelSSNValue.Text = patient.SSN;

        //marital status
        LabelMaritalStatusValue.Text = (patient.STD_MARITALSTATUS != null ? patient.STD_MARITALSTATUS.NAME : "");

        //ethnicity
        LabelEthnicityValue.Text = (patient.STD_ETHNICITY != null ? patient.STD_ETHNICITY.NAME : "");

        //gender
        LabelGenderValue.Text = (patient.STD_GENDER != null ? patient.STD_GENDER.NAME : "");

        //address type
        LabelAddressTypeValue.Text = patient.ADDRESS_TYPE;
        
        //address street 1
        LabelStreetLine1Value.Text = patient.ADDRESS_LINE1;

        //address street 2
        LabelStreetLine2Value.Text = patient.ADDRESS_LINE2;

        //address street 3
        LabelStreetLine3Value.Text = patient.ADDRESS_LINE3;

        //city
        LabelCityValue.Text = patient.CITY;

        //state
        LabelStateValue.Text = patient.STATE;

        //zip
        LabelZipValue.Text = patient.POSTAL_CODE;

        //primary phone
        LabelPrimaryPhoneNumberValue.Text = patient.PRIMARY_PHONE;
            
        //home phone
        LabelHomePhoneNumberValue.Text = patient.HOME_PHONE;

        //cell phone
        LabelCellPhoneNumberValue.Text = patient.CELL_PHONE;

        //work phone
        LabelWorkPhoneNumberValue.Text = patient.WORK_PHONE;

        //email
        TextBoxmail.MaxLength = 128;
        TextBoxmail.Text = patient.EMAIL_ADDRESS;

        //Preferred Contact Method
        TextBoxPreferredContactMethod.MaxLength = 50;
        TextBoxPreferredContactMethod.Text = patient.PreferredContactMethod;

        //get registry demographic entry for next of kin
        //REGISTRY_DEMOGRAPHIC registryDemographic = patient.REGISTRY_DEMOGRAPHICs.Where(rd => rd.STD_ADDRESSTYPE.CODE == REGISTRY_DEMOGRAPHIC.NextOfKinAddressTypeCode).FirstOrDefault();
        REGISTRY_DEMOGRAPHIC registryDemographic = patient.REGISTRY_DEMOGRAPHICs.Where(rd => rd.STD_ADDRESSTYPE_ID == (int)REGISTRY_DEMOGRAPHIC.AddressType.NOK).FirstOrDefault();
        
        if (registryDemographic != null)
        {
            //next of kin First Name
            TextBoxNextOfKinFirstName.MaxLength = 100;
            TextBoxNextOfKinFirstName.Text = registryDemographic.NEXT_OF_KIN_FNAME;

            //next of kin Last Name
            TextBoxNextOfKinLastName.MaxLength = 100;
            TextBoxNextOfKinLastName.Text = registryDemographic.NEXT_OF_KIN_LNAME;

            //TO DO Add next of kin first name text box
            
            //next of kin relationship
            TextBoxNextOfKinRelationship.MaxLength = 100;
            TextBoxNextOfKinRelationship.Text = registryDemographic.NEXT_OF_KIN_RELATIONSHIP;

            //next of kin street 1
            TextBoxNextOfKinNextOfKinStreetLine1.MaxLength = 100;
            TextBoxNextOfKinNextOfKinStreetLine1.Text = registryDemographic.ADDRESS_LINE1;

            //next of kin street 2
            TextBoxNextOfKinAddressStreetLine2.MaxLength = 100;
            TextBoxNextOfKinAddressStreetLine2.Text = registryDemographic.ADDRESS_LINE2;
            
            //next of kin street 3
            TextBoxNextOfKinAddressStreetLine3.MaxLength = 100;
            TextBoxNextOfKinAddressStreetLine3.Text = registryDemographic.ADDRESS_LINE3;

            //next of kin city
            TextBoxNextOfKinCity.MaxLength = 60;
            TextBoxNextOfKinCity.Text = registryDemographic.CITY;

            //next of kin state
            TextBoxNextOfKinState.MaxLength = 50;
            TextBoxNextOfKinState.Text = registryDemographic.STATE;
            
            //next of kin zip
            TextBoxNextOfKinZip.MaxLength = 20;
            TextBoxNextOfKinZip.Text = registryDemographic.POSTAL_CODE;

            //next of kin phone
            TextBoxNextOfKinPhone.MaxLength = 50;
            TextBoxNextOfKinPhone.Text = registryDemographic.HOME_PHONE;

            //next of kin email
            TextBoxNextOfKinEmail.MaxLength = 128;
            TextBoxNextOfKinEmail.Text = registryDemographic.EMAIL_ADDRESS;
        
            //Battle
            if (registryDemographic.SERVICE_CONNECTED_IND.HasValue)
            {
                RadioButtonListBattle.Text = registryDemographic.SERVICE_CONNECTED_IND.Value.ToString();
            } 
        }

        //OEF or OIF 
        if (patient.OEF_OIF_IND.HasValue)
        {
            LabelOEFOIFValue.Text = (patient.OEF_OIF_IND.Value.ToString() == "1" ? "Yes" : "No");
        }
        else
        {
            LabelOEFOIFValue.Text = "";
        }

        //service
        LabelServiceValue.Text = (patient.STD_SERVICEBRANCH != null ? patient.STD_SERVICEBRANCH.NAME : "");

        //Living Arrangement
        if (patient.LIVING_ARRANGEMENT_TEXT == "" || patient.LIVING_ARRANGEMENT_TEXT == null)
        {
            RadioButtonListLivingArrangement.ClearSelection();
        }
        else
        {
            RadioButtonListLivingArrangement.Text = patient.LIVING_ARRANGEMENT_TEXT;
        }

        int status = ReviewEmail.LoadControlsFromObject(referral);
        if (status == 0) status = ReviewContact.LoadControlsFromObject(referral);
        if (status == 0) status = ReviewNextOfKin.LoadControlsFromObject(referral);
        if (status == 0) status = ReviewOther.LoadControlsFromObject(referral);

        return status;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eyeExam"></param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(PATIENT patient, REFERRAL referral)
    {
        //update object with data from controls

        //email
        patient.EMAIL_ADDRESS = TextBoxmail.Text;

        //Preferred Contact Method
        patient.PreferredContactMethod = TextBoxPreferredContactMethod.Text;

        //get registry demographic entry for next of kin
        //REGISTRY_DEMOGRAPHIC registryDemographic = referral.REGISTRY_DEMOGRAPHICs.Where(rd => rd.STD_ADDRESSTYPE.CODE == "NOK").FirstOrDefault();
        REGISTRY_DEMOGRAPHIC registryDemographic = patient.REGISTRY_DEMOGRAPHICs.Where(rd => rd.STD_ADDRESSTYPE.CODE == REGISTRY_DEMOGRAPHIC.NextOfKinAddressTypeCode).FirstOrDefault();

        if (registryDemographic == null)
        {
            registryDemographic = new REGISTRY_DEMOGRAPHIC();

            STD_ADDRESSTYPE newAddressType = new PatientManager().getNewAddressType(REGISTRY_DEMOGRAPHIC.NextOfKinAddressTypeCode);

            registryDemographic.STD_ADDRESSTYPE_ID = newAddressType.ID;
            
            STD_REGISTRY newRegistry = new PatientManager().getNewRegistry(_currentRegistry);

            registryDemographic.STD_REGISTRY_ID = newRegistry.ID ;

            patient.REGISTRY_DEMOGRAPHICs.Add(registryDemographic);

        }

        //next of kin First Name
        registryDemographic.NEXT_OF_KIN_FNAME = TextBoxNextOfKinFirstName.Text;

        //next of kin Last Name
        registryDemographic.NEXT_OF_KIN_LNAME = TextBoxNextOfKinLastName.Text;

        //next of kin relationship
        registryDemographic.NEXT_OF_KIN_RELATIONSHIP = TextBoxNextOfKinRelationship.Text;

        //next of kin street 1
        registryDemographic.ADDRESS_LINE1 = TextBoxNextOfKinNextOfKinStreetLine1.Text;

        //next of kin street 2
        registryDemographic.ADDRESS_LINE2 = TextBoxNextOfKinAddressStreetLine2.Text;

        //next of kin street 3
        registryDemographic.ADDRESS_LINE3 = TextBoxNextOfKinAddressStreetLine3.Text;

        //next of kin city
        registryDemographic.CITY = TextBoxNextOfKinCity.Text;

        //next of kin state
        registryDemographic.STATE = TextBoxNextOfKinState.Text;

        //next of kin zip
        registryDemographic.POSTAL_CODE = TextBoxNextOfKinZip.Text;

        //next of kin phone
        registryDemographic.HOME_PHONE = TextBoxNextOfKinPhone.Text;

        //next of kin email
        registryDemographic.EMAIL_ADDRESS = TextBoxNextOfKinEmail.Text;

        //Battle
        registryDemographic.SERVICE_CONNECTED_IND = 
            Helpers.ConvertStringToNullable<char>(RadioButtonListBattle.SelectedValue);
        
        //Living Arrangement
        if (RadioButtonListLivingArrangement.Text == "")
        {
            patient.LIVING_ARRANGEMENT_TEXT = null;
        }
        else
        {
            patient.LIVING_ARRANGEMENT_TEXT = RadioButtonListLivingArrangement.Text;
        }

        int status = ReviewEmail.LoadObjectFromControls(referral);
        if (status == 0) status = ReviewContact.LoadObjectFromControls(referral);
        if (status == 0) status = ReviewNextOfKin.LoadObjectFromControls(referral);
        if (status == 0) status = ReviewOther.LoadObjectFromControls(referral); 

        return status;
    }

    private void PopulateDropDowns(PATIENT patient)
    {

        //Example
        //if (DropDownListGender.Items.Count == 1)
        //{
        //    DropDownListGender.DataSourceID = "ObjectDataSourceGender";
        //    DropDownListGender.DataBind();
        //}

    }

    #endregion
}
