﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.DVEIR;
using VeteransAffairs.Registries.BusinessManager.Utilities;



public partial class controls_ReviewCommentsEntryUserControl : BaseClinicalDataControl
{
    
    //_showCommentControl determines if the whole control is visible
    private bool _showCommentControl = false;
    
    private bool _commentContentShowOnLoad = false;
    private bool _commentIsEditable = false;
    private bool _commentIsResovleEditable = false;
    private bool _commentIsApproveEditable = false;
    private bool _showCommentAddButton = false;
    private bool _showCommentEditButton = false;
    private bool _showCommentBackButton = false;

    private bool _setFocusOnLoad = false;

    public controls_ReviewCommentsEntryUserControl()
    {
        // allow commenting even on read-only pages
        OverrideReadOnly = true;
    }

    /// <summary>
    /// Event to raise when the back button is clicked.
    /// </summary>
    public event GoToReviewCommentListClickEvent GoToReviewCommentList;

    public delegate void GoToReviewCommentListClickEvent(object sender, GoToReviewCommentListEventArgs e);


    protected void Page_Load(object sender, EventArgs e)
    {
        string scriptName = "__dveirReviewCommentsUserControlJQueryScript.js";
        if (!Page.ClientScript.IsClientScriptIncludeRegistered(scriptName))
        {
            Page.ClientScript.RegisterClientScriptInclude(Page.GetType(), scriptName, "controls/dveirReviewCommentsUserControlJQueryScript.js");
        }

        Page.PreRender += new EventHandler(Page_PreRender);

    }

    void Page_PreRender(object sender, EventArgs e)
    {
        //set control state

        PanelReviewComment.Visible = _showCommentControl;

        if (_commentContentShowOnLoad)
        {
            PanelReviewCommentsContent.Style.Add("display", "block");
        }
        else
        {
            PanelReviewCommentsContent.Style.Remove("display");
            PanelReviewCommentsContent.Style.Add("display", "none");
        }
        
        TextBoxReviewComment.Visible = _commentIsEditable;
        LabelReviewCommentReadOnly.Visible = !_commentIsEditable;

        CheckBoxResolved.Enabled = _commentIsResovleEditable;
        CheckBoxApproved.Enabled = _commentIsApproveEditable;

        ImageButtonCommentAdd.Visible = _showCommentAddButton;
        ImageButtonCommentOpen.Visible = _showCommentEditButton;
        ImageButtonCommentReturnToList.Visible = _showCommentBackButton;

        if (_setFocusOnLoad)
        {
            if (_commentIsEditable)
            {
                //TextBoxReviewComment.Focus();

                ScrollTo(TextBoxReviewComment.ClientID,-130);
            }
            else
            {
                //ImageButtonCommentOpen.Focus();

                ScrollTo(ImageButtonCommentOpen.ClientID,-40);
            }

 
        }
    }

    public void ScrollTo(string ControlClientId, int pixelsAdditionalScrollBy)
    {

        string scriptName = "__ScrollTo";

        if (!Page.ClientScript.IsClientScriptBlockRegistered(scriptName))
        {
            Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName, 
            
            string.Format(@" 
 
                <script type='text/javascript'>  
         
                        $(document).ready(function() {{ 
                                var element = document.getElementById('{0}'); 
                                element.scrollIntoView(); 
                                element.focus();
                                window.scrollBy(0, {1}); 
                        }}); 
         
                </script> 
         
            ", ControlClientId, pixelsAdditionalScrollBy), false);
            
        }

        ;
    }

    #region Postback Events

    protected void ImageButtonCommentReturnToList_Click(object sender, ImageClickEventArgs e)
    {
            //Raise an Event to give notice that a item has been selected
 
            //create an instance of the custom eventArgs in order to populate the id selected
            GoToReviewCommentListEventArgs eventArgs = new GoToReviewCommentListEventArgs();
            eventArgs.Type = HiddenFieldReviewCommentType.Value;
            eventArgs.ReviewCommentID = Helpers.getIntFromString(HiddenFieldReviewCommentId.Value);

            if (this.GoToReviewCommentList != null) //check for listeners
            {
               

                //raise event
                this.GoToReviewCommentList(this, eventArgs);
            }

            this.RaiseBubbleEvent(this, eventArgs);

    }

    #endregion

    #region Private Methods

    #endregion


    #region Postback Events


    #endregion


    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Exam entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(ER_ENCOUNTER encounter)
    {
        TextBoxReviewComment.ToolTip = LocationFriendlyName + " Review Comments";
        LabelReviewCommentReadOnly.ToolTip = LocationFriendlyName + " Review Comments";

        RegularExpressionValidatorTextBoxReviewComment.ErrorMessage = LocationFriendlyName + " Review Comment needs to be less than 1000 characters.";
        
        //find the review comment attached to encounter by matching the locationPage and anchorId that is set on
        //the control at design time
        ER_ENCOUNTER_REVIEWCOMMENT reviewCommentFound =
            (from rC in encounter.ER_ENCOUNTER_REVIEWCOMMENTs
            where rC.LOCATIONPAGE == _locationPage.GetStringValue()
            &&
            rC.ANCHORid == _anchorId
            select rC).FirstOrDefault();

        if (reviewCommentFound != null)
        {
            TextBoxReviewComment.Text = reviewCommentFound.COMMENT_TEXT;
            LabelReviewCommentReadOnly.Text = reviewCommentFound.COMMENT_TEXT;

            CheckBoxResolved.Checked = reviewCommentFound.RESOLVED_FLAG.GetValueOrDefault();
            CheckBoxApproved.Checked = reviewCommentFound.APPROVED_FLAG.GetValueOrDefault();

            HiddenFieldReviewCommentId.Value = reviewCommentFound.ID.ToString();
            HiddenFieldReviewCommentType.Value = "encounter";

            if (reviewCommentFound.IsSelectedFromCommentsTab)
            {
                _setFocusOnLoad = true;
            }
        }

        

        SetControlState(reviewCommentFound, encounter.REFERRAL.STD_REFERRALSTS_ID);

        return 0;
    }

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="exam">Referral entity used to populate controls</param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(REFERRAL referral)
    {
        TextBoxReviewComment.ToolTip = LocationFriendlyName + " Review Comments";
        LabelReviewCommentReadOnly.ToolTip = LocationFriendlyName + " Review Comments";

        RegularExpressionValidatorTextBoxReviewComment.ErrorMessage = LocationFriendlyName + " Review Comment needs to be less than 1000 characters.";

        //find the review comment attached to referral by matching the anchorId that is set on
        //the control at design time
        ER_REFERRAL_REVIEWCOMMENT reviewCommentFound =
            (from rC in referral.ER_REFERRAL_REVIEWCOMMENTs
             where rC.ANCHORid == _anchorId
             select rC).FirstOrDefault();

        if (reviewCommentFound != null)
        {
            TextBoxReviewComment.Text = reviewCommentFound.COMMENT_TEXT;
            LabelReviewCommentReadOnly.Text = reviewCommentFound.COMMENT_TEXT;

            CheckBoxResolved.Checked = reviewCommentFound.RESOLVED_FLAG.GetValueOrDefault();
            CheckBoxApproved.Checked = reviewCommentFound.APPROVED_FLAG.GetValueOrDefault();

            HiddenFieldReviewCommentId.Value = reviewCommentFound.ID.ToString();
            HiddenFieldReviewCommentType.Value = "referral";

            if (reviewCommentFound.IsSelectedFromCommentsTab)
            {
                _setFocusOnLoad = true;
            }
        }

        SetControlState(reviewCommentFound, referral.STD_REFERRALSTS_ID);

        return 0;
    }

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="eye">Exam used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref ER_ENCOUNTER encounter, BaseClinicalDataControlIdentifiers controlIdentifier)
    {

        int returnStatus = 0;

        _anchorId = controlIdentifier.AnchorId;
        _locationFriendlyName = controlIdentifier.LocationFriendlyName;
        // The max length allowed in the database is 50 characters.
        if (_locationFriendlyName.Length > 50)
        {
            _locationFriendlyName = _locationFriendlyName.Substring(0, 50);
        }
        _locationPage = controlIdentifier.LocationPage;

        //find the review comment attached to encounter by matching the locationPage and anchorId that is set on
        //the control at design time
        ER_ENCOUNTER_REVIEWCOMMENT reviewCommentFound =
            (from rC in encounter.ER_ENCOUNTER_REVIEWCOMMENTs
             where rC.LOCATIONPAGE == _locationPage.GetStringValue()
             &&
             rC.ANCHORid == _anchorId
             select rC).FirstOrDefault();

        if (reviewCommentFound != null)
        {
            reviewCommentFound.COMMENT_TEXT = TextBoxReviewComment.Text;

            reviewCommentFound.RESOLVED_FLAG = CheckBoxResolved.Checked;
            reviewCommentFound.APPROVED_FLAG = CheckBoxApproved.Checked;

        }
        else
        {
            if (TextBoxReviewComment.Text != "")
            {
                ER_ENCOUNTER_REVIEWCOMMENT newReviewComment = new ER_ENCOUNTER_REVIEWCOMMENT();

                newReviewComment.ANCHORid = _anchorId;
                newReviewComment.COMMENT_TEXT = TextBoxReviewComment.Text;
                //need to get the server date
                newReviewComment.DATE_ENTERED = DateTime.Now;
                newReviewComment.LOCATION_FRIENDLY_NAME = _locationFriendlyName;
                newReviewComment.LOCATIONPAGE = _locationPage.GetStringValue();

                newReviewComment.APPROVED_FLAG = false;
                newReviewComment.RESOLVED_FLAG = false;

                encounter.ER_ENCOUNTER_REVIEWCOMMENTs.Add(newReviewComment);
            }
        }
       
        return returnStatus;

    }

    /// <summary>
    /// Method to populate an entity object from the user control's form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="patient">Patient used to fill witn form data</param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(REFERRAL referral)
    {
        int returnStatus = 0;

        //find the review comment attached to referral by matching the locationPage and anchorId that is set on
        //the control at design time
        ER_REFERRAL_REVIEWCOMMENT reviewCommentFound =
            (from rC in referral.ER_REFERRAL_REVIEWCOMMENTs
             where rC.ANCHORid == _anchorId
             select rC).FirstOrDefault();

        if (reviewCommentFound != null)
        {
            reviewCommentFound.COMMENT_TEXT = TextBoxReviewComment.Text;
            reviewCommentFound.RESOLVED_FLAG = CheckBoxResolved.Checked;
            reviewCommentFound.APPROVED_FLAG = CheckBoxApproved.Checked;
        }
        else
        {
            if (TextBoxReviewComment.Text != "")
            {
                ER_REFERRAL_REVIEWCOMMENT newReviewComment = new ER_REFERRAL_REVIEWCOMMENT();

                newReviewComment.ANCHORid = _anchorId;
                newReviewComment.COMMENT_TEXT = TextBoxReviewComment.Text;
                newReviewComment.DATE_ENTERED = DateTime.Now;
                newReviewComment.LOCATION_FRIENDLY_NAME = _locationFriendlyName;

                newReviewComment.APPROVED_FLAG = false;
                newReviewComment.RESOLVED_FLAG = false;

                referral.ER_REFERRAL_REVIEWCOMMENTs.Add(newReviewComment);
            }
        }

        return returnStatus;
    }

    private void SetControlState(ER_ENCOUNTER_REVIEWCOMMENT reviewComment, int referralStatusId)
    {
        UserAccountManager _currentUser = (UserAccountManager)HttpContext.Current.User;
        bool reviewCommentExists = (reviewComment != null);
        
       
        switch ((DVEIRReferralStatus)referralStatusId)
        {
            case DVEIRReferralStatus.DVEIR_NEW:

                _showCommentControl = false;
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;

                break;


            case DVEIRReferralStatus.DVEIR_IN_PROCESS:

                _showCommentControl = false; 
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;

                break;

            case DVEIRReferralStatus.DVEIR_RETURNED:


                if (reviewCommentExists)
                {
                    _showCommentControl = true;

                    if (reviewComment.APPROVED_FLAG != true || reviewComment.APPROVED_FLAG.HasValue != true)
                    {
                        _commentContentShowOnLoad = true;
                    }
                }

                _commentIsEditable = false;

                if (
                    _currentUser.IsSuperUser
                    ||
                    _currentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    )
                {
                    _commentIsResovleEditable = true;
                }

                _commentIsApproveEditable = false;
                
                _showCommentAddButton = false;
                
                _showCommentEditButton = true;
                
                _showCommentBackButton = true;
                
                break;

            case DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW:

                _showCommentControl = true;

                //initialize to not show comment.
                _commentContentShowOnLoad = false;

                //set show comment to true if the comment exists for following conditions only
                if (reviewCommentExists)
                {
                    if (reviewComment.APPROVED_FLAG.HasValue != true || reviewComment.APPROVED_FLAG != true)
                    {
                        _commentContentShowOnLoad = true;
                    }
                }
                

                if (
                    _currentUser.IsSuperUser
                    ||
                    //_currentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    _commentIsEditable = true;

                }

                _commentIsResovleEditable = false;

                if (
                    _currentUser.IsSuperUser
                    ||
                    //_currentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    _commentIsApproveEditable = true;
                }

                if (reviewCommentExists)
                {
                    _showCommentEditButton = true; 
                }
                else 
                { 
                    _showCommentAddButton = true;
                }
                
                _showCommentBackButton = true;

                
                break;

            case DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR:

                _showCommentControl = true;

                //initialize to not show comment.
                _commentContentShowOnLoad = false;

                //set show comment to true if the comment exists for following conditions only
                if (reviewCommentExists)
                {
                    if (reviewComment.APPROVED_FLAG.HasValue != true || reviewComment.APPROVED_FLAG != true)
                    {
                        _commentContentShowOnLoad = true;
                    }
                }


                if (
                    _currentUser.IsSuperUser
                    ||
                    //_currentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    _commentIsEditable = true;

                }

                _commentIsResovleEditable = false;

                if (
                    _currentUser.IsSuperUser
                    ||
                    //_currentUser.IsInRole((int)DVEIRRoles.Abstractor)
                    //||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
                    )
                {
                    _commentIsApproveEditable = true;
                }

                if (reviewCommentExists)
                {
                    _showCommentEditButton = true;
                }
                else
                {
                    _showCommentAddButton = true;
                }

                _showCommentBackButton = true;

                //_showCommentControl = false;
                //_commentContentShowOnLoad = false;
                //_commentIsEditable = false;
                //_commentIsResovleEditable = false;
                //_commentIsApproveEditable = false;
                //_showCommentAddButton = false;
                //_showCommentEditButton = false;
                //_showCommentBackButton = false;

                
                break;

            case DVEIRReferralStatus.DVEIR_COMPLETED:
                
                _showCommentControl = false;
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;

                break;

        }

        if (reviewCommentExists)
        {
            if (reviewComment.IsSelectedFromCommentsTab)
            {
                _commentContentShowOnLoad = true;
            }
        }

    }

    private void SetControlState(ER_REFERRAL_REVIEWCOMMENT reviewComment, int referralStatusId)
    {
        UserAccountManager _currentUser = (UserAccountManager)HttpContext.Current.User;
        bool reviewCommentExists = (reviewComment != null);

        switch ((DVEIRReferralStatus)referralStatusId)
        {
            case DVEIRReferralStatus.DVEIR_NEW:
                _showCommentControl = false;
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;
                break;
            case DVEIRReferralStatus.DVEIR_IN_PROCESS:
                _showCommentControl = false;
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;
                break;
            case DVEIRReferralStatus.DVEIR_RETURNED:
                if (reviewCommentExists)
                {
                    _showCommentControl = true;
                    if (reviewComment.APPROVED_FLAG != true || reviewComment.APPROVED_FLAG.HasValue != true)
                        _commentContentShowOnLoad = true;
                }
                _commentIsEditable = false;
                if (_currentUser.IsSuperUser ||
                    _currentUser.IsInRole((int)DVEIRRoles.Abstractor))
                    _commentIsResovleEditable = true;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = true;
                _showCommentBackButton = true;
                break;
            case DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW:
                _showCommentControl = true;
                //initialize to not show comment.
                _commentContentShowOnLoad = false;
                //set show comment to true if the comment exists for following conditions only
                if (reviewCommentExists)
                {
                    if (reviewComment.APPROVED_FLAG.HasValue != true || reviewComment.APPROVED_FLAG != true)
                        _commentContentShowOnLoad = true;
                }
                if (_currentUser.IsSuperUser ||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional))
                    _commentIsEditable = true;
                _commentIsResovleEditable = false;
                if (_currentUser.IsSuperUser ||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional))
                    _commentIsApproveEditable = true;
                if (reviewCommentExists)
                    _showCommentEditButton = true;
                else
                    _showCommentAddButton = true;
                _showCommentBackButton = true;
                break;
            case DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR:
                _showCommentControl = true;
                //initialize to not show comment.
                _commentContentShowOnLoad = false;
                //set show comment to true if the comment exists for following conditions only
                if (reviewCommentExists)
                {
                    if (reviewComment.APPROVED_FLAG.HasValue != true || reviewComment.APPROVED_FLAG != true)
                        _commentContentShowOnLoad = true;
                }
                if (_currentUser.IsSuperUser ||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional))
                    _commentIsEditable = true;
                _commentIsResovleEditable = false;
                if (_currentUser.IsSuperUser ||
                    _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional))
                    _commentIsApproveEditable = true;
                if (reviewCommentExists)
                    _showCommentEditButton = true;
                else
                    _showCommentAddButton = true;
                _showCommentBackButton = true;
                break;
            case DVEIRReferralStatus.DVEIR_COMPLETED:
                _showCommentControl = false;
                _commentContentShowOnLoad = false;
                _commentIsEditable = false;
                _commentIsResovleEditable = false;
                _commentIsApproveEditable = false;
                _showCommentAddButton = false;
                _showCommentEditButton = false;
                _showCommentBackButton = false;
                break;
        }
        if (reviewCommentExists)
        {
            if (reviewComment.IsSelectedFromCommentsTab)
                _commentContentShowOnLoad = true;
        }
    }

    #endregion

}
