﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using VeteransAffairs.Registries.BusinessManager.DVEIR;

public partial class clinicalDataUserControls_ReviewCommentsTabUserControl : System.Web.UI.UserControl
{

    /// <summary>
    /// Event to raise when a review Comment is selected.  The EventArgs for the event will contain the selected id.
    /// </summary>
    public event ReviewCommentClickEvent ReviewCommentSelected;

    public delegate void ReviewCommentClickEvent(object sender, ReviewCommentClickEventArgs e);

    public class ReviewCommentClickEventArgs : EventArgs
    {
        public bool ReferralComment { get; set; }
        public int ReviewCommentID { get; set; }
    }

    public int LoadWithReviewCommentID { get; set; }

    public string LoadWithReviewCommentType { get; set; }

    #region Postback Events

    protected void Page_Load(object sender, EventArgs e)
    {
        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewReviewComments.ClientID + "');", true);
    }

    #endregion

    #region Private Methods


    #endregion

    #region Object/Form Mapping

    /// <summary>
    /// Method to populate form elements from entity parameter
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="encounter"></param>
    /// <returns>int</returns>
    public int LoadControlsFromObject(DVEIR_REFERRAL referral)
    {

        //populate controls

        HiddenFieldReferralId.Value = referral.REFERRAL_ID.ToString();

        GridViewReviewComments.DataBind();

        //DVEIRReviewCommentsManager man = new DVEIRReviewCommentsManager();

        //IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> revComments = man.SelectByReferralId(1872,"ID",1,10);

        if (LoadWithReviewCommentID > 0)
        {
            SetRowValueValueByKey(GridViewReviewComments, LoadWithReviewCommentID.ToString());
        }
        else
        {
            GridViewReviewComments.SelectedIndex = -1;
        }

        return 0;
    }

    public void SetRowValueValueByKey(GridView GridView, string DataKeyValue)
    {
        int intSelectedIndex = 0;
        int intPageIndex = 0;
        int intGridViewPages = GridView.PageCount;

        // Loop thru each page in the GridView 
        for (int intPage = 0; intPage < intGridViewPages; intPage++)
        {
            // Set the current GridView page 
            GridView.PageIndex = intPage;
            // Bind the GridView to the current page 
            GridView.DataBind();
            // Loop thru each DataKey in the GridView 
            for (int i = 0; i < GridView.DataKeys.Count; i++)
            {
                if (Convert.ToString(GridView.DataKeys[i].Value) == DataKeyValue)
                {
                    // If it is a match set the variables and exit 
                    intSelectedIndex = i;
                    intPageIndex = intPage;
                    break;
                }
            }
        }

        // Set the GridView to the values found 
        GridView.PageIndex = intPageIndex;
        GridView.SelectedIndex = intSelectedIndex;

        GridView.DataBind();
        
//        //focus on the select button of the row selected
//        GridViewRow Row = GridView.Rows[GridView.SelectedIndex];

//        Button SelectButton = (Button)Row.FindControl("ButtonGoToReviewComment");
//        string scriptName = "__FocusGridViewButton";

//        if (!Page.ClientScript.IsClientScriptBlockRegistered(scriptName))
//        {
//            Page.ClientScript.RegisterClientScriptBlock(Page.GetType(), scriptName,

//            string.Format(@" 
// 
//                <script type='text/javascript'>  
//         
//                        $(document).ready(function() {{ 
//                                var element = document.getElementById('{0}'); 
//                                element.scrollIntoView(); 
//                                element.focus();
//                                window.scrollBy(0, {1}); 
//                        }}); 
//         
//                </script> 
//         
//            ", SelectButton.ClientID, 0), false);

//        }

//        ;

    } 

    /// <summary>
    /// Method to populate an entity object (passed by reference) form the user controls form elements
    /// Returns a status code, any value other than 0 is an error
    /// </summary>
    /// <param name="reviewComments"></param>
    /// <returns>int</returns>
    public int LoadObjectFromControls(ref DVEIR_REFERRAL referral)
    {
        int returnStatus = 0;

        //update object with data from controls


        //JH 7-27-2010: 
        //This tab will not be read only.  Commenting out the section below that updates the entity with the values from the checkboxes.
        //
        //foreach (GridViewRow Row in GridViewReviewComments.Rows)
        //{
        //    //Load form elemets data to objects
        //    bool reviewCommentResolved = ((CheckBox)Row.FindControl("CheckBoxReviewCommentResolved")).Checked;
        //    bool reviewCommentApproved = ((CheckBox)Row.FindControl("CheckBoxReviewCommentApproved")).Checked;
            
        //    int reviewCommentId = (int)GridViewReviewComments.DataKeys[Row.RowIndex].Value;

        //    ER_ENCOUNTER_REVIEWCOMMENT reviewCommentFound =
        //            referral.ER_ENCOUNTERs.SelectMany(p => p.ER_ENCOUNTER_REVIEWCOMMENTs).Where(a => a.ID == reviewCommentId).FirstOrDefault();

        //    if (reviewCommentFound != null)
        //    {
        //        reviewCommentFound.RESOLVED_FLAG = reviewCommentResolved;
        //        reviewCommentFound.APPROVED_FLAG = reviewCommentApproved;
        //    }
   
        //}

        return returnStatus;

    }

    protected void GridViewReviewComments_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "GoToReviewComment")
        {
            string arg = Convert.ToString(e.CommandArgument);
            if (arg != null)
            {
                //Raise an Event to give notice that a item has been selected
                if (this.ReviewCommentSelected != null) //check for listeners
                {
                    string[] argArr = arg.Split(';');
                    string type = argArr[0];
                    int id = Convert.ToInt32(argArr[1]);

                    //create an instance of the custom eventArgs in order to populate the id selected
                    ReviewCommentClickEventArgs eventArgs = new ReviewCommentClickEventArgs();
                    eventArgs.ReferralComment = type == "referral";
                    eventArgs.ReviewCommentID = id;

                    //raise event
                    this.ReviewCommentSelected(this, eventArgs);
                }
            }

        }

    }

    private void PopulateDropDowns(ER_ENCOUNTER encounter)
    {
      
    }

    public void ClearReviewCommentsGridSelection()
    {
        GridViewReviewComments.SelectedIndex = -1;

    }


    #endregion


    protected void GridViewReviewComments_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        //JH 7-27-2010: 
        //This tab will not be read only.  Commenting out the section below that sets enabled status of checkboxes
        //
        //CheckBox checkBoxResolve = (CheckBox)e.Row.FindControl("CheckBoxReviewCommentResolved");
        //CheckBox checkBoxApprove = (CheckBox)e.Row.FindControl("CheckBoxReviewCommentApproved");

        //if (checkBoxResolve != null && checkBoxApprove != null)
        //{
        //    checkBoxResolve.Enabled = false;
        //    checkBoxApprove.Enabled = false;

        //    UserAccountManager _currentUser = (UserAccountManager)HttpContext.Current.User;

        //    if (
        //        _currentUser.IsSuperUser
        //        ||
        //        _currentUser.IsInRole((int)DVEIRRoles.Abstractor)
        //        )
        //    {
        //        checkBoxResolve.Enabled = true;
        //    }

        //    if (
        //        _currentUser.IsSuperUser
        //        ||
        //        _currentUser.IsInRole((int)DVEIRRoles.MedicalProfessional)
        //        )
        //    {
        //        checkBoxApprove.Enabled = true;
        //    }
        //}
    }
}
