﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using DVEIRUserActivityType = BusinessManagerClasses.DVEIRUserActivityType;

public partial class AdminEditUsersRoles : BasePage
{
    USER userEntry;
    UsersManager usersManager = new UsersManager();

    public override string MenuItemName()
    {
        return "DVEIR ADMIN USER ROLE";
    }

    public override string AppObjectCode()
    {
        return "DVEIR ADMIN USER ROLE";
    }

    public override string TopMenuName()
    {
        return "Administration";
    }

    #region protected methods

    protected void Page_Load(object sender, EventArgs e)
    {
        HiddenFieldRegistryName.Value = CurrentRegistry;
              

        if (!IsPostBack)
        {
            HiddenFieldBreadCrumbBaseText.Value = LabelBreadCrumb.Text;
            
            //if page is loading for first time get ID from querystring and set to hidden field to use on postbacks
            int? userID = Convert.ToInt32(Request.QueryString["id"]);
            HiddenFieldUserID.Value = userID.GetValueOrDefault().ToString();

            //if userID is not supplied or zero then display a list of all user roles
            if (userID == null || userID == 0)
            {
                userID = 0;

                //set page display for list of all user roles
                SetFormDisplay(false);
            }
            else
            {
                //set page to display for single user role management
                SetFormDisplay(true);
            }
        }

        if (HiddenFieldUserID.Value != "0")
        {
            //Create User Entity object and Set properties to state of form
            getEntry(Convert.ToInt32(HiddenFieldUserID.Value));

            if (LabelBreadCrumb != null)
            {
                LabelBreadCrumb.Text = HiddenFieldBreadCrumbBaseText.Value + " > User List > " + userEntry.USERNAME + " Roles"; 
            }
        }

        if (IsPostBack && userEntry != null)
        {
            LoadObjectFromControls();
        }

        Page.ClientScript.RegisterStartupScript(this.GetType(), "UpdateGridView", "UpdateGridView('" + GridViewUserRoles.ClientID + "');", true);
    }

    protected void getEntry(int id)
    {
        userEntry = usersManager.SelectByID(CurrentRegistry, id);

        //initial display panels
        PanelNoAccess.Visible = false;
        PanelIdNotFound.Visible = false;
        PanelObjectContent.Visible = false;

        if (userEntry == null)
        {
            PanelObjectContent.Visible = false;

            if (usersManager.UserIDExists(id))
            {
                PanelNoAccess.Visible = true;
            }
            else
            {
                PanelIdNotFound.Visible = true;
            }
        }
        else
        {
            PanelObjectContent.Visible = true;
        }
    }

    protected void ButtonDownloadToExcel_Click(object sender, EventArgs e)
    {
        UsersManager usersManager = new UsersManager();
        int countAll = usersManager.SelectListUsersVISNsCount(HiddenFieldRegistryName.Value, "", "", 0, 0);
        GridViewUserRoles.PageSize = countAll;
        GridViewUserRoles.DataBind();
        PrepareGridViewForExport(GridViewUserRoles);
        ExportGridViewToExcel(GridViewUserRoles, "DVEIR User Roles.xls");
    }

    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string s1 = string.Empty; // e.Row.Cells[0].Text;
            HyperLink hl = e.Row.Cells[0].Controls[0] as HyperLink;
            if (hl != null)
            {
                s1 = hl.Text;
                hl.ToolTip = "Edit Roles for ID " + s1;
            }
        }
    }

    #endregion protected methods

    #region Postback Events

    protected void ButtonSave_Click(object sender, EventArgs e)
    {
        int returnSaveValue = 0;

        returnSaveValue = usersManager.Update(userEntry);

        this.SuppressSaveMessage = false;
        this.IsSaveSuccess = (returnSaveValue > 0);

        if (this.IsSaveSuccess)
        {
            CreateLoggingRecord(DVEIRUserActivityType.ADMIN_EDIT_USER_ROLES_SAVE, "username: " + userEntry.USERNAME);
        }

        //Set Broken Rule Error Display
        if (userEntry.BrokenRules.Count == 0)
        {
            ErrorList1.Visible = false;
        }
        else
        {
            ErrorList1.Visible = true;
        }

        ErrorList1.BrokenRules = userEntry.BrokenRules;
        
        //Refresh entity root data
        if (returnSaveValue != 0)
        {
            getEntry(Convert.ToInt32(returnSaveValue));
            var master = (RegistryMaster)Master;
            master.SetDirtyFlag(false);
            //(this.Master as RegistryMaster).SetDirtyFlag(false);
        }
    }

    protected void ButtonClose_Click(object sender, EventArgs e)
    {
        this.Redirect("AdminEditUsersRoles.aspx");
    }

    protected void ButtonClear_Click(object sender, EventArgs e)
    {
        this.TextBoxSearchCriteria.Text = "";
        HiddenFieldClearFlag.Value = "1";
        
    }

    protected void ButtonSearch_Click(object sender, EventArgs e)
    {
        
        userEntry = usersManager.SelectByUsername(TextBoxSearchCriteria.Text);

        if (userEntry != null)
        {
            GridViewUserRoles.DataSourceID = "ObjectDataSourceUsersRoles";
            GridViewUserRoles.DataBind();
            GridViewUserRoles.PageIndex = 0;
            HiddenFieldClearFlag.Value = "0";
        }

    }

    protected void SetFormDisplay(bool isSingleUserRoleManagement)
    {
        //LabelBreadCrumb = (Label)Master.FindControl("LabelBreadCrumb");

        if (isSingleUserRoleManagement)
        {
            //set page to display for single user role management
            PanelListUserRoles.Visible = false;
            PanelRoleInformation.Visible = true;

            //Modify breadcrumb to display editing user id
            //LabelBreadCrumb.Text = HiddenFieldBreadCrumbBaseText.Value + " > Edit Roles for User ID : " + HiddenFieldUserID.Value;

        }
        else
        {
            //set page display for list of all user roles
            PanelListUserRoles.Visible = true;
            PanelRoleInformation.Visible = false;

            //Modify breadcrumb to display adding new
            LabelBreadCrumb.Text = HiddenFieldBreadCrumbBaseText.Value; // + " > User Roles ";
            
        }
    }

    protected void CheckBoxRoleDVEIRAdministrator_CheckedChanged(object sender, EventArgs e)
    {
        var userRole = new USER_ROLE
                           {
                               STD_ROLE_ID = (int) RoleCode.DVEIR_ADMINISTRATOR
                           };

        if (CheckBoxRoleDVEIRAdministrator.Checked &&
                !userEntry.USER_ROLEs.Contains(userRole))
        {
            userEntry.USER_ROLEs.Add(userRole);
        }
    }

    protected void CheckBoxRoleDVEIRDataAbstractor_CheckedChanged(object sender, EventArgs e)
    {
        var userRole = new USER_ROLE
        {
            STD_ROLE_ID = (int)RoleCode.DVEIR_DATA_ABSTRACTOR
        };

        if (CheckBoxRoleDVEIRDataAbstractor.Checked &&
                !userEntry.USER_ROLEs.Contains(userRole))
        {
            userEntry.USER_ROLEs.Add(userRole);
        }
    }

    protected void CheckBoxRoleDVEIRMedicalProfessional_CheckedChanged(object sender, EventArgs e)
    {
        var userRole = new USER_ROLE
                            {
                                STD_ROLE_ID = (int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL
                            };

        if (CheckBoxRoleDVEIRMedicalProfessional.Checked &&
                !userEntry.USER_ROLEs.Contains(userRole))
        {
            userEntry.USER_ROLEs.Add(userRole);
        }
    }

    #endregion

    #region Object/Form Mapping

    private void LoadControlsFromObject()
    {
        //Registry
        HiddenFieldRegistryName.Value = CurrentRegistry;

        //Username:
        LabelUsername.Text = userEntry.USERNAME;

        //Full Name:
        LabelFullName.Text = userEntry.FULL_NAME;
        
        //Employee Number:
        LabelEmployeeNumber.Text = userEntry.EMPLOYEE_NUMBER;

        //Job Title:
        LabelJobTitle.Text = userEntry.JOB_TITLE;

        //userid 
        HiddenFieldUserID.Value = userEntry.USER_ID.ToString();

        //User Roles
        foreach (USER_ROLE userRole in userEntry.USER_ROLEs)
        {
            //DVEIR_ADMINISTRATOR = 19,
            //DVEIR_DATA_Abstractor = 21,
            //DVEIR_MEDICAL_PROFESSIONAL = 24

            switch (userRole.STD_ROLE_ID)
            {
                case (int)RoleCode.DVEIR_ADMINISTRATOR: //19
                    CheckBoxRoleDVEIRAdministrator.Checked = true;
                    break;
                case (int)RoleCode.DVEIR_DATA_ABSTRACTOR: //21
                    CheckBoxRoleDVEIRDataAbstractor.Checked = true;
                    break;
                case (int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL: //24
                    CheckBoxRoleDVEIRMedicalProfessional.Checked = true;
                    break;
            }
        }
    }

    private void LoadObjectFromControls()
    {
        //Remove USER Role Entries as needed  
        if (CheckBoxRoleDVEIRAdministrator.Checked == false)
        {
            userEntry.RemoveRoleFromUser(userEntry, (int)RoleCode.DVEIR_ADMINISTRATOR);
        }
        if (CheckBoxRoleDVEIRDataAbstractor.Checked == false)
        {
            userEntry.RemoveRoleFromUser(userEntry, (int)RoleCode.DVEIR_DATA_ABSTRACTOR);
        }
        if (CheckBoxRoleDVEIRMedicalProfessional.Checked == false)
        {
            userEntry.RemoveRoleFromUser(userEntry, (int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
        }

        //Add User Roles as needed
        //DVEIR_ADMINISTRATOR = 19,
        //DVEIR_DATA_Abstractor = 21,
        //DVEIR_MEDICAL_PROFESSIONAL = 24

        if (CheckBoxRoleDVEIRAdministrator.Checked == true)
        {
            UpdateUserRoleNotVisnBased(userEntry, (int)RoleCode.DVEIR_ADMINISTRATOR);
        }
      
        if (CheckBoxRoleDVEIRDataAbstractor.Checked == true)
        {
            UpdateUserRoleNotVisnBased(userEntry, (int)RoleCode.DVEIR_DATA_ABSTRACTOR);
        }
        
        if (CheckBoxRoleDVEIRMedicalProfessional.Checked == true)
        {
            UpdateUserRoleNotVisnBased(userEntry, (int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
        }
    }

    #endregion

    #region private methods

    private void Page_PreRender(object sender, EventArgs e)
    {
        if (userEntry != null)
        {
            //This is the last event in the page life cycle where changes can be made to the page object
            //so enforce business rules on the object (which could possibly change the propertie of the object
            //then load the form controls from the object

            //Enforce business rules on the referral object
            //need to call EnforceBusinessRules method in the ReferralManager because some rules require a database call
            //pass in the local instance of referral to the method.  The method will return the object with modifications due to business rules.
            if (IsPostBack)
            {
                int ret;
                ret = usersManager.EnforceBusinessRules(ref userEntry);
            }

            if (!IsPostBack)
            {
                //Load Controls from the Object
                LoadControlsFromObject();
            }
        }
    }

    private void UpdateUserRoleNotVisnBased(USER userEntity, int roleID)
    {
        if (!userEntity.USER_ROLEs.Any(e => e.STD_ROLE_ID == roleID && e.INACTIVE_FLAG == false))
        {
            USER_ROLE role = new USER_ROLE();
            role.STD_ROLE_ID = roleID;
            role.STD_INSTITUTION_ID = STD_INSTITUTION.TopInstitutionId;
            userEntry.USER_ROLEs.Add(role);
        }
    }

    #endregion private methods
}

