﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Text;
using System.Data;
using System.Data.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy.DVEIR
{
    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIREncounterManager : BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;
        private UsersManager _usersManager = new UsersManager();
        private DVEIRCommonManager _commonManager = new DVEIRCommonManager();

        public DVEIREncounterManager() 
        {
            _defaultSortField = "ER_ENCOUNTER_ID";
            
        }

        private IQueryable<ER_ENCOUNTER> LinqAll()
        {
            //populate LinqAll
            IQueryable<ER_ENCOUNTER> tempLinq = (from e in _db.ER_ENCOUNTERs
                                                 select e);
            return tempLinq;

        }

        private void SetLoadWith(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_DIAG_DTLs);
            lo.LoadWith<ER_DIAG_DTL>(e => e.STD_ICD_CODE);
            lo.LoadWith<ER_ENCOUNTER>(e => e.REFERRAL);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_OUTCOME);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_PROCEDURE_DTLs);
            lo.LoadWith<ER_PROCEDURE_DTL>(e => e.STD_CPT_Code);
            lo.LoadWith<ER_ENCOUNTER>(e => e.PROVIDER);
            lo.LoadWith<PROVIDER>(e => e.PROVIDER_CLASS_DTLs);
            lo.LoadWith<PROVIDER_CLASS_DTL>(e => e.STD_PROVIDER_CLASS);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_EXAM);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }

        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_DIAG_DTLs);
            lo.LoadWith<ER_DIAG_DTL>(e => e.STD_ICD_CODE);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_EXAM);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_OUTCOME);
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_PROCEDURE_DTLs);
            lo.LoadWith<ER_PROCEDURE_DTL>(e => e.STD_CPT_Code);
            lo.LoadWith<ER_ENCOUNTER>(e => e.PROVIDER);
            
            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        private void SetLoadWithForEtlListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.REFERRAL);
            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.ER_DIAG_DTL_ETLs);
            lo.LoadWith<ER_DIAG_DTL_ETL>(e => e.STD_ICD_CODE);
            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.ER_EXAM_ETL);
            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.ER_OUTCOME_ETL);
            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.ER_PROCEDURE_DTL_ETLs);
            lo.LoadWith<ER_PROCEDURE_DTL_ETL>(e => e.STD_CPT_Code);
            lo.LoadWith<ER_ENCOUNTER_ETL>(e => e.PROVIDER);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        #region Method for Check Existence
        
        public bool ItemExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.ER_ENCOUNTERs where e.ER_ENCOUNTER_ID == id select e).Count();
                
                if (count > 0)
                {
                    return true;
                }
                else 
                {
                    return false;
                }
            }
        }

        #endregion

        #region Methods for Select

        public IEnumerable<ER_ENCOUNTER> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_ENCOUNTER> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
                 
            }
        }

        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }

        private IQueryable<ER_ENCOUNTER> SelectLinqFilter()
        {
            IQueryable<ER_ENCOUNTER> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByDate

        /// <summary>
        /// Search encounters IN ER_ENCOUNTER_ETL table by referral ID and encounter date
        /// </summary>
        /// <param name="referralId"></param>
        /// <param name="encounterDate"></param>
        /// <returns></returns>
        public IEnumerable<ER_ENCOUNTER_ETL> SelectByDate(int referralId, DateTime encounterDate)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForEtlListViews(_db);
                return SelectByDateLinqFilter(referralId, encounterDate).OrderBy("VISIT_DATE").ToList();
            }
        }

        private IQueryable<ER_ENCOUNTER_ETL> SelectByDateLinqFilter(int referralId, DateTime encounterDate)
        {
            IQueryable<ER_ENCOUNTER_ETL> linqFilter = (from e in _db.ER_ENCOUNTER_ETLs
                                                       where e.REFERRAL_ID == referralId
                                                       && e.VISIT_DATE >= encounterDate.Date
                                                       && e.VISIT_DATE < encounterDate.Date.AddDays(1)
                                                       select e);

            return linqFilter;
        }
        #endregion

        #region Methods for SelectByID

        public ER_ENCOUNTER SelectByID(int id)
        {
           bool refresh = true;

           if (DVEIRGlobal.CurrentEncounter != null)
            {
                if (DVEIRGlobal.CurrentEncounter.ER_ENCOUNTER_ID == id)
                {
                    refresh = false;
                }
            }

            if (refresh)
            {
                using (_db = GetDataContext())
                {
                    SetLoadWith(_db);
                    ER_ENCOUNTER encounter = SelectByIDLinqFilter(id).SingleOrDefault();

                    if (encounter != null)
                    {
                        encounter.SetAsChangeTrackingRoot(true);

                        int status = encounter.EnforceBusinessRules(false, _db);
                    }

                    DVEIRGlobal.CurrentEncounter = encounter;

                }
            }
            return DVEIRGlobal.CurrentEncounter;
        }

        private IQueryable<ER_ENCOUNTER> SelectByIDLinqFilter(int id)
        {
            IQueryable<ER_ENCOUNTER> linqFilter = LinqAll();
            
            linqFilter = from t in linqFilter where t.ER_ENCOUNTER_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for EncounterExists

        /// <summary>
        /// Check if an encounter already exists in the database.
        /// </summary>
        /// <param name="referralId"></param>
        /// <param name="providerId"></param>
        /// <param name="sta6a"></param>
        /// <param name="visitDate"></param>
        /// <returns></returns>
        public bool EncounterExists(int referralId, int providerId, 
                                        string sta6a, DateTime visitDate)
        {
            bool exists = false;

            List<ER_ENCOUNTER> encounters = SelectByReferralID(referralId, string.Empty, 0, 10).ToList();

            foreach (ER_ENCOUNTER encounter in encounters)
            {
                if (encounter.PROVIDER_ID == providerId
                    && encounter.STA6A == sta6a
                    && encounter.VISIT_DATE == visitDate)
                {
                    exists = true;
                    break;
                }
            }

            return exists;
        }

        #endregion

        #region Methods for SelectByReferralID

        public IEnumerable<ER_ENCOUNTER> SelectByReferralID(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;
            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_ENCOUNTER> entities;
                entities = SelectByReferralIDLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                foreach (var erEncounter in entities)
                {
                    var  user = _usersManager.SelectByUsername(erEncounter.UPDATEDBY);
                    if ( user!= null )
                    {
                        erEncounter.UPDATEDBY = user.FULL_NAME;
                    }
                }
                
                return entities;
            }
        }

        public int SelectByReferralIDCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectByReferralIDLinqFilter(id).Count();
            }
        }

        private IQueryable<ER_ENCOUNTER> SelectByReferralIDLinqFilter(int id)
        {
            IQueryable<ER_ENCOUNTER> linqFilter = LinqAll();

            linqFilter = from t in linqFilter 
                         where t.REFERRAL.REFERRAL_ID == id
                          && 
                          ( 
                              //show all encounters when in completed status  
                              t.REFERRAL.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_COMPLETED
                              ||
                              //show non sent encounters when in ready to send status
                              (
                                t.REFERRAL.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR
                                &&
                                (t.SendTo_DoD_DVEIR_FLAG == null
                                    || t.SendTo_DoD_DVEIR_FLAG == false)
                              )
                              ||
                              //for all other statuses only show encounters that are not marked ready and not sent
                              (
                                  (t.Ready_to_Send_to_DVEIR_FLAG == null
                                    ||  t.Ready_to_Send_to_DVEIR_FLAG == false)
                                  && 
                                  (t.SendTo_DoD_DVEIR_FLAG == null
                                    || t.SendTo_DoD_DVEIR_FLAG == false)
                              )  
                          )
                         select t;

            return linqFilter;
        }

        #endregion

        #region Methods for SelectAllProviders
        public List<PROVIDER> SelectAllProviders()
        {
            List<PROVIDER> allProviders = null;
            using (_db = GetDataContext())
            {
                allProviders = (from p in _db.PROVIDERs 
                                where p.LAST_NAME != "NULL VALUE"
                                select p).OrderBy("LAST_NAME").ToList();
            }

            return allProviders;
        }
        #endregion

        #region Methods for SelectByPatientID

        public IEnumerable<ER_ENCOUNTER> SelectByPatientID(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_ENCOUNTER> entities;
                entities = SelectByPatientIDLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;

            }
        }

        public int SelectByPatientIDCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectByPatientIDLinqFilter(id).Count();
            }
        }

        private IQueryable<ER_ENCOUNTER> SelectByPatientIDLinqFilter(int id)
        {
            IQueryable<ER_ENCOUNTER> linqFilter = LinqAll();
           
            linqFilter = from t in linqFilter where t.REFERRAL.PATIENT.PATIENT_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for Update


        public int Update(ER_ENCOUNTER encounter)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                int status = encounter.EnforceBusinessRules(true, _db);
                
                if (encounter.BrokenRules.Count == 0)
                {
                    encounter.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.
                    
                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManager.ResetGlobalVariables();

                        //if changes present submit changes
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = encounter.ER_ENCOUNTER_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (System.Data.SqlClient.SqlException e)
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException e)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepCurrentValues);
                            
                            try
                            {
                                _db.SubmitChanges(ConflictMode.FailOnFirstConflict);

                                returnStatus =
                                eventArgs.SavedItemId = encounter.ER_ENCOUNTER_ID;
                                eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                                RaiseSaveEvent(this, eventArgs);
                            }
                            catch
                            {
                                returnStatus = -1;
                                eventArgs.SavedItemId = -1;
                                eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                                RaiseSaveEvent(this, eventArgs);
                            }
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }
            return returnStatus;
        }

        #endregion

        #region Methods to change foreign key references


        #endregion

        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref ER_ENCOUNTER encounter) 
        {
            using (_db = GetDataContext())
            {
                int status = encounter.EnforceBusinessRules(false, _db);
            }

            return 0;
        }


        #endregion

        #region Data View Classes

        public List<ViewErDiagDtl> GetEncounterDiagnosisCodes(int encounterId)
        {
            List<ViewErDiagDtl> result;
            using (_db = GetDataContext())
            {

                result = 
                    (
                    from diags in _db.ER_DIAG_DTLs
                    where diags.ER_ENCOUNTER_ID == encounterId
                    select new ViewErDiagDtl 
                        {
                            ER_DIAG_DTL_ID = diags.ER_DIAG_DTL_ID,
                            COMMENT_TEXT = diags.COMMENT_TEXT,
                            ROW_VERSION = diags.ROW_VERSION,
                            CREATED = diags.CREATED,
                            UPDATED = diags.UPDATED,
                            CREATEDBY = diags.CREATEDBY,
                            UPDATEDBY = diags.UPDATEDBY,
                            ER_ENCOUNTER_ID = diags.ER_ENCOUNTER_ID.Value,
                            CODE = diags.STD_ICD_CODE == null ? "":diags.STD_ICD_CODE.CODE,
                            DESCRIPTION_TEXT = diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODE_DESCRIPTION,
                            CodeWithDescription = diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODE + " " + diags.STD_ICD_CODE.CODE_DESCRIPTION,
                            ICD_CODESET = diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODESET 
                        }
                    ).ToList();
            }
            return result;
        }

        /// <summary>
        /// VHAISPBLANCT | RTC 29158 | Display code description for "parent" ICD codes
        /// </summary>
        /// <param name="encounterId"></param>
        /// <returns></returns>
        public List<ViewErDiagDtl> GetEncounterDiagnosisCodesCustom(int encounterId)
        {
            List<ViewErDiagDtl> result = GetEncounterDiagnosisCodes(encounterId);

            if (result != null)
            {
                foreach (ViewErDiagDtl dtl in result)
                {
                    if (!string.IsNullOrEmpty(dtl.CODE))
                    {
                        string customDescription = string.Empty;
                        string code = dtl.CODE;

                        while (code.Contains("."))
                        {
                            code = code.Substring(0, code.Length - 1);

                            List<ViewStdIcdCode> subResult;
                            using (_db = GetDataContext())
                            {
                                subResult =
                                (
                                    from codes in _db.STD_ICD_CODEs
                                    where codes.CODE == code
                                    select new ViewStdIcdCode
                                    {
                                        ICD_CODE_ID = codes.ICD_CODE_ID,
                                        CODE = codes.CODE,
                                        CODE_DESCRIPTION = codes.CODE_DESCRIPTION,
                                        CODESET = codes.CODESET,
                                        CREATED = codes.CREATED,
                                        UPDATED = codes.UPDATED,
                                        CREATEDBY = codes.CREATEDBY,
                                        UPDATEDBY = codes.UPDATEDBY,
                                        ROW_VERSION = codes.ROW_VERSION,
                                        DVEIR_FLAG = codes.DVEIR_FLAG,
                                        EFR_FLAG = codes.EFR_FLAG,
                                        TBI_FLAG = codes.TBI_FLAG
                                    }
                                ).ToList();
                            }

                            if (subResult != null)
                            {
                                foreach (ViewStdIcdCode c in subResult)
                                {
                                    if (string.IsNullOrEmpty(customDescription))
                                    {
                                        if (!string.IsNullOrEmpty(c.CODE_DESCRIPTION))
                                        {
                                            customDescription += "(" + c.CODE_DESCRIPTION;
                                        }
                                    }
                                    else
                                    {
                                        if (!string.IsNullOrEmpty(c.CODE_DESCRIPTION))
                                        {
                                            customDescription += " | " + c.CODE_DESCRIPTION;
                                        }
                                    }
                                }
                            }
                        }

                        if (!string.IsNullOrEmpty(customDescription))
                        {
                            customDescription += ")";
                            dtl.DESCRIPTION_TEXT += " " + customDescription;
                        }
                    }
                }
            }

            return result;
        }

        public class ViewErDiagDtl
        {
            public int ER_DIAG_DTL_ID { get; set; }
            public string COMMENT_TEXT { get; set; }
            public Binary ROW_VERSION { get; set; }
            public DateTime CREATED { get; set; }
            public DateTime UPDATED { get; set; }
            public string CREATEDBY { get; set; }
            public string UPDATEDBY { get; set; }
            public int ER_ENCOUNTER_ID { get; set; }
            public string CODE { get; set; }
            public string DESCRIPTION_TEXT { get; set; }
            public string CodeWithDescription { get; set; }
            public string ICD_CODESET { get; set; }
        }

        public class ViewStdIcdCode
        {
            public int ICD_CODE_ID { get; set; }
            public string CODE { get; set; }
            public string CODE_DESCRIPTION { get; set; }
            public string CODESET { get; set; }
            public DateTime CREATED { get; set; }
            public DateTime UPDATED { get; set; }
            public string CREATEDBY { get; set; }
            public string UPDATEDBY { get; set; }
            public Binary ROW_VERSION { get; set; }
            public bool DVEIR_FLAG { get; set; }
            public bool EFR_FLAG { get; set; }
            public bool TBI_FLAG { get; set; }
        }

        public List<ViewErDiagDtl> GetEncounterDiagnositicProcedureCodes(int encounterId)
        {
            List<ViewErDiagDtl> result;
            using (_db = GetDataContext())
            {
                result = (
                    from diags in _db.ER_PROCEDURE_DTLs
                    where diags.ER_ENCOUNTER_ID == encounterId
                            && diags.ICD9_FLAG == true
                    //join c in _db.STD_ER_ICD9_CODEs
                    //    on
                    //    diags.CODE equals c.CODE into sr
                    //from x in sr.DefaultIfEmpty()
                    //where x.CODE == null || x.EYE_PROCEDURE_FLAG == true

                    select new ViewErDiagDtl
                        {
                            ER_DIAG_DTL_ID = diags.ER_PROCEDURE_DTL_ID,
                            COMMENT_TEXT = diags.COMMENT_TEXT,
                            ROW_VERSION = diags.ROW_VERSION,
                            CREATED = diags.CREATED,
                            UPDATED = diags.UPDATED,
                            CREATEDBY = diags.CREATEDBY,
                            UPDATEDBY = diags.UPDATEDBY,
                            ER_ENCOUNTER_ID = diags.ER_ENCOUNTER_ID.Value,

                            //VHAISPBLANCT | RTC 33465 | Use ER_PROCEDURE_DTL code and description because ICD_CODE_ID is always NULL
                            CODE = diags.CODE == null ? "" : diags.CODE,
                            DESCRIPTION_TEXT = diags.DESCRIPTION_TEXT == null ? "" : diags.DESCRIPTION_TEXT,
                            CodeWithDescription = (diags.CODE == null ? "" : diags.CODE) + " " + (diags.DESCRIPTION_TEXT == null ? "" : diags.DESCRIPTION_TEXT),
                            ICD_CODESET = diags.ICD_CODESET == null ? "" : diags.ICD_CODESET.Value.ToString()

                            //CODE = diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODE,
                            //DESCRIPTION_TEXT =  diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODE_DESCRIPTION,
                            //CodeWithDescription = diags.STD_ICD_CODE == null ? "" : diags.CODE + " " + diags.DESCRIPTION_TEXT,
                            //ICD_CODESET = diags.STD_ICD_CODE == null ? "" : diags.STD_ICD_CODE.CODESET
                        }

                    ).ToList();
            }

            return result;
        }

        public class ViewErProcedureDtl
        {
            public int ER_PROCEDURE_DTL_ID { get; set; }
            public string COMMENT_TEXT { get; set; }
            public Binary ROW_VERSION { get; set; }
            public DateTime CREATED { get; set; }
            public DateTime UPDATED { get; set; }
            public string CREATEDBY { get; set; }
            public string UPDATEDBY { get; set; }
            public int ER_ENCOUNTER_ID { get; set; }
            public string CODE { get; set; }
            public string DESCRIPTION_TEXT { get; set; }
            public string CodeWithDescription { get; set; }
        }

        public List<ViewErProcedureDtl> GetEncounterProcedureCodes(int encounterId)
        {
            List<ViewErProcedureDtl> result;
            using (_db = GetDataContext())
            {

                result = (
                    from procs in _db.ER_PROCEDURE_DTLs
                    where procs.ER_ENCOUNTER_ID == encounterId
                    && procs.ICD9_FLAG == false
                    //join c in _db.STD_ER_CPT4_CODEs
                    //    on
                    //    procs.CODE equals c.CODE into sr
                    //from x in sr.DefaultIfEmpty()

                    select new ViewErProcedureDtl
                        {
                            ER_PROCEDURE_DTL_ID = procs.ER_PROCEDURE_DTL_ID,
                            COMMENT_TEXT = procs.COMMENT_TEXT,
                            ROW_VERSION = procs.ROW_VERSION,
                            CREATED = procs.CREATED,
                            UPDATED = procs.UPDATED,
                            CREATEDBY = procs.CREATEDBY,
                            UPDATEDBY = procs.UPDATEDBY,
                            ER_ENCOUNTER_ID = procs.ER_ENCOUNTER_ID.Value,
                            CODE = procs.STD_CPT_Code == null ? "" : procs.STD_CPT_Code.code,
                            DESCRIPTION_TEXT = procs.STD_CPT_Code == null ? "" : procs.STD_CPT_Code.code_description,
                            CodeWithDescription = procs.STD_CPT_Code == null ? "" : procs.CODE + " " + procs.DESCRIPTION_TEXT
                        }

                    ).ToList();
            }

            return result;
        }
        
        #endregion

    }
}

