﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Text;
using System.Data;
using System.Data.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy.DVEIR
{
    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIRHistoryManager : BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;

        public DVEIRHistoryManager() 
        {
            _defaultSortField = "ER_HISTORY_ID";
            
        }

        private IQueryable<ER_HISTORY> LinqAll()
        {
            //populate LinqAll
            IQueryable<ER_HISTORY> tempLinq = (from e in _db.ER_HISTORies
                                                 select e);
            return tempLinq;

        }

        private void SetLoadWith(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_DIAG_DTLs);
            lo.LoadWith<ER_PRE_INJURY_DIAG_DTL>(e => e.STD_ER_ICD9_CODE);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;
        }

        private void SetLoadWithOcular(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_DIAG_DTLs);

            lo.AssociateWith<ER_HISTORY>(
                e => e.ER_PRE_INJURY_DIAG_DTLs.Where(a => a.LEFT_FLAG.Equals(true)
                                                        || a.RIGHT_FLAG.Equals(true)));

            lo.LoadWith<ER_PRE_INJURY_DIAG_DTL>(e => e.STD_ER_ICD9_CODE);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;
        }

        private void SetLoadWithSurgery(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_PROCEDURE_DTLs);

            lo.AssociateWith<ER_HISTORY>(
                e => e.ER_PRE_INJURY_PROCEDURE_DTLs.Where(a => a.LEFT_FLAG.Equals(true)
                                                            || a.RIGHT_FLAG.Equals(true)));

            lo.LoadWith<ER_PRE_INJURY_PROCEDURE_DTL>(e => e.STD_ER_CPT4_CODE);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;
        }

        private void SetLoadWithImaging(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_IMAGING_DTLs);

            lo.AssociateWith<ER_HISTORY>(
                e => e.ER_PRE_INJURY_IMAGING_DTLs.Where(a => a.LEFT_FLAG.Equals(true)
                                                            || a.RIGHT_FLAG.Equals(true)));

            lo.LoadWith<ER_PRE_INJURY_IMAGING_DTL>(e => e.STD_ER_IMG_MTHD);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;
        }

        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_DIAG_DTLs);
            lo.LoadWith<ER_PRE_INJURY_DIAG_DTL>(e => e.STD_ER_ICD9_CODE);
            //lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_PROCEDURE_DTLs);
            //lo.LoadWith<ER_HISTORY>(e => e.ER_PRE_INJURY_IMAGING_DTLs);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        #region Method for Check Existence
        
        public bool ItemExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.ER_HISTORies where e.ER_HISTORY_ID == id select e).Count();
                
                if (count > 0)
                {
                    return true;
                }
                else 
                {
                    return false;
                }
            }
        }

        #endregion

        #region Methods for Select

        public IEnumerable<ER_HISTORY> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_HISTORY> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
                 
            }
        }

        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }

        private IQueryable<ER_HISTORY> SelectLinqFilter()
        {
            IQueryable<ER_HISTORY> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByID

        public ER_HISTORY SelectByID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWith(_db);
                ER_HISTORY history = SelectByIDLinqFilter(id).SingleOrDefault();

                if (history != null)
                {
                    history.SetAsChangeTrackingRoot(true);

                    int status = history.EnforceBusinessRules(false, _db);
                }
                
                return history;

            }

        }

        private IQueryable<ER_HISTORY> SelectByIDLinqFilter(int id)
        {
            IQueryable<ER_HISTORY> linqFilter = LinqAll();
            
            linqFilter = from t in linqFilter where t.ER_HISTORY_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for SelectByPatientID

        public ER_HISTORY SelectByPatientID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWith(_db);
                ER_HISTORY history = SelectByPatientIDLinqFilter(id).SingleOrDefault();

                if (history != null)
                {
                    history.SetAsChangeTrackingRoot(true);

                    int status = history.EnforceBusinessRules(false, _db);
                }

                return history;

            }

        }

        public ER_HISTORY SelectOcularHistoryByPatientID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithOcular(_db);
                ER_HISTORY history = SelectByPatientIDLinqFilter( id ).SingleOrDefault();

                if (history != null)
                {
                    history.SetAsChangeTrackingRoot(true);

                    int status = history.EnforceBusinessRules(false, _db);
                }
                return history;
            }
        }

        public ER_HISTORY SelectSurgeryHistoryByPatientID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithSurgery(_db);
                ER_HISTORY history = SelectByPatientIDLinqFilter(id).SingleOrDefault();

                if (history != null)
                {
                    history.SetAsChangeTrackingRoot(true);

                    int status = history.EnforceBusinessRules(false, _db);
                }
                return history;
            }
        }

        public ER_HISTORY SelectImagingHistoryByPatientID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithImaging(_db);
                ER_HISTORY history = SelectByPatientIDLinqFilter(id).SingleOrDefault();

                if (history != null)
                {
                    history.SetAsChangeTrackingRoot(true);

                    int status = history.EnforceBusinessRules(false, _db);
                }
                return history;
            }
        }
        
        private IQueryable<ER_HISTORY> SelectByPatientIDLinqFilter(int id)
        {
            IQueryable<ER_HISTORY> linqFilter = LinqAll();

            linqFilter = from t in linqFilter where t.PATIENT_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for Update


        public int Update(ER_HISTORY history)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                int status = history.EnforceBusinessRules(true, _db);
                
                if (history.BrokenRules.Count == 0)
                {
                    DVEIRGlobal.CurrentPatient = null;

                    history.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.
                    
                    try
                    {
                        _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                        returnStatus =
                        eventArgs.SavedItemId = history.ER_HISTORY_ID;
                        eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                        RaiseSaveEvent(this, eventArgs);
                    }
                    catch (ChangeConflictException)
                    {
                        _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                        returnStatus =
                        eventArgs.SavedItemId = history.ER_HISTORY_ID;
                        eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                        RaiseSaveEvent(this, eventArgs);
                    }
                    catch
                    {
                        returnStatus = 0;
                        eventArgs.SavedItemId = 0;
                        eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                        RaiseSaveEvent(this, eventArgs);
                    }
                }
            }
            return returnStatus;
        }

        #endregion

        #region Methods to change foreign key references


        #endregion


        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref ER_HISTORY history) 
        {
            using (_db = GetDataContext())
            {
                int status = history.EnforceBusinessRules(false, _db);
            }

            return 0;
        }


        #endregion

    }
}

