﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Text;
using System.Data;
using System.Data.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy.DVEIR
{
    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIROutcomeManager : BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;
        private DVEIRCommonManager _commonManagerDveir = new DVEIRCommonManager();

        public DVEIROutcomeManager() 
        {
            _defaultSortField = "ER_OUTCOME_ID";
            
        }

        private IQueryable<ER_OUTCOME> LinqAll()
        {
           

            //populate LinqAll
            IQueryable<ER_OUTCOME> tempLinq = (from e in _db.ER_OUTCOMEs
                                                 select e);
                  
            return tempLinq;

        }

        private void SetLoadWith(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            lo.LoadWith<ER_OUTCOME>(e => e.ER_PROSTHETIC_DTLs);
            lo.LoadWith<ER_PROSTHETIC_DTL>(e => e.STD_ER_PROSTHETIC_TYPE);
          
            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        #region Method for Check Existence
        
        public bool ItemExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.ER_OUTCOMEs where e.ER_OUTCOME_ID == id select e).Count();
                
                if (count > 0)
                {
                    return true;
                }
                else 
                {
                    return false;
                }
            }
        }

        #endregion

        #region Methods for Select

        public IEnumerable<ER_OUTCOME> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_OUTCOME> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
                 
            }
        }

        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }

        private IQueryable<ER_OUTCOME> SelectLinqFilter()
        {
            IQueryable<ER_OUTCOME> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByID

        public ER_OUTCOME SelectByID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWith(_db);
                ER_OUTCOME outcome = SelectByIDLinqFilter(id).SingleOrDefault();

                if (outcome != null)
                {
                    outcome.SetAsChangeTrackingRoot(true);

                    int status = outcome.EnforceBusinessRules(false, _db);
                }
                
                return outcome;

            }

        }

        private IQueryable<ER_OUTCOME> SelectByIDLinqFilter(int id)
        {
            IQueryable<ER_OUTCOME> linqFilter = LinqAll();
            
            linqFilter = from t in linqFilter where t.ER_OUTCOME_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for SelectByPatientID

        public IEnumerable<ER_OUTCOME> SelectByPatientID(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_OUTCOME> entities;
                entities = SelectByPatientIDLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;

            }
        }

        public int SelectByPatientIDCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }

        private IQueryable<ER_OUTCOME> SelectByPatientIDLinqFilter(int id)
        {
            IQueryable<ER_OUTCOME> linqFilter = LinqAll();
           
            linqFilter = from t in linqFilter where t.PATIENT_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for Update


        public int Update(ER_OUTCOME outcome)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                int status = outcome.EnforceBusinessRules(true, _db);
                
                if (outcome.BrokenRules.Count == 0)
                {
                    outcome.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.
                    
                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();

                        //if changes are present call submit changes
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = outcome.ER_OUTCOME_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = outcome.ER_OUTCOME_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }
            return returnStatus;
        }

       
        #endregion

        #region Methods to change foreign key references

        /// <summary>
        /// Method to return a new selected VFQ questionnaire (STD_ER_VFQ_QUESTIONNAIRE_TYPE) in order to modify the linq entity.
        /// </summary>
        public STD_ER_VFQ_QUESTIONNAIRE_TYPE getNewVFQQuestionnaireType(int id)
        {
            STD_ER_VFQ_QUESTIONNAIRE_TYPE newEntity;

            using (_db = GetDataContext())
            {
                newEntity = (from se in _db.STD_ER_VFQ_QUESTIONNAIRE_TYPEs where se.ID == id select se).FirstOrDefault();
            }

            return newEntity;

        }


        #endregion


        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref ER_OUTCOME outcome) 
        {
            using (_db = GetDataContext())
            {
                int status = outcome.EnforceBusinessRules(false, _db);
            }

            return 0;
        }


        #endregion


    }
}

