﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Linq.Expressions;
using System.Text;
using System.Data.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy
{
    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIRPatientManager : BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;
        private DVEIR.DVEIRCommonManager _commonManagerDveir = new DVEIR.DVEIRCommonManager();

        public DVEIRPatientManager() 
        {
            _defaultSortField = "PATIENT_ID";

        }

        private IQueryable<PATIENT> LinqAll()
        {
            //populate LinqAll
            IQueryable<PATIENT> tempLinq = (from e in _db.PATIENTs
                                                 select e);

            //add all business filtering rules 
            
            //filter on DVEIR patients in patient registry
            tempLinq = (from p in tempLinq
                        //JH - 8/26/2010 modifying linq query so that multiple referrals are not returned.  
                        //Just returning patient records that have DVEIR referrals
                        where p.REFERRALs.OfType<DVEIR_REFERRAL>().Any()
                        select p).Distinct();

            return tempLinq;

        }

        private void SetLoadWith(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            lo.LoadWith<PATIENT>(e => e.STD_INSTITUTION);
            lo.LoadWith<STD_INSTITUTION>(e => e.STD_INSTITUTION_PARENT);
            lo.LoadWith<PATIENT>(e => e.STD_COMBATLOCATION);
            lo.LoadWith<PATIENT>(e => e.STD_GENDER);
            lo.LoadWith<PATIENT>(e => e.STD_MARITALSTATUS);
            lo.LoadWith<PATIENT>(e => e.STD_RACE);
            lo.LoadWith<PATIENT>(e => e.STD_ETHNICITY);
            lo.LoadWith<PATIENT>(e => e.STD_SERVICEBRANCH);
            lo.LoadWith<PATIENT>(e => e.ER_PROBLEM_LISTs);

            lo.LoadWith<PATIENT>(e => e.REGISTRY_DEMOGRAPHICs);
            lo.AssociateWith<PATIENT>(p => p.REGISTRY_DEMOGRAPHICs.Where(rd => rd.STD_REGISTRY.ID == _user.RegistryId));

            lo.LoadWith<REGISTRY_DEMOGRAPHIC>(e => e.STD_ADDRESSTYPE);
            //lo.LoadWith<REGISTRY_DEMOGRAPHIC>(e => e.STD_STATE);


            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }

        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            lo.LoadWith<PATIENT>(e => e.STD_INSTITUTION);
            lo.LoadWith<STD_INSTITUTION>(e => e.STD_INSTITUTION_PARENT);
            lo.LoadWith<PATIENT>(e => e.STD_SERVICEBRANCH);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }

        #region Method for Check Existence
        
        public bool PatientExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.PATIENTs where e.PATIENT_ID == id select e).Count();
                
                if (count > 0)
                {
                    return true;
                }
                else 
                {
                    return false;
                }
            }
        }

        #endregion

        #region Methods for Select

        public IEnumerable<PATIENT> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<PATIENT> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
                 
            }
        }

        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }

        private IQueryable<PATIENT> SelectLinqFilter()
        {
            IQueryable<PATIENT> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByID

        public PATIENT SelectByID(int id)
        {
             bool refresh = true;

             if (DVEIR.DVEIRGlobal.CurrentPatient != null)
            {
                if (DVEIR.DVEIRGlobal.CurrentPatient.PATIENT_ID == id)
                {
                    refresh = false;
                }
            }

           if (refresh)
           {
               using (_db = GetDataContext())
               {
                   SetLoadWith(_db);
                   PATIENT patient = SelectByIDLinqFilter(id).SingleOrDefault();

                   if (patient != null)
                   {
                       patient.SetAsChangeTrackingRoot(true);

                       int status = patient.EnforceBusinessRules(true, _db);
                   }

                   DVEIR.DVEIRGlobal.CurrentPatient = patient;
               }
           }
           return DVEIR.DVEIRGlobal.CurrentPatient;
        }

        private IQueryable<PATIENT> SelectByIDLinqFilter(int id)
        {
            IQueryable<PATIENT> linqFilter = LinqAll();

            linqFilter = from t in linqFilter where t.PATIENT_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for SelectBySearchCriteria

        public IEnumerable<PATIENT> SelectBySearchCriteria(string searchStringFirstName,
            string searchStringLastName, string searchStringSSN, string searchStringICN,
            string searchStringDateOfBirth,
            string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(searchStringFirstName)
                && string.IsNullOrEmpty(searchStringLastName)
                    && string.IsNullOrEmpty(searchStringSSN)
                        && string.IsNullOrEmpty(searchStringICN)
                            && string.IsNullOrEmpty(searchStringDateOfBirth))
            {
                return null;
            }

            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                IEnumerable<PATIENT> entities;
                entities = SelectBySearchCriteriaLinqFilter(searchStringFirstName,
          searchStringLastName, searchStringSSN, searchStringICN, searchStringDateOfBirth).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }

        public IEnumerable<PATIENT> SelectBySearchCriteria(string searchStringVISN,
            string searchStringVAMCID, 
            string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(searchStringVISN)
                && string.IsNullOrEmpty(searchStringVAMCID))
            {
                return null;
            }

            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                IEnumerable<PATIENT> entities;
                entities = SelectBySearchCriteriaLinqFilter(searchStringVISN,
          searchStringVAMCID).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }

        public int SelectBySearchCriteriaCount(string searchStringFirstName,
            string searchStringLastName, string searchStringSSN, string searchStringICN, string searchStringDateOfBirth, string sort, int startRow, int maxRows)
        {

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectBySearchCriteriaLinqFilter(searchStringFirstName,
          searchStringLastName, searchStringSSN, searchStringICN, searchStringDateOfBirth).Count();
            }
        }

        public int SelectBySearchCriteriaCount(string searchStringVISN,
            string searchStringVAMCID, string sort, int startRow, int maxRows)
        {

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectBySearchCriteriaLinqFilter(searchStringVISN,
          searchStringVAMCID).Count();
            }
        }

        private IQueryable<PATIENT> SelectBySearchCriteriaLinqFilter(string searchStringFirstName,
            string searchStringLastName, string searchStringSSN, string searchStringICN, string searchStringDateOfBirth)
        {
            IQueryable<PATIENT> linqFilter = LinqAll();

            if (!String.IsNullOrEmpty(searchStringFirstName))
            {
                linqFilter = from t in linqFilter where t.FIRST_NAME.Contains(searchStringFirstName.Trim()) select t;

            }

            if (!String.IsNullOrEmpty(searchStringLastName))
            {
                linqFilter = from t in linqFilter where t.LAST_NAME.Contains(searchStringLastName.Trim()) select t;

            }

            if (!String.IsNullOrEmpty(searchStringSSN))
            {
                //string formattedSSN = String.Format("{0:000-00-0000}", int.Parse(searchStringSSN.Trim().Replace("-", "")));
                string formattedSSN = searchStringSSN.Trim().Replace("-", "");
                linqFilter = from t in linqFilter where t.SSN.Replace("-","").Equals(formattedSSN) select t;

            }

            if (!String.IsNullOrEmpty(searchStringICN))
            {
                linqFilter = from t in linqFilter where t.PATIENT_ICN.Equals(searchStringICN.Trim()) select t;

            }

            if (!String.IsNullOrEmpty(searchStringDateOfBirth))
            {
                linqFilter = from t in linqFilter where t.BIRTH_DATE.Equals(searchStringDateOfBirth.Trim()) select t;

            }

            //linqFilter = from t in linqFilter where t.LAST_NAME.Contains(searchString) select t;

            return linqFilter;
        }

        private IQueryable<PATIENT> SelectBySearchCriteriaLinqFilter(string searchStringVISN,
            string searchStringVAMCID)
        {
            IQueryable<PATIENT> linqFilter = LinqAll();

            if (!String.IsNullOrEmpty(searchStringVISN))
            {
                linqFilter = from t in linqFilter where t.STD_INSTITUTION.STD_INSTITUTION_PARENT.NAME.Equals("VISN " + searchStringVISN.Trim()) select t;

            }

            if (!String.IsNullOrEmpty(searchStringVAMCID))
            {
                linqFilter = from t in linqFilter where t.STD_INSTITUTION.STATIONNUMBER.StartsWith(searchStringVAMCID.Trim()) select t;

            }

            return linqFilter;
        }

        #endregion

        #region Methods for Update

        public int Update(PATIENT patient)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                int status = patient.EnforceBusinessRules(true, _db);
                patient.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.

                //Check if any actual changes will occur
                ChangeSet changeSet = _db.GetChangeSet();

                if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                || changeSet.Updates.Count > 0)
                {
                    _commonManagerDveir.ResetGlobalVariables();

                    //if changes present submit changes
                    try
                    {
                        _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                        returnStatus =
                        eventArgs.SavedItemId = patient.PATIENT_ID;
                        eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                        RaiseSaveEvent(this, eventArgs);
                    }
                    catch (ChangeConflictException)
                    {
                        _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                        returnStatus =
                        eventArgs.SavedItemId = patient.PATIENT_ID;
                        eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                        RaiseSaveEvent(this, eventArgs);
                    }
                    
                    catch
                    {
                        returnStatus = -1;
                        eventArgs.SavedItemId = -1;
                        eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                        RaiseSaveEvent(this, eventArgs);
                    }
                }

            }

            return returnStatus;
        }

        #endregion

        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref PATIENT patient)
        {
            using (_db = GetDataContext())
            {
                int status = patient.EnforceBusinessRules(false, _db);
            }

            return 0;
        }


        #endregion
        #region Methods to change foreign key references
        #endregion
    }
}
