﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Text;
using VeteransAffairs.Registries.Business;
using System.Data.Linq;
using VeteransAffairs.Registries.Validation;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy.DVEIR
{
    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIRReferralReviewCommentsManager: BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;
        private DVEIRCommonManager _commonManagerDveir = new DVEIRCommonManager();

        public DVEIRReferralReviewCommentsManager() 
        {
            _defaultSortField = "ID";
        }

        #region Methods for Set Load
        private IQueryable<ER_REFERRAL_REVIEWCOMMENT> LinqAll()
        {
            //populate LinqAll
            IQueryable<ER_REFERRAL_REVIEWCOMMENT> tempLinq = (from e in _db.ER_REFERRAL_REVIEWCOMMENTs
                                                               select e);
            return tempLinq;
        }
        private void SetLoadWith(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;
        }
        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }
        #endregion

        #region Method for Check Existence
        public bool ReviewCommentExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.ER_REFERRAL_REVIEWCOMMENTs where e.ID == id select e).Count();

                if (count > 0)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }
        #endregion

        #region Methods for Select

        public IEnumerable<ER_REFERRAL_REVIEWCOMMENT> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;
            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<ER_REFERRAL_REVIEWCOMMENT> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }
        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }
        private IQueryable<ER_REFERRAL_REVIEWCOMMENT> SelectLinqFilter()
        {
            IQueryable<ER_REFERRAL_REVIEWCOMMENT> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByID

        public ER_REFERRAL_REVIEWCOMMENT SelectByID(int id)
        {
            using (_db = GetDataContext())
            {
                SetLoadWith(_db);
                ER_REFERRAL_REVIEWCOMMENT reviewComment;

                if (id == 0)
                {
                    reviewComment = new ER_REFERRAL_REVIEWCOMMENT();
                    _db.ER_REFERRAL_REVIEWCOMMENTs.Attach(reviewComment);

                    reviewComment.SetAsChangeTrackingRoot(EntityState.New, true);
                    reviewComment.SetAsInsertOnSubmit();
                }
                else
                {
                    reviewComment = SelectByIDLinqFilter(id).SingleOrDefault();

                    if (reviewComment != null)
                    {
                        reviewComment.SetAsChangeTrackingRoot(true);

                    }

                }

                if (reviewComment != null)
                {
                    int status = reviewComment.EnforceBusinessRules(false, _db);
                }

                return reviewComment;

            }

        }
        private IQueryable<ER_REFERRAL_REVIEWCOMMENT> SelectByIDLinqFilter(int id)
        {
            IQueryable<ER_REFERRAL_REVIEWCOMMENT> linqFilter = LinqAll();

            linqFilter = from t in linqFilter where t.ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for SelectByReferral

        public IEnumerable<ER_REFERRAL_REVIEWCOMMENT> SelectByReferral(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;
            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                IEnumerable<ER_REFERRAL_REVIEWCOMMENT> entities;
                entities = SelectByReferralLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }

        public int SelectByReferralCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectByReferralLinqFilter(id).Count();
            }
        }

        private IQueryable<ER_REFERRAL_REVIEWCOMMENT> SelectByReferralLinqFilter(int id)
        {
            IQueryable<ER_REFERRAL_REVIEWCOMMENT> linqFilter = LinqAll();

            linqFilter = from t in linqFilter where t.REFERRAL.REFERRAL_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for Update

        /// <summary>
        /// Update Review Comment. Returns id of ER_REFERRAL_REVIEWCOMMENT if save is sucessful, returns 0 if no changes were present, returns -1 if a save error occurred
        /// </summary>
        /// <param name="reviewComment">Instance of ER_PATIENT_REVIEWCOMMENT to save</param>
        /// <returns>int</returns>
        public int Update(ER_REFERRAL_REVIEWCOMMENT reviewComment)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                reviewComment.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.

                int status = reviewComment.EnforceBusinessRules(true, _db);

                if (reviewComment.BrokenRules.Count == 0)
                {
                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();

                        //call submit changes if changes exist
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = reviewComment.ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = reviewComment.ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }
            return returnStatus;
        }

        /// <summary>
        /// Update Review Comments. Returns 1 if save is sucessful, returns 0 if no changes were present, returns -1 if a save error occurred
        /// </summary>
        /// <param name="reviewComment">Instance of IEnumerable<ER_REFERRAL_REVIEWCOMMENT> to save</param>
        /// <returns>int</returns>
        public int Update(IEnumerable<ER_REFERRAL_REVIEWCOMMENT> reviewComments)
        {
            int returnStatus = 0;
            int businessRulesReturn = 0;
            ValidationErrorsCollection AggregateBrokenRules = new ValidationErrorsCollection();

            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                foreach (ER_REFERRAL_REVIEWCOMMENT reviewComment in reviewComments)
                {
                    reviewComment.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.
                    businessRulesReturn = reviewComment.EnforceBusinessRules(true, _db);

                    AggregateBrokenRules.Concat(reviewComment.BrokenRules);
                }


                if (AggregateBrokenRules.Count == 0)
                {

                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();

                        //call submit changes if changes exist
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = 1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = 1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }
            return returnStatus;
        }

        #endregion

        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref DVEIR_REFERRAL referral)
        {
            using (_db = GetDataContext())
            {
                int status = referral.EnforceBusinessRules(false, _db);
            }

            return 0;
        }

        #endregion
    }
}
