﻿using DataAccess;
using Managers.Encounter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Telerik.Web.UI;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

/// <summary>
/// Summary description for EncounterOperationAvailabilityFactory
/// </summary>
namespace Factories.EncounterOperations
{
    public class EncounterOperationAvailabilityFactory
    {
        /// <summary>
        /// Default Constructor
        /// </summary>
        public EncounterOperationAvailabilityFactory()
        {
        }

        /// <summary>
        /// Accepts a user object, encounterId and a reference to the Encounter operations toolbar control.
        /// 
        /// Based on the User's access level and the Encounter's status - this method will determine which options to keep visible on the toolbar.
        /// 
        /// Returns true if the record is editable in its current state.
        /// </summary>
        /// <param name="currentUser"></param>
        /// <param name="encounter"></param>
        /// <param name="encounterOperationsToolbar"></param>
        /// <returns>boolean if record is editable in the current state with the current user.</returns>
        public static bool ConfigureEncounterOperationMenu(UserAccountManager currentUser, ER_ENCOUNTER_MOD encounter, ref RadToolBar encounterOperationsToolbar)
        {
            bool isEditable = false;
            switch (encounter.EncounterStatusID)
            {
                case EncounterStatusType.NEW:
                    #region Role Determination 

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - New

                        isEditable = true;

                        // should have SAVE and CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - New

                        isEditable = true;

                        // should have SAVE and CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - New

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                case EncounterStatusType.IN_PROC:
                    #region Role Determination

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - In Proc

                        isEditable = true;

                        // should have SAVE, READY FOR REVIEW, and CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - In Proc

                        isEditable = true;

                        // should have SAVE, REVIEW, and CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - In Proc

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                case EncounterStatusType.REVIEW:
                    #region Role Determination

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - In Review

                        isEditable = true;

                        // should have SAVE, RETURN, SEND TO DVEIVR, CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - In Review

                        isEditable = true;

                        // SAVE, RETURN, SEND TO DVEIVR, CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - In Review

                        isEditable = false;

                        // should have CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                case EncounterStatusType.SEND_BACK:
                    #region Role Determination

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - SEND BACK

                        isEditable = true;

                        // should have SAVE, REVIEW, CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - SEND BACK

                        isEditable = true;

                        // should have SAVE, REVIEW, AND CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = true;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - SEND BACK

                        isEditable = false;

                        // CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                case EncounterStatusType.READY_SEND:
                    #region Role Determination

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - READY TO SEND

                        isEditable = false;

                        // should have CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - READY TO SEND

                        isEditable = false;

                        // should have CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - READY TO SEND

                        isEditable = false;

                        // CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                case EncounterStatusType.COMPLETED:
                    #region Role Determination

                    if (currentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
                    {
                        #region Admin - COMPLETED

                        isEditable = false;

                        // should have CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
                    {
                        #region Abstractor - COMPLETED

                        isEditable = false;

                        // should have CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }

                        #endregion
                    }
                    else if (currentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
                    {
                        #region Med Prof - COMPLETED

                        isEditable = false;

                        // CANCEL
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }
                    else
                    {
                        #region No Known Role - Hide all but Cancel

                        isEditable = false;

                        // Cancel
                        foreach (var item in encounterOperationsToolbar.GetAllItems())
                        {
                            switch (Int32.Parse(item.Value))
                            {
                                case EncounterOperationSelectionValues.SAVE:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                                    item.Visible = false;
                                    break;
                                case EncounterOperationSelectionValues.CANCEL:
                                    item.Visible = true;
                                    break;
                                default:
                                    // should never happen
                                    break;
                            }
                        }
                        #endregion
                    }

                    #endregion
                    break;
                default:
                    // not sure yet.
                    break;
            }

            return isEditable;
        }
    }
}