﻿using Managers.Exam;

namespace Factories
{
    /// <summary>
    /// Summary description for ExamEventFactory
    /// </summary>
    public class ExamEventFactory
    {
        private const string BasePath = "~/Controls/ExamTab/";
        private const string ControlExt = ".ascx";
        public static ExamEventFactoryResult GetExamEventFactoryResult(string currentUser, string categoryCode)
        {
            ExamEventFactoryResult result = new ExamEventFactoryResult();

            switch (categoryCode)
            {
                case "AngRecesss":
                    result.EventControlLocation = BasePath + "ExamTabAngleRecessionUserControl" + ControlExt;
                    result.EventViewManager = new AngleRecessionEventViewManager(currentUser);
                    break;
                case "AntChamber":
                    result.EventControlLocation = BasePath + "ExamTabAnteriorChamberUserControl" + ControlExt;
                    result.EventViewManager = new AnteriorChamberEventViewManager(currentUser);
                    break;
                case "Brow":
                    result.EventControlLocation = BasePath + "ExamTabBrowUserControl" + ControlExt;
                    result.EventViewManager = new BrowEventViewManager(currentUser);
                    break;
                case "ColorVis":
                    result.EventControlLocation = BasePath + "ExamTabColorVisionUserControl" + ControlExt;
                    result.EventViewManager = new ColorVisionEventViewManager(currentUser);
                    break;
                case "ConfrontVisF":
                    result.EventControlLocation = BasePath + "ExamTabConfrontationalVisualFieldUserControl" + ControlExt;
                    result.EventViewManager = new ConfrontationalVisualFieldEventViewManager(currentUser);
                    break;
                case "Conjuctiva":
                    result.EventControlLocation = BasePath + "ExamTabConjuctivaUserControl" + ControlExt;
                    result.EventViewManager = new ConjuctivaEventViewManager(currentUser);
                    break;
                case "Cornea":
                    result.EventControlLocation = BasePath + "ExamTabCorneaUserControl" + ControlExt;
                    result.EventViewManager = new CorneaEventViewManager(currentUser);
                    break;
                case "DepthPerc":
                    result.EventControlLocation = BasePath + "ExamTabDepthPerceptionUserControl" + ControlExt;
                    result.EventViewManager = new DepthPerceptionEventViewManager(currentUser);
                    break;
                case "FuncTestF":
                    result.EventControlLocation = BasePath + "ExamTabFunctionalTestingUserControl" + ControlExt;
                    result.EventViewManager = new FunctionalTestingEventViewManager(currentUser);
                    break;
                case "Globe":
                    result.EventControlLocation = BasePath + "ExamTabGlobeUserControl" + ControlExt;
                    result.EventViewManager = new GlobeEventViewManager(currentUser);
                    break;
                case "ImgAncilTest":
                    result.EventControlLocation = BasePath + "ExamTabImagingMethodologyUserControl" + ControlExt;
                    result.EventViewManager = new ImagingMethodologyEventViewManager(currentUser);
                    break;
                case "IntraocularPres":
                    result.EventControlLocation = BasePath + "ExamTabIntraocularPressureUserControl" + ControlExt;
                    result.EventViewManager = new IntraocularPressureEventViewManager(currentUser);
                    break;
                case "Iris":
                    result.EventControlLocation = BasePath + "ExamTabIrisUserControl" + ControlExt;
                    result.EventViewManager = new IrisEventViewManager(currentUser);
                    break;
                case "KinecticVisF":
                    result.EventControlLocation = BasePath + "ExamTabKineticVisualFieldsUserControl" + ControlExt;
                    result.EventViewManager = new KineticVisualFieldsEventViewManager(currentUser);
                    break;
                case "Lens":
                    result.EventControlLocation = BasePath + "ExamTabLensUserControl" + ControlExt;
                    result.EventViewManager = new LensEventViewManager(currentUser);
                    break;
                case "Lids":
                    result.EventControlLocation = BasePath + "ExamTabLidsUserControl" + ControlExt;
                    result.EventViewManager = new LidsEventViewManager(currentUser);
                    break;
                case "MotilityExMusc":
                    result.EventControlLocation = BasePath + "ExamTabMotilityUserControl" + ControlExt;
                    result.EventViewManager = new MotilityEventViewManager(currentUser);
                    break;
                case "OpticDisc":
                    result.EventControlLocation = BasePath + "ExamTabOpticDiscUserControl" + ControlExt;
                    result.EventViewManager = new OpticDiscEventViewManager(currentUser);
                    break;
                case "Orbit":
                    result.EventControlLocation = BasePath + "ExamTabOrbitUserControl" + ControlExt;
                    result.EventViewManager = new OrbitEventViewManager(currentUser);
                    break;
                case "ProstheticDevOrder":
                    result.EventControlLocation = BasePath + "ExamTabProstheticDevicesUserControl" + ControlExt;
                    result.EventViewManager = new ProstheticDevicesEventViewManager(currentUser);
                    break;
                case "Pupil":
                    result.EventControlLocation = BasePath + "ExamTabPupilUserControl" + ControlExt;
                    result.EventViewManager = new PupilEventViewManager(currentUser);
                    break;
                case "RelAffPupilDef":
                    result.EventControlLocation = BasePath + "ExamTabRelativeAfferentPupilDefectUserControl" + ControlExt;
                    result.EventViewManager = new RelativeAfferentPupilDefectEventViewManager(currentUser);
                    break;
                case "Retina":
                    result.EventControlLocation = BasePath + "ExamTabRetinaUserControl" + ControlExt;
                    result.EventViewManager = new RetinaEventViewManager(currentUser);
                    break;
                case "Scelera":
                    result.EventControlLocation = BasePath + "ExamTabSceleraUserControl" + ControlExt;
                    result.EventViewManager = new SceleraEventViewManager(currentUser);
                    break;
                case "StaticThresPer":
                    result.EventControlLocation = BasePath + "ExamTabStaticThresholdPerimetryUserControl" + ControlExt;
                    result.EventViewManager = new StaticThresholdPerimetryEventViewManager(currentUser);
                    break;
                case "Symptoms":
                    result.EventControlLocation = BasePath + "ExamTabSymptomsUserControl" + ControlExt;
                    result.EventViewManager = new SymptomsEventViewManager(currentUser);
                    break;
                case "VisualAcuity":
                    result.EventControlLocation = BasePath + "ExamTabVisualAcuityUserControl" + ControlExt;
                    result.EventViewManager = new VisualAcuityEventViewManager(currentUser);
                    break;
                case "VisFuncQues":
                    result.EventControlLocation = BasePath + "ExamTabVisualFunctioningUserControl" + ControlExt;
                    result.EventViewManager = new VisualFunctioningEventViewManager(currentUser);
                    break;
                case "Vitreous":
                    result.EventControlLocation = BasePath + "ExamTabVitreousUserControl" + ControlExt;
                    result.EventViewManager = new VirtreousEventViewManager(currentUser);
                    break;
                default:
                    result = null;
                    break;
            }

            return result;
        }
    }
}