﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Configuration;
using VeteransAffairs.Registries.Business;
using Legacy.DVEIR;

/// <summary>
/// Summary description for Helpers
/// </summary>
public static class Helpers
{
    private static string[] _countries = new string[] { 
         "Afghanistan", "Albania", "Algeria", "American Samoa", "Andorra", 
         "Angola", "Anguilla", "Antarctica", "Antigua And Barbuda", "Argentina", 
         "Armenia", "Aruba", "Australia", "Austria", "Azerbaijan",
		   "Bahamas", "Bahrain", "Bangladesh", "Barbados", "Belarus",
		   "Belgium", "Belize", "Benin", "Bermuda", "Bhutan",
		   "Bolivia", "Bosnia Hercegovina", "Botswana", "Bouvet Island", "Brazil",
		   "Brunei Darussalam", "Bulgaria", "Burkina Faso", "Burundi", "Byelorussian SSR",
		   "Cambodia", "Cameroon", "Canada", "Cape Verde", "Cayman Islands",
		   "Central African Republic", "Chad", "Chile", "China", "Christmas Island",
		    "Colombia", "Comoros", "Congo", "Cook Islands",
		   "Costa Rica", "Cote D'Ivoire", "Croatia", "Cuba", "Cyprus",
		   "Czech Republic", "Czechoslovakia", "Denmark", "Djibouti", "Dominica",
		   "Dominican Republic", "East Timor", "Ecuador", "Egypt", "El Salvador",
		   "England", "Equatorial Guinea", "Eritrea", "Estonia", "Ethiopia",
		   "Falkland Islands", "Faroe Islands", "Fiji", "Finland", "France",
		   "Gabon", "Gambia", "Georgia", "Germany", "Ghana",
		   "Gibraltar", "Great Britain", "Greece", "Greenland", "Grenada",
		   "Guadeloupe", "Guam", "Guatemela", "Guernsey", "Guiana",
		   "Guinea", "Guinea-Bissau", "Guyana", "Haiti", "Heard Islands",
		   "Honduras", "Hong Kong", "Hungary", "Iceland", "India",
		   "Indonesia", "Iran", "Iraq", "Ireland", "Isle Of Man",
		   "Israel", "Italy", "Jamaica", "Japan", "Jersey",
		   "Jordan", "Kazakhstan", "Kenya", "Kiribati", "Korea, South",
		   "Korea, North", "Kuwait", "Kyrgyzstan", "Lao People's Dem. Rep.", "Latvia",
		   "Lebanon", "Lesotho", "Liberia", "Libya", "Liechtenstein",
		   "Lithuania", "Luxembourg", "Macau", "Macedonia", "Madagascar",
		   "Malawi", "Malaysia", "Maldives", "Mali", "Malta",
		   "Mariana Islands", "Marshall Islands", "Martinique", "Mauritania", "Mauritius",
		   "Mayotte", "Mexico", "Micronesia", "Moldova", "Monaco",
		   "Mongolia", "Montserrat", "Morocco", "Mozambique", "Myanmar",
		   "Namibia", "Nauru", "Nepal", "Netherlands", "Netherlands Antilles",
		   "Neutral Zone", "New Caledonia", "New Zealand", "Nicaragua", "Niger",
		   "Nigeria", "Niue", "Norfolk Island", "Northern Ireland", "Norway",
		   "Oman", "Pakistan", "Palau", "Panama", "Papua New Guinea",
		   "Paraguay", "Peru", "Philippines", "Pitcairn", "Poland",
		   "Polynesia", "Portugal", "Puerto Rico", "Qatar", "Reunion",
		   "Romania", "Russian Federation", "Rwanda", "Saint Helena", "Saint Kitts",
		   "Saint Lucia", "Saint Pierre", "Saint Vincent", "Samoa", "San Marino",
		   "Sao Tome and Principe", "Saudi Arabia", "Scotland", "Senegal", "Seychelles",
		   "Sierra Leone", "Singapore", "Slovakia", "Slovenia", "Solomon Islands",
		   "Somalia", "South Africa", "South Georgia", "Spain", "Sri Lanka",
		   "Sudan", "Suriname", "Svalbard", "Swaziland", "Sweden",
		   "Switzerland", "Syrian Arab Republic", "Taiwan", "Tajikista", "Tanzania",
		   "Thailand", "Togo", "Tokelau", "Tonga", "Trinidad and Tobago",
		   "Tunisia", "Turkey", "Turkmenistan", "Turks and Caicos Islands", "Tuvalu",
		   "Uganda", "Ukraine", "United Arab Emirates", "United Kingdom", "United States",
		   "Uruguay", "Uzbekistan", "Vanuatu", "Vatican City State", "Venezuela",
		   "Vietnam", "Virgin Islands", "Wales", "Western Sahara", "Yemen",
		   "Yugoslavia", "Zaire", "Zambia", "Zimbabwe"};

    /// <summary>
    /// Returns an array with all countries
    /// </summary>     
    public static StringCollection GetCountries()
    {
        StringCollection countries = new StringCollection();
        countries.AddRange(_countries);
        return countries;
    }
    /// <summary>
    /// Returns sorted list of all countries
    /// </summary>
    /// <param name="insertEmpty"></param>
    /// <returns></returns>
    public static SortedList GetCountries(bool insertEmpty)
    {
        SortedList countries = new SortedList();
        if (insertEmpty)
            countries.Add("", "Please select one...");
        foreach (String country in _countries)
            countries.Add(country, country);
        return countries;
    }

    public static void SetUpHealthFactorCheckBoxes(Control container, int factorId)
    {

        foreach (Control ctl in container.Controls)
        {
            if (ctl is CheckBoxList)
            {
                CheckBoxList boxList = ctl as CheckBoxList;

                ListItem item = boxList.Items.FindByValue(factorId.ToString());
                if (item != null)
                {
                    //if we find factor nothing else to do here 
                    item.Selected = true;
                    return;
                }
            }
            else
            {
                if (ctl.Controls.Count > 0)
                    SetUpHealthFactorCheckBoxes(ctl, factorId);
            }
        }
    }
    /// <summary>
    /// Utility to build a control list of pre-defined control type within a container(usually panel)
    /// </summary>
    /// <param name="container"></param>
    /// <param name="type"></param>
    /// <returns></returns>
    public static List<Control> BuildControlList(Control container, Type type)
    {
        List<Control> controlList = new List<Control>();

        AddControlsToList(container.Controls, type, controlList);

        return controlList;
    }
    private static void AddControlsToList(ControlCollection collection, Type type, List<Control> listToFill)
    {
        foreach (Control control in collection)
        {
            if (control.GetType().Equals(type))
            {

                listToFill.Add(control);


            }
            else
            {
                if (control.Controls.Count > 0)
                    AddControlsToList(control.Controls, type, listToFill);

            }
        }
    }


    public static Boolean isInteger(string valueToCheck)
    {
        int _tempInt;

        try
        {
            _tempInt = Convert.ToInt32(valueToCheck);
            return true;
        }
        catch
        {
            return false;
        }

    }

    public static Boolean isString(string valueToCheck)
    {
        string _tempString;

        try
        {
            _tempString = valueToCheck.ToString();
            return true;
        }
        catch
        {
            return false;
        }

    }

    public static string GetConfigEntry(string key)
    {
        string Val = string.Empty;
        if (WebConfigurationManager.AppSettings.Count > 0)
            Val = WebConfigurationManager.AppSettings.Get(key);

        return Val;
    }

    /// <summary>
    /// useful function to convert to various nullable types from string
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="input"></param>
    /// <returns></returns>
    public static T? ConvertStringToNullable<T>(string input) where T : struct
    {
        T? result = null;

        TypeConverter converter = TypeDescriptor.GetConverter(typeof(T));

        if (string.IsNullOrEmpty(input))
        {
            result = null;
        }
        else
        {
            try
            {
                result = (T)converter.ConvertFromString(input);
            }
            catch
            {
                result = null;
            }

        }

        return result;
    }
    public static Nullable<T> ToNullable<T>(this string s) where T : struct
    {
        Nullable<T> result = new Nullable<T>();
        try
        {
            if (!string.IsNullOrEmpty(s) && s.Trim().Length > 0)
            {
                TypeConverter conv = TypeDescriptor.GetConverter(typeof(T));
                result = (T)conv.ConvertFrom(s);
            }
        }
        catch { }
        return result;
    }

    public static int getIntFromString(string inString)
    {
        int returnInt = 0;

        int tempInt;
        bool result = int.TryParse(inString, out tempInt);
        if (result)
        {
            returnInt = tempInt;
        }

        return returnInt;
    }

    public static int? getNullableIntFromString(string inString)
    {
        int? returnInt = null;

        int tempInt;
        bool result = int.TryParse(inString, out tempInt);
        if (result)
        {
            returnInt = tempInt;
        }

        return returnInt;
    }

    public static string getStringFromNullableInt(int? inInt)
    {
        string returnString = "";

        if (inInt.HasValue)
        {
            returnString = inInt.ToString();
        }

        return returnString;
    }

    public static DateTime? getNullableDateFromString(string inString)
    {
        DateTime? returnDate = null;

        DateTime tempDate;
        bool result = DateTime.TryParse(inString, out tempDate);
        if (result)
        {
            returnDate = tempDate;
        }

        return returnDate;
    }

    public static Char? getNullableCharFromString_old(string inString)
    {
        Char? returnChar = null;

        Char tempChar;
        bool result = Char.TryParse(inString, out tempChar);
        if (result)
        {
            returnChar = tempChar;
        }

        return returnChar;
    }

    public static string getShortDateStringFromNullableDate(DateTime? inDate)
    {
        string returnString = "";

        if (inDate.HasValue)
        {
            returnString = Convert.ToDateTime(inDate).ToShortDateString();

        }

        return returnString;
    }

    public static string getDateTimeStringFromNullableDate(DateTime? inDate)
    {
        string returnString = "";

        if (inDate.HasValue)
        {
            returnString = Convert.ToDateTime(inDate).ToShortDateString() 
                + " " + Convert.ToDateTime(inDate).ToShortTimeString();

        }

        return returnString;
    }


    /// <summary>
    /// Load object for Test Performed, Test Result and Comments
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="examDetailCategory"></param>
    /// <param name="examDetailValue"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    /// <param name="controlType"></param>
    public static void LoadOneObjectForExamDetail(ref ER_ENCOUNTER encounter,
        int examDetailCategory, string examDetailValue, 
        bool rightEye, bool leftEye,
        DveirBaseControlType controlType)
    {
        if (encounter.ER_EXAM.ER_EXAM_DTLs.Count > 0)
        {
            foreach (ER_EXAM_DTL item in encounter.ER_EXAM.ER_EXAM_DTLs)
            {
                if (item.STD_EXAM_LOOKUP_ID_CATEGORY == examDetailCategory
                    && item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye)
                {
                    item.SetAsDeleteOnSubmit();
                }
            }
        }

        if (examDetailCategory > 0 && !string.IsNullOrEmpty(examDetailValue))
        {
            ER_EXAM_DTL newExamDetail = new ER_EXAM_DTL();
            newExamDetail.ER_EXAM_ID = encounter.ER_EXAM.ER_EXAM_ID;
            newExamDetail.STD_EXAM_LOOKUP_ID_CATEGORY = examDetailCategory;
            newExamDetail.EXAM_DATE = encounter.ER_EXAM.EXAM_DATE;
            newExamDetail.RIGHT_FLAG = rightEye;
            newExamDetail.LEFT_FLAG = leftEye;
            if (controlType == DveirBaseControlType.TextBox)
            {
                newExamDetail.COMMENT_TEXT = examDetailValue;
            }
            else
            {
                newExamDetail.STD_EXAM_LOOKUP_ID_VALUE = Convert.ToInt32(examDetailValue);
            }

            encounter.ER_EXAM.ER_EXAM_DTLs.Add(newExamDetail);
            encounter.ER_EXAM.ER_EXAM_DTLs.Last().SetAsInsertOnSubmit();
        }

    }


    public static string GetExamDetailValue(ER_ENCOUNTER encounter, 
        int examDetailCategory, DveirBaseControlType controlType, bool rightEye, bool leftEye)
    {
        string results = string.Empty;

        if (encounter != null)
        {
            if (encounter.ER_EXAM != null)
            {
                foreach (ER_EXAM_DTL item in encounter.ER_EXAM.ER_EXAM_DTLs)
                {
                    if (item.STD_EXAM_LOOKUP_ID_CATEGORY == examDetailCategory
                        && item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye)
                    {
                        if (item.STD_EXAM_LOOKUP_ID_VALUE > 0 && controlType != DveirBaseControlType.TextBox)
                        {
                            results = item.STD_EXAM_LOOKUP_ID_VALUE.ToString();
                        }
                        else if (controlType == DveirBaseControlType.TextBox)
                        {
                            results = item.COMMENT_TEXT;
                        }
                        break;
                    }
                }
            }
        }

        return results;
    }

    /// <summary>
    /// This function will remove a record from ER_EXAM_DTL table
    /// </summary>
    /// <param name="encounter"></param>
    /// <param name="examDetailCategory"></param>
    /// <param name="examDetailValue"></param>
    /// <param name="rightEye"></param>
    /// <param name="leftEye"></param>
    public static void RemoveOneRowForExamDetail(ref ER_ENCOUNTER encounter,
    int examDetailCategory, string examDetailValue,
    bool rightEye, bool leftEye)
    {
        if (encounter.ER_EXAM.ER_EXAM_DTLs.Count > 0)
        {
            foreach (ER_EXAM_DTL item in encounter.ER_EXAM.ER_EXAM_DTLs)
            {
                if (item.STD_EXAM_LOOKUP_ID_CATEGORY == examDetailCategory
                    && item.RIGHT_FLAG == rightEye && item.LEFT_FLAG == leftEye)
                {
                    item.SetAsDeleteOnSubmit();
                    break;
                }
            }
        }
    }

}