﻿using System.Collections.Generic;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for ProviderViewManager
    /// </summary>
    public class ChiefComplaintViewManager : BaseViewManager
    {
        private readonly ReviewerCommentViewManager reviewerCommentViewManager;
        
        public ChiefComplaintViewManager(UserAccountManager currentUser) : base(currentUser)
        {
            this.reviewerCommentViewManager = new ReviewerCommentViewManager(currentUser);
        }

        public ChiefComplaintViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = new ReviewerCommentViewManager(currentUser, dataAccessManager);
        }

        public ChiefComplaintViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager, ReviewerCommentViewManager reviewerCommentViewManager) : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = reviewerCommentViewManager;
        }

        public ChiefComplaintViewModel GetChiefComplaintViewModel(int referralId, int encounterId)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComment = DataAccessManager.GetReviewCommentsByEncounterId(encounterId);
            string patientName = GetPatientName(referralId);

            return GetChiefComplaintViewModel(patientName, dbEncounter, reviewComment);
        }

        public ChiefComplaintViewModel GetChiefComplaintViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEncounter,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewerComments)
        {
            ChiefComplaintViewModel chiefComplaintView = new ChiefComplaintViewModel();
            if (dbEncounter != null)
            {
                chiefComplaintView.IsReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0);

                chiefComplaintView.ReferralId = dbEncounter.REFERRAL_ID;
                chiefComplaintView.EncounterId = dbEncounter.ER_ENCOUNTER_ID;
                chiefComplaintView.PatientName = patientName;
                chiefComplaintView.ChiefComplaintText = dbEncounter.CHIEF_COMPLAINT_TEXT;
                chiefComplaintView.HasReadingProblem = GetIndicatorValue(dbEncounter.TBI_READ_PRBLM_IND);
                chiefComplaintView.HasDiplopia = GetIndicatorValue(dbEncounter.TBI_DIPLOPIA_IND);
                chiefComplaintView.HasDazzlingGlare = GetIndicatorValue(dbEncounter.TBI_DAZZLING_IND);
                chiefComplaintView.HasPhotophobia = GetIndicatorValue(dbEncounter.TBI_PHOTOPHOBIA_IND);
                chiefComplaintView.HasEyeStrain = GetIndicatorValue(dbEncounter.TBI_EYE_STRAIN_IND);
                chiefComplaintView.HasBlurredVision = GetIndicatorValue(dbEncounter.TBI_BLURRED_VISION_IND);
                chiefComplaintView.HasOtherCondition = GetIndicatorValue(dbEncounter.OTHER_NEUROLOGICAL_IND);
                chiefComplaintView.Comments = dbEncounter.NEUROLOGICAL_COMMENT_TEXT;

                ViewModels.Encounter.EncounterStatusType encounterStatusType =
                    GetEncounterStatusType(dbEncounter.EncounterStatusID ?? 0);

                chiefComplaintView.AllowReviewerComments =
                    reviewerCommentViewManager.ShowReviewerComment(encounterStatusType);

                chiefComplaintView.ChiefComplaintTextReviewer = chiefComplaintView.AllowReviewerComments
                    ? reviewerCommentViewManager.GetReviewerCommentViewModel(
                        dbEncounter.REFERRAL_ID,
                        dbEncounter.ER_ENCOUNTER_ID,
                        patientName,
                        "ENC_CHIEFCOMP",
                        "N",
                        ReviewCommentAnchorType.ENCOUNTER_CHIEF_COMPLAINT_TAB,
                        null,
                        "Chief Complaint Tab > Chief Complaint",
                        encounterStatusType,
                        reviewerComments)
                    : new ReviewerCommentViewModel();

                chiefComplaintView.CommentReviewer = chiefComplaintView.AllowReviewerComments
                    ? reviewerCommentViewManager.GetReviewerCommentViewModel(
                        dbEncounter.REFERRAL_ID,
                        dbEncounter.ER_ENCOUNTER_ID,
                        patientName,
                        "ENC_VIS_COND",
                        "N",
                        ReviewCommentAnchorType.ENCOUNTER_CHIEF_COMPLAINT_TAB,
                        null,
                        "Chief Complaint Tab > Visual Conditions",
                        encounterStatusType,
                        reviewerComments)
                    : new ReviewerCommentViewModel();
            }

            return chiefComplaintView;
        }

        public void SaveChiefComplaintView(ChiefComplaintViewModel viewModel)
        {
            // Get the encounter
            var dbEncounter = DataAccessManager.GetEncounterById(viewModel.ReferralId, viewModel.EncounterId);

            // Update the exisiting encounter
            if (dbEncounter != null)
            {
                dbEncounter.CHIEF_COMPLAINT_TEXT = viewModel.ChiefComplaintText;
                dbEncounter.TBI_READ_PRBLM_IND = GetIndicatorString(viewModel.HasReadingProblem);
                dbEncounter.TBI_DIPLOPIA_IND = GetIndicatorString(viewModel.HasDiplopia);
                dbEncounter.TBI_DAZZLING_IND = GetIndicatorString(viewModel.HasDazzlingGlare);
                dbEncounter.TBI_PHOTOPHOBIA_IND = GetIndicatorString(viewModel.HasPhotophobia);
                dbEncounter.TBI_EYE_STRAIN_IND = GetIndicatorString(viewModel.HasEyeStrain);
                dbEncounter.TBI_BLURRED_VISION_IND = GetIndicatorString(viewModel.HasBlurredVision);
                dbEncounter.OTHER_NEUROLOGICAL_IND = GetIndicatorString(viewModel.HasOtherCondition);
                dbEncounter.NEUROLOGICAL_COMMENT_TEXT = viewModel.Comments;

                DataAccessManager.SaveEncounter(dbEncounter);
            }
        }
    }
}