﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using Managers.Exam;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.Encounter;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using ViewModels.Exam;
using ER_ENCOUNTER_REVIEWCOMMENT = DataAccess.ER_ENCOUNTER_REVIEWCOMMENT;
using PROVIDER = DataAccess.PROVIDER;
using STD_INSTITUTION = VeteransAffairs.Registries.Business.STD_INSTITUTION;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for EncounterViewManager
    /// </summary>
    public class EncounterViewManager : BaseViewManager
    {
        private ProviderViewManager providerViewManager;
        private ChiefComplaintViewManager chiefComplaintViewManager;
        private ExamViewManager examViewManager;
        private DiagnosisViewManager diagnosisViewManager;
        private ProceduresViewManager proceduresViewManager;
        private HistoryViewManager historyViewManager;
        private ReviewerCommentViewManager reviewerCommentViewManager;

        public EncounterViewManager(UserAccountManager currentUser) : base(currentUser)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
            providerViewManager = new ProviderViewManager(CurrentUser, DataAccessManager);
            chiefComplaintViewManager = new ChiefComplaintViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            examViewManager = new ExamViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            diagnosisViewManager = new DiagnosisViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            proceduresViewManager = new ProceduresViewManager(CurrentUser, DataAccessManager);
            historyViewManager = new HistoryViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
        }

        public EncounterViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) : base(currentUser, dataAccessManager)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
            providerViewManager = new ProviderViewManager(CurrentUser, DataAccessManager);
            chiefComplaintViewManager = new ChiefComplaintViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            examViewManager = new ExamViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            diagnosisViewManager = new DiagnosisViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
            proceduresViewManager = new ProceduresViewManager(CurrentUser, DataAccessManager);
            historyViewManager = new HistoryViewManager(CurrentUser, DataAccessManager, reviewerCommentViewManager);
        }

        public EncounterViewModel GetEncounterViewModel(int referralId, int encounterId)
        {
            EncounterViewModel encounterViewModel = null;
            
            DateTime referralDate;
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId, out referralDate);
            string patientFullName = GetPatientName(referralId);

            if (dbEncounter != null)
            {
                IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = DataAccessManager.GetReviewCommentsByEncounterId(encounterId).ToList();
                IEnumerable<STD_INSTITUTION> facilityList = DropDownLists.GetVISNStationOnlyList().ToArray();
                ViewModels.Encounter.EncounterStatusType statusType = GetEncounterStatusType(dbEncounter.EncounterStatusID ?? 0);
                bool isReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0);

                // General View
                GeneralViewModel generalViewModel = 
                    GetGeneralViewModel(patientFullName, dbEncounter, facilityList, reviewComments, referralDate, isReadOnly);

                // Provider View
                ProviderViewModel providerViewModel = 
                    providerViewManager.GetProviderViewModel(patientFullName, dbEncounter, facilityList);

                // Chief Complaint View
                ChiefComplaintViewModel chiefComplaintViewModel = 
                    chiefComplaintViewManager.GetChiefComplaintViewModel(patientFullName, dbEncounter, reviewComments);

                // Exam View
                ExamViewModel examViewModel = 
                    examViewManager.GetExamViewModel(patientFullName, dbEncounter, reviewComments);

                // Diagnosis View
                DiagnosisViewModel diagnosisView = 
                    diagnosisViewManager.GetDiagnosisViewModel(patientFullName, dbEncounter, reviewComments);

                // Procedures View
                ProceduresViewModel proceduresView = proceduresViewManager.GetProceduresViewModel(patientFullName, dbEncounter);

                // History View
                HistoryViewModel historyViewModel = historyViewManager.GetHistoryViewModel(patientFullName, dbEncounter, reviewComments);

                bool allowReviewerComments = reviewerCommentViewManager.ShowReviewerComment(statusType);

                IEnumerable<EncounterOperation> availableOperations = GetAvailableOperations(statusType);
                encounterViewModel = new EncounterViewModel()
                {
                    ReferralId = referralId,
                    EncounterId = encounterId,
                    PatientName = patientFullName,
                    StatusType = statusType,
                    AllowReviewerComments = allowReviewerComments,
                    AvaiilableEncounterOperations = availableOperations,
                    GeneralView = generalViewModel,
                    ProviderView = providerViewModel,
                    ChiefComplaintView = chiefComplaintViewModel,
                    ExamView = examViewModel,
                    DiagnosisView = diagnosisView,
                    ProceduresView = proceduresView,
                    HistoryView = historyViewModel
                };
            }

            return encounterViewModel;
        }

        public EncounterViewModel RefreshReviewerComments(EncounterViewModel encounterViewModel)
        {
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments =
                DataAccessManager.GetReviewCommentsByEncounterId(encounterViewModel.EncounterId);

            encounterViewModel.GeneralView.ReviewCommentSummaries = GetReviewCommentSummaries(reviewComments);

            return encounterViewModel;
        }

        public void SaveGeneralView(GeneralViewModel generalView)
        {
            if (!generalView.IsReadOnly)
            {
                var dbEncounter = DataAccessManager.GetEncounterById(generalView.ReferralId, generalView.EncounterId);

                dbEncounter.VISIT_DATE = generalView.VisitDate;
                dbEncounter.InstitutionID = generalView.Facility != null ? generalView.Facility.Id : (int?) null;
                dbEncounter.STA6A = generalView.Facility != null ? generalView.Facility.StationNumber : String.Empty;
                dbEncounter.EncounterTypeInOut = generalView.EncounterType;

                DataAccessManager.SaveEncounter(dbEncounter);
            }
        }

        public EncounterViewModel SaveEncounterStatus(EncounterViewModel viewModel)
        {
            var dbEncounter = DataAccessManager.GetEncounterById(viewModel.ReferralId, viewModel.EncounterId);

            if (dbEncounter != null)
            {
                dbEncounter.EncounterStatusID = (int) viewModel.StatusType;

                // Assign encounter when setting to In Progress and if it is not already assigned
                if (viewModel.StatusType == ViewModels.Encounter.EncounterStatusType.InProc &&
                    !dbEncounter.AssignedTo.HasValue)
                {
                    dbEncounter.AssignedTo = CurrentUser.UserId;
                    dbEncounter.AssignedDate = DateTime.Now;
                }

                DataAccessManager.SaveEncounter(dbEncounter);
            }

            // Update availble operations
            viewModel.AvaiilableEncounterOperations = GetAvailableOperations(viewModel.StatusType);

            return viewModel;
        }

        private GeneralViewModel GetGeneralViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEncounter,
            IEnumerable<STD_INSTITUTION> facilities,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments,
            DateTime referralDate,
            bool isReadonly)
        {
            IEnumerable<Facility> facilityList = facilities.Select(s => new Facility()
            {
                Id = s.ID,
                Name = s.FullVISNName,
                StationNumber = s.STATIONNUMBER
            }).ToList();

            var reviewSummaries = GetReviewCommentSummaries(reviewComments);

            return new GeneralViewModel()
            {
                ReferralId = dbEncounter.REFERRAL_ID,
                EncounterId = dbEncounter.ER_ENCOUNTER_ID,
                PatientName = patientName,
                IsReadOnly = isReadonly,
                VisitDate = dbEncounter.VISIT_DATE,
                FacilityId = dbEncounter.InstitutionID ?? 0,
                EncounterType = dbEncounter.EncounterTypeInOut,
                DateReferred = referralDate,
                Facilities = facilityList,
                ReviewCommentSummaries = reviewSummaries
            };
        }

        private List<ReviewCommentSummary> GetReviewCommentSummaries(IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments)
        {
            var reviewSummaries = reviewComments.Select(r => new ReviewCommentSummary()
            {
                CommentLocation = r.LOCATION_FRIENDLY_NAME,
                IsResolved = r.RESOLVED_FLAG ?? false,
                ResolvedBy = r.RESOLVED_BY,
                ResolvedDate = r.DATE_RESOLVED,
                IsApproved = r.APPROVED_FLAG ?? false,
                ApprovedBy = r.APPROVED_BY,
                ApprovedDate = r.DATE_APPROVED,
                AnchorId = r.ANCHOR_ID ?? 0,
                QuestionCode = r.QUESTION_CODE,
                LateralityCode = r.LATERALITY_CODE,
                GroupingId = r.GROUPING_ID,
                CreatedDate = r.CREATED,
                CreatedBy = r.CREATEDBY,
                UpdatedDate = r.UPDATED,
                UpdatedBy = r.UPDATEDBY
            }).ToList();
            return reviewSummaries;
        }

        private IEnumerable<EncounterOperation> GetAvailableOperations(ViewModels.Encounter.EncounterStatusType statusType)
        {
            List<EncounterOperation> availableOperations =
                new List<EncounterOperation>();

            bool isAdmin = CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR);
            bool isAbstractor = CurrentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR);
            bool isMedicalProfessional = CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);

            availableOperations.Add(EncounterOperation.Cancel);

            switch (statusType)
            {
                case ViewModels.Encounter.EncounterStatusType.New:
                    if (isAdmin || isAbstractor)
                    {
                        availableOperations.Add(EncounterOperation.Save);
                    }
                    break;
                case ViewModels.Encounter.EncounterStatusType.InProc:
                case ViewModels.Encounter.EncounterStatusType.SendBack:
                    if (isAdmin || isAbstractor)
                    {
                        availableOperations.Add(EncounterOperation.Save);
                        availableOperations.Add(EncounterOperation.ReadyForReview);
                    }
                    break;
                case ViewModels.Encounter.EncounterStatusType.Review:
                    if (isAdmin || isMedicalProfessional)
                    {
                        availableOperations.Add(EncounterOperation.Save);
                        availableOperations.Add(EncounterOperation.ReturnToAbstractor);
                        availableOperations.Add(EncounterOperation.SendToDVEIVR);
                    }
                    break;
            }

            return availableOperations;
        }
    }
}