﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;
using ViewModels.EncounterDetail;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for ProceduresViewManager
    /// </summary>
    public class ProceduresViewManager : BaseViewManager
    {
        public ProceduresViewManager(UserAccountManager currentUser) : base(currentUser)
        {
        }

        public ProceduresViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) : base(currentUser, dataAccessManager)
        {
        }

        public ProceduresViewModel GetProceduresViewModel(int referralId, int encounterId)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            string patientName = GetPatientName(referralId);

            return GetProceduresViewModel(patientName, dbEncounter);
        }

        public ProceduresViewModel GetProceduresViewModel(string patientName, ER_ENCOUNTER_MOD dbEncounter)
        {
            var viewModel = new ProceduresViewModel();

            var dbLateralities = DataAccessManager.GetAllLataralities();

            if (dbEncounter != null)
            {
                viewModel.ReferralId = dbEncounter.REFERRAL_ID;
                viewModel.EncounterId = dbEncounter.ER_ENCOUNTER_ID;
                viewModel.PatientName = patientName;
                viewModel.IsReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0);
                viewModel.VisitId = dbEncounter.UniqueVHAEncounterIDPrimaryKey;
                viewModel.AvailableLateralities = MapToViewModelLateralities(dbLateralities);
                viewModel.InitialLateralityCode = Laterality.LateralityCodes.NotSpecified;

                viewModel = GetProceduresSummaries(viewModel);
            }

            return viewModel;
        }

        public ProceduresViewModel GetProceduresSummaries(ProceduresViewModel viewModel)
        {
            IEnumerable<ENCOUNTER_DETAILS> encounterDetails = DataAccessManager.GetEncounterDetails(viewModel.VisitId, EncounterDetailType.CPT);

            var proceduresSummaries = encounterDetails.Select(d => new ProceduresSummary()
            {
                Id = d.ID,
                LateralityCode = d.LATERALITY_CODE,
                Laterality = d.LATERALITY != null ? d.LATERALITY.LATERALITY_NAME : string.Empty,
                Name = d.Name,
                Code = d.Value,
                CodeDescription = d.Description,
                CodeType = d.Type
            });

            viewModel.ProceduresSummaries = proceduresSummaries.ToList();

            return viewModel;
        }

        public void SaveProceduresDetail(ProceduresViewModel viewModel)
        {
            if (viewModel.ProcedureSummary != null)
            {
                var dbDetail = new ENCOUNTER_DETAILS()
                {
                    VisitIdentifier = viewModel.VisitId,
                    ID = viewModel.ProcedureSummary.Id,
                    LATERALITY_CODE = viewModel.ProcedureSummary.LateralityCode,
                    Name = viewModel.ProcedureSummary.Name,
                    Value = viewModel.ProcedureSummary.Code,
                    Description = viewModel.ProcedureSummary.CodeDescription,
                    Type = EncounterDetailType.CPT
                };

                DataAccessManager.SaveEncounterDetail(viewModel.EncounterId, dbDetail);
            }
        }

        public void DeleteProceduresDetail(int encounterId, int encounterDetailId)
        {
            DataAccessManager.RemoveEncounterDetail(encounterId, encounterDetailId);
        }
    }
}