﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager.MDWS;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class FunctionalTestingEventViewManager : BaseEventViewManager<FunctionalTestingEventModel>
    {
        public override string CategoryCode
        {
            get { return "FuncTestF"; }
        }

        public FunctionalTestingEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <returns>The functional testing event model</returns>
        public override FunctionalTestingEventModel GetDefaultViewModel(FunctionalTestingEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                FunctionalTestingEventModel.QuestionCodes.TestPerformed,
                FunctionalTestingEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                FunctionalTestingEventModel.QuestionCodes.TestResult,
                FunctionalTestingEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.EventComment);

            viewModel.VergenceRangesType = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.VergenceRangesType);
            viewModel.VergenceRanges = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.VergenceRanges);
            viewModel.NearPointConvergence = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergence);
            viewModel.NearPointConvergenceUnit = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergenceUnit);
            viewModel.AccommodativeAmplitude = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitude);
            viewModel.AccommodativeAmplitudeUnit = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitudeUnit);
            viewModel.AccommodativeRanges = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeRanges);
            viewModel.RefractionResults = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.RefractionResults);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The functional testing event model</returns>
        public override FunctionalTestingEventModel GetLoadedViewModel(FunctionalTestingEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.TestResult);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.EventComment);

            viewModel.VergenceRangesType = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.VergenceRangesType);
            viewModel.VergenceRanges = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.VergenceRanges);
            viewModel.NearPointConvergence = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergence);
            viewModel.NearPointConvergenceUnit = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergenceUnit);
            viewModel.AccommodativeAmplitude = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitude);
            viewModel.AccommodativeAmplitudeUnit = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitudeUnit);
            viewModel.AccommodativeRanges = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeRanges);
            viewModel.RefractionResults = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.RefractionResults);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(FunctionalTestingEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId,
                lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                FunctionalTestingEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Vergence Ranges Type
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.VergenceRangesType, referralId,
                    encounterId, lateralityCode));

                // Vergence Ranges Number Format
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.VergenceRanges, referralId, encounterId,
                    lateralityCode));

                // Near Point of Convergence
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.NearPointConvergence, referralId, encounterId,
                    lateralityCode));

                // Near Point of Convergence - Unit
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.NearPointConvergenceUnit, referralId,
                    encounterId, lateralityCode));

                // Accommodative Amplitude
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.AccommodativeAmplitude, referralId,
                    encounterId, lateralityCode));

                // Accommodative Amplitude - Unit
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.AccommodativeAmplitudeUnit,
                    referralId, encounterId, lateralityCode));

                // Accommodative Ranges
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.AccommodativeRanges, referralId, encounterId,
                    lateralityCode));

                // Refraction Results For Fused Cross Cylinder
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.RefractionResults, referralId, encounterId,
                    lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, FunctionalTestingEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, FunctionalTestingEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetVergenceRanges(responses));
            descriptionList.Add(GetUnit(responses, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitude, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitudeUnit));
            descriptionList.Add(GetUnit(responses, FunctionalTestingEventModel.QuestionCodes.NearPointConvergence, FunctionalTestingEventModel.QuestionCodes.NearPointConvergenceUnit));

            return descriptionList;
        }

        /// <summary>Gets the unit.</summary>
        /// <param name="responses">The responses.</param>
        /// <param name="number">The number.</param>
        /// <param name="unit">The unit.</param>
        /// <returns></returns>
        private string GetUnit(IEnumerable<RESPONSE> responses, string number, string unit)
        {
            string description = string.Empty;
            var numberResponse = responses.FirstOrDefault(r => r.QUESTION_CODE == number);
            var unitResponse = responses.FirstOrDefault(r => r.QUESTION_CODE == unit);

            if (numberResponse != null && unitResponse != null)
            {
                description =
                    numberResponse.QUESTION.QUESTION_TEXT
                    + ": "
                    + numberResponse.RESPONSE_TEXT
                    + " "
                    + unitResponse.QUESTION_CHOICE.CHOICE_NAME;
            }

            return description;
        }

        /// <summary>Gets the vergence ranges.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        private string GetVergenceRanges(IEnumerable<RESPONSE> responses)
        {
            string description = string.Empty;
            var vrTypeResponse = responses.FirstOrDefault(r => r.QUESTION_CODE == FunctionalTestingEventModel.QuestionCodes.VergenceRangesType);
            var vrNumberResponse = responses.FirstOrDefault(r => r.QUESTION_CODE == FunctionalTestingEventModel.QuestionCodes.VergenceRanges);

            if (vrTypeResponse != null  && vrNumberResponse != null)
            {
                description =
                    vrNumberResponse.QUESTION.QUESTION_TEXT
                    + ": "
                    + vrTypeResponse.QUESTION_CHOICE.CHOICE_NAME
                    + " "
                    + vrNumberResponse.RESPONSE_TEXT;
            }

            return description;
        }
    }
}