﻿using System.Collections.Generic;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class ImagingMethodologyEventViewManager : BaseEventViewManager<ImagingAncillaryEventModel>
    {
        public override string CategoryCode
        {
            get { return "ImgAncilTest"; }
        }

        public ImagingMethodologyEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The imaging ancillary event model</returns>
        public override ImagingAncillaryEventModel GetDefaultViewModel(ImagingAncillaryEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.ImagingMethodology = MapToMultiSelectQuestion(categoryQuestionList, ImagingAncillaryEventModel.QuestionCodes.ImagingMethodology);

            viewModel.TestDates =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestDateSuffix);

            viewModel.TestResults =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestResultSuffix);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The imaging ancillary event model</returns>
        public override ImagingAncillaryEventModel GetLoadedViewModel(ImagingAncillaryEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));

            viewModel.ImagingMethodology = MapToMultiSelectQuestion(categoryQuestionList, responseList, ImagingAncillaryEventModel.QuestionCodes.ImagingMethodology);

            viewModel.TestDates =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestDateSuffix);

            viewModel.TestResults =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestResultSuffix);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(ImagingAncillaryEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Imaging Methodology
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.ImagingMethodology, referralId, encounterId, lateralityCode));

            // Test Dates
            responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(viewModel.TestDates, viewModel.ImagingMethodology, referralId, encounterId, lateralityCode));

            // Test Results
            responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(viewModel.TestResults, viewModel.ImagingMethodology, referralId, encounterId, lateralityCode));

            return responsesToBeAdded;
        }

        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, ImagingAncillaryEventModel.QuestionCodes.ImagingMethodology));

            return descriptionList;
        }
    }
}