﻿using System.Collections.Generic;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Prosthetic Devices Event View Manager
    /// </summary>
    public class ProstheticDevicesEventViewManager : BaseEventViewManager<ProstheticDevicesEventModel>
    {
        public override string CategoryCode
        {
            get { return "ProstheticDevOrder"; }
        }

        public ProstheticDevicesEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override ProstheticDevicesEventModel GetDefaultViewModel(ProstheticDevicesEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);
            viewModel.Options = MapToMultiSelectQuestion(categoryQuestionList, ProstheticDevicesEventModel.QuestionCodes.Options);
            viewModel.ImplantTypes = MapToMultiSelectQuestion(categoryQuestionList, ProstheticDevicesEventModel.QuestionCodes.ImplantTypes);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override ProstheticDevicesEventModel GetLoadedViewModel(ProstheticDevicesEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.Options = MapToMultiSelectQuestion(categoryQuestionList, responseList, ProstheticDevicesEventModel.QuestionCodes.Options);
            viewModel.ImplantTypes = MapToMultiSelectQuestion(categoryQuestionList, responseList, ProstheticDevicesEventModel.QuestionCodes.ImplantTypes);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(ProstheticDevicesEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Options
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.Options, referralId, encounterId, lateralityCode));

            // Implant Types
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.ImplantTypes, referralId, encounterId, lateralityCode));

            return responsesToBeAdded;
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, ProstheticDevicesEventModel.QuestionCodes.Options));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, ProstheticDevicesEventModel.QuestionCodes.ImplantTypes));

            return descriptionList;
        }
    }
}