﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Pupil Event View Manager
    /// </summary>
    public class RelativeAfferentPupilDefectEventViewManager : BaseEventViewManager<RelativeAfferentPupilDefectEventModel>
    {
        public override string CategoryCode
        {
            get { return "RelAffPupilDef"; }
        }

        public RelativeAfferentPupilDefectEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override RelativeAfferentPupilDefectEventModel GetDefaultViewModel(RelativeAfferentPupilDefectEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                RelativeAfferentPupilDefectEventModel.QuestionCodes.TestPerformed,
                RelativeAfferentPupilDefectEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                RelativeAfferentPupilDefectEventModel.QuestionCodes.TestResult,
                RelativeAfferentPupilDefectEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToYesNoQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.VisitData);
            viewModel.ADP = MapToSingleSelectQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADP);
            viewModel.ADPLaterality = MapToTextQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADPLaterality);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override RelativeAfferentPupilDefectEventModel GetLoadedViewModel(RelativeAfferentPupilDefectEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed =
                MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestPerformed);

            viewModel.TestResult =
                MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestResult);

            viewModel.EventComment =
                MapToTextQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.EventComment);

            viewModel.VisitData =
                MapToYesNoQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.VisitData);

            viewModel.ADP =
                MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADP);

            viewModel.ADPLaterality =
                MapToTextQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADPLaterality);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(RelativeAfferentPupilDefectEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                RelativeAfferentPupilDefectEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                // Visit Data
                responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.VisitData, referralId, encounterId, lateralityCode));

                // ADP
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.ADP, referralId, encounterId, lateralityCode));

                if (viewModel.ADP.IsAnswered)
                {
                    // ADP Laterality
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.ADPLaterality, referralId, encounterId, lateralityCode));
                }
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, RelativeAfferentPupilDefectEventModel.QuestionCodes.VisitData));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADP));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADPLaterality));

            return descriptionList;
        }
    }
}