﻿using System.Collections.Generic;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Symptoms Event View Manager
    /// </summary>
    public class SymptomsEventViewManager : BaseEventViewManager<SymptomsEventModel>
    {
        public override string CategoryCode
        {
            get { return "Symptoms"; }
        }

        public SymptomsEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override SymptomsEventModel GetDefaultViewModel(SymptomsEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, SymptomsEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override SymptomsEventModel GetLoadedViewModel(SymptomsEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, responseList, SymptomsEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(SymptomsEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Visit Data
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.VisitData, referralId, encounterId, lateralityCode));

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, SymptomsEventModel.QuestionCodes.VisitData));

            return descriptionList;
        }
    }
}