﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Visual Acuity Event View Manager
    /// </summary>
    public class VisualAcuityEventViewManager : BaseEventViewManager<VisualAcuityEventModel>
    {
        public override string CategoryCode
        {
            get { return "VisualAcuity"; }
        }

        public VisualAcuityEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override VisualAcuityEventModel GetDefaultViewModel(VisualAcuityEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.TestPerformed,
                VisualAcuityEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.EventComment);

            // Universal
            viewModel.Scale = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Scale);
            viewModel.Distance = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Distance);
            viewModel.Correction = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Correction);
            viewModel.ExamMethod = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.ExamMethod);
            viewModel.ChartType = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.ChartType);
            viewModel.PrintSample = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.PrintSample);
            viewModel.Sphere = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Sphere);
            viewModel.Cylinder = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Cylinder);
            viewModel.Axis = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Axis);
            viewModel.Unknown = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Unknown);
            viewModel.ADD = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.ADD);
            viewModel.RefractionType = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Type);

            // US Scale
            viewModel.USChartDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USChartDistance);
            viewModel.USAcuity = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USAcuity);
            viewModel.USDifferentDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USDifferentDistance);
            viewModel.USPerformDistance = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USPerformDistance);
            viewModel.USPerformDistance.ResponseText = "20";
            viewModel.USAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USAdjustment);
            viewModel.USUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USUnits);

            // Metric Scale
            viewModel.MetricChartDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricChartDistance);
            viewModel.MetricAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricAcuity);
            viewModel.MetricDifferentDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricDifferentDistance);
            viewModel.MetricPerformDistance = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricPerformDistance);
            viewModel.MetricAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricAdjustment);
            viewModel.MetricUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricUnits);

            // Jaeger Scale
            viewModel.JaegerPerformDistance = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerPerformDistance);
            viewModel.JaegerAcuity = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerAcuity);
            viewModel.JaegerAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerAdjustment);
            viewModel.JaegerUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerUnits);

            viewModel.JaegerChartType = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.JaegerChartType,
                VisualAcuityEventModel.ChoiceCodes.JaegerChartType.Jaeger);

            viewModel.JaegerPrintSamples = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.JaegerPrintSamples,
                VisualAcuityEventModel.ChoiceCodes.JaegerPrintSample.Jaeger);

            // Decimal Scale
            viewModel.DecimalAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalAcuity);
            viewModel.DecimalAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalAdjustment);
            viewModel.DecimalUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalUnits);

            // M Unit Scale
            viewModel.MUnitAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitAcuity);
            viewModel.MUnitAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitAdjustment);
            viewModel.MUnitUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitUnits);

            // Non-Numeric Scale
            viewModel.NonNumericAcuity = MapToMultiSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.NonNumericAcuity);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override VisualAcuityEventModel GetLoadedViewModel(VisualAcuityEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.TestPerformed);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.EventComment);

            // Universal
            viewModel.Scale = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Scale);
            viewModel.Distance = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Distance);
            viewModel.Correction = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Correction);
            viewModel.ExamMethod = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.ExamMethod);
            viewModel.ChartType = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.ChartType);
            viewModel.PrintSample = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.PrintSample);
            viewModel.Sphere = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Sphere);
            viewModel.Cylinder = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Cylinder);
            viewModel.Axis = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Axis);
            viewModel.Unknown = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Unknown);
            viewModel.ADD = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.ADD);
            viewModel.RefractionType = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Type);

            // US Scale
            viewModel.USChartDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USChartDistance);
            viewModel.USAcuity = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USAcuity);
            viewModel.USDifferentDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USDifferentDistance);
            viewModel.USPerformDistance = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USPerformDistance);
            viewModel.USAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USAdjustment);
            viewModel.USUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USUnits);

            // Metric Scale
            viewModel.MetricChartDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricChartDistance);
            viewModel.MetricAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricAcuity);
            viewModel.MetricDifferentDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricDifferentDistance);
            viewModel.MetricPerformDistance = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricPerformDistance);
            viewModel.MetricAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricAdjustment);
            viewModel.MetricUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricUnits);

            // Jaeger Scale
            viewModel.JaegerPerformDistance = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerPerformDistance);
            viewModel.JaegerAcuity = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerAcuity);
            viewModel.JaegerAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerAdjustment);
            viewModel.JaegerUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerUnits);
            viewModel.JaegerChartType = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerChartType);
            viewModel.JaegerPrintSamples = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerPrintSamples);

            // Decimal Scale
            viewModel.DecimalAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalAcuity);
            viewModel.DecimalAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalAdjustment);
            viewModel.DecimalUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalUnits);

            // M Unit Scale
            viewModel.MUnitAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitAcuity);
            viewModel.MUnitAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitAdjustment);
            viewModel.MUnitUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitUnits);

            // Non-Numeric Scale
            viewModel.NonNumericAcuity = MapToMultiSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.NonNumericAcuity);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(VisualAcuityEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                VisualAcuityEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Universal
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Scale, referralId, encounterId, lateralityCode));
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Distance, referralId, encounterId, lateralityCode));
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Correction, referralId, encounterId, lateralityCode));
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.ExamMethod, referralId, encounterId, lateralityCode));

                if (viewModel.IsJaegerScale)
                {
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.JaegerChartType, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.JaegerPrintSamples, referralId, encounterId, lateralityCode));
                }
                else
                {
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.ChartType, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.PrintSample, referralId, encounterId, lateralityCode));
                }

                responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.Unknown, referralId, encounterId, lateralityCode));
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.ADD, referralId, encounterId, lateralityCode));
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.RefractionType, referralId, encounterId, lateralityCode));
                
                // Don't save refraction if unknown
                if (!viewModel.Unknown.IsYesSelected)
                {
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Sphere, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Cylinder, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Axis, referralId, encounterId, lateralityCode));
                }

                // US Scale
                if (viewModel.IsUSScale)
                {
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.USChartDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.USAcuity, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.USDifferentDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.USPerformDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.USAdjustment, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.USUnits, referralId, encounterId, lateralityCode));
                }

                // Metric Scale
                if (viewModel.IsMetricScale)
                {
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.MetricChartDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.MetricAcuity, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.MetricDifferentDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.MetricPerformDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.MetricAdjustment, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.MetricUnits, referralId, encounterId, lateralityCode));
                }

                // Jaeger Scale
                if (viewModel.IsJaegerScale)
                {
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.JaegerPerformDistance, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.JaegerAcuity, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.JaegerAdjustment, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.JaegerUnits, referralId, encounterId, lateralityCode));
                }

                // Decimal Scale
                if (viewModel.IsDecimalScale)
                {
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.DecimalAcuity, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.DecimalAdjustment, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.DecimalUnits, referralId, encounterId, lateralityCode));
                }

                // M Unit Scale
                if (viewModel.IsMUnitScale)
                {
                    responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.MUnitAcuity, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.MUnitAdjustment, referralId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.MUnitUnits, referralId, encounterId, lateralityCode));
                }

                // Non-Numeric Scale
                if (viewModel.IsNonNumericScale)
                {
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.NonNumericAcuity, referralId, encounterId, lateralityCode));
                }
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.Scale));
            descriptionList.AddRange(GetAcuity(responses));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.Distance));

            return descriptionList;
        }

        /// <summary>Gets the acuity.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        private IEnumerable<string> GetAcuity(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();
            var scale = responses.FirstOrDefault(r => r.QUESTION_CODE == VisualAcuityEventModel.QuestionCodes.Scale);

            if (scale != null)
            {
                if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.USScale)
                {
                    descriptionList.Add(GetUSAcuity(responses));
                }
                else if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.MetricScale)
                {
                    descriptionList.Add(GetMetricAcuity(responses));
                }
                else if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.JaegerScale)
                {
                    descriptionList.Add(GetTextQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.JaegerPerformDistance));
                    descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.JaegerAcuity));
                }
                else if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.DecimalScale)
                {
                    descriptionList.Add(GetTextQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.DecimalAcuity));
                }
                else if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.MUnitScale)
                {
                    descriptionList.Add(GetTextQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.MUnitAcuity));
                }
                else if (scale.QUESTION_CHOICE_CODE == VisualAcuityEventModel.ChoiceCodes.Scale.NonNumericScale)
                {
                    descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, VisualAcuityEventModel.QuestionCodes.NonNumericAcuity));
                }
            }

            return descriptionList;
        }

        /// <summary>Gets the us acuity.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        private string GetUSAcuity(IEnumerable<RESPONSE> responses)
        {
            string description = string.Empty;

            var performDistance = responses.FirstOrDefault(r => r.QUESTION_CODE == VisualAcuityEventModel.QuestionCodes.USPerformDistance);
            var acuity = responses.FirstOrDefault(r => r.QUESTION_CODE == VisualAcuityEventModel.QuestionCodes.USAcuity);

            if (performDistance != null && acuity != null)
            {
                description =
                    acuity.QUESTION.QUESTION_TEXT
                    + ": "
                    + performDistance.RESPONSE_TEXT
                    + " / "
                    + acuity.QUESTION_CHOICE.DISPLAY_TEXT;
            }

            return description;
        }

        /// <summary>Gets the metric acuity.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        private string GetMetricAcuity(IEnumerable<RESPONSE> responses)
        {
            string description = string.Empty;

            var performDistance = responses.FirstOrDefault(r => r.QUESTION_CODE == VisualAcuityEventModel.QuestionCodes.MetricPerformDistance);
            var acuity = responses.FirstOrDefault(r => r.QUESTION_CODE == VisualAcuityEventModel.QuestionCodes.MetricAcuity);

            if (performDistance != null && acuity != null)
            {
                description =
                    acuity.QUESTION.QUESTION_TEXT
                    + ": "
                    + performDistance.QUESTION_CHOICE.DISPLAY_TEXT
                    + " / "
                    + acuity.RESPONSE_TEXT;
            }

            return description;
        }
    }
}