﻿using System.Collections.Generic;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Visual Functioning Event View Manager
    /// </summary>
    public class VisualFunctioningEventViewManager : BaseEventViewManager<VisualFunctioningEventModel>
    {
        public override string CategoryCode
        {
            get { return "VisFuncQues"; }
        }

        public VisualFunctioningEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override VisualFunctioningEventModel GetDefaultViewModel(VisualFunctioningEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);
            viewModel.Questionnaire = MapToSingleSelectQuestion(categoryQuestionList, VisualFunctioningEventModel.QuestionCodes.Questionnaire);
            viewModel.Score = MapToTextQuestion(categoryQuestionList, VisualFunctioningEventModel.QuestionCodes.Score);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override VisualFunctioningEventModel GetLoadedViewModel(VisualFunctioningEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.Questionnaire = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualFunctioningEventModel.QuestionCodes.Questionnaire);
            viewModel.Score = MapToTextQuestion(categoryQuestionList, responseList, VisualFunctioningEventModel.QuestionCodes.Score);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(VisualFunctioningEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Questionnaire
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Questionnaire, referralId, encounterId, lateralityCode));

            // Score
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Score, referralId, encounterId, lateralityCode));

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, VisualFunctioningEventModel.QuestionCodes.Questionnaire));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, VisualFunctioningEventModel.QuestionCodes.Score));

            return descriptionList;
        }
    }
}