﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;

namespace Managers
{
    /// <summary>
    /// Summary description for MyTasksViewManager
    /// </summary>
    public class MyTasksViewManager : BaseViewManager
    {
        public MyTasksViewManager(UserAccountManager currentUser)
            : base(currentUser)
        {
        }

        public MyTasksViewModel GetMyTaskViewModel()
        {
            bool showDataAbstractorTasks = false;
            bool showReviewerTasks = false;
            bool showAdminTasks = false;

            IEnumerable<UserViewModel> assignableUsers = new UserViewModel[0];

            IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> dataAbstractorEncounters =
                new usp_GetEncountersByUserIdAndStatusId_Result[0];

            IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> reviewableEncounters =
                new usp_GetEncountersByUserIdAndStatusId_Result[0];

            if (CurrentUser.IsInRole((int) RoleCode.DVEIR_ADMINISTRATOR))
            {
                showAdminTasks = true;
                showReviewerTasks = true;
                showDataAbstractorTasks = true;
            }

            if (CurrentUser.IsInRole((int) RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
            {
                showReviewerTasks = true;
            }

            if (CurrentUser.IsInRole((int) RoleCode.DVEIR_DATA_ABSTRACTOR))
            {
                showDataAbstractorTasks = true;
            }

            assignableUsers = showAdminTasks ? GetUsersByRole("DVEIR DATA ABSTRACTOR") : assignableUsers;

            dataAbstractorEncounters = showDataAbstractorTasks
                ? DataAccessManager.GetEncountersByUserIdAndStatusId(CurrentUser.UserId, 0)
                : dataAbstractorEncounters;

            reviewableEncounters = showReviewerTasks
                ? DataAccessManager.GetEncountersByUserIdAndStatusId(0, (int) EncounterStatusType.Review)
                : reviewableEncounters;

            var statuses = DataAccessManager.GetEncounterStatuses();

            var workflowAdmins = new MyTasksViewModel.WorkflowAdminsModel()
            {
                AssignableUsers = assignableUsers,
                ReassignEncounterStatus = statuses.ToDictionary(k => k.ID, v => v.DESCRIPTION)
            };

            return new MyTasksViewModel()
            {
                ShowNoAccess = !(showAdminTasks || showDataAbstractorTasks || showReviewerTasks),
                ShowAdministratorTasks = showAdminTasks,
                ShowDataAbstractorTasks = showDataAbstractorTasks,
                ShowReviewerTasks = showReviewerTasks,
                EncounterStatusesFilters = GetEncounterStatuses(),
                DataAbstractorEncounters = dataAbstractorEncounters.Select(MapToEncounterSummary),
                ReviewableEncounters = reviewableEncounters.Select(MapToEncounterSummary),
                WorkflowAdmin = workflowAdmins
            };
        }

        public MyTasksViewModel RefreshDataAbstractorTasks(MyTasksViewModel viewModel)
        {
            IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> dataAbstractorEncounters =
                new usp_GetEncountersByUserIdAndStatusId_Result[0];

                dataAbstractorEncounters = viewModel.ShowDataAbstractorTasks
                    ? DataAccessManager.GetEncountersByUserIdAndStatusId(CurrentUser.UserId, 0)
                    : dataAbstractorEncounters;

            viewModel.DataAbstractorEncounters = dataAbstractorEncounters.Select(MapToEncounterSummary);

            return viewModel;
        }

        public MyTasksViewModel RefreshReviewerTasks(MyTasksViewModel viewModel)
        {
            IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> reviewableEncounters =
                new usp_GetEncountersByUserIdAndStatusId_Result[0];

            reviewableEncounters = viewModel.ShowReviewerTasks
                ? DataAccessManager.GetEncountersByUserIdAndStatusId(0, (int)EncounterStatusType.Review)
                : reviewableEncounters;

            viewModel.ReviewableEncounters = reviewableEncounters.Select(MapToEncounterSummary);

            return viewModel;
        }

        public MyTasksViewModel.WorkflowAdminsModel UpdateEncounterAssignment(
            MyTasksViewModel.WorkflowAdminsModel adminsModel, int encounterId, int? assignedUserId)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(encounterId);

            dbEncounter.AssignedTo = assignedUserId;
            dbEncounter.AssignedDate = DateTime.Now;

            DataAccessManager.SaveEncounter(dbEncounter);

            return adminsModel;
        }

        public MyTasksViewModel.WorkflowAdminsModel SaveEncounterStatus(
            MyTasksViewModel.WorkflowAdminsModel adminsModel, int encounterId, int statusTypeId)
        {
            var dbEncounter = DataAccessManager.GetEncounterById(encounterId);

            if (dbEncounter != null)
            {
                dbEncounter.UPDATED = DateTime.Now;
                dbEncounter.UPDATEDBY = CurrentUser.UserName;
                dbEncounter.EncounterStatusID = statusTypeId;

                DataAccessManager.SaveEncounter(dbEncounter);
            }

            // Clear model
            adminsModel.EncounterStatusChangeEncounterId = null;
            adminsModel.EncounterStatusChangeAssignedToId = null;
            adminsModel.EncounterStatusChangeStatusId = null;

            return adminsModel;
        }

        private MyTasksViewModel.EncounterSummary MapToEncounterSummary(usp_GetEncountersByUserIdAndStatusId_Result dbEncounter)
        {
            return new MyTasksViewModel.EncounterSummary()
            {
                ReferralId = dbEncounter.REFERRAL_ID,
                EncounterId = dbEncounter.ER_ENCOUNTER_ID,
                EncounterDate = dbEncounter.ENCOUNTER_DATE_TIME,
                PatientName = dbEncounter.PatientName,
                Facility = dbEncounter.STATIONNUMBER,
                LastUpdatedDate = dbEncounter.LAST_UPDATED_DATE,
                UpdatedBy = dbEncounter.UPDATED_BY,
                EncounterStatusId = dbEncounter.ENCOUNTER_STATUS_ID ?? 0,
                EncounterStatus = dbEncounter.ENCOUNTER_STATUS,
            };
        }

        public MyTasksViewModel.WorkflowAdminsModel GetDataAbstractorEncounters(
            MyTasksViewModel.WorkflowAdminsModel adminModel, 
            int userId)
        {
            var encounters = userId > 0 
                ? DataAccessManager.GetEncountersByUserIdAndStatusId(userId, 0)
                : new usp_GetEncountersByUserIdAndStatusId_Result[0 ];
            adminModel.DataAbstractorEncounters = encounters.Select(MapToEncounterSummary);
            return adminModel;
        }

        public MyTasksViewModel.WorkflowAdminsModel GetEncounterForStatusChange(
            MyTasksViewModel.WorkflowAdminsModel adminsModel, int encounterId)
        {
            var encounter = DataAccessManager.GetEncounterById(encounterId);

            if (encounter != null)
            {
                var user = DataAccessManager.GetAllUsers().FirstOrDefault(u => u.USER_ID == encounter.AssignedTo);
                adminsModel.EncounterStatusChangeEncounterId = encounter.ER_ENCOUNTER_ID;
                adminsModel.EncounterStatusChangeAssignedToId = user != null ? user.FULL_NAME : "Not Assigned";
                adminsModel.EncounterStatusChangeStatusId = encounter.EncounterStatusID;
            }
            else
            {
                adminsModel.EncounterStatusChangeEncounterId = null;
                adminsModel.EncounterStatusChangeAssignedToId = null;
                adminsModel.EncounterStatusChangeStatusId = null;
            }

            return adminsModel;
        }
    }
}