﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Referral;
using ViewModels.Encounter;

namespace Managers.Referral
{
    /// <summary>
    /// Summary description for ReferralViewManager
    /// </summary>
    public class ReferralSearchViewManager : BaseViewManager
    {
        public ReferralSearchViewManager(UserAccountManager currentUser)
            : base(currentUser)
        {
        }

        public ReferralSearchViewModel GetReferralSearchViewModel()
        {
            IEnumerable<usp_GetVISNs_Result> visns = DataAccessManager.GetAllVisns();

            return new ReferralSearchViewModel()
            {
                VISNs = visns.ToDictionary(k => k.ID, v => v.NAME),
                EncounterStatuses = GetEncounterStatuses()
            };
        }

        public ReferralSearchViewModel RefreshFacilities(ReferralSearchViewModel viewModel)
        {
            var facilities = DataAccessManager.GetFacilitiesByVisnId(viewModel.SelectedVisn);

            viewModel.Facilities = facilities.ToDictionary(k => k.ID, v => v.VISTANAME);

            return viewModel;
        }

        public ReferralSearchViewModel RefreshReferralSummaries(ReferralSearchViewModel viewModel)
        {
            int encounterStatusId = viewModel.SelectedEncounterStatus;
            int visnId = viewModel.SelectedVisn;
            int facilityId = viewModel.SelectedFacility;
            string patientNameSSN = viewModel.PatientNameSSN;

            var referralSummaries = DataAccessManager.GetReferralsByCriteria(encounterStatusId, visnId, facilityId, patientNameSSN);

            viewModel.ReferralSearchSummaries = referralSummaries.Select(
                r => new ReferralSearchSummary()
                {
                    ReferralId = r.REFERRAL_ID,
                    ReferralSTSId = r.STD_REFERRALSTS_ID,
                    ReferralDate = r.REFERRAL_DATE,
                    PatientSSN = r.PatientSSN,
                    PatientName = r.PatientName,
                    StationNumber = r.STATIONNUMBER,
                    VISNName = r.VISN_NAME,
                    TotalEncounterCount = r.TotalEncounterCount ?? 0
                });

            return viewModel;
        }
    }
}