﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Referral;
using ViewModels.Encounter;

namespace Managers.Referral
{
    /// <summary>
    /// Summary description for ReferralViewManager
    /// </summary>
    public class ReferralViewManager : BaseViewManager
    {
        public ReferralViewManager(UserAccountManager currentUser)
            : base(currentUser)
        {
        }

        public ReferralDetailViewModel GetReferralDetailViewModel(int referralId)
        {
            IEnumerable<usp_GetEncountersByReferralId_Result> encounterResults =
                DataAccessManager.GetEncountersByReferralId(referralId);

            IEnumerable<PROBLEM_LIST> problems = DataAccessManager.GetProblemListByReferralId(referralId);

            string patientName = GetPatientName(referralId);

            IEnumerable<UserViewModel> users = GetUsersByRole("DVEIR DATA ABSTRACTOR");

            return new ReferralDetailViewModel()
            {
                ReferralId = referralId,
                PatientName = patientName,
                IsAdminUser = CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR),
                AssignableUsers = users,
                Encounters = encounterResults.Select(MapToEncounterSummary),
                ActiveProblems = problems.Where(p => p.ActiveFlag == 1).Select(MapToProblemSummary),
                InactiveProblems = problems.Where(p => p.ActiveFlag == 0).Select(MapToProblemSummary)
            };
        }

        public void UpdateEncounterAssignment(int referralId, int encounterId, int? assignedUserId)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);

            dbEncounter.AssignedTo = assignedUserId;
            dbEncounter.AssignedDate = DateTime.Now;

            DataAccessManager.SaveEncounter(dbEncounter);
        }

        public ReferralDetailViewModel RefreshEncounterSummaries(ReferralDetailViewModel viewModel)
        {
            IEnumerable<usp_GetEncountersByReferralId_Result> encounterResults =
                DataAccessManager.GetEncountersByReferralId(viewModel.ReferralId);

            viewModel.Encounters = encounterResults.Select(MapToEncounterSummary).ToArray();

            return viewModel;
        }

        public int AddNewEncounter(int referralId, int institutionId, DateTime visitDate, string encounterType)
        {
            int patientSid = DataAccessManager.GetPatientSIDByReferralId(referralId);
            usp_GetInstitutionNameAndSta3nByID_Result facility = DataAccessManager.GetFacilityNameAndSta3nById(institutionId);

            var encounter = new ER_ENCOUNTER_MOD()
            {
                REFERRAL_ID = referralId,
                PatientSID = patientSid,
                EncounterStatusID = (int)EncounterStatusType.New,
                EncounterTypeInOut = encounterType,
                InstitutionID = institutionId,
                STA6A = facility.STATIONNUMBER,
                UniqueVHAEncounterIDPrimaryKey = Guid.NewGuid().ToString(),
                VISIT_DATE = visitDate,
                AssignedTo = CurrentUser.UserId,
                AssignedDate = DateTime.Now
            };

            DataAccessManager.SaveEncounter(encounter);

            return encounter.ER_ENCOUNTER_ID;
        }

        private ReferralDetailViewModel.ProblemSummary MapToProblemSummary(PROBLEM_LIST dbProblem)
        {
            return new ReferralDetailViewModel.ProblemSummary()
            {
                Description = dbProblem.PROBLEM_DESCRIPTION,
                OnsetDate = dbProblem.OnsetDate,
                LastUpdatedDate = dbProblem.LastModifiedDate,
                FacilityName = dbProblem.FACILITY_NAME
            };
        }

        private ReferralDetailViewModel.EncounterSummary MapToEncounterSummary(usp_GetEncountersByReferralId_Result dbEncounter)
        {
            return new ReferralDetailViewModel.EncounterSummary()
            {
                Id = dbEncounter.ER_ENCOUNTER_ID,
                EncounterDate = dbEncounter.ENCOUNTER_DATE_TIME,
                StopCode = dbEncounter.STOP_CODE,
                StopDescription = dbEncounter.STOP_CODE_DESC,
                LastUpdatedDate = dbEncounter.LAST_UPDATED_DATE,
                UpdatedBy = dbEncounter.UPDATED_BY,
                StatusId = dbEncounter.ENCOUNTER_STATUS_ID,
                Status = dbEncounter.ENCOUNTER_STATUS,
                AssignedToUserId = dbEncounter.ASSIGNED_TO_ID,
                AssignedToUserName = dbEncounter.ASSIGNED_TO_NAME
            };
        }
    }
}