﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels.Encounter
{
    [Serializable]
    public class ExamViewModel
    {
        public int ReferralId { get; set; }

        public int EncounterId { get; set; }

        public string PatientName { get; set; }

        public EncounterStatusType EncounterStatusType { get; set; }

        public bool IsReadOnly { get; set; }

        public bool IsCurrentlyInitialize { get; set; }

        public bool HasSelectedCategory
        {
            get { return SelectedCategory != null; }
        }

        public ExamEventCategory SelectedCategory
        {
            get { return EventCategories.FirstOrDefault(c => c.Code == SelectedCategoryCode); ; }
        }

        public string SelectedCategoryCode { get; set; }

        public string SelectedLateralityCode { get; set; }

        public long SelectedGroupingId { get; set; }

        public IEnumerable<ExamViewEventSummary> EventSummaries { get; set; }

        public List<ExamViewEventSummary> EventSummariesOrdered
        {
            get { return EventSummaries.OrderBy(s => s.CategorySortOrder).ThenBy(s => s.LateralitySortOrder).ToList(); }
        } 

        public IEnumerable<ExamEventCategory> EventCategories { get; set; }

        public IEnumerable<ExamEventCategory> EventCategoriesOrdered
        {
            get { return EventCategories.OrderBy(c => c.SortOrder); }
        }

        public bool AllowReviewerComments { get; set; }

        public ReviewerCommentViewModel ReviewerCommentView { get; set; }
    }

    [Serializable]
    public class ExamViewEventSummary
    {
        private IEnumerable<string> descriptionList; 

        public string Name { get; set; }
        public string CategoryCode { get; set; }
        public string LateralityCode { get; set; }
        public string Laterality { get; set; }
        public long GroupingId { get; set; }

        /// <summary>Gets the first 2 lines of the description.</summary>
        /// <value>The short description.</value>
        public string ShortDescription
        {
            get
            {
                string displayDescription = string.Empty;
                int count = 0;

                foreach (var description in descriptionList)
                {
                    count++;
                    displayDescription += description;
                    if (count < 2 || count != descriptionList.Count())
                    {
                        displayDescription += "<br />";
                    }
                    else
                    {
                        break;
                    }
                }

                return displayDescription;
            }
        }

        /// <summary>Gets the full description.</summary>
        /// <value>The full description.</value>
        public string FullDescription
        {
            get
            {
                string displayDescription = string.Empty;
                int count = 0;

                foreach (var description in descriptionList)
                {
                    count++;
                    displayDescription += description;
                    if (count != descriptionList.Count())
                    {
                        displayDescription += "<br />";
                    }
                    else
                    {
                        break;
                    }
                }

                return displayDescription;
            }
        }

        public IEnumerable<string> DescriptionList
        {
            set { descriptionList = value.ToList(); }
        }
        public string ReviewComment { get; set; }
        public bool IsResolved { get; set; }
        public bool IsApproved { get; set; }
        public DateTime CreatedDate { get; set; }
        public string CreatedBy { get; set; }
        public int CategorySortOrder { get; set; }
        public int LateralitySortOrder { get; set; }
        public bool IsReadOnly { get; set; }
        public bool AllowDelete { get; set; }
    }

    [Serializable]
    public class ExamEventCategory
    {
        public string Code { get; set; }
        public string Name { get; set; }
        public int SortOrder { get; set; }
    }
}