﻿using System;
using System.Linq;
using System.Collections.Generic;

namespace ViewModels.Encounter
{
    /// <summary>
    /// Summary description for Provider View Model
    /// </summary>
    [Serializable]
    public class ProviderViewModel
    {
        public int ReferralId { get; set; }

        public int EncounterId { get; set; }

        public string PatientName { get; set; }

        private IEnumerable<ProviderSummary> _providerSummaries;

        private Dictionary<string, string> _facilities; 

        private Dictionary<int, string> _providers; 

        public string EncounterVisitId { get; set; }

        public string CurrentFacility { get; set; }

        public bool IsReadOnly { get; set; }

        public bool HasPrimary { get; set; }

        public Dictionary<string, string> Facilities
        {
            get
            {
                return _facilities != null
                    ? _facilities.OrderBy(f => f.Value).ToDictionary(f => f.Key, f => f.Value)
                    : new Dictionary<string, string>();
            }
            set
            {
                _facilities = value;
            }
        }

        public Dictionary<int, string> Providers
        {
             get
            {
                return _providers != null
                    ? _providers.OrderBy(f => f.Value).ToDictionary(f => f.Key, f => f.Value)
                    : new Dictionary<int, string>();
            }
            set
            {
                _providers = value;
            }
        }

        public IEnumerable<ProviderSummary> ProviderSummaries
        {
            get
            {
                return _providerSummaries != null
                    ? _providerSummaries.OrderByDescending(p => p.IsPrimary).ThenBy(p => p.Name).ToArray()
                    : new ProviderSummary[0];
            }

            set { _providerSummaries = value; }
        }
    }

    [Serializable]
    public class ProviderSummary
    {
        public int Id { get; set; }

        public bool IsPrimary { get; set; }

        public string Name { get; set; }

        public int Sta3n { get; set; }

        public string InstitutionName { get; set; }

        public string Address1 { get; set; }

        public string Address2 { get; set; }

        public string City { get; set; }

        public string State { get; set; }

        public string Zip { get; set; }

        public string BusinessPhone { get; set; }

        public string Fax { get; set; }
    }
}
