﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels.Exam
{
    /// <summary>
    /// Summary description for Kinetic Visual Fields Event
    /// </summary>
    [Serializable]
    public class KineticVisualFieldsEventModel : BaseEventModel
    {
        public MultiSelectQuestion Methodology { get; set; }

        public Dictionary<string, TextQuestion>  Results { get; set; }

        public List<MethodologyView> MethodologyViews
        {
            get
            {
                return Methodology.SelectedChoicesOrderd.Select(
                    c =>
                        new MethodologyView()
                        {
                            ChoiceCode = c.Code,
                            RequiresComment = c.RequiresComment,
                            DisplayText = c.DisplayText,
                            Comment = c.Comment,
                            Result = Results.ContainsKey(c.Code) ? Results[c.Code].ResponseText : string.Empty
                        })
                    .ToList();
            }
        } 

        public class MethodologyView
        {
            public string ChoiceCode { get; set; }
            public string DisplayText { get; set; }
            public bool RequiresComment { get; set; }
            public string Comment { get; set; }
            public string Result { get; set; }
        }

        public static IEnumerable<string> ShowTestDetailValues
        {
            get
            {
                return new[]
                {
                    ChoiceCodes.TestPerformed.Yes
                };
            }
        }

        public static class QuestionCodes
        {
            public static string TestPerformed = "KVF_TP";

            public static string TestResult = "KVF_TR";

            public static string EventComment = "KVF_C";

            public static string Methodology = "KVF_M";

            public static string ResultSuffix = "_R";
        }

        public static class ChoiceCodes
        {
            public static class TestPerformed
            {
                public static string Yes = "KVF_TP_Y";

                public static string NotDocumented = "KVF_TP_ND";
            }

            public static class TestResult
            {
                public static string Normal = "KVF_TR_N";

                public static string Abnormal = "KVF_TR_A";
            }
        }
    }
}