﻿using System;
using System.Collections.Generic;
using System.Linq;
using ViewModels.Encounter;

namespace ViewModels
{
    /// <summary>
    /// 
    /// </summary>
    [Serializable]
    public class MyTasksViewModel
    {
        /// <summary>Initializes a new instance of the <see cref="MyTasksViewModel"/> class.</summary>
        public MyTasksViewModel()
        {
            ShowNoAccess = true;
            ShowDataAbstractorTasks = false;
            ShowReviewerTasks = false;
            ShowAdministratorTasks = false;
        }
        
        private IEnumerable<EncounterSummary> _dataAbstractorEncounters;

        private IEnumerable<EncounterSummary> _reviewableEncounters;

        public bool ShowNoAccess { get; set; }

        public bool ShowDataAbstractorTasks { get; set; }

        public bool ShowReviewerTasks { get; set; }

        public bool ShowAdministratorTasks { get; set; }

        public Dictionary<int, string> EncounterStatusesFilters { get; set; }

        public int SelectedEncounterStatus { get; set; }

        public IEnumerable<EncounterSummary> DataAbstractorEncounters
        {
            get
            {
                return _dataAbstractorEncounters != null
                    ? _dataAbstractorEncounters.Where(e => SelectedEncounterStatus == 0 || e.EncounterStatusId == SelectedEncounterStatus)
                    : new EncounterSummary[0];
            }
            set { _dataAbstractorEncounters = value.ToArray(); }
        }

        public IEnumerable<EncounterSummary> ReviewableEncounters
        {
            get { return _reviewableEncounters ?? new EncounterSummary[0]; }
            set { _reviewableEncounters = value.ToArray(); }
        }

        public WorkflowAdminsModel WorkflowAdmin { get; set; }

        [Serializable]
        public class WorkflowAdminsModel
        {
            private IEnumerable<UserViewModel> _assignableUsers;

            private IEnumerable<EncounterSummary> _dataAbstractorEncounters;

            /// <summary>Gets or sets the assignable users.</summary>
            /// <value>The assignable users.</value>
            public IEnumerable<UserViewModel> AssignableUsers
            {
                get { return _assignableUsers ?? new UserViewModel[0]; }
                set { _assignableUsers = value.ToArray(); }
            }

            public Dictionary<int, string> ReassignEncounterStatus { get; set; }

            public IEnumerable<EncounterSummary> DataAbstractorEncounters
            {
                get { return _dataAbstractorEncounters ?? new EncounterSummary[0]; }
                set { _dataAbstractorEncounters = value.ToArray(); }
            }

            public string EncounterStatusChangeAssignedToId { get; set; }

            public int? EncounterStatusChangeStatusId { get; set; }

            public int? EncounterStatusChangeEncounterId { get; set; }
        }

        [Serializable]
        public class EncounterSummary
        {
            public int ReferralId { get; set; }

            public int EncounterId { get; set; }

            public DateTime EncounterDate { get; set; }

            public string PatientName { get; set; }

            public string Facility { get; set; }

            public DateTime LastUpdatedDate { get; set; }

            public string UpdatedBy { get; set; }

            public string EncounterStatus { get; set; }

            public int EncounterStatusId { get; set; }
        }
    }
}