﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels.Referral
{
    /// <summary>
    /// 
    /// </summary>
    [Serializable]
    public class ReferralDetailViewModel
    {
        private IEnumerable<UserViewModel> _assignableUsers;

        private IEnumerable<EncounterSummary> _encounters;

        private IEnumerable<ProblemSummary> _activeProblems;

        private IEnumerable<ProblemSummary> _inactiveProblems; 

        /// <summary>Initializes a new instance of the <see cref="ReferralDetailViewModel"/> class.</summary>
        public ReferralDetailViewModel()
        {
            ReferralId = 0;
            IsAdminUser = false;
        }

        public int ReferralId { get; set; }

        public string PatientName { get; set; }

        public bool IsAdminUser { get; set; }

        /// <summary>Gets or sets the assignable users.</summary>
        /// <value>The assignable users.</value>
        public IEnumerable<UserViewModel> AssignableUsers
        {
            get { return _assignableUsers ?? new UserViewModel[0]; }
            set { _assignableUsers = value.ToArray(); }
        }

        /// <summary>Gets or sets the encounters.</summary>
        /// <value>The encounters.</value>
        public IEnumerable<EncounterSummary> Encounters
        {
            get { return _encounters ?? new EncounterSummary[0]; }
            set { _encounters = value.ToArray(); }
        }

        /// <summary>Gets or sets the active problems.</summary>
        /// <value>The active problems.</value>
        public IEnumerable<ProblemSummary> ActiveProblems
        {
            get { return _activeProblems ?? new ProblemSummary[0]; }
            set { _activeProblems = value.ToArray(); }
        }

        /// <summary>Gets or sets the inactive problems.</summary>
        /// <value>The inactive problems.</value>
        public IEnumerable<ProblemSummary> InactiveProblems
        {
            get { return _inactiveProblems ?? new ProblemSummary[0]; }
            set { _inactiveProblems = value.ToArray(); }
        }

        /// <summary>
        /// 
        /// </summary>
        [Serializable]
        public class EncounterSummary
        {
            /// <summary>Gets or sets the identifier.</summary>
            /// <value>The identifier.</value>
            public int Id { get; set; }

            /// <summary>Gets or sets the encounter date.</summary>
            /// <value>The encounter date.</value>
            public DateTime EncounterDate { get; set; }

            /// <summary>Gets or sets the stop code.</summary>
            /// <value>The stop code.</value>
            public string StopCode { get; set; }

            /// <summary>Gets or sets the stop description.</summary>
            /// <value>The stop description.</value>
            public string StopDescription { get; set; }

            /// <summary>Gets or sets the last updated date.</summary>
            /// <value>The last updated date.</value>
            public DateTime LastUpdatedDate { get; set; }

            /// <summary>Gets or sets the updated by.</summary>
            /// <value>The updated by.</value>
            public string UpdatedBy { get; set; }

            /// <summary>Gets or sets the status identifier.</summary>
            /// <value>The status identifier.</value>
            public int? StatusId { get; set; }

            /// <summary>Gets or sets the status.</summary>
            /// <value>The status.</value>
            public string Status { get; set; }

            /// <summary>Gets or sets the assigned to user identifier.</summary>
            /// <value>The assigned to user identifier.</value>
            public int? AssignedToUserId { get; set; }

            /// <summary>Gets or sets the name of the assigned to user.</summary>
            /// <value>The name of the assigned to user.</value>
            public string AssignedToUserName { get; set; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Serializable]
        public class ProblemSummary
        {
            /// <summary>Gets or sets the description.</summary>
            /// <value>The description.</value>
            public string Description { get; set; }

            /// <summary>Gets or sets the onset date.</summary>
            /// <value>The onset date.</value>
            public DateTime? OnsetDate { get; set; }

            /// <summary>Gets or sets the last updated date.</summary>
            /// <value>The last updated date.</value>
            public DateTime? LastUpdatedDate { get; set; }

            /// <summary>Gets or sets the name of the facility.</summary>
            /// <value>The name of the facility.</value>
            public string FacilityName { get; set; }
        }
    }
}