﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels.Reporting
{
    /// <summary>
    /// 
    /// </summary>
    [Serializable]
    public class DoDInterfaceTransferReportViewModel
    {
        /// <summary>The _dod interface transfer</summary>
        private IEnumerable<DoDInterfaceTransfer> _dodInterfaceTransfer;

        /// <summary>Gets or sets the date from.</summary>
        /// <value>The date from.</value>
        public DateTime DateFrom { get; set; }

        /// <summary>Gets or sets the date to.</summary>
        /// <value>The date to.</value>
        public DateTime DateTo { get; set; }

        /// <summary>Gets or sets the do d interface transfers.</summary>
        /// <value>The do d interface transfers.</value>
        public IEnumerable<DoDInterfaceTransfer> DoDInterfaceTransfers
        {
            get { return _dodInterfaceTransfer ?? new DoDInterfaceTransfer[0]; }
            set { _dodInterfaceTransfer = value.ToArray(); }
        }

        public IEnumerable<DodInterfaceTransferAggregates> Aggregates
        {
            get
            {
                return new[]
                {
                    new DodInterfaceTransferAggregates()
                    {
                        TotalPatients = DoDInterfaceTransfers.Select(d => d.PatientId).Distinct().Count(),
                        TotalEncounters = DoDInterfaceTransfers.Select(d => d.EncounterId).Distinct().Count()
                    }
                };
            }
        }   
    }

    /// <summary>
    /// 
    /// </summary>
    [Serializable]
    public class DoDInterfaceTransfer
    {
        /// <summary>Gets or sets the patient identifier.</summary>
        /// <value>The patient identifier.</value>
        public int PatientId { get; set; }

        /// <summary>Gets or sets the first name.</summary>
        /// <value>The first name.</value>
        public string FirstName { get; set; }

        /// <summary>Gets or sets the last name.</summary>
        /// <value>The last name.</value>
        public string LastName { get; set; }

        /// <summary>Gets or sets the gender.</summary>
        /// <value>The gender.</value>
        public string Gender { get; set; }

        /// <summary>Gets or sets the referral identifier.</summary>
        /// <value>The referral identifier.</value>
        public int ReferralId { get; set; }

        /// <summary>Gets or sets the encounter identifier.</summary>
        /// <value>The encounter identifier.</value>
        public int EncounterId { get; set; }

        /// <summary>Gets or sets the encounter date.</summary>
        /// <value>The encounter date.</value>
        public DateTime EncounterDate { get; set; }

        /// <summary>Gets or sets the transfer date.</summary>
        /// <value>The transfer date.</value>
        public DateTime TransferDate { get; set; }
    }

    [Serializable]
    public class DodInterfaceTransferAggregates
    {
        public int TotalPatients { get; set; }

        public int TotalEncounters { get; set; }
    }
}