﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using Managers.Encounter;
using ViewModels.Encounter;

namespace Controls.Encounter
{
    public partial class ChiefComplaintControl : BaseWebControl
    {
        public ChiefComplaintViewModel ChiefComplaintViewModel
        {
            get { return ViewState["ChiefComplaintViewModel"] as ChiefComplaintViewModel; }
            set { ViewState["ChiefComplaintViewModel"] = value; }
        }

        public IEnumerable<string> ReviewerCommentAjaxControls
        {
            get { return ViewState["ReviewerCommentAjaxControls"] as IEnumerable<string> ?? new string[0]; }
            set { ViewState["ReviewerCommentAjaxControls"] = value.ToList(); }
        }

        public event EventHandler ReviewerCommentSaved;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack && ChiefComplaintViewModel != null)
            {
                //Populate Complaint
                txtChiefComplaint.Text = ChiefComplaintViewModel.ChiefComplaintText;

                //Populate Checkboxes
                TBI_READ_PRBLM_IND.Checked = ChiefComplaintViewModel.HasReadingProblem;
                TBI_DIPLOPIA_IND.Checked = ChiefComplaintViewModel.HasDiplopia;
                TBI_DAZZLING_IND.Checked = ChiefComplaintViewModel.HasDazzlingGlare;
                TBI_PHOTOPHOBIA_IND.Checked = ChiefComplaintViewModel.HasPhotophobia;
                TBI_EYE_STRAIN_IND.Checked = ChiefComplaintViewModel.HasEyeStrain;
                TBI_BLURRED_VISION_IND.Checked = ChiefComplaintViewModel.HasBlurredVision;
                OTHER_NEUROLOGICAL_IND.Checked = ChiefComplaintViewModel.HasOtherCondition;

                //Populate Comments
                txtComments.Text = ChiefComplaintViewModel.Comments;

                // Set Review Comments
                rccChiefComplaint.ReviewerCommentViewModel = ChiefComplaintViewModel.ChiefComplaintTextReviewer;
                rccVisualConditions.ReviewerCommentViewModel = ChiefComplaintViewModel.CommentReviewer;

                rccChiefComplaint.Visible = ChiefComplaintViewModel.AllowReviewerComments;
                rccVisualConditions.Visible = ChiefComplaintViewModel.AllowReviewerComments;

                rccChiefComplaint.AjaxUpdateControls = ReviewerCommentAjaxControls;
                rccVisualConditions.AjaxUpdateControls = ReviewerCommentAjaxControls;

                // Set Readonly
                if (ChiefComplaintViewModel.IsReadOnly)
                {
                    txtChiefComplaint.ReadOnly = true;
                    txtComments.ReadOnly = true;

                    TBI_READ_PRBLM_IND.ReadOnly = true;
                    TBI_DIPLOPIA_IND.ReadOnly = true;
                    TBI_DAZZLING_IND.ReadOnly = true;
                    TBI_PHOTOPHOBIA_IND.ReadOnly = true;
                    TBI_EYE_STRAIN_IND.ReadOnly = true;
                    TBI_BLURRED_VISION_IND.ReadOnly = true;
                    OTHER_NEUROLOGICAL_IND.ReadOnly = true;

                    TBI_READ_PRBLM_IND.Enabled = TBI_READ_PRBLM_IND.Checked;
                    TBI_DIPLOPIA_IND.Enabled = TBI_DIPLOPIA_IND.Checked;
                    TBI_DAZZLING_IND.Enabled = TBI_DAZZLING_IND.Checked;
                    TBI_PHOTOPHOBIA_IND.Enabled = TBI_PHOTOPHOBIA_IND.Checked;
                    TBI_EYE_STRAIN_IND.Enabled = TBI_EYE_STRAIN_IND.Checked;
                    TBI_BLURRED_VISION_IND.Enabled = TBI_BLURRED_VISION_IND.Checked;
                    OTHER_NEUROLOGICAL_IND.Enabled = OTHER_NEUROLOGICAL_IND.Checked;
                }
            }
        }

        public void SaveChiefComplaint()
        {
            if (ChiefComplaintViewModel != null)
            {
                //Set Complaint Text
                ChiefComplaintViewModel.ChiefComplaintText = txtChiefComplaint.Text;

                //Set new Encounter Selections
                ChiefComplaintViewModel.HasReadingProblem = TBI_READ_PRBLM_IND.Checked;
                ChiefComplaintViewModel.HasDiplopia = TBI_DIPLOPIA_IND.Checked;
                ChiefComplaintViewModel.HasDazzlingGlare = TBI_DAZZLING_IND.Checked;
                ChiefComplaintViewModel.HasPhotophobia = TBI_PHOTOPHOBIA_IND.Checked;
                ChiefComplaintViewModel.HasEyeStrain = TBI_EYE_STRAIN_IND.Checked;
                ChiefComplaintViewModel.HasBlurredVision = TBI_BLURRED_VISION_IND.Checked;
                ChiefComplaintViewModel.HasOtherCondition = OTHER_NEUROLOGICAL_IND.Checked;

                //Set Comment Text
                ChiefComplaintViewModel.Comments = txtComments.Text;

                // Save the View
                var viewManager = new ChiefComplaintViewManager(CurrentUser);
                viewManager.SaveChiefComplaintView(ChiefComplaintViewModel);
            }
        }

        public void ShowReviewerComment(string questionCode)
        {
            if (ChiefComplaintViewModel.AllowReviewerComments &&
                questionCode == ChiefComplaintViewModel.ChiefComplaintTextReviewer.QuestionCode)
            {
                rccChiefComplaint.ReviewerCommentViewModel.ShowCommentText = true;
                rccChiefComplaint.ReviewerCommentViewModel.SetFocus = true;
                rccChiefComplaint.RefreshControl();
            }
            else if (ChiefComplaintViewModel.AllowReviewerComments &&
                     questionCode == ChiefComplaintViewModel.CommentReviewer.QuestionCode)
            {
                rccVisualConditions.ReviewerCommentViewModel.ShowCommentText = true;
                rccVisualConditions.ReviewerCommentViewModel.SetFocus = true;
                rccVisualConditions.RefreshControl();
            }
        }

        protected void rccChiefComplaint_OnReviewSavedSuccessfully(object sender, EventArgs e)
        {
            if (ReviewerCommentSaved != null)
            {
                ReviewerCommentSaved(sender, e);
            }
        }

        protected void rccVisualConditions_OnReviewSavedSuccessfully(object sender, EventArgs e)
        {
            if (ReviewerCommentSaved != null)
            {
                ReviewerCommentSaved(sender, e);
            }
        }
    }
}