﻿using BusinessManagerClasses;
using Factories;
using Managers.Encounter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using Telerik.Web.UI;
using ViewModels.Encounter;
using ViewModels.Exam;

namespace Controls.Encounter
{
    public partial class ExamControl : BaseWebControl
    {
        #region Public Properties

        public ExamViewModel ExamViewModel
        {
            get { return ViewState["ExamViewModel"] as ExamViewModel; }
            set { ViewState["ExamViewModel"] = value; }
        }

        public IEnumerable<string> ReviewerCommentAjaxControls
        {
            get { return ViewState["ReviewerCommentAjaxControls"] as IEnumerable<string> ?? new string[0]; }
            set { ViewState["ReviewerCommentAjaxControls"] = value.ToList(); }
        }

        public event EventHandler ReviewerCommentSaved;

        public event EventHandler EventSavedSuccessfully;

        #endregion

        #region Public Methods

        public bool Save()
        {
            return SaveCurrentEvent();
        }

        public void LoadEvent(string categoryCode, string lateralityCode, long groupingId, bool setReviewFocus)
        {
            // Set the current exam event category
            SetExamEventCategory(categoryCode, lateralityCode, groupingId);

            // Load the event
            LoadEventCategoryUserControl(true, false);

            // Load the reviewer comment
            LoadReviewCommentControl(setReviewFocus);

            // Update and set focus on the ddl
            ddlEventTypes.SelectedValue = categoryCode;
            ddlEventTypes.Focus();

            // Set the save buttons
            btnSaveEvent.Visible = false;
            btnUpdateEvent.Visible = !ExamViewModel.IsReadOnly;
            btnCancelUpdate.Visible = true;
        }

        #endregion

        #region Page Events

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                //LoadViewModel();
                DataBindEventSummaries();
                DataBindEventCategories();

                // Set 
                rccExamEvent.ReviewerCommentViewModel = ExamViewModel.ReviewerCommentView;
                rccExamEvent.Visible = ExamViewModel.AllowReviewerComments;

                // Set the reviewer comment ajax
                var rcAjax = ReviewerCommentAjaxControls.ToList();
                rcAjax.Add(gvEventList.ID);
                rccExamEvent.AjaxUpdateControls = rcAjax;
            }

            // Dynamically add reviewer comment ajax for update button
            foreach (var ctrlId in ReviewerCommentAjaxControls)
            {
                Control ctrl = FindParentControl(this, ctrlId);

                if (ctrl != null)
                {
                    ajaxProxyExam.AjaxSettings.AddAjaxSetting(btnUpdateEvent, ctrl);
                }
            }

            LoadEventCategoryUserControl(!ExamViewModel.IsCurrentlyInitialize, true);
        }

        protected void ddlEventTypes_OnSelectedIndexChanged(object sender, EventArgs e)
        {
            // Set the new event code
            SetExamEventCategory(ddlEventTypes.SelectedValue);
            LoadEventCategoryUserControl(true, true);
            LoadReviewCommentControl(false);

            // Set focus on the ddl
            ddlEventTypes.Focus();

            // Set Save Buttons
            btnSaveEvent.Visible = !ExamViewModel.IsReadOnly;
            btnUpdateEvent.Visible = false;
            btnCancelUpdate.Visible = false;
        }

        protected void btnSaveEvent_OnClick(object sender, EventArgs e)
        {
            SaveCurrentEvent();
        }

        protected void btnCancelUpdate_OnClick(object sender, EventArgs e)
        {
            // Simply Clear The Event Category
            ClearEventCategory();
        }

        protected void gvEventList_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "EditEvent" || e.CommandName == "ViewEvent")
            {
                string categoryCode = ((GridDataItem)e.Item).GetDataKeyValue("CategoryCode").ToString();
                string lateralityCode = ((GridDataItem)e.Item).GetDataKeyValue("LateralityCode").ToString();
                long groupingId = (long)((GridDataItem)e.Item).GetDataKeyValue("GroupingId");

                LoadEvent(categoryCode, lateralityCode, groupingId, false);
            }

            if (e.CommandName == "DeleteEvent")
            {
                string categoryCode = ((GridDataItem)e.Item).GetDataKeyValue("CategoryCode").ToString();
                string lateralityCode = ((GridDataItem)e.Item).GetDataKeyValue("LateralityCode").ToString();
                long groupingId = (long)((GridDataItem)e.Item).GetDataKeyValue("GroupingId");
                ExamEventFactoryResult factoryResult = ExamEventFactory.GetExamEventFactoryResult(CurrentUser.UserName,
                    categoryCode);

                // Get current factory and delete selected Event
                if (factoryResult != null)
                {
                    factoryResult.EventViewManager.Delete(ExamViewModel.ReferralId, ExamViewModel.EncounterId,
                        lateralityCode, categoryCode, groupingId);
                }

                // Refresh Summary Grid
                RefreshEventSummaries();
            }
        }

        protected void gvEventList_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            gvEventList.DataSource = ExamViewModel.EventSummariesOrdered;
        }

        protected void rccExamEvent_OnReviewSavedSuccessfully(object sender, EventArgs e)
        {
            RefreshEventSummaries();

            if (ReviewerCommentSaved != null)
            {
                ReviewerCommentSaved(sender, e);
            }
        }

        #endregion

        #region Private Methods

        private void DataBindEventCategories()
        {
            ddlEventTypes.Enabled = !ExamViewModel.IsReadOnly;
            ddlEventTypes.DataSource = ExamViewModel.EventCategoriesOrdered;
            ddlEventTypes.DataValueField = "Code";
            ddlEventTypes.DataTextField = "Name";
            ddlEventTypes.DataBind();
        }

        private void DataBindEventSummaries()
        {
            gvEventList.DataSource = ExamViewModel.EventSummariesOrdered;
            gvEventList.DataBind();
        }

        private void RefreshEventSummaries()
        {
            ExamViewManager viewManager = new ExamViewManager(CurrentUser);
            ExamViewModel = viewManager.RefreshEventSummaries(ExamViewModel);
            DataBindEventSummaries();
        }

        private void LoadReviewCommentControl(bool setReviewFocus)
        {
            var viewManager = new ExamViewManager(CurrentUser);
            ExamViewModel = viewManager.RefreshReviewerComment(ExamViewModel);
            ExamViewModel.ReviewerCommentView.ShowCommentText = setReviewFocus;
            ExamViewModel.ReviewerCommentView.SetFocus = setReviewFocus;

            rccExamEvent.ReviewerCommentViewModel = ExamViewModel.ReviewerCommentView;
            rccExamEvent.Visible = ExamViewModel.AllowReviewerComments;
            rccExamEvent.RefreshControl();
        }

        private void LoadEventCategoryUserControl(bool initializeControl, bool loadDefaultView)
        {
            string categoryCode = ExamViewModel.SelectedCategoryCode;
            string lateralityCode = ExamViewModel.SelectedLateralityCode;
            long groupingId = ExamViewModel.SelectedGroupingId;

            // Clear the current user control
            phEventCategory.Controls.Clear();
            pnlEyeExamEventCategory.GroupingText = string.Empty;

            // Find the user control and event manager for the question category
            ExamEventFactoryResult factoryResult = 
                ExamEventFactory.GetExamEventFactoryResult(CurrentUser.UserName, categoryCode);

            if (factoryResult != null)
            {
                var uc = (UserControl)LoadControl(factoryResult.EventControlLocation);
                uc.ID = factoryResult.EventControlId;


                if (initializeControl)
                {
                    BaseEventModel eventViewModel =
                        loadDefaultView
                            ? factoryResult.EventViewManager.GetDefaultViewModel(ExamViewModel.IsReadOnly)
                            : factoryResult.EventViewManager.GetLoadedViewModel(ExamViewModel.ReferralId,
                                ExamViewModel.EncounterId, lateralityCode, groupingId, ExamViewModel.IsReadOnly);

                    // Set the view model and validation group
                    var baseEventItem = uc as IBaseEventItem;
                    if (baseEventItem != null)
                    {
                        baseEventItem.SetViewModel(eventViewModel);
                        baseEventItem.SetValidationGroup(loadDefaultView
                            ? btnSaveEvent.ValidationGroup
                            : btnUpdateEvent.ValidationGroup);
                    }

                    ExamViewModel.IsCurrentlyInitialize = true;

                    // Get Section Name
                    pnlEyeExamEventCategory.GroupingText = ExamViewModel.SelectedCategory.Name;
                }

                // Add User Control
                phEventCategory.Controls.Add(uc);
            }
            else
            {
                ExamViewModel.IsCurrentlyInitialize = false;

                // Hide any save/update/cancel buttons
                btnSaveEvent.Visible = false;
                btnUpdateEvent.Visible = false;
                btnCancelUpdate.Visible = false;
            }
        }

        private bool SaveCurrentEvent()
        {
            bool saveSuccessful = false;

            // To add server validation
            if (!ExamViewModel.IsReadOnly)
            {
                string eventCode = ExamViewModel.SelectedCategoryCode;

                // Get current event manager, 
                ExamEventFactoryResult factoryResult = 
                    ExamEventFactory.GetExamEventFactoryResult(CurrentUser.UserName, eventCode);

                // Update and save the view model
                if (factoryResult != null)
                {
                    IBaseEventItem eventItem = (IBaseEventItem)phEventCategory.FindControl(factoryResult.EventControlId);
                    BaseEventModel viewModel = eventItem.SaveItem();
                    viewModel = factoryResult.EventViewManager.Save(viewModel, ExamViewModel.ReferralId, ExamViewModel.EncounterId);
                    
                    //eventItem.SetViewModel(viewModel);

                    // Log the save event
                    CreateLoggingRecord(ExamViewModel.ReferralId, ExamViewModel.EncounterId, ExamViewModel.PatientName, 
                        DVEIRUserActivityType.ENCOUNTER_EXAM_SAVE, string.Empty);
                }

                if (EventSavedSuccessfully != null)
                {
                    EventSavedSuccessfully(this, new EventArgs());
                }

                // On update, run reviewer comment save
                if (btnUpdateEvent.Visible && ReviewerCommentSaved != null)
                {
                    ReviewerCommentSaved(this, new EventArgs());
                }

                RefreshEventSummaries();
                ClearEventCategory();

                saveSuccessful = true;
            }

            return saveSuccessful; // Need some validation handled
        }

        private void ClearEventCategory()
        {
            // Clear current view
            SetExamEventCategory(string.Empty);

            // Event Item should be cleared and enabled
            ddlEventTypes.ClearSelection();
            ddlEventTypes.Enabled = true;
            LoadEventCategoryUserControl(false, true);

            // Clear review comment
            LoadReviewCommentControl(false);

            // Update Save Buttons
            btnSaveEvent.Visible = false;
            btnUpdateEvent.Visible = false;
            btnCancelUpdate.Visible = false;
        }

        private void SetExamEventCategory(string categoryCode)
        {
            SetExamEventCategory(categoryCode, string.Empty, 0);
        }

        private void SetExamEventCategory(string categoryCode, string lateralityCode, long groupingId)
        {
            ExamViewModel.SelectedCategoryCode = categoryCode;
            ExamViewModel.SelectedLateralityCode = lateralityCode;
            ExamViewModel.SelectedGroupingId = groupingId;
        }

        /// <summary>Finds the parent control. (This is a cheat and assumes the control id is unique)</summary>
        /// <param name="currentCtrl">The current control.</param>
        /// <param name="ctrlId">The control identifier.</param>
        /// <returns>The searched control</returns>
        private Control FindParentControl(Control currentCtrl, string ctrlId)
        {
            var searchedCtrl = currentCtrl.FindControl(ctrlId);

            if (searchedCtrl != null)
            {
                return searchedCtrl;
            }

            if (currentCtrl.Parent != null)
            {
                return FindParentControl(currentCtrl.Parent, ctrlId);
            }

            return null;
        }

        #endregion
    }
}