﻿using System;
using System.Collections.Generic;
using System.Linq;
using Managers.Encounter;
using ViewModels.Encounter;

namespace Controls.Encounter
{
    public partial class HistoryControl : BaseWebControl
    {
        /// <summary>Gets or sets the history view model.</summary>
        /// <value>The history view model.</value>
        public HistoryViewModel HistoryViewModel
        {
            get { return ViewState["HistoryViewModel"] as HistoryViewModel; }
            set { ViewState["HistoryViewModel"] = value; }
        }

        /// <summary>Gets or sets the reviewer comment ajax controls.</summary>
        /// <value>The reviewer comment ajax controls.</value>
        public IEnumerable<string> ReviewerCommentAjaxControls
        {
            get { return ViewState["ReviewerComments"] as IEnumerable<string> ?? new string[0]; } 
            set { ViewState["ReviewerComments"] = value.ToList(); }
        }

        /// <summary>Occurs when [reviewer comment saved].</summary>
        public event EventHandler ReviewerCommentSaved;

        /// <summary>Saves this instance.</summary>
        public void SaveHistory()
        {
            HistoryViewModel.History = ucHistory.GetUpdatedQuestion();
            HistoryViewModel.Injury = ucInjury.GetUpdatedQuestion();

            var viewManager = new HistoryViewManager(CurrentUser);
            viewManager.Save(HistoryViewModel);
        }

        /// <summary>Shows the reviewer comment.</summary>
        /// <param name="questionCode">The question code.</param>
        public void ShowReviewerComment(string questionCode)
        {
            if (HistoryViewModel.AllowReviewerComments &&
                questionCode == HistoryViewModel.HistoryReviewerComment.QuestionCode)
            {
                rccHistory.ReviewerCommentViewModel.ShowCommentText = true;
                rccHistory.ReviewerCommentViewModel.SetFocus = true;
                rccHistory.RefreshControl();
            }
        }

        /// <summary>Handles the Load event of the Page control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack && HistoryViewModel != null)
            {
                // Set questions
                ucHistory.Question = HistoryViewModel.History;
                ucInjury.Question = HistoryViewModel.Injury;

                // Set Visibility
                ucHistory.IsReadOnly = HistoryViewModel.IsReadOnly;
                ucInjury.IsReadOnly = HistoryViewModel.IsReadOnly;

                // Set 
                rccHistory.ReviewerCommentViewModel = HistoryViewModel.HistoryReviewerComment;
                rccHistory.Visible = HistoryViewModel.AllowReviewerComments;
                rccHistory.AjaxUpdateControls = ReviewerCommentAjaxControls;
            }
        }

        /// <summary>Handles the OnReviewSavedSuccessfully event of the rccHistory control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rccHistory_OnReviewSavedSuccessfully(object sender, EventArgs e)
        {
            if (ReviewerCommentSaved != null)
            {
                ReviewerCommentSaved(sender, e);
            }
        }
    }
}