﻿using System;
using System.Linq;
using Managers.Encounter;
using Telerik.Web.UI;
using ViewModels.Encounter;
using BusinessManagerClasses;

namespace Controls.Encounter
{
    public partial class ProcedureControl : BaseWebControl
    {
        #region Member Variables

        public ProceduresViewModel ProcedureViewModel {
            get { return ViewState["ProceduresViewModel"] as ProceduresViewModel; }
            set { ViewState["ProceduresViewModel"] = value; }
        }

        public event EventHandler ProcedureSavedSuccessfully;

        #endregion

        /// <summary>
        /// Default Page Load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadControls();
                ddlLaterality.DataSource = ProcedureViewModel.AvailableLateralitiesOrdered;
                ddlLaterality.DataValueField = "Code";
                ddlLaterality.DataTextField = "DisplayText";
                ddlLaterality.DataBind();
            }
        }

        #region Grid Events

        protected void rgProcedures_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            rgProcedures.DataSource = ProcedureViewModel.ProceduresSummaries;
        }

        /// <summary>
        /// Fired on Item Command
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rgProcedures_ItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "EditProcedure")
            {
                int id;
                string summaryId = ((GridDataItem)e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    var pSummary = ProcedureViewModel.ProceduresSummaries.First(s => s.Id == id);

                    // populate Add Panel with Edit ... 
                    rgProcedures.Visible = false;
                    pnlAddProcedureBtn.Visible = false;
                    pnlAddProcedureRegion.Visible = true;

                    hfEncounterDetailID.Value = pSummary.Id.ToString();
                    ddlLaterality.SelectedValue = pSummary.LateralityCode;
                    rtbDetailName.Text = pSummary.Name;
                    rtbDetailCode.Text = pSummary.Code;
                    rtbDescription.Text = pSummary.CodeDescription;

                    rbAddProcedureEntry.Text = "Update";
                    rbAddProcedureEntry.ToolTip = "Click to update the procedure entry";
                }
            }
            else if (e.CommandName == "DeleteProcedure")
            {
                int id;
                string summaryId = ((GridDataItem)e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    var viewManager = new ProceduresViewManager(CurrentUser);
                    viewManager.DeleteProceduresDetail(ProcedureViewModel.EncounterId, id);
                    ProcedureViewModel = viewManager.GetProceduresSummaries(ProcedureViewModel);

                    CreateLoggingRecord(ProcedureViewModel.ReferralId, ProcedureViewModel.EncounterId,ProcedureViewModel.PatientName,
                        DVEIRUserActivityType.ENCOUNTER_PROCEDURE_SAVE, "Delete Procedure");

                    LoadControls();
                }
            }
        }

        #endregion

        #region GUI Events

        /// <summary>
        /// Add procedure area to show
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbAddProcedure_Click(object sender, EventArgs e)
        {
            rgProcedures.Visible = false;
            pnlAddProcedureBtn.Visible = false;
            pnlAddProcedureRegion.Visible = true;

            rbAddProcedureEntry.Text = "Add Procedure";
            rbAddProcedureEntry.ToolTip = "Click to add procedure entry to the encounter";

            // clear values
            hfEncounterDetailID.Value = "0";
            ddlLaterality.SelectedValue = ProcedureViewModel.InitialLateralityCode;
            rtbDetailCode.Text = "";
            rtbDescription.Text = "";
            rtbDetailName.Text = "";
        }

        /// <summary>
        /// Add Procedure to the encounter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbAddProcedureEntry_Click(object sender, EventArgs e)
        {
            var summary = new ProceduresSummary();

            int detailId = 0;
            int.TryParse(hfEncounterDetailID.Value, out detailId);

            summary.Id = detailId;
            summary.LateralityCode = ddlLaterality.SelectedValue;
            summary.Name = rtbDetailName.Text;
            summary.Code = rtbDetailCode.Text;
            summary.CodeDescription = rtbDescription.Text;

            rgProcedures.Visible = true;
            pnlAddProcedureBtn.Visible = true;
            pnlAddProcedureRegion.Visible = false;

            ProcedureViewModel.ProcedureSummary = summary;

            var viewManager = new ProceduresViewManager(CurrentUser);
            viewManager.SaveProceduresDetail(ProcedureViewModel);

            if (ProcedureSavedSuccessfully != null)
            {
                ProcedureSavedSuccessfully(sender, e);
            }

            ProcedureViewModel = viewManager.GetProceduresSummaries(ProcedureViewModel);

            CreateLoggingRecord(ProcedureViewModel.ReferralId, ProcedureViewModel.EncounterId, ProcedureViewModel.PatientName,
                        DVEIRUserActivityType.ENCOUNTER_PROCEDURE_SAVE, "Save Procedure");

            ProcedureViewModel.ProcedureSummary = null;

            LoadControls();
        }

        /// <summary>
        /// Cancel adding procedure
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbCancelAddProcedureEntry_Click(object sender, EventArgs e)
        {
            rgProcedures.Visible = true;
            pnlAddProcedureBtn.Visible = true;

            // clear values
            rtbDetailCode.Text = "";
            rtbDescription.Text = "";
            rtbDetailName.Text = "";
            hfEncounterDetailID.Value = null;

            rbAddProcedureEntry.Text = "Add Procedure";
            rbAddProcedureEntry.ToolTip = "Click to add procedure entry to the encounter";

            pnlAddProcedureRegion.Visible = false;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Fills the report grid with entries from the db.
        /// </summary>
        private void LoadControls()
        {
            pnlAddProcedureBtn.Visible = !ProcedureViewModel.IsReadOnly;

            rgProcedures.MasterTableView.GetColumn("DeleteProcedureColumn").Display = !ProcedureViewModel.IsReadOnly;
            rgProcedures.MasterTableView.GetColumn("EditProcedureColumn").Display = !ProcedureViewModel.IsReadOnly;
            rgProcedures.DataSource = ProcedureViewModel.ProceduresSummaries;
            rgProcedures.DataBind();
        }

        #endregion
    }
}