﻿using DataAccess;
using Managers.Encounter;
using System;
using ViewModels.Encounter;

public partial class controls_EncounterInformationUserControl : BaseWebControl
{
    #region Member Variables

    public GeneralViewModel GeneralViewModel 
    { 
        get { return ViewState["GeneralViewModel"] as GeneralViewModel; }
        set { ViewState["GeneralViewModel"] = value; }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            RefreshView();
        }
    }

    #region Public Methods

    /// <summary>
    /// SaveEncounterInformation: receives an ER_ENCOUNTER_MOD object by reference, sets its values and returns.
    /// </summary>
    public void Save()
    {
        if (!GeneralViewModel.IsReadOnly)
        {
            if (Page.IsValid)
            {
                GeneralViewModel.VisitDate = rdtpDateOfEncounterEditable.SelectedDate ?? DateTime.Now;

                int facilityId;
                if (int.TryParse(rcbInstitution.SelectedValue, out facilityId))
                {
                    GeneralViewModel.FacilityId = facilityId;
                }

                GeneralViewModel.EncounterType = rcbEncounterType.SelectedValue;

                new EncounterViewManager(CurrentUser).SaveGeneralView(GeneralViewModel);
            }
        }
    }

    public void RefreshView()
    {
        PopulateDataFields();
        SetVisibility();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Populate controls
    /// </summary>
    private void PopulateDataFields()
    {
        lblActualDate.Text = GeneralViewModel.VisitDate.ToString();

        if (GeneralViewModel.Facility != null)
        {
            lblActualFacility.Text = GeneralViewModel.Facility.Name;
        }

        rdtpDateOfEncounterEditable.SelectedDate = GeneralViewModel.VisitDate;
        rcbInstitution.SelectedValue = GeneralViewModel.FacilityId.ToString();

        switch (GeneralViewModel.EncounterType)
        {
            case "I":
                lblEncounterType.Text = "Inpatient";
                break;
            case "O":
                lblEncounterType.Text = "Outpatient";
                break;
            default:
                lblEncounterType.Text = "";
                break;
        }

        rcbEncounterType.SelectedValue = GeneralViewModel.EncounterType;

        lblReferralDate.Text = GeneralViewModel.DateReferred.ToString();
        lblReferralDateEditable.Text = GeneralViewModel.DateReferred.ToString();
    }

    /// <summary>
    /// Determine which controls to show based on IsEditable
    /// </summary>
    private void SetVisibility()
    {
        pnlEditableEncounterInfo.Visible = !GeneralViewModel.IsReadOnly;
        pnlReadOnlyEncounterInfo.Visible = GeneralViewModel.IsReadOnly;
    }

    #endregion
}