﻿using System;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;

namespace Controls.ExamTabv2
{
    public partial class ExamMultiSelectUserControl : UserControl, IDynamicControl
    {
        #region Public Members

        public MultiSelectQuestion Question
        {
            get { return ViewState["Question"] as MultiSelectQuestion; }
            set { ViewState["Question"] = value; }
        }

        public string EmptyMessage { get; set; }

        public bool IsReadOnly { get; set; }

        public bool DisableDynamic { get; set; }

        public string CssClass
        {
            set { pnlMultiSelectWrapper.CssClass = value; }
        }

        public string OnClientLoad { get; set; }

        public string OnClientSelectedIndexChanged { get; set; }

        public DdlOnItemDataBoundExt ExtendDdlOnItemDataBound { get; set; }

        public GridOnItemDataBoundExt ExtendGridOnItemDataBound { get; set; }

        public GridOnItemCommandExt ExtendGridOnItemCommand { get; set; }

        public delegate void DdlOnItemDataBoundExt(object sender, RadComboBoxItemEventArgs e);

        public delegate void GridOnItemDataBoundExt(object sender, GridItemEventArgs e);

        public delegate void GridOnItemCommandExt(object sender, GridCommandEventArgs e);

        #endregion

        #region Public Methods

        public MultiSelectQuestion GetUpdatedQuestion()
        {
            // Just a formality right now
            return Question;
        }

        public void PerformForcedRefresh()
        {
            // Not used
        }

        #endregion

        #region Page Events

        protected void Page_Load(object sender, EventArgs e)
        {
            if ((DisableDynamic && !IsPostBack)
                || (!DisableDynamic && Question != null))
            {
                MapGrid();
            }
        }

        protected void gvMultiSelect_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                LinkButton lbtnMultiSelectEdit = (LinkButton)e.Item.FindControl("lbtnMultiSelectEdit");
                lbtnMultiSelectEdit.ToolTip = string.Format("Click To Edit The {0} Selection", Question.DisplayText);

                LinkButton lbtnMultiSelectDelete = (LinkButton)e.Item.FindControl("lbtnMultiSelectDelete");
                lbtnMultiSelectDelete.ToolTip = string.Format("Click To Delete The {0} Selection", Question.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.EditItem)
            {
                TextBox tbMultiSelectAdditionalDetails = (TextBox) e.Item.FindControl("tbMultiSelectAdditionalDetails");
                tbMultiSelectAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", Question.DisplayText);

                LinkButton lbtnMultiSelectUpdate = (LinkButton)e.Item.FindControl("lbtnMultiSelectUpdate");
                lbtnMultiSelectUpdate.ToolTip = string.Format("Click To Update The {0} Selection", Question.DisplayText);

                LinkButton lbtnMultiSelectCancel = (LinkButton)e.Item.FindControl("lbtnMultiSelectCancel");
                lbtnMultiSelectCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", Question.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox multiSelectComboBox = (RadComboBox)e.Item.FindControl("ddlMultiSelect");
                multiSelectComboBox.ToolTip = string.Format("Select The {0}", Question.DisplayText);
                DataBindSelect(multiSelectComboBox);

                TextBox tbMultiSelectAdditionalDetails = (TextBox)e.Item.FindControl("tbMultiSelectAdditionalDetails");
                tbMultiSelectAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", Question.DisplayText);

                LinkButton lbtnMultiSelectAdd = (LinkButton)e.Item.FindControl("lbtnMultiSelectAdd");
                lbtnMultiSelectAdd.ToolTip = string.Format("Click To Add The {0} Selection", Question.DisplayText);
            }

            if (ExtendGridOnItemDataBound != null)
            {
                ExtendGridOnItemDataBound(sender, e);
            }
        }

        protected void gvMultiSelect_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMultiSelect.EditIndexes.Add(e.Item.ItemIndex);
                gvMultiSelect.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                string choiceCode = ((GridDataItem) e.Item).GetDataKeyValue("Code").ToString();
                var selectedChoice = Question.AllChoices.FirstOrDefault(c => c.Code == choiceCode);

                if (selectedChoice != null)
                {
                    Question.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMultiSelect.EditIndexes.Clear();
                gvMultiSelect.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                string choiceCode = ((GridDataItem)e.Item).GetDataKeyValue("Code").ToString();
                var selectedChoice = Question.SelectedChoices.FirstOrDefault(c => c.Code == choiceCode);

                if (selectedChoice != null)
                {
                    Question.SelectedChoices.Remove(selectedChoice);

                    TextBox tbMultiSelectAdditionalDetails = (TextBox) e.Item.FindControl("tbMultiSelectAdditionalDetails");
                    selectedChoice.Comment = tbMultiSelectAdditionalDetails.Text;

                    Question.SelectedChoices.Add(selectedChoice);
                }

                gvMultiSelect.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlMultiSelect");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    Question.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbMultiSelectAdditionalDetails = (TextBox) e.Item.FindControl("tbMultiSelectAdditionalDetails");
                    selectedChoice.Comment = tbMultiSelectAdditionalDetails.Text;

                    Question.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbMultiSelectAdditionalDetails.Text = string.Empty;
                }
            }

            if (ExtendGridOnItemCommand != null)
            {
                ExtendGridOnItemCommand(sender, e);
            }

            MapGrid();
        }

        protected void ddlMultiSelect_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();

            if (ExtendGridOnItemDataBound != null)
            {
                ExtendDdlOnItemDataBound(sender, e);
            }
        }

        #endregion

        #region Private Methods

        private void MapGrid()
        {
            gvMultiSelect.MasterTableView.Caption = Question.DisplayText;
            gvMultiSelect.MasterTableView.Columns[0].HeaderText = Question.DisplayText;
            gvMultiSelect.MasterTableView.Columns[0].HeaderTooltip = Question.DisplayText;


            // Show comment column only if there are some choices that require it
            gvMultiSelect.MasterTableView.Columns[1].Visible = Question.AllChoices.Any(c => c.RequiresComment);

            if (IsReadOnly)
            {
                int editColumnNum = gvMultiSelect.MasterTableView.Columns.Count - 1;
                gvMultiSelect.MasterTableView.Columns[editColumnNum].Visible = false;
                gvMultiSelect.MasterTableView.ShowFooter = false;
            }

            gvMultiSelect.DataSource = Question.SelectedChoicesOrderd;
            gvMultiSelect.DataBind();
        }

        private void DataBindSelect(RadComboBox multiSelectComboBox)
        {
            multiSelectComboBox.MarkFirstMatch = true;
            multiSelectComboBox.IsCaseSensitive = false;
            multiSelectComboBox.OnClientBlur = "MEVIR.Common.RadComboBoxValidateText";
            multiSelectComboBox.OnClientTemplateDataBound = "MEVIR.Common.RadComboBoxValidateText";
            multiSelectComboBox.EmptyMessage = EmptyMessage ?? multiSelectComboBox.EmptyMessage;
            multiSelectComboBox.OnClientLoad = OnClientLoad ?? multiSelectComboBox.OnClientLoad;
            multiSelectComboBox.OnClientSelectedIndexChanged = 
                OnClientSelectedIndexChanged ?? multiSelectComboBox.OnClientSelectedIndexChanged;
            
            multiSelectComboBox.DataSource = Question.AvailableChoicesOrdered;
            multiSelectComboBox.DataValueField = "Code";
            multiSelectComboBox.DataTextField = "DisplayText";
            multiSelectComboBox.DataBind();
        }

        #endregion
    }
}