﻿using System;
using System.Linq;
using System.Web.UI;
using Telerik.Web.UI;
using ViewModels;

namespace Controls.ExamTabv2
{
    public partial class ExamSingleSelectUserControl : UserControl, IDynamicControl
    {
        #region Public Members

        public SingleSelectQuestion Question { get; set; }

        public string EmptyMessage { get; set; }

        public bool IsReadOnly { get; set; }

        public string LabelAssociation
        {
            get
            {
                return IsReadOnly ? tbReadOnlySingleSelect.ClientID : ddlSingleSelect.ClientID + "_Input";
            }
        }

        public string CssClass
        {
            set
            {
                pnlSingleSelect.CssClass = value;
                ddlSingleSelect.CssClass = value;
                tbReadOnlySingleSelect.CssClass = value;
            }
        }

        public string OnClientLoad
        {
            set { ddlSingleSelect.OnClientLoad = value; }
        }

        public string OnClientSelectedIndexChanged
        {
            set { ddlSingleSelect.OnClientSelectedIndexChanged = value; }
        }

        public DdlOnItemDataBoundExt ExtendDdlOnItemDataBound { get; set; }

        public delegate void DdlOnItemDataBoundExt(object sender, RadComboBoxItemEventArgs e);

        #endregion

        public SingleSelectQuestion GetUpdatedQuestion()
        {
            string selectedChoiceCode = ddlSingleSelect.SelectedValue;
            Choice selectedChoice = Question.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

            if (selectedChoice != null && selectedChoice.RequiresComment)
            {
                selectedChoice.Comment = tbOptionAdditionalDetails.Text;
            }

            Question.SelectedChoice = selectedChoice;

            return Question;
        }

        public void PerformForcedRefresh()
        {
            // Does nothing
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (Question != null)
            {
                MapSingleSelect();
            }
        }

        protected void ddlSingleSelect_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();

            if (ExtendDdlOnItemDataBound != null)
            {
                ExtendDdlOnItemDataBound(sender, e);
            }
        }

        private void MapSingleSelect()
        {
            ddlSingleSelect.MarkFirstMatch = true;
            ddlSingleSelect.IsCaseSensitive = false;
            ddlSingleSelect.OnClientBlur = "MEVIR.Common.RadComboBoxValidateText";
            ddlSingleSelect.OnClientTemplateDataBound = "MEVIR.Common.RadComboBoxValidateText";
            ddlSingleSelect.ToolTip = string.Format("Select The {0}", Question.DisplayText);
            ddlSingleSelect.EmptyMessage = EmptyMessage;
            ddlSingleSelect.DataSource = Question.AllChoicesOrdered;
            ddlSingleSelect.DataValueField = "Code";
            ddlSingleSelect.DataTextField = "DisplayText";
            ddlSingleSelect.DataBind();

            tbOptionAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", Question.DisplayText);

            if (Question.IsAnswered)
            {
                ddlSingleSelect.SelectedValue = Question.SelectedChoice.Code;
                tbOptionAdditionalDetails.Text = Question.SelectedChoice.RequiresComment 
                    ? Question.SelectedChoice.Comment 
                    : string.Empty;
            }

            if (IsReadOnly)
            {
                ddlSingleSelect.CssClass += " displayNone";
                tbReadOnlySingleSelect.ToolTip = string.Format("Read Only - {0}", Question.DisplayText);
                tbReadOnlySingleSelect.Visible = true;
                tbReadOnlySingleSelect.Text = ddlSingleSelect.Text;
                tbOptionAdditionalDetails.ReadOnly = true;
            }
        }
    }
}