﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabAngleRecessionUserControl : BaseEventItem<AngleRecessionEventModel>
    {
        #region Implmented Methods

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = AngleRecessionEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            gvVisitData.MasterTableView.Caption = EventViewModel.VisitData.DisplayText;
            gvVisitData.MasterTableView.Columns[0].HeaderText = EventViewModel.VisitData.DisplayText;
            gvVisitData.MasterTableView.Columns[2].HeaderText = EventViewModel.ClockHours.First().Value.DisplayText;
            MapMultiSelectGrid(gvVisitData, EventViewModel.VisitDataViews, EventViewModel.IsReadOnly);
        }

        protected override AngleRecessionEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Page Events

        protected void ddlVisitData_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            var choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
        }

        protected void gvVisitData_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnVisitDataEdit = (LinkButton)e.Item.FindControl("lbtnVisitDataEdit");
                lbtnVisitDataEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.VisitData.DisplayText);

                var lbtnVisitDataDelete = (LinkButton)e.Item.FindControl("lbtnVisitDataDelete");
                lbtnVisitDataDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.VisitData.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");
                tbVisitDataAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.VisitData.DisplayText);

                var clockHour1 = (RadComboBox)e.Item.FindControl("ddlClockHour1");
                clockHour1.ToolTip = string.Format("Select The Initial {0}", EventViewModel.ClockHours.First().Value.DisplayText);
                DataBindClockHourDropDownList(clockHour1, 1);

                var clockHour2 = (RadComboBox)e.Item.FindControl("ddlClockHour2");
                clockHour2.ToolTip = string.Format("Select The Ending {0}", EventViewModel.ClockHours.First().Value.DisplayText);
                DataBindClockHourDropDownList(clockHour2, 2);

                var lbtnVisitDataUpdate = (LinkButton)e.Item.FindControl("lbtnVisitDataUpdate");
                lbtnVisitDataUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.VisitData.DisplayText);

                var lbtnVisitDataCancel = (LinkButton)e.Item.FindControl("lbtnVisitDataCancel");
                lbtnVisitDataCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.VisitData.DisplayText);
                
                var dataItem = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;
                if (dataItem != null)
                {
                    MapClockHours(dataItem.ClockHour, clockHour1, clockHour2);
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                var visitDataComboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                MapSelectField(visitDataComboBox, EventViewModel.VisitData);

                var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");
                tbVisitDataAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.VisitData.DisplayText);

                var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                clockHour1.ToolTip = string.Format("Select The Initial {0}", EventViewModel.ClockHours.First().Value.DisplayText);
                DataBindClockHourDropDownList(clockHour1, 1);

                var clockHour2 = (RadComboBox)e.Item.FindControl("ddlClockHour2");
                clockHour2.ToolTip = string.Format("Select The Ending {0}", EventViewModel.ClockHours.First().Value.DisplayText);
                DataBindClockHourDropDownList(clockHour2, 2);

                var lbtnVisitDataAdd = (LinkButton)e.Item.FindControl("lbtnVisitDataAdd");
                lbtnVisitDataAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.VisitData.DisplayText);
            }
        }

        protected void gvVisitData_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvVisitData.EditIndexes.Add(e.Item.ItemIndex);
                gvVisitData.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                    EventViewModel.ClockHours[selectedChoice.Code].ResponseText = String.Empty;
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvVisitData.EditIndexes.Clear();
                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.VisitData.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.Remove(selectedChoice);
                    EventViewModel.ClockHours[selectedChoice.Code].ResponseText = string.Empty;

                    var tbVisitDataAdditionalDetails = (TextBox) e.Item.FindControl("tbVisitDataAdditionalDetails");
                    selectedChoice.Comment = tbVisitDataAdditionalDetails.Text;
                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);

                    var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                    var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                    if (clockHour1.SelectedIndex > -1 && clockHour2.SelectedIndex > -1)
                    {
                        EventViewModel.ClockHours[selectedChoice.Code].ResponseText =
                            clockHour1.SelectedValue + " - " + clockHour2.SelectedValue;
                    }
                }

                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                var comboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.VisitData.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    var tbVisitDataAdditionalDetails = (TextBox) e.Item.FindControl("tbVisitDataAdditionalDetails");
                    selectedChoice.Comment = tbVisitDataAdditionalDetails.Text;
                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);

                    var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                    var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                    if (clockHour1.SelectedIndex > -1 && clockHour2.SelectedIndex > -1)
                    {
                        EventViewModel.ClockHours[selectedChoice.Code].ResponseText =
                            clockHour1.SelectedValue + " - " + clockHour2.SelectedValue;
                    }

                    comboBox.Text = string.Empty;
                    tbVisitDataAdditionalDetails.Text = string.Empty;
                    clockHour1.SelectedIndex = -1;
                    clockHour2.SelectedIndex = -1;
                }
            }

            MapTestDetails();
        }

        #endregion

        #region Private Methods

        private void DataBindClockHourDropDownList(RadComboBox clockHourDropDownList, int startNumber)
        {

            clockHourDropDownList.MarkFirstMatch = true;
            clockHourDropDownList.IsCaseSensitive = false;
            clockHourDropDownList.OnClientBlur = "MEVIR.Common.RadComboBoxValidateText";
            clockHourDropDownList.OnClientTemplateDataBound = "MEVIR.Common.RadComboBoxValidateText";
            List<int> clockhours = new List<int>();
            for (int i = startNumber; i <= 12; i++)
            {
                clockhours.Add(i);
            }

            clockHourDropDownList.DataSource = clockhours;
            clockHourDropDownList.DataBind();
        }

        private void MapClockHours(string clockHourText, RadComboBox clockHour1, RadComboBox clockHour2)
        {
            if (!string.IsNullOrEmpty(clockHourText))
            {
                var clockHours = clockHourText.Split(new char[] {'-'});
                clockHour1.SelectedValue = clockHours[0].Trim();
                clockHour2.SelectedValue = clockHours[1].Trim();
            }
        }

        #endregion
    }
}