﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabAnteriorChamberUserControl : BaseEventItem<AnteriorChamberEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = AnteriorChamberEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            MapMultiSelectGrid(gvVisitData, EventViewModel.VisitData, EventViewModel.IsReadOnly);
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override AnteriorChamberEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Page Events

        protected void ddlVisitData_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            var choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["ShowAdditionalDetail"] = "false";
            e.Item.Attributes["ShowHeight"] = "false";
            e.Item.Attributes["ShowPercentage"] = "false";

            if (choice.RequiresComment)
            {
                if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
                {
                    e.Item.Attributes["ShowHeight"] = "true";
                }
                else if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
                {
                    e.Item.Attributes["ShowPercentage"] = "true";
                }
                else
                {
                    e.Item.Attributes["ShowAdditionalDetail"] = "true";
                }
            }
        }

        protected void gvVisitData_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnVisitDataEdit = (LinkButton)e.Item.FindControl("lbtnVisitDataEdit");
                lbtnVisitDataEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.VisitData.DisplayText);

                var lbtnVisitDataDelete = (LinkButton)e.Item.FindControl("lbtnVisitDataDelete");
                lbtnVisitDataDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.VisitData.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var choice = e.Item.DataItem as Choice;

                if (choice != null && choice.RequiresComment)
                {
                    if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
                    {
                        var tbVisitDataHeight = (RadTextBox)e.Item.FindControl("tbVisitDataHeight");
                        tbVisitDataHeight.Text = choice.Comment;
                        tbVisitDataHeight.Visible = true;

                        var reqValidator = (RequiredFieldValidator)e.Item.FindControl("reqValHeight");
                        reqValidator.Enabled = true;

                        var regExValidator = (RegularExpressionValidator)e.Item.FindControl("regExValHeight");
                        regExValidator.Enabled = true;
                    }
                    else if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
                    {
                        var tbVisitDataPercentage = (RadTextBox)e.Item.FindControl("tbVisitDataPercentage");
                        tbVisitDataPercentage.Text = choice.Comment;
                        tbVisitDataPercentage.Visible = true;

                        var reqValidator = (RequiredFieldValidator)e.Item.FindControl("reqValPercentage");
                        reqValidator.Enabled = true;

                        var regExValidator = (RegularExpressionValidator)e.Item.FindControl("regExValPercentage");
                        regExValidator.Enabled = true;
                    }
                    else
                    {
                        var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");
                        tbVisitDataAdditionalDetails.Text = choice.Comment;
                        tbVisitDataAdditionalDetails.Visible = true;
                        tbVisitDataAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.VisitData.DisplayText);
                    }
                }

                var lbtnVisitDataUpdate = (LinkButton)e.Item.FindControl("lbtnVisitDataUpdate");
                lbtnVisitDataUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.VisitData.DisplayText);

                var lbtnVisitDataCancel = (LinkButton)e.Item.FindControl("lbtnVisitDataCancel");
                lbtnVisitDataCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.VisitData.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                var visitDataComboBox = (RadComboBox)e.Item.FindControl("ddlVisitData");
                MapSelectField(visitDataComboBox, EventViewModel.VisitData);

                var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");
                tbVisitDataAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.VisitData.DisplayText);

                var lbtnVisitDataAdd = (LinkButton)e.Item.FindControl("lbtnVisitDataAdd");
                lbtnVisitDataAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.VisitData.DisplayText);
            }
        }

        protected void gvVisitData_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvVisitData.EditIndexes.Add(e.Item.ItemIndex);
                gvVisitData.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvVisitData.EditIndexes.Clear();
                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.VisitData.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {
                        if (selectedChoice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
                        {
                            var tbVisitDataHeight = (RadTextBox)e.Item.FindControl("tbVisitDataHeight");
                            comment = FormatComment(selectedChoice.Code, tbVisitDataHeight.Text);
                        }
                        else if (selectedChoice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
                        {
                            var tbVisitDataPercentage = (RadTextBox)e.Item.FindControl("tbVisitDataPercentage");
                            comment = FormatComment(selectedChoice.Code, tbVisitDataPercentage.Text);
                        }
                        else
                        {
                            var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");
                            comment = tbVisitDataAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;
                }

                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                var comboBox = (RadComboBox)e.Item.FindControl("ddlVisitData");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.VisitData.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    var tbVisitDataHeight = (RadTextBox)e.Item.FindControl("tbVisitDataHeight");
                    var tbVisitDataPercentage = (RadTextBox)e.Item.FindControl("tbVisitDataPercentage");
                    var tbVisitDataAdditionalDetails = (TextBox)e.Item.FindControl("tbVisitDataAdditionalDetails");

                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {

                        if (selectedChoice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
                        {
                            comment = FormatComment(selectedChoice.Code, tbVisitDataHeight.Text);
                        }
                        else if (selectedChoice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
                        {
                            comment = FormatComment(selectedChoice.Code, tbVisitDataPercentage.Text);
                        }
                        else
                        {
                            comment = tbVisitDataAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;
                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);


                    tbVisitDataHeight.Text = string.Empty;
                    tbVisitDataPercentage.Text = string.Empty;
                    tbVisitDataAdditionalDetails.Text = string.Empty;
                    comboBox.Text = string.Empty;
                }
            }

            MapTestDetails();
        }

        #endregion

        #region Private Methods

        private string FormatComment(string choiceCode, string commentText)
        {
            if (choiceCode == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
            {
                if (!commentText.EndsWith("mm"))
                {
                    commentText = commentText.Trim() + " mm";
                }
            }
            else if (choiceCode == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
            {
                if (!commentText.EndsWith("%"))
                {
                    commentText = commentText.Trim() + "%";
                }
            }

            return commentText;
        }

        #endregion
    }
}