﻿using System;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Globe User Control
    /// </summary>
    public partial class ExamTabIntraocularPressureUserControl : BaseEventItem<IntraocularPressureEventModel>
    {
        #region Implemented Method

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
            timeValidator.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = IntraocularPressureEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            lblMethod.Text = EventViewModel.Method.DisplayText;
            MapTextField(tbMethod, EventViewModel.Method, EventViewModel.IsReadOnly);

            lblMethodComment.Text = EventViewModel.MethodComment.DisplayText;
            MapTextField(tbMethodComment, EventViewModel.MethodComment, EventViewModel.IsReadOnly);

            lblNumber.Text = EventViewModel.Number.DisplayText;
            MapTextField(tbNumber, EventViewModel.Number, EventViewModel.IsReadOnly);

            tbTime.ToolTip = string.Format("Input The {0}", EventViewModel.Time.DisplayText);

            TimeSpan time;
            if (TimeSpan.TryParse(EventViewModel.Time.ResponseText, out time))
            {
                tbTime.SelectedTime = time;
            }

            if (EventViewModel.IsReadOnly)
            {
                tbTime.DateInput.ReadOnly = true;
                tbTime.TimePopupButton.Enabled = false;
            }
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Globe Event Model</returns>
        protected override IntraocularPressureEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);
            EventViewModel.Method.ResponseText = tbMethod.Text;
            EventViewModel.MethodComment.ResponseText = tbMethodComment.Text;
            EventViewModel.Number.ResponseText = tbNumber.Text;
            EventViewModel.Time.ResponseText =
                tbTime.SelectedTime.HasValue
                    ? tbTime.SelectedTime.Value.ToString(@"hh\:mm")
                    : string.Empty;

            return EventViewModel;
        }

        #endregion
    }
}