﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabKineticVisualFieldsUserControl : BaseEventItem<KineticVisualFieldsEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = KineticVisualFieldsEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            gvMethodology.MasterTableView.Caption = EventViewModel.Methodology.DisplayText;
            gvMethodology.MasterTableView.Columns[0].HeaderText = EventViewModel.Methodology.DisplayText;
            gvMethodology.MasterTableView.Columns[2].HeaderText = EventViewModel.Results.First().Value.DisplayText;
            MapMultiSelectGrid(gvMethodology, EventViewModel.MethodologyViews, EventViewModel.IsReadOnly);
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override KineticVisualFieldsEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void gvMethodology_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnMethodologyEdit = (LinkButton)e.Item.FindControl("lbtnMethodologyEdit");
                lbtnMethodologyEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.Methodology.DisplayText);

                var lbtnMethodologyDelete = (LinkButton)e.Item.FindControl("lbtnMethodologyDelete");
                lbtnMethodologyDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.Methodology.DisplayText);
            }

            else if (e.Item.ItemType == GridItemType.EditItem)
            {
                var tbMethodologyAdditionalDetails = (TextBox)e.Item.FindControl("tbMethodologyAdditionalDetails");
                tbMethodologyAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.Methodology.DisplayText);

                var tbResult = (TextBox)e.Item.FindControl("tbResult");
                tbResult.ToolTip = string.Format("Input The {0}", EventViewModel.Results.First().Value.DisplayText);

                var lbtnMethodologyUpdate = (LinkButton)e.Item.FindControl("lbtnMethodologyUpdate");
                lbtnMethodologyUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.Methodology.DisplayText);

                var lbtnMethodologyCancel = (LinkButton)e.Item.FindControl("lbtnMethodologyCancel");
                lbtnMethodologyCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.Methodology.DisplayText);
            }

            else if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox methodologyComboBox = (RadComboBox)e.Item.FindControl("ddlMethodology");
                MapSelectField(methodologyComboBox, EventViewModel.Methodology);

                var tbMethodologyAdditionalDetails = (TextBox)e.Item.FindControl("tbMethodologyAdditionalDetails");
                tbMethodologyAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.Methodology.DisplayText);

                var tbResult = (TextBox)e.Item.FindControl("tbResult");
                tbResult.ToolTip = string.Format("Input The {0}", EventViewModel.Results.First().Value.DisplayText);

                var lbtnMethodologyAdd = (LinkButton)e.Item.FindControl("lbtnMethodologyAdd");
                lbtnMethodologyAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.Methodology.DisplayText);
            }  
        }

        protected void gvMethodology_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMethodology.EditIndexes.Add(e.Item.ItemIndex);
                gvMethodology.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as KineticVisualFieldsEventModel.MethodologyView;

                if (selectedChoice != null)
                {
                    EventViewModel.Methodology.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.ChoiceCode);
                    EventViewModel.Results[selectedChoice.ChoiceCode].ResponseText = String.Empty;
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMethodology.EditIndexes.Clear();
                gvMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as KineticVisualFieldsEventModel.MethodologyView;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.Methodology.SelectedChoices.FirstOrDefault(
                            c => c.Code == dataView.ChoiceCode)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.Methodology.SelectedChoices.Remove(selectedChoice);
                    EventViewModel.Results[selectedChoice.Code].ResponseText = string.Empty;

                    TextBox tbMethodologyAdditionalDetails = (TextBox) e.Item.FindControl("tbMethodologyAdditionalDetails");
                    selectedChoice.Comment = tbMethodologyAdditionalDetails.Text;
                    EventViewModel.Methodology.SelectedChoices.Add(selectedChoice);

                    var result = (TextBox) e.Item.FindControl("tbResult");
                    EventViewModel.Results[selectedChoice.Code].ResponseText = result.Text;
                }

                gvMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                var comboBox = (RadComboBox) e.Item.FindControl("ddlMethodology");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.Methodology.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    var tbMethodologyAdditionalDetails = (TextBox) e.Item.FindControl("tbMethodologyAdditionalDetails");
                    selectedChoice.Comment = tbMethodologyAdditionalDetails.Text;
                    EventViewModel.Methodology.SelectedChoices.Add(selectedChoice);

                    var result = (TextBox) e.Item.FindControl("tbResult");
                    EventViewModel.Results[selectedChoice.Code].ResponseText = result.Text;

                    comboBox.Text = string.Empty;
                    tbMethodologyAdditionalDetails.Text = string.Empty;
                    result.Text = string.Empty;
                }
            }

            MapTestDetails();
        }

        protected void ddlMethodology_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            var choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
        }

        #endregion
    }
}