﻿using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabMotilityUserControl : BaseEventItem<MotilityEventModel>
    {
        #region Implemented Members

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = MotilityEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            // Pursuit
            ucPursuit.Question = EventViewModel.Pursuit;
            ucPursuit.IsReadOnly = EventViewModel.IsReadOnly;

            // Saccades
            ucSaccades.Question = EventViewModel.Saccades;
            ucSaccades.IsReadOnly = EventViewModel.IsReadOnly;

            // Nystagmus
            ucNystagmus.Question = EventViewModel.Nystagmus;
            ucNystagmus.IsReadOnly = EventViewModel.IsReadOnly;
            ucNystagmus.ExtendDdlOnItemDataBound = ucNystagmus_OnItemDataBound;

            ucNystagmusOptions.EmptyMessage = EventViewModel.NystagmusOptions.DisplayText;
            ucNystagmusOptions.Question = EventViewModel.NystagmusOptions;
            ucNystagmusOptions.IsReadOnly = EventViewModel.IsReadOnly;

            // Motility
            MapMotilityGrid();

            // Cranial Nerve
            ucCranialNerve.EmptyMessage = EventViewModel.CranialNerve.DisplayText;
            ucCranialNerve.Question = EventViewModel.CranialNerve;
            ucCranialNerve.IsReadOnly = EventViewModel.IsReadOnly;
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override MotilityEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            // Pursuit
            EventViewModel.Pursuit = ucPursuit.GetUpdatedQuestion();

            // Saccades
            EventViewModel.Saccades = ucSaccades.GetUpdatedQuestion();

            // Nystagmus
            EventViewModel.Nystagmus = ucNystagmus.GetUpdatedQuestion();
            EventViewModel.NystagmusOptions = ucNystagmusOptions.GetUpdatedQuestion();

            // Motility handled by gridview controls

            // Cranial Nerve
            EventViewModel.CranialNerve = ucCranialNerve.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void gvMotility_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnMotilityEdit = (LinkButton)e.Item.FindControl("lbtnMotilityEdit");
                lbtnMotilityEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.Motility.DisplayText);

                var lbtnMotilityDelete = (LinkButton)e.Item.FindControl("lbtnMotilityDelete");
                lbtnMotilityDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.Motility.DisplayText);
            }

            else if (e.Item.ItemType == GridItemType.EditItem)
            {
                var lbtnMotilityUpdate = (LinkButton)e.Item.FindControl("lbtnMotilityUpdate");
                lbtnMotilityUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.Motility.DisplayText);

                var lbtnMotilityCancel = (LinkButton)e.Item.FindControl("lbtnMotilityCancel");
                lbtnMotilityCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.Motility.DisplayText);

                var dataItem = e.Item.DataItem as Choice;

                if (dataItem != null && dataItem.RequiresComment)
                {
                    if (MotilityEventModel.MotilityNumberComment.Any(c => c == dataItem.Code))
                    {
                        var tbMotilityNumber = (RadTextBox)e.Item.FindControl("tbMotilityNumber");
                        tbMotilityNumber.Text = dataItem.Comment;
                        tbMotilityNumber.Visible = true;
                    }
                    else
                    {
                        var tbMethodologyAdditionalDetails = (TextBox)e.Item.FindControl("tbMotilityAdditionalDetails");
                        tbMethodologyAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.Motility.DisplayText);
                        tbMethodologyAdditionalDetails.Text = dataItem.Comment;
                        tbMethodologyAdditionalDetails.Visible = true;
                    }
                }
            }

            else if (e.Item.ItemType == GridItemType.Footer)
            {
                var motilityComboBox = (RadComboBox)e.Item.FindControl("ddlMotility");
                MapSelectField(motilityComboBox, EventViewModel.Motility);

                var tbMotilityAdditionalDetails = (TextBox)e.Item.FindControl("tbMotilityAdditionalDetails");
                tbMotilityAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.Motility.DisplayText);

                var lbtnMotilityAdd = (LinkButton)e.Item.FindControl("lbtnMotilityAdd");
                lbtnMotilityAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.Motility.DisplayText);
            }
        }

        protected void gvMotility_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMotility.EditIndexes.Add(e.Item.ItemIndex);
                gvMotility.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.Motility.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMotility.EditIndexes.Clear();
                gvMotility.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.Motility.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {
                        if (MotilityEventModel.MotilityNumberComment.Any(c => c == selectedChoice.Code))
                        {
                            var tbMotilityNumber = (RadTextBox)e.Item.FindControl("tbMotilityNumber");
                            comment = FormatComment(selectedChoice.Code, tbMotilityNumber.Text);
                        }
                        else
                        {
                            var tbMotilityAdditionalDetails = (TextBox)e.Item.FindControl("tbMotilityAdditionalDetails");
                            comment = tbMotilityAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;

                }

                gvMotility.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                var comboBox = (RadComboBox)e.Item.FindControl("ddlMotility");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.Motility.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    var tbMotilityNumber = (RadTextBox)e.Item.FindControl("tbMotilityNumber");
                    var tbMotilityAdditionalDetails = (TextBox)e.Item.FindControl("tbMotilityAdditionalDetails");

                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {
                        if (MotilityEventModel.MotilityNumberComment.Any(c => c == selectedChoice.Code))
                        {
                            comment = FormatComment(selectedChoice.Code, tbMotilityNumber.Text);
                        }
                        else
                        {
                            comment = tbMotilityAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;
                    EventViewModel.Motility.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbMotilityNumber.Text = string.Empty;
                    tbMotilityAdditionalDetails.Text = string.Empty;
                }
            }

            MapMotilityGrid();
        }

        protected void ddlMotility_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["ShowAdditionalDetails"] = "false";
            e.Item.Attributes["ShowMotilityNumber"] = "false";

            if (choice.RequiresComment)
            {
                if (MotilityEventModel.MotilityNumberComment.Any(c => c == choice.Code))
                {
                    e.Item.Attributes["ShowMotilityNumber"] = "true";
                }
                else
                {
                    e.Item.Attributes["ShowAdditionalDetails"] = "true";
                }
            }
        }

        protected void ucNystagmus_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["ExpandNystagmus"] =
                (choice.Code == MotilityEventModel.ChoiceCodes.Nystagmus.Present).ToString().ToLower();
        }

        #endregion

        #region Private Methods

        private void MapMotilityGrid()
        {
            MapMultiSelectGrid(gvMotility, EventViewModel.Motility, EventViewModel.IsReadOnly);
        }

        private string FormatComment(string choiceCode, string commentText)
        {
            if (!string.IsNullOrEmpty(commentText) && MotilityEventModel.MotilityNumberComment.Any(c => c == choiceCode))
            {
                if (!commentText.EndsWith("PD"))
                {
                    commentText = commentText.Trim() + " PD";
                }
            }

            return commentText;
        }

        #endregion
    }
}