﻿using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Prosthetic Devices User Control
    /// </summary>
    public partial class ExamTabProstheticDevicesUserControl : BaseEventItem<ProstheticDevicesEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
        }

        protected override void MapTestDetails()
        {
            ucOptions.Question = EventViewModel.Options;
            MapImplantTypesGrid();
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Lens Event Model</returns>
        protected override ProstheticDevicesEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);
            EventViewModel.Options = ucOptions.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void gvImplantTypes_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnImplantTypeEdit = (LinkButton)e.Item.FindControl("lbtnImplantTypeEdit");
                lbtnImplantTypeEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.ImplantTypes.DisplayText);

                var lbtnImplantTypeDelete = (LinkButton)e.Item.FindControl("lbtnImplantTypeDelete");
                lbtnImplantTypeDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.ImplantTypes.DisplayText);
            }
            else if (e.Item.ItemType == GridItemType.EditItem)
            {
                var tbImplantTypeAdditionalDetails = (TextBox)e.Item.FindControl("tbImplantTypeAdditionalDetails");
                tbImplantTypeAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.ImplantTypes.DisplayText);

                var lbtnImplantTypeUpdate = (LinkButton)e.Item.FindControl("lbtnImplantTypeUpdate");
                lbtnImplantTypeUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.ImplantTypes.DisplayText);

                var lbtnImplantTypeCancel = (LinkButton)e.Item.FindControl("lbtnImplantTypeCancel");
                lbtnImplantTypeCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.ImplantTypes.DisplayText);

                var dataItem = e.Item.DataItem as Choice;
                if (dataItem != null && dataItem.Code == ProstheticDevicesEventModel.ChoiceCodes.ImplantTypes.Size)
                {
                    var compValidator = (RequiredFieldValidator)e.Item.FindControl("compValImplantTypes");
                    compValidator.Enabled = true;
                }
            }
            else if (e.Item.ItemType == GridItemType.Footer)
            {
                var implantTypeComboBox = (RadComboBox)e.Item.FindControl("ddlImplantType");
                MapSelectField(implantTypeComboBox, EventViewModel.ImplantTypes);

                var tbImplantTypeAdditionalDetails = (TextBox)e.Item.FindControl("tbImplantTypeAdditionalDetails");
                tbImplantTypeAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.ImplantTypes.DisplayText);

                var lbtnImplantTypeAdd = (LinkButton)e.Item.FindControl("lbtnImplantTypeAdd");
                lbtnImplantTypeAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.ImplantTypes.DisplayText);
            }
        }

        protected void gvImplantTypes_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvImplantTypes.EditIndexes.Add(e.Item.ItemIndex);
                gvImplantTypes.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.ImplantTypes.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvImplantTypes.EditIndexes.Clear();
                gvImplantTypes.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.ImplantTypes.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.ImplantTypes.SelectedChoices.Remove(selectedChoice);

                    TextBox tbImplantTypeAdditionalDetails = (TextBox) e.Item.FindControl("tbImplantTypeAdditionalDetails");
                    selectedChoice.Comment = tbImplantTypeAdditionalDetails.Text;

                    EventViewModel.ImplantTypes.SelectedChoices.Add(selectedChoice);
                }

                gvImplantTypes.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlImplantType");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.ImplantTypes.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    var tbImplantTypeAdditionalDetails = (TextBox) e.Item.FindControl("tbImplantTypeAdditionalDetails");
                    selectedChoice.Comment = tbImplantTypeAdditionalDetails.Text;

                    EventViewModel.ImplantTypes.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbImplantTypeAdditionalDetails.Text = string.Empty;
                }
            }

            MapImplantTypesGrid();
        }

        protected void ddlImplantTypes_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
            e.Item.Attributes["CompValImplantTypeEnabled"] =
                (choice.Code == ProstheticDevicesEventModel.ChoiceCodes.ImplantTypes.Size).ToString().ToLower();
        }

        #endregion

        private void MapImplantTypesGrid()
        {
            MapMultiSelectGrid(gvImplantTypes, EventViewModel.ImplantTypes, EventViewModel.IsReadOnly);
        }
    }
}