﻿using System;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Pupil User Control
    /// </summary>
    public partial class ExamTabPupilUserControl : BaseEventItem<PupilEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;
            tbSize.ValidationGroup = ValidationGroup;
            rangeValSize.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = PupilEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            lblSize.Text = EventViewModel.Size.DisplayText;
            MapTextField(tbSize, EventViewModel.Size, EventViewModel.IsReadOnly);
            tbSize.ToolTip += " between 1 and 10";

            lblAlterationInShape.Text = EventViewModel.AlterationInShape.DisplayText;
            MapTextField(tbAlterationInShape, EventViewModel.AlterationInShape, EventViewModel.IsReadOnly);

            ucPerrlOrPerrla.Question = EventViewModel.PerrlOrPerrla;
            ucPerrlOrPerrla.IsReadOnly = EventViewModel.IsReadOnly;

            lblReactive.Text = EventViewModel.Reactive.DisplayText;
            ucReactive.Question = EventViewModel.Reactive;
            ucReactive.IsReadOnly = EventViewModel.IsReadOnly;

            lblAdie.Text = EventViewModel.Adie.DisplayText;
            ucAdie.Question = EventViewModel.Adie;
            ucAdie.IsReadOnly = EventViewModel.IsReadOnly;

            lblArgyllRobertson.Text = EventViewModel.ArgyllRobertson.DisplayText;
            ucArgyllRobertson.Question = EventViewModel.ArgyllRobertson;
            ucArgyllRobertson.IsReadOnly = EventViewModel.IsReadOnly;
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Lens Event Model</returns>
        protected override PupilEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            EventViewModel.Size.ResponseText = GetNumberResponse(tbSize.Text);
            EventViewModel.AlterationInShape.ResponseText = tbAlterationInShape.Text;

            EventViewModel.PerrlOrPerrla = ucPerrlOrPerrla.GetUpdatedQuestion();

            EventViewModel.Reactive = ucReactive.GetUpdatedQuestion();
            EventViewModel.Adie = ucAdie.GetUpdatedQuestion();
            EventViewModel.ArgyllRobertson = ucArgyllRobertson.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion
    }
}