﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    [ParseChildren(true)]
    public partial class ExamTabTestSummaryUserControl : UserControl, IDynamicControl
    {
        #region Private Members

        private bool IsInitialLoad
        {
            get { return ViewState["TestSummaryInit"] == null ? true : (bool)ViewState["TestSummaryInit"]; }

            set { ViewState["TestSummaryInit"] = value; }
        }

        private bool _forceRefresh;

        private ITemplate contents = null;

        #endregion

        #region Public Members

        [TemplateContainer(typeof(TemplateControl))]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [TemplateInstance(TemplateInstance.Single)]
        public ITemplate Contents
        {
            get { return contents; }
            set { contents = value; }
        }

        public BaseEventModel ViewModel { get; set; }

        public IEnumerable<string> TestPerformExpandValues { private get; set; }

        public IEnumerable<string> TestResultExpandValues { private get; set; }

        public string TestPerformOnClientLoad
        {
            set { ddlTestPerformed.OnClientLoad = value; }
        }

        public string TestPerformOnClientSelectedIndexChanged
        {
            set { ddlTestPerformed.OnClientSelectedIndexChanged = value; }
        }

        public string TestResultOnClientLoad
        {
            set { ddlTestResult.OnClientLoad = value; }
        }

        public string TestResultOnClientSelectedIndexChanged
        {
            set { ddlTestResult.OnClientSelectedIndexChanged = value; }
        }

        public string ValidationGroup
        {
            set
            {
                reqValLaterality.ValidationGroup = value;
                ddlLaterality.ValidationGroup = value;

                reqValTestPerformed.ValidationGroup = value;
                ddlTestPerformed.ValidationGroup = value;

                reqValTestResult.ValidationGroup = value;
                ddlTestResult.ValidationGroup = value;
            }
        }

        #endregion

        #region Public Methods

        public void UpdateViewModel(BaseEventModel viewModel)
        {
            // Get Laterality
            string lateralityCode = ddlLaterality.SelectedValue;
            viewModel.SelectedLaterality =
                viewModel.AvailableLateralities.FirstOrDefault(l => l.Code == lateralityCode);

            // Get Test Performed
            if (viewModel.HasTestPerformed)
            {
                string tpChoiceCode = ddlTestPerformed.SelectedValue;
                Choice selectedChoice = 
                    viewModel.TestPerformed.AllChoices.FirstOrDefault(c => c.Code == tpChoiceCode);

                if (selectedChoice != null && selectedChoice.RequiresComment)
                {
                    selectedChoice.Comment = tbTestPerformedCmt.Text;
                }

                viewModel.TestPerformed.SelectedChoice = selectedChoice;
            }

            // Get Test Result
            if (viewModel.HasTestResult)
            {
                string trChoiceCode = ddlTestResult.SelectedValue;
                viewModel.TestResult.SelectedChoice =
                    viewModel.TestResult.AllChoices.FirstOrDefault(c => c.Code == trChoiceCode);
            }

            // Get Test Comment
            if (viewModel.HasEventComment)
            {
                viewModel.EventComment.ResponseText = tbEventComments.Text;
            }
        }

        public void PerformForcedRefresh()
        {
            _forceRefresh = true;
            foreach (var ctrl in phTestWrapper.Controls)
            {
                var dynamicControl = ctrl as IDynamicControl;
                if (dynamicControl != null)
                {
                    dynamicControl.PerformForcedRefresh();
                }
            }
        }

        #endregion

        #region Page Events

        /// <summary>Handles the Init event of the Page control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Init(object sender, EventArgs e)
        {
            if (contents != null)
            {
                contents.InstantiateIn(phTestWrapper);
            }
        }

        /// <summary>Handles the Load event of the Page control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (ViewModel != null)
            {
                // Update view model after the first load
                if (IsInitialLoad || _forceRefresh)
                {
                    IsInitialLoad = false;
                    _forceRefresh = false;
                }
                else
                {
                    UpdateViewModel(ViewModel);
                }

                MapInputs();
                MapSelectedItems();
                SetReadOnly();
            }
        }

        /// <summary>Sets the read only view.</summary>
        private void SetReadOnly()
        {
            if (ViewModel.IsReadOnly)
            {
                ddlLaterality.CssClass += " displayNone"; // Still need it to render for javascript to execute
                tbReadOnlyLaterality.Text = ddlLaterality.Text;
                tbReadOnlyLaterality.Visible = true;

                ddlTestPerformed.CssClass += " displayNone"; // Still need it to render for javascript to execute
                tbReadOnlyTestPerformed.Text = ddlTestPerformed.Text;
                tbReadOnlyTestPerformed.Visible = true;

                ddlTestResult.CssClass += " displayNone"; // Still need it to render for javascript to execute
                tbReadOnlyTestResult.Text = ddlTestResult.Text;
                tbReadOnlyTestResult.Visible = true;

                tbTestPerformedCmt.ReadOnly = true;
                tbEventComments.ReadOnly = true;
            }
        }

        // Adds attributes to the items for the javascript to use.
        protected void ddlTestPerformed_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            // Add attribute for require comment and show test details
            Choice dataItem = (Choice)e.Item.DataItem;

            e.Item.Attributes["RequiresComment"] = dataItem.RequiresComment.ToString().ToLower();

            // If no values specified then always show details
            if (TestPerformExpandValues == null || !TestPerformExpandValues.Any())
            {
                e.Item.Attributes["TestPerformExpand"] = "true";
            }
            // If there are values specified then show the details only for those values
            else
            {
                e.Item.Attributes["TestPerformExpand"] =
                    (TestPerformExpandValues.Any(v => v == dataItem.Code)).ToString().ToLower();
            }
        }

        protected void ddlTestResult_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            // Add attribute for require comment and show test details
            Choice dataItem = (Choice)e.Item.DataItem;

            // If no values specified then always show details
            if (TestResultExpandValues == null || !TestResultExpandValues.Any())
            {
                e.Item.Attributes["TestResultExpand"] = "true";
            }
            // If there are values specified then show the details only for those values
            else
            {
                e.Item.Attributes["TestResultExpand"] = 
                    (TestResultExpandValues.Any(v => v == dataItem.Code)).ToString().ToLower();
            }
        }

        #endregion

        #region Private Methods

        private void MapSelectedItems()
        {
            if (ViewModel.SelectedLaterality != null)
            {
                ddlLaterality.SelectedValue = ViewModel.SelectedLaterality.Code;

                // Disable laterality if the view model is locked
                ddlLaterality.Enabled = !ViewModel.IsLateralityLocked;
            }

            if (ViewModel.HasTestPerformed && ViewModel.TestPerformed.IsAnswered)
            {
                ddlTestPerformed.SelectedValue = ViewModel.TestPerformed.SelectedChoice.Code;
            }

            if (ViewModel.HasTestResult && ViewModel.TestResult.IsAnswered)
            {
                ddlTestResult.SelectedValue = ViewModel.TestResult.SelectedChoice.Code;
            }

            if (ViewModel.HasEventComment && ViewModel.EventComment.IsAnswered)
            {
                tbEventComments.Text = ViewModel.EventComment.ResponseText;
            }
        }

        private void MapInputs()
        {
            ddlLaterality.DataSource = ViewModel.AvailableLateralitiesOrdered;
            ddlLaterality.DataValueField = "Code";
            ddlLaterality.DataTextField = "DisplayText";
            ddlLaterality.DataBind();

            phTestPerformed.Visible = ViewModel.HasTestPerformed;
            if (ViewModel.HasTestPerformed)
            {
                ddlTestPerformed.DataSource = ViewModel.TestPerformed.AllChoicesOrdered;
                ddlTestPerformed.DataValueField = "Code";
                ddlTestPerformed.DataTextField = "DisplayText";
                ddlTestPerformed.DataBind();
            }

            phTestResult.Visible = ViewModel.HasTestResult;
            if (ViewModel.HasTestResult)
            {
                ddlTestResult.DataSource = ViewModel.TestResult.AllChoicesOrdered;
                ddlTestResult.DataValueField = "Code";
                ddlTestResult.DataTextField = "DisplayText";
                ddlTestResult.DataBind();
            }

            phEventComments.Visible = ViewModel.HasEventComment;
            if (ViewModel.HasEventComment)
            {
                lblEventComments.Text = ViewModel.EventComment.DisplayText;
            }
        }

        #endregion
    }
}