﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Visual Acuity User Control
    /// </summary>
    public partial class ExamTabVisualAcuityUserControl : BaseEventItem<VisualAcuityEventModel>
    {
        #region Implement Methods

        protected override void MapValidationGroup()
        {
            ucTestSummary.ValidationGroup = ValidationGroup;

            tbUSPerformDistance.ValidationGroup = ValidationGroup;
            rangeValUSPerformDistance.ValidationGroup = ValidationGroup;

            tbJaegerPerformDistance.ValidationGroup = ValidationGroup;
            rangeValJaegerPerformDistance.ValidationGroup = ValidationGroup;

            tbDecimalAcuity.ValidationGroup = ValidationGroup;
            regExValDecimalAcuity.ValidationGroup = ValidationGroup;

            tbMUnitAcuity.ValidationGroup = ValidationGroup;
            regExValMUnitAcuity.ValidationGroup = ValidationGroup;

            tbSphere.ValidationGroup = ValidationGroup;
            regExSphere.ValidationGroup = ValidationGroup;

            tbCylinder.ValidationGroup = ValidationGroup;
            regExCylinderFormat.ValidationGroup = ValidationGroup;
            regExCylinderDecimal.ValidationGroup = ValidationGroup;

            tbRefractionADD.ValidationGroup = ValidationGroup;
            regExADD.ValidationGroup = ValidationGroup;

            tbAxis.ValidationGroup = ValidationGroup;
            regExAxis.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = VisualAcuityEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            // Scale
            ucScale.Question = EventViewModel.Scale;
            ucScale.IsReadOnly = EventViewModel.IsReadOnly;
            ucScale.ExtendDdlOnItemDataBound = ucScale_OnItemDataBound;

            // US Scale
            MapUSScale();

            // Metric Scale
            MapMetricScale();

            // Jaeger Scale
            MapJaegerScale();

            // Decimal Scale
            MapDecimalScale();

            // M Unit Scale
            MapMUnitScale();

            // Non-Numeric Scale
            MapNonNumericScale();

            // Qualifiers
            MapQualifiers();

            // Refraction
            MapRefraction();
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Visual Acuity Event Model</returns>
        protected override VisualAcuityEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            // Scale
            EventViewModel.Scale = ucScale.GetUpdatedQuestion();

            // US Scale
            EventViewModel.USChartDistance = ucUSChartDistance.GetUpdatedQuestion();
            EventViewModel.USDifferentDistance = ucUSDifferentDistance.GetUpdatedQuestion();
            EventViewModel.USPerformDistance.ResponseText = GetNumberResponse(tbUSPerformDistance.Text);
            EventViewModel.USAcuity = ucUSAcuity.GetUpdatedQuestion();
            EventViewModel.USAdjustment = ucUSAdjustment.GetUpdatedQuestion();
            EventViewModel.USUnits = ucUSUnits.GetUpdatedQuestion();

            // Metric Scale
            EventViewModel.MetricChartDistance = ucMetricChartDistance.GetUpdatedQuestion();
            EventViewModel.MetricDifferentDistance = ucMetricDifferentDistance.GetUpdatedQuestion();
            EventViewModel.MetricPerformDistance = ucMetricPerformDistance.GetUpdatedQuestion();
            EventViewModel.MetricAcuity.ResponseText = GetNumberResponse(tbMetricAcuity.Text);
            EventViewModel.MetricAdjustment = ucMetricAdjustment.GetUpdatedQuestion();
            EventViewModel.MetricUnits = ucMetricUnits.GetUpdatedQuestion();

            // Jaeger Scale
            EventViewModel.JaegerPerformDistance.ResponseText = GetNumberResponse(tbJaegerPerformDistance.Text);
            EventViewModel.JaegerAcuity = ucJaegerAcuity.GetUpdatedQuestion();
            EventViewModel.JaegerAdjustment = ucJaegerAdjustment.GetUpdatedQuestion();
            EventViewModel.JaegerUnits = ucJaegerUnits.GetUpdatedQuestion();

            // Decimal Scale
            EventViewModel.DecimalAcuity.ResponseText = GetTextResponse(tbDecimalAcuity);
            EventViewModel.DecimalAdjustment = ucDecimalAdjustment.GetUpdatedQuestion();
            EventViewModel.DecimalUnits = ucDecimalUnits.GetUpdatedQuestion();

            // M Unit Scale
            EventViewModel.MUnitAcuity.ResponseText = GetTextResponse(tbMUnitAcuity);
            EventViewModel.MUnitAdjustment = ucMUnitAdjustment.GetUpdatedQuestion();
            EventViewModel.MUnitUnits = ucMUnitUnits.GetUpdatedQuestion();

            // Non-Numeric Scale - Handled by the grid

            // Qualifiers
            EventViewModel.Distance = ucDistance.GetUpdatedQuestion();
            EventViewModel.Correction = ucCorrection.GetUpdatedQuestion();
            EventViewModel.ExamMethod = ucExamMethod.GetUpdatedQuestion();
            EventViewModel.ChartType = ucChartType.GetUpdatedQuestion();
            EventViewModel.PrintSample = ucPrintSamples.GetUpdatedQuestion();
            EventViewModel.JaegerChartType = ucJaegerChartType.GetUpdatedQuestion();
            EventViewModel.JaegerPrintSamples = ucJaegerPrintSamples.GetUpdatedQuestion();

            // Refraction
            EventViewModel.Unknown = ucRefractionUnknown.GetUpdatedQuestion();
            EventViewModel.Sphere.ResponseText = tbSphere.Text;
            EventViewModel.Cylinder.ResponseText = GetTextResponse(tbCylinder);
            EventViewModel.Axis.ResponseText = GetTextResponse(tbAxis);
            EventViewModel.ADD.ResponseText = GetTextResponse(tbRefractionADD);
            EventViewModel.RefractionType = ucRefractionType.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void ucScale_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;

            e.Item.Attributes["ExpandUSScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.USScale).ToString().ToLower();

            e.Item.Attributes["ExpandMetricScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.MetricScale).ToString().ToLower();

            e.Item.Attributes["ExpandJaegerScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.JaegerScale).ToString().ToLower();

            e.Item.Attributes["ExpandDecimalScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.DecimalScale).ToString().ToLower();

            e.Item.Attributes["ExpandMUnitScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.MUnitScale).ToString().ToLower();

            e.Item.Attributes["ExpandNonNumericScale"] =
                (choice.Code == VisualAcuityEventModel.ChoiceCodes.Scale.NonNumericScale).ToString().ToLower();
        }

        protected void gvNonNumericAcuity_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
            {
                var lbtnNonNumericAcuityEdit = (LinkButton)e.Item.FindControl("lbtnNonNumericAcuityEdit");
                lbtnNonNumericAcuityEdit.ToolTip = string.Format("Click To Edit The {0} Selection", EventViewModel.NonNumericAcuity.DisplayText);

                var lbtnNonNumericAcuityDelete = (LinkButton)e.Item.FindControl("lbtnNonNumericAcuityDelete");
                lbtnNonNumericAcuityDelete.ToolTip = string.Format("Click To Delete The {0} Selection", EventViewModel.NonNumericAcuity.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as Choice;

                if (dataItem != null && dataItem.RequiresComment)
                {
                    if (VisualAcuityEventModel.EnableNonNumericAcuityValidation.Any(v => v == dataItem.Code))
                    {
                        var reqValidator = (RequiredFieldValidator) e.Item.FindControl("reqValNonNumericAdditionalDetails");
                        reqValidator.Enabled = true;

                        var compValidator = (CompareValidator) e.Item.FindControl("compValNonNumericAdditionalDetails");
                        compValidator.Enabled = true;

                        var tbNonNumericAcuityNumber = (TextBox)e.Item.FindControl("tbNonNumericAcuityNumber");
                        tbNonNumericAcuityNumber.Text = dataItem.Comment;
                        tbNonNumericAcuityNumber.Visible = true;
                    }
                    else
                    {
                        var tbNonNumericAcuityAdditionalDetails = (TextBox)e.Item.FindControl("tbNonNumericAcuityAdditionalDetails");
                        tbNonNumericAcuityAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.NonNumericAcuity.DisplayText);
                        tbNonNumericAcuityAdditionalDetails.Text = dataItem.Comment;
                        tbNonNumericAcuityAdditionalDetails.Visible = true;
                    }
                }
                    
                var lbtnNonNumericAcuityUpdate = (LinkButton)e.Item.FindControl("lbtnNonNumericAcuityUpdate");
                lbtnNonNumericAcuityUpdate.ToolTip = string.Format("Click To Update The {0} Selection", EventViewModel.NonNumericAcuity.DisplayText);

                var lbtnNonNumericAcuityCancel = (LinkButton)e.Item.FindControl("lbtnNonNumericAcuityCancel");
                lbtnNonNumericAcuityCancel.ToolTip = string.Format("Click To Cancel Updating The {0} Selection", EventViewModel.NonNumericAcuity.DisplayText);
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                var ddlNonNumericAcuity = (RadComboBox)e.Item.FindControl("ddlNonNumericAcuity");
                MapSelectField(ddlNonNumericAcuity, EventViewModel.NonNumericAcuity);

                var tbNonNumericAcuityAdditionalDetails = (TextBox)e.Item.FindControl("tbNonNumericAcuityAdditionalDetails");
                tbNonNumericAcuityAdditionalDetails.ToolTip = string.Format("Input Additional Details For The {0}", EventViewModel.NonNumericAcuity.DisplayText);

                var lbtnNonNumericAcuityAdd = (LinkButton)e.Item.FindControl("lbtnNonNumericAcuityAdd");
                lbtnNonNumericAcuityAdd.ToolTip = string.Format("Click To Add The {0} Selection", EventViewModel.NonNumericAcuity.DisplayText);
            }
        }

        protected void gvNonNumericAcuity_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvNonNumericAcuity.EditIndexes.Add(e.Item.ItemIndex);
                gvNonNumericAcuity.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.NonNumericAcuity.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvNonNumericAcuity.EditIndexes.Clear();
                gvNonNumericAcuity.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.NonNumericAcuity.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {
                        if (VisualAcuityEventModel.EnableNonNumericAcuityValidation.Any(v => v == selectedChoice.Code))
                        {
                            TextBox tbNonNumericAcuityNumber = (TextBox)e.Item.FindControl("tbNonNumericAcuityNumber");
                            comment = tbNonNumericAcuityNumber.Text;
                        }
                        else
                        {
                            TextBox tbNonNumericAcuityAdditionalDetails = (TextBox)e.Item.FindControl("tbNonNumericAcuityAdditionalDetails");
                            comment = tbNonNumericAcuityAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;
                }

                gvNonNumericAcuity.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                var comboBox = (RadComboBox)e.Item.FindControl("ddlNonNumericAcuity");
                var tbNonNumericAcuityNumber = (TextBox)e.Item.FindControl("tbNonNumericAcuityNumber");
                var tbNonNumericAcuityAdditionalDetails = (TextBox)e.Item.FindControl("tbNonNumericAcuityAdditionalDetails");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.NonNumericAcuity.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    string comment = string.Empty;

                    if (selectedChoice.RequiresComment)
                    {
                        if (VisualAcuityEventModel.EnableNonNumericAcuityValidation.Any(v => v == selectedChoice.Code))
                        {
                            comment = tbNonNumericAcuityNumber.Text;
                        }
                        else
                        {
                            comment = tbNonNumericAcuityAdditionalDetails.Text;
                        }
                    }

                    selectedChoice.Comment = comment;

                    EventViewModel.NonNumericAcuity.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbNonNumericAcuityAdditionalDetails.Text = string.Empty;
                    tbNonNumericAcuityNumber.Text = string.Empty;
                }
            }

            MapNonNumericScale();
        }

        protected void ddlNonNumericAcuity_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["ShowAdditionalDetails"] = "false";
            e.Item.Attributes["ShowNumber"] = "false";

            if (choice.RequiresComment)
            {
                if (VisualAcuityEventModel.EnableNonNumericAcuityValidation.Any(v => v == choice.Code))
                {
                    e.Item.Attributes["ShowNumber"] = "true";
                }
                else
                {
                    e.Item.Attributes["ShowAdditionalDetails"] = "true";
                }
            }
        }

        #endregion

        #region Private Methods

        private void MapUSScale()
        {
            lblUSChartDistance.Text = EventViewModel.USChartDistance.DisplayText;
            ucUSChartDistance.Question = EventViewModel.USChartDistance;
            ucUSChartDistance.IsReadOnly = EventViewModel.IsReadOnly;

            lblUSDifferentDistance.Text = EventViewModel.USDifferentDistance.DisplayText;
            ucUSDifferentDistance.Question = EventViewModel.USDifferentDistance;
            ucUSDifferentDistance.IsReadOnly = EventViewModel.IsReadOnly;

            MapTextField(tbUSPerformDistance, EventViewModel.USPerformDistance, EventViewModel.IsReadOnly);
            tbUSPerformDistance.ToolTip += " between 1 and 99";
            ucUSAcuity.Question = EventViewModel.USAcuity;
            ucUSAcuity.IsReadOnly = EventViewModel.IsReadOnly;

            ucUSAdjustment.Question = EventViewModel.USAdjustment;
            ucUSAdjustment.IsReadOnly = EventViewModel.IsReadOnly;

            ucUSUnits.Question = EventViewModel.USUnits;
            ucUSUnits.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapMetricScale()
        {
            lblMetricChartDistance.Text = EventViewModel.MetricChartDistance.DisplayText;
            ucMetricChartDistance.Question = EventViewModel.MetricChartDistance;
            ucMetricChartDistance.IsReadOnly = EventViewModel.IsReadOnly;

            lblMetricDifferentDistance.Text = EventViewModel.MetricDifferentDistance.DisplayText;
            ucMetricDifferentDistance.Question = EventViewModel.MetricDifferentDistance;
            ucMetricDifferentDistance.IsReadOnly = EventViewModel.IsReadOnly;

            lblMetricAcuity.Text = EventViewModel.MetricAcuity.DisplayText;
            ucMetricPerformDistance.Question = EventViewModel.MetricPerformDistance;
            ucMetricPerformDistance.IsReadOnly = EventViewModel.IsReadOnly;
            MapTextField(tbMetricAcuity, EventViewModel.MetricAcuity, EventViewModel.IsReadOnly);

            ucMetricAdjustment.Question = EventViewModel.MetricAdjustment;
            ucMetricAdjustment.IsReadOnly = EventViewModel.IsReadOnly;

            ucMetricUnits.Question = EventViewModel.MetricUnits;
            ucMetricUnits.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapJaegerScale()
        {
            lblJaegerPerformDistance.Text = EventViewModel.JaegerPerformDistance.DisplayText;
            MapTextField(tbJaegerPerformDistance, EventViewModel.JaegerPerformDistance, EventViewModel.IsReadOnly);
            tbJaegerPerformDistance.ToolTip += " between 1 and 99";

            ucJaegerAcuity.Question = EventViewModel.JaegerAcuity;
            ucJaegerAcuity.IsReadOnly = EventViewModel.IsReadOnly;

            ucJaegerAdjustment.Question = EventViewModel.JaegerAdjustment;
            ucJaegerAdjustment.IsReadOnly = EventViewModel.IsReadOnly;

            ucJaegerUnits.Question = EventViewModel.JaegerUnits;
            ucJaegerUnits.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapDecimalScale()
        {
            lblDecimalAcuity.Text = EventViewModel.DecimalAcuity.DisplayText;
            MapTextField(tbDecimalAcuity, EventViewModel.DecimalAcuity, EventViewModel.IsReadOnly);
            tbDecimalAcuity.ToolTip += " between 0.032 and 2.000";

            ucDecimalAdjustment.Question = EventViewModel.DecimalAdjustment;
            ucDecimalAdjustment.IsReadOnly = EventViewModel.IsReadOnly;

            ucDecimalUnits.Question = EventViewModel.DecimalUnits;
            ucDecimalUnits.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapMUnitScale()
        {
            lblMUnitAcuity.Text = EventViewModel.MUnitAcuity.DisplayText;
            MapTextField(tbMUnitAcuity, EventViewModel.MUnitAcuity, EventViewModel.IsReadOnly);
            tbMUnitAcuity.ToolTip += " in the format of #.# M/#.###";

            ucMUnitAdjustment.Question = EventViewModel.MUnitAdjustment;
            ucMUnitAdjustment.IsReadOnly = EventViewModel.IsReadOnly;

            ucMUnitUnits.Question = EventViewModel.MUnitUnits;
            ucMUnitUnits.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapNonNumericScale()
        {
            MapMultiSelectGrid(gvNonNumericAcuity, EventViewModel.NonNumericAcuity, EventViewModel.IsReadOnly);
        }

        private void MapQualifiers()
        {
            ucDistance.Question = EventViewModel.Distance;
            ucDistance.IsReadOnly = EventViewModel.IsReadOnly;

            ucCorrection.Question = EventViewModel.Correction;
            ucCorrection.IsReadOnly = EventViewModel.IsReadOnly;

            ucExamMethod.Question = EventViewModel.ExamMethod;
            ucExamMethod.IsReadOnly = EventViewModel.IsReadOnly;

            ucChartType.Question = EventViewModel.ChartType;
            ucChartType.IsReadOnly = EventViewModel.IsReadOnly;

            ucPrintSamples.Question = EventViewModel.PrintSample;
            ucPrintSamples.IsReadOnly = EventViewModel.IsReadOnly;

            ucJaegerChartType.Question = EventViewModel.JaegerChartType;
            ucJaegerChartType.IsReadOnly = EventViewModel.IsReadOnly;

            ucJaegerPrintSamples.Question = EventViewModel.JaegerPrintSamples;
            ucJaegerPrintSamples.IsReadOnly = EventViewModel.IsReadOnly;
        }

        private void MapRefraction()
        {
            lblRefractionUnknown.Text = EventViewModel.Unknown.DisplayText;
            ucRefractionUnknown.Question = EventViewModel.Unknown;

            lblADD.Text = EventViewModel.ADD.DisplayText;
            MapTextField(tbRefractionADD, EventViewModel.ADD, EventViewModel.IsReadOnly);
            tbRefractionADD.ToolTip += " in the format of +##.##";

            ucRefractionType.Question = EventViewModel.RefractionType;

            lblSphere.Text = EventViewModel.Sphere.DisplayText;
            MapTextField(tbSphere, EventViewModel.Sphere, EventViewModel.IsReadOnly);
            tbSphere.ToolTip += " in the format of (+/-)##.##";

            lblCylinder.Text = EventViewModel.Cylinder.DisplayText;
            MapTextField(tbCylinder, EventViewModel.Cylinder, EventViewModel.IsReadOnly);
            tbCylinder.ToolTip += " between -19.00 and +19.75";

            lblAxis.Text = EventViewModel.Axis.DisplayText;
            MapTextField(tbAxis, EventViewModel.Axis, EventViewModel.IsReadOnly);
            tbAxis.ToolTip += " in the format of ###";
        }

        #endregion
    }
}