﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using Managers.Encounter;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;

public partial class controls_ReviewComment_ReviewerCommentControl : UserControl
{
    #region Member Variables

    public ReviewerCommentViewModel ReviewerCommentViewModel
    {
        get { return ViewState["ReviewerCommentViewModel"] as ReviewerCommentViewModel; }
        set { ViewState["ReviewerCommentViewModel"] = value; }
    }

    public IEnumerable<string> AjaxUpdateControls
    {
        get { return ViewState["AjaxUpdateControls"] as IEnumerable<string>; }
        set { ViewState["AjaxUpdateControls"] = value != null ? value.ToList() : new List<string>(); }
    } 

    public event EventHandler ReviewSavedSuccessfully;

    #endregion

    /// <summary>Default Page Load</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            LoadControl();
        }

        LoadAjaxSetting();
    }

    /// <summary>Loads the control.</summary>
    public void RefreshControl()
    {
        LoadControl();
        LoadAjaxSetting();
    }

    public void LoadControl()
    {
        PanelReviewComment.Visible = ReviewerCommentViewModel.ShowControl;
        TextBoxReviewComment.Text = ReviewerCommentViewModel.CommentText;
        LabelReviewCommentReadOnly.Text = ReviewerCommentViewModel.CommentText;

        CheckBoxResolved.Checked = ReviewerCommentViewModel.IsResolved;
        CheckBoxApproved.Checked = ReviewerCommentViewModel.IsApproved;

        ImageButtonCommentOpen.Visible = ReviewerCommentViewModel.ShowOpenButton;
        ImageButtonCommentAdd.Visible = !ReviewerCommentViewModel.ShowOpenButton;

        if (ReviewerCommentViewModel.ShowCommentText)
        {
            PanelReviewCommentsContent.Style.Add("display", "block");

            if (ReviewerCommentViewModel.SetFocus)
            {
                TextBoxReviewComment.Focus();
            }
        }
        else
        {
            PanelReviewCommentsContent.Style.Remove("display");
            PanelReviewCommentsContent.Style.Add("display", "none");
        }

        //set control state
        CheckBoxApproved.Enabled = ReviewerCommentViewModel.CanApprove;
        CheckBoxResolved.Enabled = ReviewerCommentViewModel.CanResolve;

        TextBoxReviewComment.Visible = ReviewerCommentViewModel.CanEditComment;
        LabelReviewCommentReadOnly.Visible = !ReviewerCommentViewModel.CanEditComment;
    }

    /// <summary>Save Comment</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="ImageClickEventArgs"/> instance containing the event data.</param>
    protected void ImageButtonCommentReturnToList_Click(object sender, ImageClickEventArgs e)
    {
        // Update View Model
        ReviewerCommentViewModel.CommentText = TextBoxReviewComment.Text;

        ReviewerCommentViewModel.IsApproved = CheckBoxApproved.Checked;
        ReviewerCommentViewModel.IsResolved = CheckBoxResolved.Checked;

        var viewManager = new ReviewerCommentViewManager(new UserAccountManager(ReviewerCommentViewModel.UserName, string.Empty));
        viewManager.SaveReviewerComment(ReviewerCommentViewModel);
        ReviewerCommentViewModel = viewManager.RefreshCommentViewModel(ReviewerCommentViewModel);

        // Hide the comment text
        ReviewerCommentViewModel.ShowCommentText = false;
        ReviewerCommentViewModel.SetFocus = false;
        
        RefreshControl();

        if (ReviewSavedSuccessfully != null)
        {
            ReviewSavedSuccessfully(sender, e);
        }
    }

    /// <summary>Loads the ajax setting.</summary>
    private void LoadAjaxSetting()
    {
        if (ReviewerCommentViewModel.ShowControl)
        {
            var ajaxManager = RadAjaxManager.GetCurrent(this.Page);
            ajaxManager.AjaxSettings.AddAjaxSetting(ImageButtonCommentReturnToList, PanelReviewComment, ajaxLoadingPnlReviewerCmt);

            if (AjaxUpdateControls != null)
            {
                foreach (var ctrlId in AjaxUpdateControls)
                {
                    Control ctrl = FindParentControl(this, ctrlId);

                    if (ctrl != null)
                    {
                        ajaxManager.AjaxSettings.AddAjaxSetting(ImageButtonCommentReturnToList, ctrl);
                    }
                }
            }
        }
    }

    /// <summary>Finds the parent control. (This is a cheat and assumes the control id is unique)</summary>
    /// <param name="currentCtrl">The current control.</param>
    /// <param name="ctrlId">The control identifier.</param>
    /// <returns>The searched control</returns>
    private Control FindParentControl(Control currentCtrl, string ctrlId)
    {
        var searchedCtrl = currentCtrl.FindControl(ctrlId);

        if (searchedCtrl != null)
        {
            return searchedCtrl;
        }
        
        if (currentCtrl.Parent != null)
        {
            return FindParentControl(currentCtrl.Parent, ctrlId);
        }

        return null;
    }
}