﻿using System;
using System.Linq;
using BusinessManagerClasses;
using Managers;
using Telerik.Web.UI;
using ViewModels;

public partial class controls_WorkflowAdministration_WorkflowAdministrationUserControl : BaseWebControl
{
    #region Member Variables

    public string CurrentUserName { get; set; }

    public MyTasksViewModel.WorkflowAdminsModel ViewModel
    {
        get { return ViewState["ViewModel"] as MyTasksViewModel.WorkflowAdminsModel; }
        set { ViewState["ViewModel"] = value; }
    }

    public event EventHandler EncounterAssignmentChanged;

    public event EventHandler EncounterStatusChanged;

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            PopulateDataAbstractorUsers();

            rcbCurrentEncounterStatus.DataSource = ViewModel.ReassignEncounterStatus;
            rcbCurrentEncounterStatus.DataTextField = "Value";
            rcbCurrentEncounterStatus.DataValueField = "Key";
            rcbCurrentEncounterStatus.DataBind();
        }
    }

    #region GUI Events

    /*********************************************************************
     * Encounter Reassignment Methods
     *********************************************************************/

    /// <summary>
    /// Data Abstractor List selected index changed.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbDataAbstractors_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        // show all tasking for abstractor
        if (rcbDataAbstractors.SelectedIndex > -1)
        {
            pnlAbstractorWorkload.GroupingText = "Abstractor Workload (" + rcbDataAbstractors.SelectedItem.Text + ") ";
            pnlAbstractorWorkload.Visible = true;

            FillDataAbstractorWorkloadGrid();
            rgDataAbstractorEncounters.DataBind();
            rgDataAbstractorEncounters.Focus();
        }
        else
        {
            pnlAbstractorWorkload.Visible = false;
        }
    }

    /// <summary>Perform Reassignment</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void rbCancelAssignmentList_Click(object sender, EventArgs e)
    {
        ViewModel.DataAbstractorEncounters = new MyTasksViewModel.EncounterSummary[0];

        rcbDataAbstractors.ClearSelection();

        rgDataAbstractorEncounters.DataSource = ViewModel.DataAbstractorEncounters;
        rgDataAbstractorEncounters.DataBind();

        pnlAbstractorWorkload.Visible = false;
    }

    /*********************************************************************
     * Encounter Status Changing Methods
     *********************************************************************/

    /// <summary>
    /// Search button clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbSearchEncounterID_Click(object sender, EventArgs e)
    {
        pnlEncounterInfo.Visible = false;
        pnlEncounterInfoNotFound.Visible = false;
        
        int encounterId;

        if (int.TryParse(rtbEncounterIDSearch.Text, out encounterId))
        {
            MyTasksViewManager manager = new MyTasksViewManager(CurrentUser);
            ViewModel = manager.GetEncounterForStatusChange(ViewModel, encounterId);

            if (ViewModel.EncounterStatusChangeEncounterId.HasValue)
            {
                rcbCurrentEncounterStatus.SelectedValue = ViewModel.EncounterStatusChangeStatusId.ToString();

                lblCurrentEncounterAssignedTo.Text = ViewModel.EncounterStatusChangeAssignedToId;

                pnlEncounterInfo.Visible = true;
            }
            else
            {
                pnlEncounterInfoNotFound.Visible = true;
            }
        }
        else
        {
            pnlEncounterInfoNotFound.Visible = true;
        }
    }

    /// <summary>
    /// Change Status Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbChangeStatus_Click(object sender, EventArgs e)
    {
        int statusChangeId;
        // get updated encounter status
        if (ViewModel.EncounterStatusChangeEncounterId.HasValue && int.TryParse(rcbCurrentEncounterStatus.SelectedValue, out statusChangeId))
        {
            var manager = new MyTasksViewManager(CurrentUser);
            ViewModel = manager.SaveEncounterStatus(ViewModel, ViewModel.EncounterStatusChangeEncounterId.Value, statusChangeId);

            // update and close
            lblCurrentEncounterAssignedTo.Text = "";
            rcbCurrentEncounterStatus.ClearSelection();
            rtbEncounterIDSearch.Text = "";

            pnlEncounterInfo.Visible = false;

            FillDataAbstractorWorkloadGrid();

            // Run event
            if (EncounterStatusChanged != null)
            {
                EncounterStatusChanged(sender, e);
            }
        }
    }

    /// <summary>
    /// Cancel Status Change
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbCancelChangeStatus_Click(object sender, EventArgs e)
    {
        // update and close
        lblCurrentEncounterAssignedTo.Text = "";
        rcbCurrentEncounterStatus.ClearSelection();
        rtbEncounterIDSearch.Text = "";

        pnlEncounterInfo.Visible = false;
        pnlEncounterInfoNotFound.Visible = false;
    }

    #endregion

    #region Data Abstractor Workload Grid Events

    /// <summary>
    /// Called when grid needs data source
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgDataAbstractorEncounters_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgDataAbstractorEncounters.DataSource = ViewModel.DataAbstractorEncounters;
    }

    protected void rgDataAbstractorEncounters_OnItemDataBound(object sender, GridItemEventArgs e)
    {
        if (e.Item.ItemType == GridItemType.EditItem)
        {
            var rcbAssignEncounter = (RadComboBox)e.Item.FindControl("rcbAssignEncounter");
            rcbAssignEncounter.DataSource = ViewModel.AssignableUsers;
            rcbAssignEncounter.DataValueField = "Id";
            rcbAssignEncounter.DataTextField = "FullName";
            rcbAssignEncounter.DataBind();
            rcbAssignEncounter.Focus();
        }
    }

    protected void rgDataAbstractorEncounters_OnItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "EditAssignment")
        {
            rgDataAbstractorEncounters.EditIndexes.Add(e.Item.ItemIndex);
            rgDataAbstractorEncounters.DataSource = ViewModel.DataAbstractorEncounters;
            rgDataAbstractorEncounters.DataBind();
        }

        if (e.CommandName == "UpdateAssignment")
        {
            int? assignedUserId;
            int encId = (int)((GridDataItem)e.Item).GetDataKeyValue("EncounterId");
            int referralId = (int)((GridDataItem)e.Item).GetDataKeyValue("ReferralId");

            // Get New Assigned User Id
            RadComboBox rcbAssignEncounter = (RadComboBox)e.Item.FindControl("rcbAssignEncounter");
            string selectedUser = rcbAssignEncounter.SelectedValue;
            string selectedName = rcbAssignEncounter.SelectedItem.Text;
            assignedUserId = string.IsNullOrEmpty(selectedUser) ? (int?)null : int.Parse(selectedUser);

            // Save new Assignment
            var viewManager = new MyTasksViewManager(CurrentUser);
            ViewModel = viewManager.UpdateEncounterAssignment(ViewModel, encId, assignedUserId);

            // Create logging record
            CreateLoggingRecord(referralId, encId, DVEIRUserActivityType.ENCOUNTER_ASSIGNMENT_CHANGE, "Assigned To: " + selectedName);

            // Refresh Encounter List
            rgDataAbstractorEncounters.EditIndexes.Clear();
            FillDataAbstractorWorkloadGrid();

            // update and close
            lblCurrentEncounterAssignedTo.Text = "";
            rcbCurrentEncounterStatus.ClearSelection();
            rtbEncounterIDSearch.Text = "";

            pnlEncounterInfo.Visible = false;
            pnlEncounterInfoNotFound.Visible = false;

            // Run event
            if (EncounterStatusChanged != null)
            {
                EncounterStatusChanged(sender, e);
            }
        }

        if (e.CommandName == "CancelAssignment")
        {
            rgDataAbstractorEncounters.EditIndexes.Clear();
            rgDataAbstractorEncounters.DataSource = ViewModel.DataAbstractorEncounters;
            rgDataAbstractorEncounters.DataBind();
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Populate dropdown list for data abstractor users
    /// </summary>
    private void PopulateDataAbstractorUsers()
    {
        rcbDataAbstractors.DataSource = ViewModel.AssignableUsers;
        rcbDataAbstractors.DataTextField = "FullName";
        rcbDataAbstractors.DataValueField = "Id";
        rcbDataAbstractors.DataBind();
    }

    /// <summary>
    /// Populates the Grid with Data.
    /// </summary>
    private void FillDataAbstractorWorkloadGrid()
    {
        try
        {
            // User Id
            int userId;

            int.TryParse(rcbDataAbstractors.SelectedValue, out userId);

            var manager = new MyTasksViewManager(CurrentUser);
            ViewModel = manager.GetDataAbstractorEncounters(ViewModel, userId);

            rgDataAbstractorEncounters.DataSource = ViewModel.DataAbstractorEncounters;
            rgDataAbstractorEncounters.DataBind();
        }
        catch (Exception ex)
        {
            rgDataAbstractorEncounters.DataSource = null;
        }
    }

    #endregion
}