﻿using Managers.Encounter;
using Managers.Referral;
using System;
using System.Web.UI.WebControls;
using Managers;
using Telerik.Web.UI;
using VeteransAffairs.Registries.Business;
using ViewModels;
using ViewModels.Encounter;
using EncounterStatusType = ViewModels.Encounter.EncounterStatusType;

public partial class MyTasks : BasePage
{
    #region Member Variables

    public MyTasksViewModel ViewModel
    {
        get { return ViewState["ViewModel"] as MyTasksViewModel; }
        set { ViewState["ViewModel"] = value; }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("My Tasks");

        if (!IsPostBack)
        {
            var myTaskManager = new MyTasksViewManager(CurrentUser);
            ViewModel = myTaskManager.GetMyTaskViewModel();

            #region Hide/Show Panel Regions

            pnlNoAccess.Visible = ViewModel.ShowNoAccess;
            pnlReadyForReview.Visible = ViewModel.ShowReviewerTasks;
            pnlReferralEncounters.Visible = ViewModel.ShowDataAbstractorTasks;
            pnlDataAbstrationFilterZone.Visible = ViewModel.ShowDataAbstractorTasks;
            pnlAdministrationArea.Visible = ViewModel.ShowAdministratorTasks;
            WorkflowAdmin.CurrentUserName = CurrentUser.UserName;
            WorkflowAdmin.ViewModel = ViewModel.WorkflowAdmin;

            rcbEncounterStatus.DataValueField = "Key";
            rcbEncounterStatus.DataTextField = "Value";
            rcbEncounterStatus.DataSource = ViewModel.EncounterStatusesFilters;
            rcbEncounterStatus.DataBind();

            rgMyEncounters.DataSource = ViewModel.DataAbstractorEncounters;
            rgMyEncounters.DataBind();

            rgReviewer.DataSource = ViewModel.ReviewableEncounters;
            rgReviewer.DataBind();

            #endregion
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "My Tasks";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region Grid Events

    /// <summary>
    /// Fired upon item command
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int refId = (int) ((GridDataItem) e.Item).GetDataKeyValue("ReferralId");
            int encId = (int) ((GridDataItem) e.Item).GetDataKeyValue("EncounterId");
            this.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", refId, encId));
        }
    }

    /// <summary>
    /// On Need Data Source for Grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        rgMyEncounters.DataSource = ViewModel.DataAbstractorEncounters;
    }

    /// <summary>
    /// On Item Data Bound
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;

                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Select should show for all rows
                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                ////LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                ////bool showSelect = false;

                ////var summary = item.DataItem as MyTasksViewModel.EncounterSummary;

                ////if (summary != null)
                ////{
                ////    switch ((EncounterStatusType)summary.EncounterStatusId)
                ////    {
                ////        case EncounterStatusType.New:
                ////        case EncounterStatusType.InProc:
                ////        case EncounterStatusType.SendBack:
                ////            showSelect = true;
                ////            break;
                ////    }
                ////}

                ////drillButton.Visible = showSelect;
            }
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }
    }

    #endregion

    #region Reviewer Grid Events

    protected void rgReviewer_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgReviewer.DataSource = ViewModel.ReviewableEncounters;
    }

    protected void rgReviewer_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int refId = (int)((GridDataItem)e.Item).GetDataKeyValue("ReferralId");
            int encId = (int)((GridDataItem)e.Item).GetDataKeyValue("EncounterId");
            this.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", refId, encId));
        }
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Encounter Status Change
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbEncounterStatus_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        int selectedStatus;
        int.TryParse(rcbEncounterStatus.SelectedValue, out selectedStatus);
        ViewModel.SelectedEncounterStatus = selectedStatus;
        rgMyEncounters.DataSource = ViewModel.DataAbstractorEncounters;
        rgMyEncounters.DataBind();
    }

    /// <summary>
    /// Redirect to page
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbRefreshPage_Click(object sender, EventArgs e)
    {
        this.Redirect("MyTasks.aspx");
    }

    protected void WorkflowAdmin_OnEncounterAssignmentChanged(object sender, EventArgs e)
    {
        // Refresh Data Abstractor Tasks
        var manager = new MyTasksViewManager(CurrentUser);
        ViewModel = manager.RefreshDataAbstractorTasks(ViewModel);

        rgMyEncounters.DataSource = ViewModel.DataAbstractorEncounters;
        rgMyEncounters.DataBind();
    }

    protected void WorkflowAdmin_OnEncounterStatusChanged(object sender, EventArgs e)
    {
        // Refresh Data Abstractor Tasks and Reviewerable Encounters
        var manager = new MyTasksViewManager(CurrentUser);
        ViewModel = manager.RefreshDataAbstractorTasks(ViewModel);
        ViewModel = manager.RefreshReviewerTasks(ViewModel);

        rgMyEncounters.DataSource = ViewModel.DataAbstractorEncounters;
        rgMyEncounters.DataBind();

        rgReviewer.DataSource = ViewModel.ReviewableEncounters;
        rgReviewer.DataBind();
    }

    #endregion
}