﻿using BusinessManagerClasses;
using Managers.Referral;
using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels.Referral;

public partial class ReferralEncounters : BasePage
{
    #region Member Variables

    /// <summary>Gets or sets the referral view model.</summary>
    /// <value>The referral view model.</value>
    public ReferralDetailViewModel ReferralViewModel
    {
        get { return ViewState["ReferralDetailViewModel"] as ReferralDetailViewModel; }
        set { ViewState["ReferralDetailViewModel"] = value; }
    }

    #endregion

    #region GUI Overrides

    /// <summary>
    /// TopMenuName is the menu item in the top horizontal menu where this page falls under
    /// </summary>
    /// <returns></returns>
    public override string TopMenuName()
    {
        return "Referrals";
    }

    /// <summary>MenuItemName is the value of the left menu link for this page</summary>
    /// <returns></returns>
    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    /// <summary>
    /// The list of capabilities that a user needs to access this page.  A single page may have more than one
    /// capability associated with it.
    /// </summary>
    /// <returns></returns>
    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region Page Events

    /// <summary>Default Page Load method</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
    /// <exception cref="System.Exception">Required Parameter Exception
    /// or
    /// Required Parameter Exception</exception>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);

            if (!IsPostBack)
            {
                int referralId = 0;
                string qsReferralId = Request.QueryString.Get("id");

                if (Int32.TryParse(qsReferralId, out referralId))
                {
                    // set label
                    ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("<a href=\"\\Referrals.aspx\" title=\"Back to Referral List Page\" style=\"color:#FFFFFF;\">Referrals</a> > Encounters for Referral (" + referralId + ")");

                    var viewManager = new ReferralViewManager(CurrentUser);
                    ReferralViewModel = viewManager.GetReferralDetailViewModel(referralId);

                    FillReportGrid();
                    rgReferralEncounters.DataBind();

                    FillProblemListsGrids();

                    // Create logging record
                    CreateLoggingRecord(ReferralViewModel.ReferralId, ReferralViewModel.PatientName, DVEIRUserActivityType.REFERRAL_OPEN, string.Empty);
                }
                else
                {
                    throw new Exception("Required Parameter Exception");
                }
            }
        }
        catch (Exception ex)
        {
            throw;
        }
    }

    /// <summary>Encounter Status Change</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="RadComboBoxSelectedIndexChangedEventArgs" /> instance containing the event data.</param>
    protected void rcbEncounterStatus_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        FillReportGrid();
        rgReferralEncounters.DataBind();
    }

    /// <summary>Event for Add Encounter Button Click</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
    protected void btnAddEncounter_Click(object sender, EventArgs e)
    {
        pnlReferralEncounters.Visible = false;
        pnlAddEncounter.Visible = true;

        lblReferralId.Text = ReferralViewModel.ReferralId.ToString();
    }

    /// <summary>Save Encounter Click</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
    protected void rbSaveEncounterRecord_Click(object sender, EventArgs e)
    {
        try
        {
            if (Page.IsValid)
            {
                int referralId = ReferralViewModel.ReferralId;
                int institution = Convert.ToInt32(rcbInstitution.SelectedValue);
                DateTime visitDate = rdtpVisitDate.SelectedDate ?? DateTime.Now;
                string encounterType = rcbEncounterType.SelectedValue;


                var viewManger = new ReferralViewManager(CurrentUser);
                int encounterId = viewManger.AddNewEncounter(referralId, institution, visitDate, encounterType);
                ReferralViewModel = viewManger.RefreshEncounterSummaries(ReferralViewModel);

                // Create logging record
                CreateLoggingRecord(ReferralViewModel.ReferralId, ReferralViewModel.PatientName, DVEIRUserActivityType.ENCOUNTER_ADD, string.Format("Added Encounter #{0}", encounterId));

                FillReportGrid();
                rgReferralEncounters.DataBind();

                clearForm();

                pnlReferralEncounters.Visible = true;
                pnlAddEncounter.Visible = false;
            }
        }
        catch (Exception ex)
        {
            // TODO: Display Error Label for user ... 
            Logging.WriteLogToFile("Error Manually Creating Encounter Record: " + ex.Message);
        }
    }

    /// <summary>Cancel Encounter creation</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void rbCancelEncounterRecord_Click(object sender, EventArgs e)
    {
        // clear form
        clearForm();

        pnlReferralEncounters.Visible = true;
        pnlAddEncounter.Visible = false;
    }

    #endregion

    #region Grid Events

    /// <summary>Handles the OnItemDataBound event of the rgReferralEncounters control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridItemEventArgs"/> instance containing the event data.</param>
    protected void rgReferralEncounters_OnItemDataBound(object sender, GridItemEventArgs e)
    {
        bool isAdmin = ReferralViewModel.IsAdminUser;
        if (e.Item.ItemType == GridItemType.Item || e.Item.ItemType == GridItemType.AlternatingItem)
        {
            if (isAdmin)
            {
                var summary = (ReferralDetailViewModel.EncounterSummary)e.Item.DataItem;
                Label lblAssignToName = (Label)e.Item.FindControl("lblAssignedToName");
                LinkButton lbtnChangeAssignmentName = (LinkButton)e.Item.FindControl("lbtnChangeAssignmentName");

                lblAssignToName.Visible = false;

                lbtnChangeAssignmentName.Text =
                    string.IsNullOrEmpty(summary.AssignedToUserName)
                        ? "(Assign Encounter)"
                        : summary.AssignedToUserName + " (change)";

                lbtnChangeAssignmentName.Visible = true;
            }
        }

        if (e.Item.ItemType == GridItemType.EditItem)
        {
            if (isAdmin)
            {
                RadComboBox rcbAssignEncounter = (RadComboBox)e.Item.FindControl("rcbAssignEncounter");
                rcbAssignEncounter.DataSource = ReferralViewModel.AssignableUsers;
                rcbAssignEncounter.DataValueField = "Id";
                rcbAssignEncounter.DataTextField = "FullName";
                rcbAssignEncounter.DataBind();

                var summary = (ReferralDetailViewModel.EncounterSummary)e.Item.DataItem;
                rcbAssignEncounter.SelectedValue = summary.AssignedToUserId.ToString();
            }
        }
    }

    /// <summary>On select button pushed.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridCommandEventArgs"/> instance containing the event data.</param>
    protected void rgReferralEncounters_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int encId = (int)((GridDataItem)e.Item).GetDataKeyValue("Id");
            this.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", ReferralViewModel.ReferralId, encId));
        }

        if (e.CommandName == "EditAssignment")
        {
            rgReferralEncounters.EditIndexes.Clear();
            rgReferralEncounters.EditIndexes.Add(e.Item.ItemIndex);
            FillReportGrid();
            rgReferralEncounters.DataBind();
        }

        if (e.CommandName == "UpdateAssignment")
        {
            int? assignedUserId;
            int encId = (int)((GridDataItem)e.Item).GetDataKeyValue("Id");

            // Get New Assigned User Id
            RadComboBox rcbAssignEncounter = (RadComboBox)e.Item.FindControl("rcbAssignEncounter");
            string selectedUser = rcbAssignEncounter.SelectedValue;
            string selectedName = rcbAssignEncounter.SelectedItem != null ? rcbAssignEncounter.SelectedItem.Text : "Not Assigned";
            assignedUserId = string.IsNullOrEmpty(selectedUser) ? (int?)null : int.Parse(selectedUser);

            // Save new Assignment
            var viewManager = new ReferralViewManager(CurrentUser);
            viewManager.UpdateEncounterAssignment(ReferralViewModel.ReferralId, encId, assignedUserId);

            // Create logging record
            CreateLoggingRecord(ReferralViewModel.ReferralId, encId, ReferralViewModel.PatientName, DVEIRUserActivityType.ENCOUNTER_ASSIGNMENT_CHANGE, "Assigned To: " + selectedName);

            // Refresh Encounter List
            ReferralViewModel = viewManager.RefreshEncounterSummaries(ReferralViewModel);
            rgReferralEncounters.EditIndexes.Clear();
            FillReportGrid();
            rgReferralEncounters.DataBind();
        }

        if (e.CommandName == "CancelAssignment")
        {
            rgReferralEncounters.EditIndexes.Clear();
            FillReportGrid();
            rgReferralEncounters.DataBind();
        }
    }

    /// <summary>Datasource needed.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridNeedDataSourceEventArgs"/> instance containing the event data.</param>
    protected void rgReferralEncounters_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    /// <summary>Handles the OnNeedDataSource event of the rgActiveProblems control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridNeedDataSourceEventArgs"/> instance containing the event data.</param>
    protected void rgActiveProblems_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgActiveProblems.DataSource = ReferralViewModel.ActiveProblems;
    }

    /// <summary>Handles the OnNeedDataSource event of the rgInactiveProblems control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridNeedDataSourceEventArgs"/> instance containing the event data.</param>
    protected void rgInactiveProblems_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgInactiveProblems.DataSource = ReferralViewModel.InactiveProblems;
    }

    #endregion

    #region Private Methods

    /// <summary>Populates the Grid with Data.</summary>
    private void FillReportGrid()
    {
        try
        {
            //status
            int? statusId;

            string status = rcbEncounterStatus.SelectedValue;

            switch (status.ToLower())
            {
                case "all":
                    statusId = null;
                    break;
                case "new":
                    statusId = 1;
                    break;
                case "inproc":
                    statusId = 2;
                    break;
                case "review":
                    statusId = 3;
                    break;
                case "returned":
                    statusId = 4;
                    break;
                case "send":
                    statusId = 5;
                    break;
                case "completed":
                    statusId = 6;
                    break;
                default:
                    statusId = 0;
                    break;
            }

            rgReferralEncounters.DataSource = statusId.HasValue ?
                ReferralViewModel.Encounters.Where(e => e.StatusId == statusId)
                : ReferralViewModel.Encounters;
        }
        catch (Exception ex)
        {
            rgReferralEncounters.DataSource = null;
        }
    }

    /// <summary>Clear Add Encounter Form values.</summary>
    private void clearForm()
    {
        lblReferralId.Text = "";
        rcbInstitution.ClearSelection();
        rcbInstitution.Text = "";
        rdtpVisitDate.Clear();
        rcbEncounterType.ClearSelection();
    }

    /// <summary>Fills the problem lists grids.</summary>
    private void FillProblemListsGrids()
    {
        rgActiveProblems.DataSource = ReferralViewModel.ActiveProblems;
        rgActiveProblems.DataBind();

        rgInactiveProblems.DataSource = ReferralViewModel.InactiveProblems;
        rgInactiveProblems.DataBind();
    }

    #endregion
}