﻿using System;
using BusinessManagerClasses;
using Managers.Referral;
using Telerik.Web.UI;
using ViewModels.Referral;

public partial class Referrals : BasePage
{
    #region Member Variables

    public ReferralSearchViewModel ViewModel
    {
        get { return ViewState["ReferralSearchViewModel"] as ReferralSearchViewModel; }
        set { ViewState["ReferralSearchViewModel"] = value; }
    }

    #endregion

    /// <summary>
    /// Default Page Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("Referrals");

        if (!IsPostBack)
        {
            ViewModel = new ReferralSearchViewManager(CurrentUser).GetReferralSearchViewModel();
            LoadVISNList();
            LoadEncounterStatusList();

            rgReferrals.DataSource = "";
            rgReferrals.DataBind();
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Referrals";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region GUI Events

    /// <summary>VISN Dropdown selected index changed.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="RadComboBoxSelectedIndexChangedEventArgs"/> instance containing the event data.</param>
    protected void rcbVISN_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        int selecteVisn;
        int.TryParse(rcbVISN.SelectedValue, out selecteVisn);
        ViewModel.SelectedVisn = selecteVisn;

        rcbFac.ClearSelection();

        if (selecteVisn > 0)
        {
            rcbFac.Enabled = true;
            LoadFacilityList();
        }
        else
        {
            rcbFac.Enabled = false;
        }
    }

    /// <summary>
    /// Search Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbSearch_Click(object sender, EventArgs e)
    {
        int encounterStatusId;
        int visnId;
        int facilityId;
        string patientNameSSN;

        // Get inputs
        int.TryParse(rcbReferralType.SelectedValue, out encounterStatusId);
        int.TryParse(rcbVISN.SelectedValue, out visnId);
        int.TryParse(rcbFac.SelectedValue, out facilityId);
        patientNameSSN = rtbPatientNameSSN.Text;

        // Update View Model
        ViewModel.SelectedEncounterStatus = encounterStatusId;
        ViewModel.SelectedVisn = visnId;
        ViewModel.SelectedFacility = facilityId;
        ViewModel.PatientNameSSN = patientNameSSN;

        // Fill the grid
        FillReportGrid();
        CreateLoggingRecord(DVEIRUserActivityType.PATIENT_SEARCH, string.Empty);
    }

    #endregion

    #region Grid Events

    protected void rgReferrals_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgReferrals.DataSource = ViewModel.ReferralSearchSummaries;
    }

    protected void rgReferrals_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int refId = (int)((GridDataItem)e.Item).GetDataKeyValue("ReferralId");
            if (ViewModel.SelectedEncounterStatus > 0)
            {
                this.Redirect(string.Format("ReferralEncounters.aspx?id={0}&status={1}", refId, ViewModel.SelectedEncounterStatus));
            }
            else
            {
                this.Redirect(string.Format("ReferralEncounters.aspx?id={0}", refId));
            }
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Loads the VISN Dropdown List
    /// </summary>
    private void LoadVISNList()
    {
        rcbVISN.DataSource = ViewModel.VISNs;
        rcbVISN.DataTextField = "Value";
        rcbVISN.DataValueField = "Key";
        rcbVISN.DataBind();
    }

    private void LoadEncounterStatusList()
    {
        rcbReferralType.DataSource = ViewModel.EncounterStatuses;
        rcbReferralType.DataTextField = "Value";
        rcbReferralType.DataValueField = "Key";
        rcbReferralType.DataBind();
    }

    /// <summary>
    /// call sproc to return facility list
    /// </summary>
    private void LoadFacilityList()
    {
        var viewManager = new ReferralSearchViewManager(CurrentUser);
        ViewModel = viewManager.RefreshFacilities(ViewModel);

        rcbFac.DataSource = ViewModel.Facilities;
        rcbFac.DataTextField = "Value";
        rcbFac.DataValueField = "Key";
        rcbFac.DataBind();
    }

    /// <summary>
    /// Populates the Grid with Data.
    /// </summary>
    private void FillReportGrid()
    {
        var viewManager = new ReferralSearchViewManager(CurrentUser);
        ViewModel = viewManager.RefreshReferralSummaries(ViewModel);

        rgReferrals.DataSource = ViewModel.ReferralSearchSummaries;
        rgReferrals.DataBind();
    }

    #endregion
}