﻿using Managers.Reporting;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reporting_AnnualCompletionReport : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryReportingMaster)Page.Master).SetDirtyFlag(false);

        if (!IsPostBack)
        {
            // set label
            ((RegistryReportingMaster)Page.Master).SetBreadCrumbLabel("Annual Completion Report");

            DataSourceFilters();
            InitializeFilters();
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "MEVIR_ANNUAL_COMPLETION_REPORT";
    }

    public override string AppObjectCode()
    {
        return "MEVIR_ANNUAL_COMPLETION_REPORT";
    }

    #endregion

    #region Private Methods

    private void DataSourceFilters()
    {
        // VISNS
        // call [dbo].[usp_GetVISNs] -- ID, NAME
        rcbVISNs.DataSource = ReportManager.GetVISNs();
        rcbVISNs.DataTextField = "NAME";
        rcbVISNs.DataValueField = "ID";
        rcbVISNs.DataBind();

        // Years
        // start at 2000 - this year + 1
        int endYear = DateTime.Now.Year + 1;

        var yearList = new List<int>();

        // Currently, records in legacy start at 2011, but data should go back to 2001.
        for (int year = 2000; year <= endYear; year++)
        {
            yearList.Add(year);
        }

        rcbReportYears.DataSource = yearList;
        rcbReportYears.DataBind();
    }

    /// <summary>
    /// populate VISN list
    /// </summary>
    private void InitializeFilters()
    {
        foreach (RadComboBoxItem itm in rcbVISNs.Items)
        {
            itm.Checked = true;
        }

        foreach (RadComboBoxItem itm in rcbReportYears.Items)
        {
            itm.Checked = true;
        }

        rcbReportYearDef.ClearSelection();
    }
        
    /// <summary>
    /// Shows Report Container and hides instructions
    /// </summary>
    private void ShowReportArea()
    {
        pnlFilters.Visible = true;
        pnlInstructions.Visible = false;
        pnlMainReportContainer.Visible = true;
    }

    /// <summary>
    /// Reset Report to initial view
    /// </summary>
    private void ResetReport()
    {
        // reset filters
        InitializeFilters();

        // clear grid
        rgReport.DataSource = null;

        // hide/show UI
        pnlFilters.Visible = true;
        pnlInstructions.Visible = true;
        pnlMainReportContainer.Visible = false;
    }

    /// <summary>
    /// Obtains filters and populates data grid
    /// </summary>
    private void PopulateGrid()
    {
        // perform search
        List<int> selectedYears = new List<int>();
        string selectedVisnString = "";
        string selectedVisnDisplayString = "";
        string selectedYearString = "";

        // perform search

        #region Get VISN(s)

        var visnList = rcbVISNs.CheckedItems;

        foreach (var entry in visnList)
        {
            selectedVisnString += entry.Value + ",";
            selectedVisnDisplayString += entry.Text + ", ";
        }

        if (selectedVisnString.EndsWith(","))
        {
            selectedVisnString = selectedVisnString.Substring(0, selectedVisnString.Length - 1);
        }

        if (selectedVisnDisplayString.EndsWith(", "))
        {
            selectedVisnDisplayString = selectedVisnDisplayString.Substring(0, selectedVisnDisplayString.Length - 2);
        }

        if (String.IsNullOrEmpty(selectedVisnString))
            selectedVisnString = null;

        if (String.IsNullOrEmpty(selectedVisnDisplayString))
            selectedVisnDisplayString = null;

        #endregion

        #region Get Selected Years

        // if fiscal ... add -1 to every year, and remove dups
        foreach (RadComboBoxItem rcbi in rcbReportYears.CheckedItems)
        {
            int year = Convert.ToInt32(rcbi.Value);
            selectedYears.Add(year);

            if (rcbReportYearDef.SelectedValue == "fiscal")
            {
                selectedYears.Add(year - 1);
            }
        }

        // remove dups
        selectedYears = selectedYears.Distinct().OrderBy(S => S).ToList();

        foreach (var entry in selectedYears)
        {
            selectedYearString += entry + ", ";
        }

        if (selectedYearString.EndsWith(", "))
        {
            selectedYearString = selectedYearString.Substring(0, selectedYearString.Length - 2);
        }

        if (String.IsNullOrEmpty(selectedYearString))
            selectedYearString = null;

        #endregion

        DataTable dt = ReportManager.GetAnnualCompletionReport(selectedVisnString, selectedYearString);

        lblVisnSelection.Text = selectedVisnDisplayString;
        lblReportYearDefinitionSelection.Text = rcbReportYearDef.SelectedItem.Text;
        lblYearsSelected.Text = selectedYearString;

        if (dt != null)
        {
            rgReport.DataSource = dt;
        }
    }

    /// <summary>
    /// collects data and calls export page.
    /// </summary>
    /// <param name="type"></param>
    private void PerformExport(string type)
    {
        List<int> selectedYears = new List<int>();
        string selectedVisnString = "";
        string selectedVisnDisplayString = "";
        string selectedReportDef = "";
        string selectedYearString = "";

        // perform search

        if (rcbReportYearDef.SelectedValue == "fiscal")
        {
            selectedReportDef = "fiscal";
        }
        else
        {
            selectedReportDef = "calendar";
        }

        #region Get VISN(s)

        var visnList = rcbVISNs.CheckedItems;

        foreach (var entry in visnList)
        {
            selectedVisnString += entry.Value + ",";
            selectedVisnDisplayString += entry.Text + ", ";
        }

        if (selectedVisnString.EndsWith(","))
        {
            selectedVisnString = selectedVisnString.Substring(0, selectedVisnString.Length - 1);
        }

        if (selectedVisnDisplayString.EndsWith(", "))
        {
            selectedVisnDisplayString = selectedVisnDisplayString.Substring(0, selectedVisnDisplayString.Length - 2);
        }

        if (String.IsNullOrEmpty(selectedVisnString))
            selectedVisnString = null;

        if (String.IsNullOrEmpty(selectedVisnDisplayString))
            selectedVisnDisplayString = null;

        #endregion

        #region Get Selected Years

        // if fiscal ... add -1 to every year, and remove dups
        foreach (RadComboBoxItem rcbi in rcbReportYears.CheckedItems)
        {
            int year = Convert.ToInt32(rcbi.Value);
            selectedYears.Add(year);

            if (rcbReportYearDef.SelectedValue == "fiscal")
            {
                selectedYears.Add(year - 1);
            }
        }

        // remove dups
        selectedYears = selectedYears.Distinct().OrderBy(S => S).ToList();

        foreach (var entry in selectedYears)
        {
            selectedYearString += entry + ", ";
        }

        if (selectedYearString.EndsWith(", "))
        {
            selectedYearString = selectedYearString.Substring(0, selectedYearString.Length - 2);
        }

        if (String.IsNullOrEmpty(selectedYearString))
            selectedYearString = null;

        #endregion

        Session["selectedVisnString"] = selectedVisnString;
        Session["selectedVisnDisplayString"] = selectedVisnDisplayString;
        Session["selectedYearString"] = selectedYearString;
        Session["selectedReportDef"] = selectedReportDef;

        Response.Redirect("Export/AnnualCompletionExport.aspx?mode=" + type, true);
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Filter Button Clicked Event
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnFilter_Click(object sender, EventArgs e)
    {
        ShowReportArea();

        PopulateGrid();

        rgReport.DataBind();
    }

    /// <summary>
    /// Reset button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnReset_Click(object sender, EventArgs e)
    {
        ResetReport();
    }

    /// <summary>
    /// Export Word Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExportWord_Click(object sender, EventArgs e)
    {
        PerformExport("word");
    }

    /// <summary>
    /// Export Excel Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExportExcel_Click(object sender, EventArgs e)
    {
        PerformExport("excel");
    }

    #endregion

    #region Grid Events

    /// <summary>
    /// Report Grid Needs Datasource
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgReport_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        PopulateGrid();
    }

    #endregion 
}