﻿using System;
using System.Web;
using Managers.Reporting;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.Reporting;
using DVEIRUserActivityType = BusinessManagerClasses.DVEIRUserActivityType;

public partial class Reporting_DoDInterfaceTransferReport : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    public DoDInterfaceTransferReportViewModel ReportViewModel
    {
        get { return ViewState["ReportViewModel"] as DoDInterfaceTransferReportViewModel; }
        set { ViewState["ReportViewModel"] = value; }
    }

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryReportingMaster)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryReportingMaster)Page.Master).SetBreadCrumbLabel("DoD Interface Transfer Report");

        if (!IsPostBack)
        {
            ReportViewModel = new ReportManager(_user).GetDoDInterfaceTransferReportViewModel();
        }

        compValTodayDateFrom.ValueToCompare = DateTime.Today.ToShortDateString();
        compValTodayDateTo.ValueToCompare = DateTime.Today.ToShortDateString();
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        // TODO Change
        return "DVEIR_REPORTING";
    }

    public override string AppObjectCode()
    {
        // TODO Change
        return "DVEIR_REPORTING";
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Shows Report Container and hides instructions
    /// </summary>
    private void showReportArea()
    {
        pnlFilters.Visible = true;
        pnlMainReportContainer.Visible = true;
    }

    /// <summary>
    /// Reset Report to initial view
    /// </summary>
    private void resetReport()
    {
        // reset filters
        rdpDateFrom.Clear();
        rdpDateTo.Clear();

        // clear grid
        rgReport.DataSource = null;
        rgAgg.DataSource = null;

        // hide/show UI
        pnlFilters.Visible = true;
        pnlMainReportContainer.Visible = false;
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Filter Button Clicked Event
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnFilter_Click(object sender, EventArgs e)
    {
        // Get user inputs
        ReportViewModel.DateFrom = rdpDateFrom.SelectedDate ?? DateTime.Now;
        ReportViewModel.DateTo = rdpDateTo.SelectedDate ?? DateTime.Now;

        // Show Report Area
        showReportArea();

        // perform search
        ReportViewModel = new ReportManager(_user).GetDoDInterfaceTransferReport(ReportViewModel);

        // Map Results
        rgReport.DataSource = ReportViewModel.DoDInterfaceTransfers;
        rgReport.DataBind();

        // Map Totals
        rgAgg.DataSource = ReportViewModel.Aggregates;
        rgAgg.DataBind();

        lblCriteriaDateFrom.Text = ReportViewModel.DateFrom.ToShortDateString();
        lblCriteriaDateTo.Text = ReportViewModel.DateTo.ToShortDateString();

        CreateLoggingRecord(DVEIRUserActivityType.REPORT_DOD_INTERFACE_TRANSFER_RUN, string.Empty);
    }

    /// <summary>
    /// Reset button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnReset_Click(object sender, EventArgs e)
    {
        resetReport();
    }

    #endregion

    protected void rgReport_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgReport.DataSource = ReportViewModel.DoDInterfaceTransfers;
    }

    protected void btnExportWord_Click(object sender, EventArgs e)
    {
        performExport("word");
    }

    protected void btnExportExcel_Click(object sender, EventArgs e)
    {
        performExport("excel");
    }

    /// <summary>
    /// collects data and calls export page.
    /// </summary>
    /// <param name="type"></param>
    private void performExport(string type)
    {
        Session["DoDInterfaceTransferReport"] = ReportViewModel;
        Response.Redirect("Export/DoDInterfaceTransferExport.aspx?mode=" + type, true);
    }
}