﻿using Managers.Reporting;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class Reporting_Export_AnnualCompletionExport : System.Web.UI.Page
{
    protected string imagePath = "";
    protected string selectedReportDef;
    protected string selectedYearString;
    protected string selectedVisnString;
    protected string selectedVisnDisplayString;

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        string exportMode = "word";
        this.EnableViewState = false;

        if (!String.IsNullOrEmpty(Request.QueryString.Get("mode")))
        {
            if (Request.QueryString.Get("mode") == "word")
            {
                exportMode = "word";
            }
            else if (Request.QueryString.Get("mode") == "excel")
            {
                exportMode = "excel";
            }
        }

        // get session
        selectedVisnDisplayString = ((Session["selectedVisnDisplayString"] != null) ? Session["selectedVisnDisplayString"].ToString() : null);
        selectedVisnString = ((Session["selectedVisnString"] != null) ? Session["selectedVisnString"].ToString() : null);
        selectedYearString = ((Session["selectedYearString"] != null) ? Session["selectedYearString"].ToString() : null);
        selectedReportDef = ((Session["selectedReportDef"] != null) ? Session["selectedReportDef"].ToString() : null);

        DataTable dt = ReportManager.GetAnnualCompletionReport(selectedVisnString, selectedYearString);

        StringBuilder sb = new StringBuilder();

        if (dt != null)
        {
            if (dt.Rows != null && dt.Rows.Count > 0)
            {
                foreach (DataRow dr in dt.Rows)
                {
                    string reportYear = "";
                    string uniquePatientsSeenWithinYear = "";
                    string uniqueEncountersWithinYear = "";
                    string completed = "";
                    string notCompleted = "";

                    reportYear = (dr["ReportYear"] != null) ? dr["ReportYear"].ToString() : "";
                    uniquePatientsSeenWithinYear = (dr["UniquePatientsSeenWithinYear"] != null) ? string.Format("{0:#,#}", dr["UniquePatientsSeenWithinYear"]) : "";
                    uniqueEncountersWithinYear = (dr["UniqueEncountersWithinYear"] != null) ? string.Format("{0:#,#}", dr["UniqueEncountersWithinYear"]) : "";
                    completed = (dr["Completed"] != null) ? string.Format("{0:#,#}", dr["Completed"]) : "";
                    notCompleted = (dr["NotCompleted"] != null) ? string.Format("{0:#,#}", dr["NotCompleted"]) : "";

                    sb.Append("<tr>");
                    sb.Append("<td style=\"vertical-align:top;\" class=\"dataTableCell\">" + reportYear + "</td>");
                    sb.Append("<td style=\"vertical-align:top;\" class=\"dataTableCell\">" + uniquePatientsSeenWithinYear + "</td>");
                    sb.Append("<td style=\"vertical-align:top;\" class=\"dataTableCell\">" + uniqueEncountersWithinYear + "</td>");
                    sb.Append("<td style=\"vertical-align:top;\" class=\"dataTableCell\">" + completed + "</td>");
                    sb.Append("<td style=\"vertical-align:top;\" class=\"dataTableCell\">" + notCompleted + "</td>");
                    sb.Append("</tr>");
                }
            }
        }

        lblDataRows.Text = sb.ToString();

        imagePath = ToAbsoluteUrl(Page.ResolveUrl("~/images/VA Logo.JPG"));

        Response.Clear();
        Response.Charset = "";

        switch (exportMode)
        {
            case "word":
                Response.ContentType = "application/vnd.ms-word";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.doc");
                break;
            case "excel":
                Response.ContentType = "application/vnd.ms-excel";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.xls");
                break;
        }
        System.IO.StringWriter sw = new System.IO.StringWriter();
        System.Web.UI.HtmlTextWriter hw = new System.Web.UI.HtmlTextWriter(sw);
        Page.RenderControl(hw);
        Response.Write(sw.ToString());
        Response.End();

        // clear session variables
        clearSessionValues();
    }

    public override void VerifyRenderingInServerForm(Control control)
    {
        /* Verifies that the control is rendered */
    }

    /// <summary>
    /// Converts the provided app-relative path into an absolute Url containing the 
    /// full host name
    /// </summary>
    /// <param name="relativeUrl">App-Relative path</param>
    /// <returns>Provided relativeUrl parameter as fully qualified Url</returns>
    /// <example>~/path/to/foo to http://www.web.com/path/to/foo</example>
    private string ToAbsoluteUrl(string relativeUrl)
    {
        if (string.IsNullOrEmpty(relativeUrl))
            return relativeUrl;

        if (HttpContext.Current == null)
            return relativeUrl;

        if (relativeUrl.StartsWith("/"))
            relativeUrl = relativeUrl.Insert(0, "~");
        if (!relativeUrl.StartsWith("~/"))
            relativeUrl = relativeUrl.Insert(0, "~/");

        var url = HttpContext.Current.Request.Url;
        var port = url.Port != 80 ? (":" + url.Port) : String.Empty;

        return String.Format("{0}://{1}{2}{3}",
            url.Scheme, url.Host, port, VirtualPathUtility.ToAbsolute(relativeUrl));
    }

    /// <summary>
    /// clear passed session values
    /// </summary>
    private void clearSessionValues()
    {
        try
        {
            Session.Remove("selectedVisnString");
            Session.Remove("selectedYearString");
            Session.Remove("selectedReportDef");
            Session.Remove("selectedVisnDisplayString");
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Exporting Annual Completion Admin Report: " + ex.Message);
        }
    }
}