﻿using Managers.Reporting;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using ViewModels.Reporting;

public partial class Reporting_Export_DoDInterfaceTransferExport : System.Web.UI.Page
{
    private DoDInterfaceTransferReportViewModel viewModel;
    protected string imagePath = "";

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        string exportMode = "word";
        this.EnableViewState = false;

        if (!String.IsNullOrEmpty(Request.QueryString.Get("mode")))
        {
            if (Request.QueryString.Get("mode") == "word")
            {
                exportMode = "word";
            }
            else if (Request.QueryString.Get("mode") == "excel")
            {
                exportMode = "excel";
            }
        }

        viewModel = Session["DoDInterfaceTransferReport"] as DoDInterfaceTransferReportViewModel;

        StringBuilder sbDataRows = new StringBuilder();

        if (viewModel != null)
        {
            if (viewModel.DoDInterfaceTransfers != null)
            {

                foreach (var transfer in viewModel.DoDInterfaceTransfers)
                {
                    sbDataRows.Append("<tr>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.PatientId + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.FirstName + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.LastName + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.Gender + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.ReferralId + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.EncounterId + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.EncounterDate.ToShortDateString() + "</td>");
                    sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + transfer.TransferDate.ToShortDateString() + "</td>");
                    sbDataRows.Append("</tr>");
                }
            }

            // Criteria
            lblCriteriaDateFrom.Text = viewModel.DateFrom.ToShortDateString();
            lblCriteriaDateTo.Text = viewModel.DateTo.ToShortDateString();

            // Aggregates
            if (viewModel.Aggregates.Any())
            {
                var agg = viewModel.Aggregates.First();
                lblTotalPatients.Text = agg.TotalPatients.ToString();
                lblTotalEncounters.Text = agg.TotalEncounters.ToString();
            }

            // Data rows
            lblDataRows.Text = sbDataRows.ToString();
            imagePath = ToAbsoluteUrl(Page.ResolveUrl("~/images/VA Logo.JPG"));
        }

        Response.Clear();
        Response.Charset = "";

        switch (exportMode)
        {
            case "word":
                Response.ContentType = "application/vnd.ms-word";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.doc");
                break;
            case "excel":
                Response.ContentType = "application/vnd.ms-excel";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.xls");
                break;
        }

        System.IO.StringWriter sw = new System.IO.StringWriter();
        System.Web.UI.HtmlTextWriter hw = new System.Web.UI.HtmlTextWriter(sw);
        Page.RenderControl(hw);
        Response.Write(sw.ToString());
        Response.End();
    }

    public override void VerifyRenderingInServerForm(Control control)
    {
        /* Verifies that the control is rendered */
    }

    /// <summary>
    /// Converts the provided app-relative path into an absolute Url containing the 
    /// full host name
    /// </summary>
    /// <param name="relativeUrl">App-Relative path</param>
    /// <returns>Provided relativeUrl parameter as fully qualified Url</returns>
    /// <example>~/path/to/foo to http://www.web.com/path/to/foo</example>
    private string ToAbsoluteUrl(string relativeUrl)
    {
        if (string.IsNullOrEmpty(relativeUrl))
            return relativeUrl;

        if (HttpContext.Current == null)
            return relativeUrl;

        if (relativeUrl.StartsWith("/"))
            relativeUrl = relativeUrl.Insert(0, "~");
        if (!relativeUrl.StartsWith("~/"))
            relativeUrl = relativeUrl.Insert(0, "~/");

        var url = HttpContext.Current.Request.Url;
        var port = url.Port != 80 ? (":" + url.Port) : String.Empty;

        return String.Format("{0}://{1}{2}{3}",
            url.Scheme, url.Host, port, VirtualPathUtility.ToAbsolute(relativeUrl));
    }
}