﻿using Managers.Reporting;
using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using Telerik.Web.UI;

public partial class Reporting_Export_PatientOverviewExport : System.Web.UI.Page
{
    protected string imagePath = "";
    protected string selectedReportDef;
    protected string selectedYearString;
    protected string selectedVisnString;
    protected string selectedVisnDisplayString;
    protected string totalRegistryPatientCount;

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            string exportMode = "word";
            this.EnableViewState = false;

            if (!String.IsNullOrEmpty(Request.QueryString.Get("mode")))
            {
                if (Request.QueryString.Get("mode") == "word")
                {
                    exportMode = "word";
                }
                else if (Request.QueryString.Get("mode") == "excel")
                {
                    exportMode = "excel";
                }
            }

            // get session
            selectedVisnDisplayString = ((Session["selectedVisnDisplayString"] != null) ? Session["selectedVisnDisplayString"].ToString() : null);
            selectedVisnString = ((Session["selectedVisnString"] != null) ? Session["selectedVisnString"].ToString() : null);
            selectedYearString = ((Session["selectedYearString"] != null) ? Session["selectedYearString"].ToString() : null);
            selectedReportDef = ((Session["selectedReportDef"] != null) ? Session["selectedReportDef"].ToString() : null);

            totalRegistryPatientCount = string.Format("{0:#,#}", ReportManager.GetTotalNumberOfReferralsForRegistry());
            populateGrid();
            //rgAgg.DataBind();

            imagePath = ToAbsoluteUrl(Page.ResolveUrl("~/images/VA Logo.JPG"));

            Response.Clear();
            Response.Charset = "";

            switch (exportMode)
            {
                case "word":
                    Response.ContentType = "application/vnd.ms-word";
                    Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.doc");
                    break;
                case "excel":
                    Response.ContentType = "application/vnd.ms-excel";
                    Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.xls");
                    break;
            }
            System.IO.StringWriter sw = new System.IO.StringWriter();
            System.Web.UI.HtmlTextWriter hw = new System.Web.UI.HtmlTextWriter(sw);
            Page.RenderControl(hw);
            Response.Write(sw.ToString());
            Response.End();

            // clear session variables
            clearSessionValues();
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Generating Patient Overview Report Export: " + ex.Message);
        }
    }

    public override void VerifyRenderingInServerForm(Control control)
    {
        /* Verifies that the control is rendered */
    }

    /// <summary>
    /// Converts the provided app-relative path into an absolute Url containing the 
    /// full host name
    /// </summary>
    /// <param name="relativeUrl">App-Relative path</param>
    /// <returns>Provided relativeUrl parameter as fully qualified Url</returns>
    /// <example>~/path/to/foo to http://www.web.com/path/to/foo</example>
    private string ToAbsoluteUrl(string relativeUrl)
    {
        if (string.IsNullOrEmpty(relativeUrl))
            return relativeUrl;

        if (HttpContext.Current == null)
            return relativeUrl;

        if (relativeUrl.StartsWith("/"))
            relativeUrl = relativeUrl.Insert(0, "~");
        if (!relativeUrl.StartsWith("~/"))
            relativeUrl = relativeUrl.Insert(0, "~/");

        var url = HttpContext.Current.Request.Url;
        var port = url.Port != 80 ? (":" + url.Port) : String.Empty;

        return String.Format("{0}://{1}{2}{3}",
            url.Scheme, url.Host, port, VirtualPathUtility.ToAbsolute(relativeUrl));
    }

    /// <summary>
    /// clear passed session values
    /// </summary>
    private void clearSessionValues()
    {
        try
        {
            Session.Remove("selectedVisnString");
            Session.Remove("selectedYearString");
            Session.Remove("selectedReportDef");
            Session.Remove("selectedVisnDisplayString");
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Exporting Admin Report: " + ex.Message);
        }
    }

    /// <summary>
    /// Obtains filters and populates data grid
    /// </summary>
    private void populateGrid()
    {
        DataTable dt = ReportManager.GetPatientOverviewReport(selectedVisnString, selectedYearString);
        StringBuilder sb = new StringBuilder();
        int patientCount = 0;
        int encounterCount = 0;

        if (dt != null)
        {
            if (dt.Rows != null && dt.Rows.Count > 0)
            {
                foreach (DataRow dr in dt.Rows)
                {
                    string visnName = dr["VISN_NAME"].ToString();
                    int totalRegistryPatientCount = (dr["TotalRegistryPatientCount"] != null) ? (int)dr["TotalRegistryPatientCount"] : 0;
                    int totalEncounterCount = (dr["TotalEncounterCount"] != null) ? (int)dr["TotalEncounterCount"] : 0;

                    sb.Append("<tr>");
                    sb.Append("<td class=\"dataTableCell\">" + visnName + "</td>");
                    sb.Append("<td class=\"dataTableCell\">" + string.Format("{0:#,#}", totalRegistryPatientCount) + "</td>");
                    sb.Append("<td class=\"dataTableCell\">" + string.Format("{0:#,#}", totalEncounterCount) + "</td>");
                    sb.Append("</tr>");

                    patientCount += totalRegistryPatientCount;
                    encounterCount += totalEncounterCount;
                }
            }
        }

        // add footer
        sb.Append("<tr>");
        sb.Append("<td class=\"dataTableCell\" style=\"background-color:#CCCCCC; font-weight: bold;\">Total Counts for Matching Criteria: </td>");
        sb.Append("<td class=\"dataTableCell\" style=\"background-color:#CCCCCC; font-weight: bold;\">" + string.Format("{0:#,#}", patientCount) + "</td>");
        sb.Append("<td class=\"dataTableCell\" style=\"background-color:#CCCCCC; font-weight: bold;\">" + string.Format("{0:#,#}", encounterCount) + "</td>");
        sb.Append("</tr>");

        litDataRows.Text = sb.ToString();
    }

    #region Grid Events

    protected void rgAgg_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        populateGrid();
    }

    #endregion
}
