﻿using Managers.Reporting;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class Reporting_Export_UniquePatientExport : System.Web.UI.Page
{
    protected string imagePath = "";

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        string exportMode = "word";
        this.EnableViewState = false;

        if (!String.IsNullOrEmpty(Request.QueryString.Get("mode")))
        {
            if (Request.QueryString.Get("mode") == "word")
            {
                exportMode = "word";
            }
            else if (Request.QueryString.Get("mode") == "excel")
            {
                exportMode = "excel";
            }
        }

        // perform search
        DataSet ds = ReportManager.GetUniquePatientReport();

        StringBuilder sbAggregateData = new StringBuilder();
        StringBuilder sbDataRows = new StringBuilder();

        if (ds != null)
        {
            if (ds.Tables != null)
            {
                // Code / Count Data
                if (ds.Tables[0] != null)
                {
                    if (ds.Tables[0].Rows != null && ds.Tables[0].Rows.Count > 0)
                    {
                        foreach (DataRow dr in ds.Tables[0].Rows)
                        {
                            string codeType = "";
                            string codeValue = "";
                            string name = "";
                            string total = "";

                            codeType = (dr["CodeType"] != null) ? dr["CodeType"].ToString() : "";
                            codeValue = (dr["CodeValue"] != null) ? dr["CodeValue"].ToString() : "";
                            name = (dr["Name"] != null) ? dr["Name"].ToString() : "";
                            total = (dr["Total"] != null) ? string.Format("{0:#,#}", dr["Total"]) : "";

                            sbDataRows.Append("<tr>");
                            sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + codeType + "</td>");
                            sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + codeValue + "</td>");
                            sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + name + "</td>");
                            sbDataRows.Append("  <td style=\"vertical-align:top;\" class=\"dataTableCell\">" + total + "</td>");
                            sbDataRows.Append("</tr>");
                        }
                    }
                }

                // Aggregate Data
                if (ds.Tables[1] != null)
                {
                    if (ds.Tables[1].Rows != null && ds.Tables[1].Rows.Count > 0)
                    {
                        foreach (DataRow dr in ds.Tables[1].Rows)
                        {
                            string totalEncounterCount = "";
                            string totalUniquePatientEncounterCount = "";

                            //string.Format("{0:0,0}", dt.Rows[0][0])
                            totalEncounterCount = (dr["TotalEncounterCount"] != null) ? string.Format("{0:#,#}", dr["TotalEncounterCount"]) : "";
                            totalUniquePatientEncounterCount = (dr["TotalUniquePatientEncounterCount"] != null) ? string.Format("{0:0,0}", dr["TotalUniquePatientEncounterCount"]) : "";

                            sbAggregateData.Append("<tr>");
                            sbAggregateData.Append("  <td colspan=\"2\" style=\"vertical-align:top;\" class=\"dataTableCell\">" + totalEncounterCount + "</td>");
                            sbAggregateData.Append("  <td colspan=\"2\" style=\"vertical-align:top;\" class=\"dataTableCell\">" + totalUniquePatientEncounterCount + "</td>");
                            sbAggregateData.Append("</tr>");
                        }
                    }
                }
            }
        }

        lblAggDataRows.Text = sbAggregateData.ToString();
        lblDataRows.Text = sbDataRows.ToString();

        imagePath = ToAbsoluteUrl(Page.ResolveUrl("~/images/VA Logo.JPG"));

        Response.Clear();
        Response.Charset = "";

        switch (exportMode)
        {
            case "word":
                Response.ContentType = "application/vnd.ms-word";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.doc");
                break;
            case "excel":
                Response.ContentType = "application/vnd.ms-excel";
                Response.AppendHeader("content-disposition", "attachment; filename=VA_EIDS_PatientOverviewExport.xls");
                break;
        }
        System.IO.StringWriter sw = new System.IO.StringWriter();
        System.Web.UI.HtmlTextWriter hw = new System.Web.UI.HtmlTextWriter(sw);
        Page.RenderControl(hw);
        Response.Write(sw.ToString());
        Response.End();
    }

    public override void VerifyRenderingInServerForm(Control control)
    {
        /* Verifies that the control is rendered */
    }

    /// <summary>
    /// Converts the provided app-relative path into an absolute Url containing the 
    /// full host name
    /// </summary>
    /// <param name="relativeUrl">App-Relative path</param>
    /// <returns>Provided relativeUrl parameter as fully qualified Url</returns>
    /// <example>~/path/to/foo to http://www.web.com/path/to/foo</example>
    private string ToAbsoluteUrl(string relativeUrl)
    {
        if (string.IsNullOrEmpty(relativeUrl))
            return relativeUrl;

        if (HttpContext.Current == null)
            return relativeUrl;

        if (relativeUrl.StartsWith("/"))
            relativeUrl = relativeUrl.Insert(0, "~");
        if (!relativeUrl.StartsWith("~/"))
            relativeUrl = relativeUrl.Insert(0, "~/");

        var url = HttpContext.Current.Request.Url;
        var port = url.Port != 80 ? (":" + url.Port) : String.Empty;

        return String.Format("{0}://{1}{2}{3}",
            url.Scheme, url.Host, port, VirtualPathUtility.ToAbsolute(relativeUrl));
    }
}