﻿using Managers.Reporting;
using System;
using System.Data;
using System.Web;
using System.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reporting_UniquePatientReport : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryReportingMaster)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryReportingMaster)Page.Master).SetBreadCrumbLabel("Unique Patient Report");
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "MEVIR_UNIQUE_PATIENT_REPORT";
    }

    public override string AppObjectCode()
    {
        return "MEVIR_UNIQUE_PATIENT_REPORT";
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Shows Report Container and hides instructions
    /// </summary>
    private void showReportArea()
    {
        pnlFilters.Visible = true;
        pnlInstructions.Visible = false;
        pnlMainReportContainer.Visible = true;
    }

    /// <summary>
    /// Reset Report to initial view
    /// </summary>
    private void resetReport()
    {
        // reset filters

        // clear grid

        // hide/show UI
        pnlFilters.Visible = true;
        pnlInstructions.Visible = true;
        pnlMainReportContainer.Visible = false;
    }

    /// <summary>
    /// Obtains filters and populates data grid
    /// </summary>
    private void populateGrid()
    {
        // perform search
        DataSet ds = ReportManager.GetUniquePatientReport();

        if (ds != null)
        {
            if (ds.Tables != null)
            {
                if (ds.Tables[0] != null)
                {
                    rgReport.DataSource = ds.Tables[0];
                }

                if (ds.Tables[1] != null)
                {
                    rgAgg.DataSource = ds.Tables[1];
                }
            }
        }
    }

    /// <summary>
    /// collects data and calls export page.
    /// </summary>
    /// <param name="type"></param>
    private void performExport(string type)
    {
        Response.Redirect("Export/UniquePatientExport.aspx?mode=" + type, true);
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Filter Button Clicked Event
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnFilter_Click(object sender, EventArgs e)
    {
        showReportArea();
        populateGrid();
        rgReport.DataBind();
        rgAgg.DataBind();
    }

    /// <summary>
    /// Export Word Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExportWord_Click(object sender, EventArgs e)
    {
        performExport("word");
    }

    /// <summary>
    /// Export Excel Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExportExcel_Click(object sender, EventArgs e)
    {
        performExport("excel");
    }

    #endregion

    #region Grid Events

    /// <summary>
    /// Report Grid Needs Datasource
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgReport_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        populateGrid();
    }

    #endregion 
}