﻿using System;
using System.Web;
using Managers.Reporting;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.Reporting;
using DVEIRUserActivityType = BusinessManagerClasses.DVEIRUserActivityType;

public partial class Reporting_UserEventLogReport : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    public UserEventLogReportViewModel ReportViewModel
    {
        get { return ViewState["ReportViewModel"] as UserEventLogReportViewModel; }
        set { ViewState["ReportViewModel"] = value; }
    }

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryReportingMaster)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryReportingMaster)Page.Master).SetBreadCrumbLabel("User Event Log Report");

        if (!IsPostBack)
        {
            ReportViewModel = new ReportManager(_user).GetUserEventLogReportViewModel();
            rcbUsername.DataSource = ReportViewModel.Users;
            rcbUsername.DataValueField = "Key";
            rcbUsername.DataTextField = "Value";
            rcbUsername.DataBind();
        }

        compValTodayDateFrom.ValueToCompare = DateTime.Today.ToShortDateString();
        compValTodayDateTo.ValueToCompare = DateTime.Today.ToShortDateString();
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        // TODO Change
        return "DVEIR_REPORTING";
    }

    public override string AppObjectCode()
    {
        // TODO Change
        return "DVEIR_REPORTING";
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Shows Report Container and hides instructions
    /// </summary>
    private void showReportArea()
    {
        pnlFilters.Visible = true;
        pnlMainReportContainer.Visible = true;
    }

    /// <summary>
    /// Reset Report to initial view
    /// </summary>
    private void resetReport()
    {
        // reset filters
        rcbUsername.Text = string.Empty;
        rcbUsername.ClearSelection();
        tbReferralId.Text = string.Empty;
        tbEncounterId.Text = string.Empty;
        rdpDateFrom.Clear();
        rdpDateTo.Clear();

        // clear grid
        rgReport.DataSource = null;

        // hide/show UI
        pnlFilters.Visible = true;
        pnlMainReportContainer.Visible = false;
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Filter Button Clicked Event
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnFilter_Click(object sender, EventArgs e)
    {
        // Get user inputs
        ReportViewModel.Username = rcbUsername.SelectedValue;
        ReportViewModel.ReferralId = !string.IsNullOrEmpty(tbReferralId.Text) ? int.Parse(tbReferralId.Text) : 0;
        ReportViewModel.EncounterId = !string.IsNullOrEmpty(tbEncounterId.Text) ? int.Parse(tbEncounterId.Text) : 0;
        ReportViewModel.DateFrom = rdpDateFrom.SelectedDate ?? DateTime.Now;
        ReportViewModel.DateTo = rdpDateTo.SelectedDate ?? DateTime.Now;

        // Show Report Area
        showReportArea();

        // perform search
        ReportViewModel = new ReportManager(_user).GetUserEventLogReport(ReportViewModel);

        // Map Results
        rgReport.DataSource = ReportViewModel.UserEventLogs;
        rgReport.DataBind();

        lblCriteriaUsername.Text = ReportViewModel.Username;
        lblCriteriaReferralId.Text = ReportViewModel.ReferralId > 0 ? ReportViewModel.ReferralId.ToString() : string.Empty;
        lblCriteriaEncounterId.Text = ReportViewModel.EncounterId > 0 ? ReportViewModel.EncounterId.ToString() : string.Empty;
        lblCriteriaDateFrom.Text = ReportViewModel.DateFrom.ToShortDateString();
        lblCriteriaDateTo.Text = ReportViewModel.DateTo.ToShortDateString();

        CreateLoggingRecord(DVEIRUserActivityType.REPORT_USER_EVENT_LOG_RUN, string.Empty);
    }

    /// <summary>
    /// Reset button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnReset_Click(object sender, EventArgs e)
    {
        resetReport();
    }

    #endregion

    protected void rgReport_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        rgReport.DataSource = ReportViewModel.UserEventLogs;
    }

    protected void btnExportWord_Click(object sender, EventArgs e)
    {
        performExport("word");
    }

    protected void btnExportExcel_Click(object sender, EventArgs e)
    {
        performExport("excel");
    }

    /// <summary>
    /// collects data and calls export page.
    /// </summary>
    /// <param name="type"></param>
    private void performExport(string type)
    {
        Session["UserEventLogReport"] = ReportViewModel;
        Response.Redirect("Export/UserEventLogExport.aspx?mode=" + type, true);
    }
}