﻿using System;
using System.Web.UI;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

/// <summary>
/// Summary description for Base Event Item
/// </summary>
public abstract class BaseEventItem<T> : UserControl, IBaseEventItem, IDynamicControl
    where T : BaseEventModel
{
    private T _eventViewModel;

    private string _validationGroup;

    #region Protected Shared Members

    protected T EventViewModel
    {
        get { return ViewState["ViewModel"] as T; }
        set { ViewState["ViewModel"] = value; }
    }

    protected string ValidationGroup
    {
        get { return ViewState["EventValidationGroup"] as string; } 
        set { ViewState["EventValidationGroup"] = value; }
    }

    protected bool ForceRefresh;

    #endregion

    #region Abstract Methods

    protected abstract T SaveItem();

    protected abstract void MapValidationGroup();

    protected abstract void MapTestSummary();

    protected abstract void MapTestDetails();

    protected abstract void PerformForcedRefresh();

    #endregion

    #region Shared Page Methods

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        if (ValidationGroup == null || ForceRefresh)
        {
            ValidationGroup = _validationGroup;
        }

        if (EventViewModel == null || ForceRefresh)
        {
            // Initialize the view model state
            EventViewModel = _eventViewModel;
            ForceRefresh = false;
        }

        if (EventViewModel != null)
        {
            MapTestSummary();
            MapTestDetails();
        }

        if (!string.IsNullOrEmpty(ValidationGroup))
        {
            MapValidationGroup();
        }
    }

    #endregion

    #region Implemented Methods

    void IDynamicControl.PerformForcedRefresh()
    {
        PerformForcedRefresh();
    }

    void IBaseEventItem.SetValidationGroup(string validationGroup)
    {
        _validationGroup = validationGroup;
    }

    void IBaseEventItem.SetViewModel(BaseEventModel viewModel)
    {
        T derivedViewModel = viewModel as T;

        if (derivedViewModel != null)
        {
            _eventViewModel = derivedViewModel;
            ForceRefresh = true;
            PerformForcedRefresh();
        }
    }

    BaseEventModel IBaseEventItem.SaveItem()
    {
        EventViewModel = SaveItem();
        return EventViewModel;
    }

    #endregion

    #region Shared Methods

    protected virtual void MapSelectField(RadComboBox comboBox, SingleSelectQuestion singleSelectQuestion)
    {
        comboBox.DataSource = singleSelectQuestion.AllChoicesOrdered;
        comboBox.DataValueField = "Code";
        comboBox.DataTextField = "DisplayText";
        comboBox.DataBind();

        if (singleSelectQuestion.IsAnswered)
        {
            comboBox.SelectedValue = singleSelectQuestion.SelectedChoice.Code;
        }
    }
   
    protected virtual void MapSelectField(RadComboBox comboBox, MultiSelectQuestion multiSelectQuestion)
    {
        comboBox.DataSource = multiSelectQuestion.AvailableChoicesOrdered;
        comboBox.DataValueField = "Code";
        comboBox.DataTextField = "DisplayText";
        comboBox.DataBind();
    }

    #endregion

}