﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;


namespace DataAccess
{
    /// <summary>
    /// Summary description for Data Access Manager
    /// </summary>
    public class DataAccessManager
    {
        public DataAccessManager()
        {
            //
            // TODO: Add constructor logic here
            //
        }


        internal IEnumerable<QUESTION> GetQuestionsByCategory(string categoryCode)
        {
            IEnumerable<QUESTION> questionList = new List<QUESTION>();

            using (var ctx = new MEVIREntities())
            {
                // Eager Load question list with choices
                questionList = ctx.QUESTIONS
                    .Where(q => q.QUESTION_CATEGORY_CODE.Equals(categoryCode))
                    .Include(q => q.QUESTION_CHOICES)
                    .ToList();
            }

            return questionList;
        }

        internal IEnumerable<LATERALITY> GetAllLataralities()
        {
            IEnumerable<LATERALITY> lateralityList = new List<LATERALITY>();

            using (var ctx = new MEVIREntities())
            {
                lateralityList = ctx.LATERALITIES.ToList();
            }

            return lateralityList;
        }

        internal IEnumerable<RESPONSE> GetResponseList(int patientId, int encounterId, string lateralityCode, string categoryCode)
        {
            IEnumerable<RESPONSE> responseList = new List<RESPONSE>();

            using (var ctx = new MEVIREntities())
            {
                responseList = ctx.RESPONSES.Where(
                    r =>
                        r.REFERRAL_ID == patientId &&
                        r.ENCOUNTER_ID == encounterId &&
                        r.LATERALITY_CODE.Equals(lateralityCode) &&
                        r.QUESTION.QUESTION_CATEGORY_CODE.Equals(categoryCode))
                    .Include(r => r.LATERALITY)
                    .Include(r => r.QUESTION)
                    .Include(r => r.QUESTION_CHOICE)
                    .ToList();
            }

            return responseList;
        }

        internal bool SaveAndReplaceResponses(IEnumerable<int> removeResponseIds, IEnumerable<RESPONSE> responsesToBeAdded)
        {
            bool saveSuccessful = false;

            try
            {
                using (var ctx = new MEVIREntities())
                {
                    // Set old responses to removed
                    foreach (var id in removeResponseIds)
                    {
                        RESPONSE responseToBeRemoved = ctx.RESPONSES.FirstOrDefault(r => r.RESPONSE_ID == id);

                        if (responseToBeRemoved != null)
                        {
                            ctx.RESPONSES.Remove(responseToBeRemoved);
                        }
                    }

                    // Set new responses to be added
                    foreach (var response in responsesToBeAdded.Where(r => r != null))
                    {
                        ctx.RESPONSES.Add(response);
                    }

                    // Save Changes
                    ctx.SaveChanges();
                    saveSuccessful = true;
                }

            }
            catch (Exception)
            {
                throw;
            }

            return saveSuccessful;
        }

        internal bool RemoveResponses(IEnumerable<int> removeResponseIds)
        {
            bool deleteSuccessful = false;

            try
            {
                using (var ctx = new MEVIREntities())
                {
                    // Set old responses to removed
                    foreach (var id in removeResponseIds)
                    {
                        RESPONSE responseToBeRemoved = ctx.RESPONSES.FirstOrDefault(r => r.RESPONSE_ID == id);

                        if (responseToBeRemoved != null)
                        {
                            ctx.RESPONSES.Remove(responseToBeRemoved);
                        }
                    }

                    ctx.SaveChanges();
                }

                deleteSuccessful = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            return deleteSuccessful;
        }

        internal IEnumerable<QUESTION_CATEGORY> GetQuestoionCategoriesBySection(string section)
        {
            IEnumerable<QUESTION_CATEGORY> questionCategories = new List<QUESTION_CATEGORY>();
            
            using (var ctx = new MEVIREntities())
            {
                // Eager Load question list with choices
                questionCategories = ctx.QUESTION_CATEGORIES.Where(c => c.QUESTION_SECTION_CODE == section).ToList();
            }

            return questionCategories;
        }

        internal IEnumerable<RESPONSE> GetResponsesByEncounter(int patientId, int encounterId)
        {
            IEnumerable<RESPONSE> responses = new List<RESPONSE>();

            using (var ctx = new MEVIREntities())
            {
                // Eager Load question list with choices
                responses = ctx.RESPONSES
                    .Where(r => r.REFERRAL_ID == patientId && r.ENCOUNTER_ID == encounterId)
                    .Include(r => r.QUESTION)
                    .ToList();
            }

            return responses;
        }
    }
}