﻿using Managers.Exam;

namespace Factories
{
    /// <summary>
    /// Summary description for ExamEventFactory
    /// </summary>
    public class ExamEventFactory
    {
        private const string BasePath = "~/controls/ExamTabv2/";
        private const string ControlExt = ".ascx";
        public static ExamEventFactoryResult GetExamEventFactoryResult(string categoryCode)
        {
            ExamEventFactoryResult result = new ExamEventFactoryResult();

            switch (categoryCode)
            {
                case "AngRecesss":
                    result.EventControlLocation = BasePath + "ExamTabAngleRecessionUserControl" + ControlExt;
                    result.EventViewManager = new AngleRecessionEventViewManager();
                    break;
                case "AntChamber":
                    result.EventControlLocation = BasePath + "ExamTabAnteriorChamberUserControl" + ControlExt;
                    result.EventViewManager = new AnteriorChamberEventViewManager();
                    break;
                case "Brow":
                    result.EventControlLocation = BasePath + "ExamTabBrowUserControl" + ControlExt;
                    result.EventViewManager = new BrowEventViewManager();
                    break;
                case "ColorVis":
                    result.EventControlLocation = BasePath + "ExamTabColorVisionUserControl" + ControlExt;
                    result.EventViewManager = new ColorVisionEventViewManager();
                    break;
                case "ConfrontVisF":
                    result.EventControlLocation = BasePath + "ExamTabConfrontationalVisualFieldUserControl" + ControlExt;
                    result.EventViewManager = new ConfrontationalVisualFieldEventViewManager();
                    break;
                case "Conjuctiva":
                    result.EventControlLocation = BasePath + "ExamTabConjuctivaUserControl" + ControlExt;
                    result.EventViewManager = new ConjuctivaEventViewManager();
                    break;
                case "Cornea":
                    result.EventControlLocation = BasePath + "ExamTabCorneaUserControl" + ControlExt;
                    result.EventViewManager = new CorneaEventViewManager();
                    break;
                case "DepthPerc":
                    result.EventControlLocation = BasePath + "ExamTabDepthPerceptionUserControl" + ControlExt;
                    result.EventViewManager = new DepthPerceptionEventViewManager();
                    break;
                case "FuncTestF":
                    result.EventControlLocation = BasePath + "ExamTabFunctionalTestingUserControl" + ControlExt;
                    result.EventViewManager = new FunctionalTestingEventViewManager();
                    break;
                case "Globe":
                    result.EventControlLocation = BasePath + "ExamTabGlobeUserControl" + ControlExt;
                    result.EventViewManager = new GlobeEventViewManager();
                    break;
                case "ImgAncilTest":
                    result.EventControlLocation = BasePath + "ExamTabImagingMethodologyUserControl" + ControlExt;
                    result.EventViewManager = new ImagingMethodologyEventViewManager();
                    break;
                case "IntraocularPres":
                    result.EventControlLocation = BasePath + "ExamTabIntraocularPressureUserControl" + ControlExt;
                    result.EventViewManager = new IntraocularPressureEventViewManager();
                    break;
                case "Iris":
                    result.EventControlLocation = BasePath + "ExamTabIrisUserControl" + ControlExt;
                    result.EventViewManager = new IrisEventViewManager();
                    break;
                case "KinecticVisF":
                    result.EventControlLocation = BasePath + "ExamTabKineticVisualFieldsUserControl" + ControlExt;
                    result.EventViewManager = new KineticVisualFieldsEventViewManager();
                    break;
                case "Lens":
                    result.EventControlLocation = BasePath + "ExamTabLensUserControl" + ControlExt;
                    result.EventViewManager = new LensEventViewManager();
                    break;
                case "Lids":
                    result.EventControlLocation = BasePath + "ExamTabLidsUserControl" + ControlExt;
                    result.EventViewManager = new LidsEventViewManager();
                    break;
                case "MotilityExMusc":
                    result.EventControlLocation = BasePath + "ExamTabMotilityUserControl" + ControlExt;
                    result.EventViewManager = new MotilityEventViewManager();
                    break;
                case "OpticDisc":
                    result.EventControlLocation = BasePath + "ExamTabOpticDiscUserControl" + ControlExt;
                    result.EventViewManager = new OpticDiscEventViewManager();
                    break;
                case "Orbit":
                    result.EventControlLocation = BasePath + "ExamTabOrbitUserControl" + ControlExt;
                    result.EventViewManager = new OrbitEventViewManager();
                    break;
                case "ProstheticDevOrder":
                    result.EventControlLocation = BasePath + "ExamTabProstheticDevicesUserControl" + ControlExt;
                    result.EventViewManager = new ProstheticDevicesEventViewManager();
                    break;
                case "Pupil":
                    result.EventControlLocation = BasePath + "ExamTabPupilUserControl" + ControlExt;
                    result.EventViewManager = new PupilEventViewManager();
                    break;
                case "RelAffPupilDef":
                    result.EventControlLocation = BasePath + "ExamTabRelativeAfferentPupilDefectUserControl" + ControlExt;
                    result.EventViewManager = new RelativeAfferentPupilDefectEventViewManager();
                    break;
                case "Retina":
                    result.EventControlLocation = BasePath + "ExamTabRetinaUserControl" + ControlExt;
                    result.EventViewManager = new RetinaEventViewManager();
                    break;
                case "Scelera":
                    result.EventControlLocation = BasePath + "ExamTabSceleraUserControl" + ControlExt;
                    result.EventViewManager = new SceleraEventViewManager();
                    break;
                case "StaticThresPer":
                    result.EventControlLocation = BasePath + "ExamTabStaticThresholdPerimetryUserControl" + ControlExt;
                    result.EventViewManager = new StaticThresholdPerimetryEventViewManager();
                    break;
                case "Symptoms":
                    result.EventControlLocation = BasePath + "ExamTabSymptomsUserControl" + ControlExt;
                    result.EventViewManager = new SymptomsEventViewManager();
                    break;
                case "VisualAcuity":
                    result.EventControlLocation = BasePath + "ExamTabVisualAcuityUserControl" + ControlExt;
                    result.EventViewManager = new VisualAcuityEventViewManager();
                    break;
                case "VisFuncQues":
                    result.EventControlLocation = BasePath + "ExamTabVisualFunctioningUserControl" + ControlExt;
                    result.EventViewManager = new VisualFunctioningEventViewManager();
                    break;
                case "Vitreous":
                    result.EventControlLocation = BasePath + "ExamTabVitreousUserControl" + ControlExt;
                    result.EventViewManager = new VirtreousEventViewManager();
                    break;
                default:
                    result = null;
                    break;
            }

            return result;
        }
    }
}