﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    public class ExamViewManager
    {
        private DataAccessManager dataAccessManager = new DataAccessManager();
        
        public ExamViewModel GetExamViewModel(int patientId, int encounterId)
        {
            ExamViewModel examViewModel = new ExamViewModel()
            {
                PatientId = patientId,
                EncounterId = encounterId
            };

            List<QUESTION_CATEGORY> questionCategories = dataAccessManager.GetQuestoionCategoriesBySection(examViewModel.QuestionSection).ToList();
            List<LATERALITY> lateralities = dataAccessManager.GetAllLataralities().ToList();
            List<RESPONSE> responses = dataAccessManager.GetResponsesByEncounter(patientId, encounterId).ToList();

            examViewModel.PatientId = patientId;
            examViewModel.EncounterId = encounterId;
            examViewModel.EventCategories = MapToEvenCategories(questionCategories);
            examViewModel.EventSummaries = MapToeEventSummaries(questionCategories, lateralities, responses);

            return examViewModel;
        }

        private IEnumerable<ExamViewEventSummary> MapToeEventSummaries(IEnumerable<QUESTION_CATEGORY> questionCategories, IEnumerable<LATERALITY> lateralities, IEnumerable<RESPONSE> responses)
        {
            List<ExamViewEventSummary> eventSummaryList = new List<ExamViewEventSummary>();

            var questionCategoriesInResponses = questionCategories.Where(
                c => responses.Any(r => r.QUESTION.QUESTION_CATEGORY_CODE == c.QUESTION_CATEGORY_CODE));
            var lateralitiesInReponses =
                lateralities.Where(l => responses.Any(r => r.LATERALITY_CODE == l.LATERALITY_CODE));

            foreach (var questionCategory in questionCategoriesInResponses)
            {
                foreach (var laterality in lateralitiesInReponses)
                {
                    RESPONSE response = responses.FirstOrDefault(r => 
                            r.LATERALITY_CODE == laterality.LATERALITY_CODE && 
                            r.QUESTION.QUESTION_CATEGORY_CODE == questionCategory.QUESTION_CATEGORY_CODE);

                    if (response != null)
                    {
                        var eventSummary = new ExamViewEventSummary()
                        {
                            Name = questionCategory.CATEGORY_NAME,
                            CategoryCode = questionCategory.QUESTION_CATEGORY_CODE,
                            CategorySortOrder = questionCategory.SORT_ORDER ?? 0,
                            Laterality = laterality.LATERALITY_NAME,
                            LateralityCode = laterality.LATERALITY_CODE,
                            LateralitySortOrder = laterality.LATERALITY_ID,
                            CreatedBy = response.CREATED_BY,
                            CreatedDate = response.CREATED ?? DateTime.Now
                        };

                        eventSummaryList.Add(eventSummary);
                    }
                }
            }

            return eventSummaryList;
        }

        private IEnumerable<ExamEventCategory> MapToEvenCategories(IEnumerable<QUESTION_CATEGORY> questionCategories)
        {
            return questionCategories.Select(
                c => new ExamEventCategory()
                {
                    Code = c.QUESTION_CATEGORY_CODE, 
                    Name = c.CATEGORY_NAME,
                    SortOrder = c.SORT_ORDER ?? 0
                }).ToList();
        }


    }
}