﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class FunctionalTestingEventViewManager : BaseEventViewManager<FunctionalTestingEventModel>
    {
        public override FunctionalTestingEventModel GetDefaultViewModel()
        {
            var viewModel = new FunctionalTestingEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                FunctionalTestingEventModel.QuestionCodes.TestPerformed,
                FunctionalTestingEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                FunctionalTestingEventModel.QuestionCodes.TestResult,
                FunctionalTestingEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.EventComment);

            viewModel.VergenceRangesType = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.VergenceRangesType);
            viewModel.VergenceRanges = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.VergenceRanges);
            viewModel.NearPointConvergence = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergence);
            viewModel.NearPointConvergenceUnit = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergenceUnit);
            viewModel.AccommodativeAmplitude = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitude);
            viewModel.AccommodativeAmplitudeUnit = MapToSingleSelectQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitudeUnit);
            viewModel.AccommodativeRanges = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.AccommodativeRanges);
            viewModel.RefractionResults = MapToTextQuestion(categoryQuestionList, FunctionalTestingEventModel.QuestionCodes.RefractionResults);

            return viewModel;
        }

        public override FunctionalTestingEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new FunctionalTestingEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.TestResult);

                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.EventComment);

                viewModel.VergenceRangesType = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.VergenceRangesType);
                viewModel.VergenceRanges = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.VergenceRanges);
                viewModel.NearPointConvergence = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergence);
                viewModel.NearPointConvergenceUnit = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.NearPointConvergenceUnit);
                viewModel.AccommodativeAmplitude = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitude);
                viewModel.AccommodativeAmplitudeUnit = MapToSingleSelectQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeAmplitudeUnit);
                viewModel.AccommodativeRanges = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.AccommodativeRanges);
                viewModel.RefractionResults = MapToTextQuestion(categoryQuestionList, responseList, FunctionalTestingEventModel.QuestionCodes.RefractionResults);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(FunctionalTestingEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId,
                    lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    FunctionalTestingEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Vergence Ranges Type
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.VergenceRangesType, patientId,
                        encounterId, lateralityCode));

                    // Vergence Ranges Number Format
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.VergenceRanges, patientId, encounterId,
                        lateralityCode));

                    // Near Point of Convergence
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.NearPointConvergence, patientId, encounterId,
                        lateralityCode));

                    // Near Point of Convergence - Unit
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.NearPointConvergenceUnit, patientId,
                        encounterId, lateralityCode));

                    // Accommodative Amplitude
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.AccommodativeAmplitude, patientId,
                        encounterId, lateralityCode));

                    // Accommodative Amplitude - Unit
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.AccommodativeAmplitudeUnit,
                        patientId, encounterId, lateralityCode));

                    // Accommodative Ranges
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.AccommodativeRanges, patientId, encounterId,
                        lateralityCode));

                    // Refraction Results For Fused Cross Cylinder
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.RefractionResults, patientId, encounterId,
                        lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}