﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class ImagingMethodologyEventViewManager : BaseEventViewManager<ImagingAncillaryEventModel>
    {
        public override ImagingAncillaryEventModel GetDefaultViewModel()
        {
            var viewModel = new ImagingAncillaryEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.ImagingMethodology = MapToMultiSelectQuestion(categoryQuestionList, ImagingAncillaryEventModel.QuestionCodes.ImagingMethodology);

            viewModel.TestDates =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestDateSuffix);

            viewModel.TestResults =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestResultSuffix);

            return viewModel;
        }

        public override ImagingAncillaryEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new ImagingAncillaryEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));

                viewModel.ImagingMethodology = MapToMultiSelectQuestion(categoryQuestionList, responseList, ImagingAncillaryEventModel.QuestionCodes.ImagingMethodology);

                viewModel.TestDates =
                    MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestDateSuffix);

                viewModel.TestResults =
                    MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.ImagingMethodology, ImagingAncillaryEventModel.QuestionCodes.TestResultSuffix);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(ImagingAncillaryEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();
                
                // Imaging Methodology
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.ImagingMethodology, patientId, encounterId, lateralityCode));

                // Test Dates
                responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(result.TestDates, result.ImagingMethodology, patientId, encounterId, lateralityCode));

                // Test Results
                responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(result.TestResults, result.ImagingMethodology, patientId, encounterId, lateralityCode));

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}