﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Visual Acuity Event View Manager
    /// </summary>
    public class VisualAcuityEventViewManager : BaseEventViewManager<VisualAcuityEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The retina view model</returns>
        public override VisualAcuityEventModel GetDefaultViewModel()
        {
            var viewModel = new VisualAcuityEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.TestPerformed,
                VisualAcuityEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.TestResult,
                VisualAcuityEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.EventComment);

            // Universal
            viewModel.Scale = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Scale);
            viewModel.Distance = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Distance);
            viewModel.Correction = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.Correction);
            viewModel.ExamMethod = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.ExamMethod);
            viewModel.ChartType = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.ChartType);
            viewModel.PrintSample = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.PrintSample);

            // US Scale
            viewModel.USChartDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USChartDistance);
            viewModel.USAcuity = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USAcuity);
            viewModel.USDifferentDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USDifferentDistance);
            viewModel.USPerformDistance = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USPerformDistance);
            viewModel.USPerformDistance.ResponseText = "20";
            viewModel.USAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USAdjustment);
            viewModel.USUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.USUnits);

            // Metric Scale
            viewModel.MetricChartDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricChartDistance);
            viewModel.MetricAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricAcuity);
            viewModel.MetricDifferentDistance = MapToYesNoQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricDifferentDistance);
            viewModel.MetricPerformDistance = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricPerformDistance);
            viewModel.MetricAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricAdjustment);
            viewModel.MetricUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MetricUnits);

            // Jaeger Scale
            viewModel.JaegerPerformDistance = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerPerformDistance);
            viewModel.JaegerAcuity = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerAcuity);
            viewModel.JaegerAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerAdjustment);
            viewModel.JaegerUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.JaegerUnits);
            
            viewModel.JaegerChartType = MapToSingleSelectQuestion(
                categoryQuestionList, 
                VisualAcuityEventModel.QuestionCodes.JaegerChartType,
                VisualAcuityEventModel.ChoiceCodes.JaegerChartType.Jaeger);

            viewModel.JaegerPrintSamples = MapToSingleSelectQuestion(
                categoryQuestionList,
                VisualAcuityEventModel.QuestionCodes.JaegerPrintSamples,
                VisualAcuityEventModel.ChoiceCodes.JaegerPrintSample.Jaeger);

            // Decimal Scale
            viewModel.DecimalAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalAcuity);
            viewModel.DecimalAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalAdjustment);
            viewModel.DecimalUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.DecimalUnits);

            // M Unit Scale
            viewModel.MUnitAcuity = MapToTextQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitAcuity);
            viewModel.MUnitAdjustment = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitAdjustment);
            viewModel.MUnitUnits = MapToSingleSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.MUnitUnits);

            // Non-Numeric Scale
            viewModel.NonNumericAcuity = MapToMultiSelectQuestion(categoryQuestionList, VisualAcuityEventModel.QuestionCodes.NonNumericAcuity);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The retina view model</returns>
        public override VisualAcuityEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new VisualAcuityEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.TestResult);

                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.EventComment);

                // Universal
                viewModel.Scale = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Scale);
                viewModel.Distance = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Distance);
                viewModel.Correction = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.Correction);
                viewModel.ExamMethod = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.ExamMethod);
                viewModel.ChartType = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.ChartType);
                viewModel.PrintSample = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.PrintSample);

                // US Scale
                viewModel.USChartDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USChartDistance);
                viewModel.USAcuity = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USAcuity);
                viewModel.USDifferentDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USDifferentDistance);
                viewModel.USPerformDistance = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USPerformDistance);
                viewModel.USAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USAdjustment);
                viewModel.USUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.USUnits);

                // Metric Scale
                viewModel.MetricChartDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricChartDistance);
                viewModel.MetricAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricAcuity);
                viewModel.MetricDifferentDistance = MapToYesNoQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricDifferentDistance);
                viewModel.MetricPerformDistance = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricPerformDistance);
                viewModel.MetricAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricAdjustment);
                viewModel.MetricUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MetricUnits);

                // Jaeger Scale
                viewModel.JaegerPerformDistance = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerPerformDistance);
                viewModel.JaegerAcuity = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerAcuity);
                viewModel.JaegerAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerAdjustment);
                viewModel.JaegerUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerUnits);
                viewModel.JaegerChartType = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerChartType);
                viewModel.JaegerPrintSamples = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.JaegerPrintSamples);

                // Decimal Scale
                viewModel.DecimalAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalAcuity);
                viewModel.DecimalAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalAdjustment);
                viewModel.DecimalUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.DecimalUnits);

                // M Unit Scale
                viewModel.MUnitAcuity = MapToTextQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitAcuity);
                viewModel.MUnitAdjustment = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitAdjustment);
                viewModel.MUnitUnits = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.MUnitUnits);

                // Non-Numeric Scale
                viewModel.NonNumericAcuity = MapToMultiSelectQuestion(categoryQuestionList, responseList, VisualAcuityEventModel.QuestionCodes.NonNumericAcuity);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(VisualAcuityEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    VisualAcuityEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId, lateralityCode));

                    // Universal
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Scale, patientId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Distance, patientId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Correction, patientId, encounterId, lateralityCode));
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.ExamMethod, patientId, encounterId, lateralityCode));

                    if (result.IsJaegerScale)
                    {
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.JaegerChartType, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.JaegerPrintSamples, patientId, encounterId, lateralityCode));
                    }
                    else
                    {
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.ChartType, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.PrintSample, patientId, encounterId, lateralityCode));
                    }

                    // US Scale
                    if (result.IsUSScale)
                    {
                        responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.USChartDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.USAcuity, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.USDifferentDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.USPerformDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.USAdjustment, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.USUnits, patientId, encounterId, lateralityCode));
                    }

                    // Metric Scale
                    if (result.IsMetricScale)
                    {
                        responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.MetricChartDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.MetricAcuity, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.MetricDifferentDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.MetricPerformDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.MetricAdjustment, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.MetricUnits, patientId, encounterId, lateralityCode));
                    }

                    // Jaeger Scale
                    if (result.IsJaegerScale)
                    {
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.JaegerPerformDistance, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.JaegerAcuity, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.JaegerAdjustment, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.JaegerUnits, patientId, encounterId, lateralityCode));
                    }

                    // Decimal Scale
                    if (result.IsDecimalScale)
                    {
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.DecimalAcuity, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.DecimalAdjustment, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.DecimalUnits, patientId, encounterId, lateralityCode));
                    }

                    // M Unit Scale
                    if (result.IsMUnitScale)
                    {
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.MUnitAcuity, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.MUnitAdjustment, patientId, encounterId, lateralityCode));
                        responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.MUnitUnits, patientId, encounterId, lateralityCode));
                    }

                    // Non-Numeric Scale
                    if (result.IsNonNumericScale)
                    {
                        responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.NonNumericAcuity, patientId, encounterId, lateralityCode));
                    }
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}