﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels.Exam
{
    /// <summary>
    /// Summary description for BaseEvent
    /// </summary>
    [Serializable]
    public abstract class BaseEventModel
    {
        /// <summary>Gets the category code.</summary>
        /// <value>The category code.</value>
        public abstract string CategoryCode { get; }

        /// <summary>Gets or sets the event date.</summary>
        /// <value>The event date.</value>
        public DateTime? EventDate { get; set; }

        /// <summary>Gets a value indicating whether this instance has test performed.</summary>
        /// <value>
        /// <c>true</c> if this instance has test performed; otherwise, <c>false</c>.
        /// </value>
        public bool HasTestPerformed
        {
            get { return TestPerformed != null; }
        }

        /// <summary>Gets or sets the test performed.</summary>
        /// <value>The test performed.</value>
        public SingleSelectQuestion TestPerformed { get; set; }

        /// <summary>Gets a value indicating whether this instance has test result.</summary>
        /// <value>
        /// <c>true</c> if this instance has test result; otherwise, <c>false</c>.
        /// </value>
        public bool HasTestResult
        {
            get { return TestResult != null; }
        }

        /// <summary>Gets or sets the test result.</summary>
        /// <value>The test result.</value>
        public SingleSelectQuestion TestResult { get; set; }

        public bool HasEventComment
        {
            get { return EventComment != null; }
        }

        public TextQuestion EventComment { get; set; }

        public bool IsLateralityLocked { get; set; }

        /// <summary>Gets or sets the selected laterality.</summary>
        /// <value>The selected laterality.</value>
        public Laterality SelectedLaterality { get; set; }

        public List<Laterality> AvailableLateralities { get; private set; }

        public IEnumerable<Laterality> AvailableLateralitiesOrdered
        {
            get { return AvailableLateralities.OrderBy(l => l.SortOrder); }
        }

        /// <summary>Initializes a new instance of the <see cref="BaseEventModel"/> class.</summary>
        protected BaseEventModel()
        {
            AvailableLateralities = new List<Laterality>();
        }

        public static class LateralityCodes
        {
            public static string LeftEye = "L";

            public static string RightEye = "R";

            public static string BothEyes = "B";

            public static string NotSpecified = "N";
        }
    }
}