﻿using Managers.Encounter;
using Managers.Referral;
using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

public partial class MyTasks : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;
    protected string status;

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("My Tasks");

        #region Hide/Show Panel Regions

        if (CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR))
        {
            pnlNoAccess.Visible = false;
            pnlReadyForReview.Visible = true;
            pnlReferralEncounters.Visible = true;
            pnlAdministrationArea.Visible = true;

            pnlDataAbstrationFilterZone.Visible = true;

            fillReportGrid();
            fillReviewGrid();
        }
        else if (CurrentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR))
        {
            pnlNoAccess.Visible = false;
            pnlReadyForReview.Visible = false;
            pnlReferralEncounters.Visible = true;
            pnlAdministrationArea.Visible = false;

            pnlDataAbstrationFilterZone.Visible = true;

            fillReportGrid();
        }
        else if (CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
        {
            pnlNoAccess.Visible = false;
            pnlReadyForReview.Visible = true;
            pnlReferralEncounters.Visible = false;
            pnlAdministrationArea.Visible = false;

            pnlDataAbstrationFilterZone.Visible = false;

            fillReviewGrid();
        }
        else
        {
            // nothing ... no show
            pnlReadyForReview.Visible = false;
            pnlReferralEncounters.Visible = false;
            pnlAdministrationArea.Visible = false;

            pnlDataAbstrationFilterZone.Visible = false;

            pnlNoAccess.Visible = true;
        }

        #endregion
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "My Tasks";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region Grid Events

    /// <summary>
    /// Fired upon item command
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int rowIndex = Convert.ToInt32(e.CommandArgument);
            int refId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["REFERRAL_ID"];
            int encId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ER_ENCOUNTER_ID"];
            Response.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", refId, encId));
        }
    }

    /// <summary>
    /// On Need Data Source for Grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        fillReportGrid();
    }

    /// <summary>
    /// On Item Data Bound
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyEncounters_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                bool showSelect = false;

                switch (item["ENCOUNTER_STATUS_Report"].Text)
                {
                    case "New":
                        showSelect = true;
                        break;
                    case "In Process":
                        showSelect = true;
                        break;
                    case "Ready for Review":
                        showSelect = false;
                        break;
                    case "Returned":
                        showSelect = true;
                        break;
                    case "Ready to Send":
                        showSelect = false;
                        break;
                    case "Completed":
                        showSelect = false;
                        break;
                    default:
                        showSelect = false;
                        break;
                }


                drillButton.Visible = showSelect;
            }

            //if (isExport && e.Item is GridDataItem)
            //{
            //    foreach (TableCell cell in e.Item.Cells)
            //    {
            //        //Approach 1
            //        cell.Style["mso-number-format"] = @"\@";
            //    }
            //}

            // footer
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }
    }

    #endregion

    #region Reviewer Grid Events

    protected void rgReviewer_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        fillReviewGrid();
    }

    protected void rgReviewer_ItemDataBound(object sender, GridItemEventArgs e)
    {

    }

    protected void rgReviewer_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int rowIndex = Convert.ToInt32(e.CommandArgument);
            int refId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["REFERRAL_ID"];
            int encId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ER_ENCOUNTER_ID"];
            Response.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", refId, encId));
        }
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Encounter Status Change
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbEncounterStatus_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        fillReportGrid();
        rgMyEncounters.DataBind();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Populates the Grid with Data.
    /// </summary>
    private void fillReportGrid()
    {
        try
        {
            //status
            int statusId = 0;

            status = rcbEncounterStatus.SelectedValue;

            switch (status.ToLower())
            {
                case "all":
                    statusId = 0;
                    break;
                case "new":
                    statusId = 1;
                    break;
                case "inproc":
                    statusId = 2;
                    break;
                case "review":
                    statusId = 3;
                    break;
                case "returned":
                    statusId = 4;
                    break;
                case "send":
                    statusId = 5;
                    break;
                case "completed":
                    statusId = 6;
                    break;
                default:
                    statusId = 0;
                    break;
            }

            rgMyEncounters.DataSource = ReferralDataManager.GetCurrentUserWorkload(CurrentUser.UserId, statusId);
        }
        catch (Exception ex)
        {
            rgMyEncounters.DataSource = null;
        }
    }

    /// <summary>
    /// Populates the Review Grid with Data.
    /// </summary>
    private void fillReviewGrid()
    {
        try
        {
            if (CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) || CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL))
            {
                rgReviewer.DataSource = EncounterDataManager.GetEncountersByStatusId(EncounterStatusType.REVIEW);
            }
        }
        catch (Exception ex)
        {
            rgReviewer.DataSource = null;
        }
    }

    /// <summary>
    /// Clear Add Encounter Form values.
    /// </summary>
    private void clearForm()
    {
        //lblReferralId.Text = "";
        //rcbInstitution.ClearSelection();
        //rcbInstitution.Text = "";
        //rdtpVisitDate.Clear();
        //rcbEncounterType.ClearSelection();
    }

    #endregion
}