﻿using DataAccess;
using Managers.Encounter;
using System;
using Telerik.Web.UI;

public partial class controls_EncounterProviderTab_EncounterProviderControl : System.Web.UI.UserControl
{
    #region Getter / Setters

    public ER_ENCOUNTER_MOD EncounterMod { get; set; }
    public string NPI { get; set; }
    public bool IsEditable { get; set; }

    #endregion

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            if (!IsPostBack)
            {
                loadData();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(ex.Message);
        }
    }

    #region Provider Grid Events

    /// <summary>
    /// Grid needs datasource
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgProviders_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        rgProviders.DataSource = "";
    }

    /// <summary>
    /// Item Command Event on the Provider Grid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgProviders_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        int encounterStaffId;

        if (e.CommandName == "ExecuteReport")
        {
            if (e.Item is GridDataItem)
            {
                //item = ((GridDataItem)e.Item);

                //int rowIndex = Convert.ToInt32(e.CommandArgument);
                //reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];
                //hfReportId.Value = reportId.ToString();

                //pnlPreviewArea.Visible = true;

                //loadReportTemplate(reportId);
                // load data ... show grid, bind
            }
        }
        else if (e.CommandName == "DeleteProvider")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                encounterStaffId = (int)item.GetDataKeyValue("Id");

                try
                {
                    EncounterDataManager.DeleteEncounterProvider(encounterStaffId);
                    loadData();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
        else if (e.CommandName == "EditReport")
        {
            if (e.Item is GridDataItem)
            {
                //item = ((GridDataItem)e.Item);

                //int rowIndex = Convert.ToInt32(e.CommandArgument);
                //reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];

                //Response.Redirect("AdhocReportingAdministration.aspx?mode=edit&id=" + reportId);
            }
        }
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Add Provider Button Clicked ... show panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddProvider_Click(object sender, EventArgs e)
    {
        rgProviders.Visible = false;
        pnlAddProviderBtn.Visible = false;

        pnlAddProviderRegion.Visible = true;

        // Default view
        rcbInstitution.SelectedValue = EncounterMod.STA6A;

        // Query for providers based on Encounter data
        rcbProviders.DataSource = EncounterDataManager.GetAllProviders(EncounterMod.STA6A);
        rcbProviders.DataBind();

        rcbProviders.Enabled = true;
    }

    /// <summary>
    /// Fired when institution is binding to data ... setting default selected value to what is in the encounter
    /// Also - binding the providers for the default institution
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbInstitution_DataBound(object sender, EventArgs e)
    {
        rcbInstitution.SelectedValue = EncounterMod.STA6A;

        // Query for providers based on Encounter data
        rcbProviders.DataSource = EncounterDataManager.GetAllProviders(EncounterMod.STA6A);
        rcbProviders.DataBind();

        rcbProviders.Enabled = true;
    }

    /// <summary>
    /// Selected index changed on Institution List
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbInstitution_SelectedIndexChanged(object sender, Telerik.Web.UI.RadComboBoxSelectedIndexChangedEventArgs e)
    {
        // query
        rcbProviders.ClearSelection();
        rcbProviders.Enabled = !String.IsNullOrEmpty(rcbInstitution.SelectedValue);
        rcbProviders.DataSource = EncounterDataManager.GetAllProviders(rcbInstitution.SelectedValue);
        rcbProviders.DataBind();
    }

    /// <summary>
    /// Selected index changed on provider list
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbProviders_SelectedIndexChanged(object sender, Telerik.Web.UI.RadComboBoxSelectedIndexChangedEventArgs e)
    {
        // if not null ... show "Add" button
        rbAddProviderEntry.Enabled = !String.IsNullOrEmpty(rcbProviders.SelectedValue);
    }

    /// <summary>
    /// Add Provider Entry Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddProviderEntry_Click(object sender, EventArgs e)
    {
        // Add entry to the LocalProviderList object ...
        EncounterDataManager.InsertEncounterProvider(new ENCOUNTER_STAFF() { StaffName = rcbProviders.SelectedItem.Text, VisitIdentifier = EncounterMod.UniqueVHAEncounterIDPrimaryKey, StaffSID = Convert.ToInt32(rcbProviders.SelectedValue) });

        rgProviders.Visible = true;
        pnlAddProviderBtn.Visible = true;

        pnlAddProviderRegion.Visible = false;

        rcbProviders.ClearSelection();
        rcbProviders.Enabled = false;
        rcbInstitution.ClearSelection();

        loadData();
    }

    /// <summary>
    /// Cancel Adding Providers
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbCancelAddProviderEntry_Click(object sender, EventArgs e)
    {
        rgProviders.Visible = true;
        pnlAddProviderBtn.Visible = true;

        pnlAddProviderRegion.Visible = false;

        rcbProviders.ClearSelection();
        rcbProviders.Enabled = false;
        rcbInstitution.ClearSelection();
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// TODO
    /// </summary>
    /// <param name="passedEncounter"></param>
    public void SaveEncounterProviderInformation(ref ER_ENCOUNTER_MOD passedEncounter)
    {
        string test = "";
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Private method to load data and initialize the form.
    /// </summary>
    private void loadData()
    {
        pnlAddProviderBtn.Visible = IsEditable;
        rcbProviders.Enabled = !String.IsNullOrEmpty(rcbInstitution.SelectedValue);
        ProviderRequiredFieldValidator.Enabled = !String.IsNullOrEmpty(rcbInstitution.SelectedValue);

        rgProviders.DataSource = EncounterDataManager.GetProvidersForVisitIdentifier(EncounterMod.UniqueVHAEncounterIDPrimaryKey);
        rgProviders.DataBind();
    }

    #endregion
}