﻿using System;
using System.EnterpriseServices;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;

namespace Controls.ExamTabv2
{
    public partial class ExamMultiSelectUserControl : UserControl, IDynamicControl
    {
        #region Public Members

        public MultiSelectQuestion Question { get; set; }

        public string EmptyMessage { get; set; }

        public string CssClass
        {
            set { pnlMultiSelectWrapper.CssClass = value; }
        }

        public string OnClientLoad { get; set; }

        public string OnClientSelectedIndexChanged { get; set; }

        public DdlOnItemDataBoundExt ExtendDdlOnItemDataBound { get; set; }

        public GridOnItemDataBoundExt ExtendGridOnItemDataBound { get; set; }

        public GridOnItemCommandExt ExtendGridOnItemCommand { get; set; }

        public delegate void DdlOnItemDataBoundExt(object sender, RadComboBoxItemEventArgs e);

        public delegate void GridOnItemDataBoundExt(object sender, GridItemEventArgs e);

        public delegate void GridOnItemCommandExt(object sender, GridCommandEventArgs e);

        #endregion

        #region Public Methods

        public MultiSelectQuestion GetUpdatedQuestion()
        {
            // Just a formality right now
            return Question;
        }

        public void PerformForcedRefresh()
        {
            // Not used
        }

        #endregion

        #region Page Events

        protected void Page_Load(object sender, EventArgs e)
        {
            if (Question != null)
            {
                MapGrid();
            }
        }

        protected void gvMultiSelect_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox multiSelectComboBox = (RadComboBox) e.Item.FindControl("ddlMultiSelect");
                DataBindSelect(multiSelectComboBox);
            }

            if (ExtendGridOnItemDataBound != null)
            {
                ExtendGridOnItemDataBound(sender, e);
            }
        }

        protected void gvMultiSelect_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMultiSelect.EditIndexes.Add(e.Item.ItemIndex);
                gvMultiSelect.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    Question.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMultiSelect.EditIndexes.Clear();
                gvMultiSelect.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataItem = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataItem != null
                        ? Question.SelectedChoices.FirstOrDefault(c => c.Code == dataItem.Code)
                        : null;

                if (selectedChoice != null)
                {
                    Question.SelectedChoices.Remove(selectedChoice);

                    TextBox tbMultiSelectComment = (TextBox) e.Item.FindControl("tbMultiSelectComment");
                    selectedChoice.Comment = tbMultiSelectComment.Text;

                    Question.SelectedChoices.Add(selectedChoice);
                }

                gvMultiSelect.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlMultiSelect");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    Question.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbMultiSelectComment = (TextBox) e.Item.FindControl("tbMultiSelectComment");
                    selectedChoice.Comment = tbMultiSelectComment.Text;

                    Question.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbMultiSelectComment.Text = string.Empty;
                }
            }

            if (ExtendGridOnItemCommand != null)
            {
                ExtendGridOnItemCommand(sender, e);
            }

            MapGrid();
        }

        protected void ddlMultiSelect_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();

            if (ExtendGridOnItemDataBound != null)
            {
                ExtendDdlOnItemDataBound(sender, e);
            }
        }

        #endregion

        #region Private Methods

        private void MapGrid()
        {
            gvMultiSelect.MasterTableView.Columns[0].HeaderText = Question.DisplayText;

            // Show comment column only if there are some choices that require it
            gvMultiSelect.MasterTableView.Columns[1].Visible =
                Question.AllChoices.Any(c => c.RequiresComment);

            gvMultiSelect.DataSource = Question.SelectedChoicesOrderd;
            gvMultiSelect.DataBind();
        }

        private void DataBindSelect(RadComboBox multiSelectComboBox)
        {
            multiSelectComboBox.EmptyMessage = EmptyMessage ?? multiSelectComboBox.EmptyMessage;
            multiSelectComboBox.OnClientLoad = OnClientLoad ?? multiSelectComboBox.OnClientLoad;
            multiSelectComboBox.OnClientSelectedIndexChanged = 
                OnClientSelectedIndexChanged ?? multiSelectComboBox.OnClientSelectedIndexChanged;
            
            multiSelectComboBox.DataSource = Question.AvailableChoicesOrdered;
            multiSelectComboBox.DataValueField = "Code";
            multiSelectComboBox.DataTextField = "DisplayText";
            multiSelectComboBox.DataBind();
        }

        #endregion
    }
}