﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabAngleRecessionUserControl : BaseEventItem<AngleRecessionEventModel>
    {
        #region Implmented Methods

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override void MapValidationGroup()
        {
            // No validation group to set here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = AngleRecessionEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            DataBindVisitDataGrid();
        }

        protected override AngleRecessionEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Page Events

        protected void ddlVisitData_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
        }

        protected void gvVisitData_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;

                var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                DataBindClockHourDropDownList(clockHour1, 1);

                var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                DataBindClockHourDropDownList(clockHour2, 2);

                if (dataItem != null)
                {
                    MapClockHours(dataItem.ClockHour, clockHour1, clockHour2);
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox visitDataComboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                MapSelectField(visitDataComboBox, EventViewModel.VisitData);

                var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                DataBindClockHourDropDownList(clockHour1, 1);

                var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                DataBindClockHourDropDownList(clockHour2, 2);
            }
        }

        protected void gvVisitData_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvVisitData.EditIndexes.Add(e.Item.ItemIndex);
                gvVisitData.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                    EventViewModel.ClockHours[selectedChoice.Code].ResponseText = String.Empty;
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvVisitData.EditIndexes.Clear();
                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as AngleRecessionEventModel.VisitDataView;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.VisitData.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.Remove(selectedChoice);
                    EventViewModel.ClockHours[selectedChoice.Code].ResponseText = string.Empty;

                    TextBox tbVisitDataComment = (TextBox) e.Item.FindControl("tbVisitDataComment");
                    selectedChoice.Comment = tbVisitDataComment.Text;
                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);

                    var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                    var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                    if (clockHour1.SelectedIndex > -1 && clockHour2.SelectedIndex > -1)
                    {
                        EventViewModel.ClockHours[selectedChoice.Code].ResponseText =
                            clockHour1.SelectedValue + " - " + clockHour2.SelectedValue;
                    }
                }

                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.VisitData.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbVisitDataComment = (TextBox) e.Item.FindControl("tbVisitDataComment");
                    selectedChoice.Comment = tbVisitDataComment.Text;
                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);

                    var clockHour1 = (RadComboBox) e.Item.FindControl("ddlClockHour1");
                    var clockHour2 = (RadComboBox) e.Item.FindControl("ddlClockHour2");
                    if (clockHour1.SelectedIndex > -1 && clockHour2.SelectedIndex > -1)
                    {
                        EventViewModel.ClockHours[selectedChoice.Code].ResponseText =
                            clockHour1.SelectedValue + " - " + clockHour2.SelectedValue;
                    }

                    comboBox.Text = string.Empty;
                    tbVisitDataComment.Text = string.Empty;
                    clockHour1.SelectedIndex = -1;
                    clockHour2.SelectedIndex = -1;
                }
            }

            DataBindVisitDataGrid();
        }

        #endregion

        #region Private Methods

        private void DataBindVisitDataGrid()
        {
            gvVisitData.DataSource = EventViewModel.VisitDataViews;
            gvVisitData.DataBind();
        }

        private void DataBindClockHourDropDownList(RadComboBox clockHourDropDownList, int startNumber)
        {
            List<int> clockhours = new List<int>();
            for (int i = startNumber; i <= 12; i++)
            {
                clockhours.Add(i);
            }

            clockHourDropDownList.DataSource = clockhours;
            clockHourDropDownList.DataBind();
        }

        private void MapClockHours(string clockHourText, RadComboBox clockHour1, RadComboBox clockHour2)
        {
            if (!string.IsNullOrEmpty(clockHourText))
            {
                var clockHours = clockHourText.Split(new char[] {'-'});
                clockHour1.SelectedValue = clockHours[0].Trim();
                clockHour2.SelectedValue = clockHours[1].Trim();
            }
        }

        #endregion
    }
}