﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabAnteriorChamberUserControl : BaseEventItem<AnteriorChamberEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            // Does nothing here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = AnteriorChamberEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            DataBindVisitDataGrid();
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override AnteriorChamberEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Page Events

        protected void ddlVisitData_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();

            e.Item.Attributes["ReqValVDCommentEnabled"] = "false";
            e.Item.Attributes["RegExValHeightEnabled"] = "false";
            e.Item.Attributes["RegExValPercentageEnabled"] = "false";

            // Add attributes so javascript can enable the validators
            if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
            {
                e.Item.Attributes["ReqValVDCommentEnabled"] = "true";
                e.Item.Attributes["RegExValHeightEnabled"] = "true";
            }

            if (choice.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
            {
                e.Item.Attributes["ReqValVDCommentEnabled"] = "true";
                e.Item.Attributes["RegExValPercentageEnabled"] = "true";
            }
        }

        protected void gvVisitData_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as Choice;

                if (dataItem != null && dataItem.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
                {
                    var reqValidator = (RequiredFieldValidator) e.Item.FindControl("reqValVDComment");
                    reqValidator.Enabled = true;

                    var regExValidator = (RegularExpressionValidator) e.Item.FindControl("regExValHeight");
                    regExValidator.Enabled = true;
                }

                if (dataItem != null &&
                    dataItem.Code == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
                {
                    var reqValidator = (RequiredFieldValidator) e.Item.FindControl("reqValVDComment");
                    reqValidator.Enabled = true;

                    var regExValidator = (RegularExpressionValidator) e.Item.FindControl("regExValHeight");
                    regExValidator.Enabled = true;
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox visitDataComboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                MapSelectField(visitDataComboBox, EventViewModel.VisitData);
            }
        }

        protected void gvVisitData_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvVisitData.EditIndexes.Add(e.Item.ItemIndex);
                gvVisitData.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvVisitData.EditIndexes.Clear();
                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.VisitData.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.VisitData.SelectedChoices.Remove(selectedChoice);

                    TextBox tbVisitDataComment = (TextBox) e.Item.FindControl("tbVisitDataComment");
                    selectedChoice.Comment = FormatComment(selectedChoice.Code, tbVisitDataComment.Text);

                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);
                }

                gvVisitData.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlVisitData");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.VisitData.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbVisitDataComment = (TextBox) e.Item.FindControl("tbVisitDataComment");
                    selectedChoice.Comment = FormatComment(selectedChoice.Code, tbVisitDataComment.Text);

                    EventViewModel.VisitData.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbVisitDataComment.Text = string.Empty;
                }
            }

            DataBindVisitDataGrid();
        }

        #endregion

        #region Private Methods

        private void DataBindVisitDataGrid()
        {
            gvVisitData.DataSource = EventViewModel.VisitData.SelectedChoicesOrderd;
            gvVisitData.DataBind();
        }

        private string FormatComment(string choiceCode, string commentText)
        {
            if (choiceCode == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaHeight)
            {
                if (!commentText.EndsWith("mm"))
                {
                    commentText = commentText.Trim() + " mm";
                }
            }
            else if (choiceCode == AnteriorChamberEventModel.ChoiceCodes.VisitData.HyphemaPercentage)
            {
                if (!commentText.EndsWith("%"))
                {
                    commentText = commentText.Trim() + "%";
                }
            }

            return commentText;
        }

        #endregion
    }
}