﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabImagingMethodologyUserControl : BaseEventItem<ImagingAncillaryEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            // Nothing Here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
        }

        protected override void MapTestDetails()
        {
            DataBindImagingMethodologyGrid();
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override ImagingAncillaryEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Page Events

        protected void gvImagingMethodology_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as ImagingAncillaryEventModel.ImagingMethodologyView;

                DateTime testDate;
                if (dataItem != null && DateTime.TryParse(dataItem.TestDate, out testDate))
                {
                    var tbTestDate = (RadDatePicker) e.Item.FindControl("tbTestDate");
                    tbTestDate.SelectedDate = testDate;
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox imagingMethodologyComboBox = (RadComboBox) e.Item.FindControl("ddlImagingMethodology");
                MapSelectField(imagingMethodologyComboBox, EventViewModel.ImagingMethodology);
            }
        }

        protected void gvImagingMethodology_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvImagingMethodology.EditIndexes.Add(e.Item.ItemIndex);
                gvImagingMethodology.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as ImagingAncillaryEventModel.ImagingMethodologyView;

                if (selectedChoice != null)
                {
                    EventViewModel.ImagingMethodology.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.ChoiceCode);
                    EventViewModel.TestDates[selectedChoice.ChoiceCode].ResponseText = String.Empty;
                    EventViewModel.TestResults[selectedChoice.ChoiceCode].ResponseText = String.Empty;
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvImagingMethodology.EditIndexes.Clear();
                gvImagingMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as ImagingAncillaryEventModel.ImagingMethodologyView;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.ImagingMethodology.SelectedChoices.FirstOrDefault(
                            c => c.Code == dataView.ChoiceCode)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.ImagingMethodology.SelectedChoices.Remove(selectedChoice);
                    EventViewModel.TestDates[selectedChoice.Code].ResponseText = string.Empty;
                    EventViewModel.TestResults[selectedChoice.Code].ResponseText = string.Empty;

                    TextBox tbMethodologyComment = (TextBox) e.Item.FindControl("tbMethodologyComment");
                    selectedChoice.Comment = tbMethodologyComment.Text;
                    EventViewModel.ImagingMethodology.SelectedChoices.Add(selectedChoice);

                    var testDate = (RadDatePicker) e.Item.FindControl("tbTestDate");
                    EventViewModel.TestDates[selectedChoice.Code].ResponseText =
                        testDate.SelectedDate.HasValue ? testDate.SelectedDate.Value.ToShortDateString() : string.Empty;

                    var testResult = (TextBox) e.Item.FindControl("tbTestResult");
                    EventViewModel.TestResults[selectedChoice.Code].ResponseText = testResult.Text;
                }

                gvImagingMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlImagingMethodology");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.ImagingMethodology.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbMethodologyComment = (TextBox) e.Item.FindControl("tbMethodologyComment");
                    selectedChoice.Comment = tbMethodologyComment.Text;
                    EventViewModel.ImagingMethodology.SelectedChoices.Add(selectedChoice);

                    var testDate = (RadDatePicker) e.Item.FindControl("tbTestDate");
                    EventViewModel.TestDates[selectedChoice.Code].ResponseText = 
                        testDate.SelectedDate.HasValue ? testDate.SelectedDate.Value.ToShortDateString() : string.Empty;

                    var testResult = (TextBox) e.Item.FindControl("tbTestResult");
                    EventViewModel.TestResults[selectedChoice.Code].ResponseText = testResult.Text;

                    comboBox.Text = string.Empty;
                    tbMethodologyComment.Text = string.Empty;
                    testDate.Clear();
                    testResult.Text = string.Empty;
                }
            }

            DataBindImagingMethodologyGrid();
        }

        protected void ddlImagingMethodology_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
        }

        #endregion

        private void DataBindImagingMethodologyGrid()
        {
            gvImagingMethodology.DataSource = EventViewModel.ImagingMethodologyViews;
            gvImagingMethodology.DataBind();
        }
    }
}