﻿using System;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Globe User Control
    /// </summary>
    public partial class ExamTabIntraocularPressureUserControl : BaseEventItem<IntraocularPressureEventModel>
    {
        #region Implemented Method

        protected override void MapValidationGroup()
        {
            // Nothing Here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = IntraocularPressureEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            lblMethod.Text = EventViewModel.Method.DisplayText;
            tbMethod.Text = EventViewModel.Method.ResponseText;

            lblMethodComment.Text = EventViewModel.MethodComment.DisplayText;
            tbMethodComment.Text = EventViewModel.MethodComment.ResponseText;

            lblNumber.Text = EventViewModel.Number.DisplayText;
            tbNumber.Text = EventViewModel.Number.ResponseText;

            lblTime.Text = EventViewModel.Time.DisplayText;

            TimeSpan time;
            if (TimeSpan.TryParse(EventViewModel.Time.ResponseText, out time))
            {
                tbTime.SelectedTime = time;
            }
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Globe Event Model</returns>
        protected override IntraocularPressureEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);
            EventViewModel.Method.ResponseText = tbMethod.Text;
            EventViewModel.MethodComment.ResponseText = tbMethodComment.Text;
            EventViewModel.Number.ResponseText = tbNumber.Text;
            EventViewModel.Time.ResponseText =
                tbTime.SelectedTime.HasValue
                    ? tbTime.SelectedTime.Value.ToString(@"hh\:mm")
                    : string.Empty;

            return EventViewModel;
        }

        #endregion
    }
}