﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabKineticVisualFieldsUserControl : BaseEventItem<KineticVisualFieldsEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            // Nothing to do here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = KineticVisualFieldsEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            DataBindMethodologyGrid();
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override KineticVisualFieldsEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void gvMethodology_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox imagingMethodologyComboBox = (RadComboBox) e.Item.FindControl("ddlMethodology");
                MapSelectField(imagingMethodologyComboBox, EventViewModel.Methodology);
            }
        }

        protected void gvMethodology_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMethodology.EditIndexes.Add(e.Item.ItemIndex);
                gvMethodology.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as KineticVisualFieldsEventModel.MethodologyView;

                if (selectedChoice != null)
                {
                    EventViewModel.Methodology.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.ChoiceCode);
                    EventViewModel.Results[selectedChoice.ChoiceCode].ResponseText = String.Empty;
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMethodology.EditIndexes.Clear();
                gvMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as KineticVisualFieldsEventModel.MethodologyView;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.Methodology.SelectedChoices.FirstOrDefault(
                            c => c.Code == dataView.ChoiceCode)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.Methodology.SelectedChoices.Remove(selectedChoice);
                    EventViewModel.Results[selectedChoice.Code].ResponseText = string.Empty;

                    TextBox tbMethodologyComment = (TextBox) e.Item.FindControl("tbMethodologyComment");
                    selectedChoice.Comment = tbMethodologyComment.Text;
                    EventViewModel.Methodology.SelectedChoices.Add(selectedChoice);

                    var result = (TextBox) e.Item.FindControl("tbResult");
                    EventViewModel.Results[selectedChoice.Code].ResponseText = result.Text;
                }

                gvMethodology.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox) e.Item.FindControl("ddlMethodology");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.Methodology.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbMethodologyComment = (TextBox) e.Item.FindControl("tbMethodologyComment");
                    selectedChoice.Comment = tbMethodologyComment.Text;
                    EventViewModel.Methodology.SelectedChoices.Add(selectedChoice);

                    var result = (TextBox) e.Item.FindControl("tbResult");
                    EventViewModel.Results[selectedChoice.Code].ResponseText = result.Text;

                    comboBox.Text = string.Empty;
                    tbMethodologyComment.Text = string.Empty;
                    result.Text = string.Empty;
                }
            }

            DataBindMethodologyGrid();
        }

        protected void ddlMethodology_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
        }

        #endregion

        private void DataBindMethodologyGrid()
        {
            gvMethodology.DataSource = EventViewModel.MethodologyViews;
            gvMethodology.DataBind();
        }
    }
}