﻿using System;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Pupil User Control
    /// </summary>
    public partial class ExamTabPupilUserControl : BaseEventItem<PupilEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            tbSize.ValidationGroup = ValidationGroup;
            rangeValSize.ValidationGroup = ValidationGroup;
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = PupilEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            lblSize.Text = EventViewModel.Size.DisplayText;
            tbSize.Text = EventViewModel.Size.ResponseText;

            lblAlterationInShape.Text = EventViewModel.AlterationInShape.DisplayText;
            tbAlterationInShape.Text = EventViewModel.AlterationInShape.ResponseText;

            lblPerrlOrPerrla.Text = EventViewModel.PerrlOrPerrla.DisplayText;
            ucPerrlOrPerrla.Question = EventViewModel.PerrlOrPerrla;

            lblReactive.Text = EventViewModel.Reactive.DisplayText;
            ucReactive.SelectedValue = EventViewModel.Reactive.IsYesSelected;

            lblAdie.Text = EventViewModel.Adie.DisplayText;
            ucAdie.SelectedValue = EventViewModel.Adie.IsYesSelected;

            lblArgyllRobertson.Text = EventViewModel.ArgyllRobertson.DisplayText;
            ucArgyllRobertson.SelectedValue = EventViewModel.ArgyllRobertson.IsYesSelected;
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Lens Event Model</returns>
        protected override PupilEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            EventViewModel.Size.ResponseText = tbSize.Text;
            EventViewModel.AlterationInShape.ResponseText = tbAlterationInShape.Text;

            EventViewModel.PerrlOrPerrla = ucPerrlOrPerrla.GetUpdatedQuestion();

            EventViewModel.Reactive.IsYesSelected = ucReactive.SelectedValue;
            EventViewModel.Adie.IsYesSelected = ucAdie.SelectedValue;
            EventViewModel.ArgyllRobertson.IsYesSelected = ucArgyllRobertson.SelectedValue;

            return EventViewModel;
        }

        #endregion
    }
}